/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.common;

/**
 * Exception thrown in gvSIGMobile 
 * @author mmontesinos
 *
 */
public class GvsigMobileException extends Exception {
	
	/**
	 * Default constructor
	 *
	 */
	public GvsigMobileException(){
		super();
	}

	/**
	 * Creates a new exception from a gvSIGMobile error code passed as a parameter
	 * It generates an error string with the error code and the corresponding
	 * error description appended
	 *
	 * @param errorCode Error Code (as defined in gvSIGMobile GvsigMobileGUIConstants class)
	 */
	public GvsigMobileException(int errorCode) {
		super("Error " + errorCode + ". " +  GvsigMobileGUIConstants.getErrorText(errorCode));
	}

	/**
	 * Creates a new exception from a gvSIGMobile error code passed as a parameter
	 * and a text description
	 * It generates an error string with the error code, the corresponding error
	 * description appended and the text message passed as a parameter
	 *
	 * @param errorCode Error Code (as defined in gvSIGMobile GvsigMobileGUIConstants class)
	 * @param message Additional error description to be generated
	 */
	public GvsigMobileException(int errorCode, String message) {
		super("Error " + errorCode + ". " +  GvsigMobileGUIConstants.getErrorText(errorCode) + " (" + message + ")");
	}

	/**
	 * Creates a new exception with the message passed as a parameter
	 *
	 * @param message Text of the message for the exception
	 */
	public GvsigMobileException(String message) {
		this(GvsigMobileGUIConstants.ERROR_UNSPECIFIED, message);
	}
	
	/**
	 * Creates a new exception with a message and the detail of the cause
	 *
	 * @param message Text of the message for the exception
	 * @param cause Cause of the exception
	 */
	public GvsigMobileException(String message, Throwable cause) {
		super(message, cause);
	}

}
