/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.bar;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;


/**
 * 
 * This class represents the status bar shown at the bottom of the application window
 * to display info: coordinates, distances, area, GPS information, etc.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.bar.StatusBarItem
 * 
 * @author jcarrasco
 *
 */
public class StatusBar extends Component {
	
	private static Logger logger = Logger.getLogger(StatusBar.class);
	
	public final static int STATUS_BAR_HEIGHT = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public final static int STATUS_BAR_TEXT_LOWER_GAP = 5 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	
	public static Image bgStrip = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "statusbar.png");
	// private String message = "";
	// private Image icon = null;
	// private Image subIcon = null;
	// private String oldMessage = "";
	// private Image oldIcon = null;
	// private Image oldSubIcon = null;

	private int bgStripLength = bgStrip.getWidth(null);
	private Color textColor = Color.WHITE;
	private ArrayList activeBars = new ArrayList();
	private HashMap removedBars = new HashMap();
	
	/**
	 * The Id of the whole status bars used in application
	 */
	public static final int MEASURES_BAR = 0;
	public static final int GPS_BAR = 1;
	/**
	 * Array with all the status bars that not disappear when selected tool changes
	 */
	public static final int[] PERMANENT_BARS = {GPS_BAR };
	
	/**
	 * Returns false if the indicated bar should hide when tool changes
	 */
	public boolean isPermanent(int id){
		boolean ispermanent = false;
		for (int i = 0; i<PERMANENT_BARS.length; i++){
			if (PERMANENT_BARS[i]==id){
				ispermanent = true;
				break;
			}
		}
		return ispermanent;
	}
	

	/**
	 * Adds a new bar to the statusbar stack
	 * if it is not yet on top of the stack
	 */ 
	public void addBar(int id){
		
		StatusBarItem aux = topBar();
		if (aux == null || aux.getId() != id){
			StatusBarItem aux2 = (StatusBarItem)removedBars.remove(new Integer(id));
			if (aux2==null)
				activeBars.add(new StatusBarItem(id));
			else
				activeBars.add(aux2);
		}
		refresh();
	}
	/**
	 * Removes the bar with the indicated id 
	 */
	public void removeBar(int id){
		//Stack auxStack = new Stack();
		for(int i = activeBars.size()-1;i>=0;i--){
			StatusBarItem aux = (StatusBarItem)activeBars.get(i);
			if (aux.getId() == id){
				activeBars.remove(i);
				if (isPermanent(id))
					removedBars.put(new Integer(id),aux);
				break;
			} 
		}
		refresh();
		
	}
	
	/**
	 * Removes bars that must disappear when tool changes
	 *
	 */
	public void removeNonPermanentBars(){
		for (int i = activeBars.size()-1;i>=0;i--){
			StatusBarItem aux = (StatusBarItem)activeBars.get(i);
			if (!isPermanent(aux.getId())){
				activeBars.remove(i);
			} 
		}
		refresh();
	}

	
	private void refresh() {
		if (topBar() != null) {
			setVisible(true);
		} else {
			setVisible(false);
		}
		repaint();
	}
	
	public StatusBarItem topBar(){
		if (activeBars.size()==0)
			return null;
		return (StatusBarItem)activeBars.get(activeBars.size()-1);
	}

	public StatusBar(int w, int h) {
		
		StatusBarItem gpsBar = new GPSStatusBarItem();
		
		ImageStatusBarIcon isbi =
			new ImageStatusBarIcon(
					ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0));
		ImageStatusBarIcon isbsi =
			new ImageStatusBarIcon(
					ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, GPSIcons.getSubIcon(0)));
		
		gpsBar.setIcon(isbi);
		gpsBar.setSubIcon(isbsi);
		
		removedBars.put(new Integer(StatusBar.GPS_BAR),gpsBar);
		reorder(w, h);
	}
	
	public void reorder(int w, int h) {
		setBounds(0, h - STATUS_BAR_HEIGHT, w, STATUS_BAR_HEIGHT);
	}
	
	public void setMessage(int id, String msg) {
		// logger.debug("StatusBar " + id + " setting message: " + msg);
		StatusBarItem bar = getStatusBar(id);
		if (bar!=null) {
			// logger.debug("bar is not null");
			bar.setMessage(msg);
			if (bar == topBar()){
				repaint();
				// logger.debug("bar is top bar");
			}
		}
	}
	/**
	 * Concatenates the new message with previous one if exists.
	 * 
	 * @param id
	 * @param msg
	 */
	public void concatMessage(int id, String msg){
		StatusBarItem bar = getStatusBar(id);
		if (bar!=null) {
			// logger.debug("bar is not null");
			String previous = bar.getMessage();
			bar.setMessage(previous + " " + msg);
			if (bar == topBar()) {
				repaint();
				// logger.debug("bar is top bar");
			}
		}
	}
	public StatusBarItem getStatusBar(int id){
		StatusBarItem res = null;
		for(int i = activeBars.size()-1;i>=0;i--){
			StatusBarItem aux = (StatusBarItem)activeBars.get(i);
			if (aux.getId() == id){
				res = aux;
				break;
			} 
		}
		return res;
	}
	
	/**
	 * Set icon shown on the right side of the status bar.
	 * 
	 * @param id status bar id, to indicate status bar type
	 * @param res_folder resources folder (usually IconResourceReader.IMG_SKIN_PATH)
	 * @param f_name icon file name
	 */
	public void setIcon(int id, String res_folder, String f_name) {
		
		StatusBarItem bar = getStatusBar(id);
		Image aux = ResourceReader.getResourceImage(res_folder, f_name);
		ImageStatusBarIcon isbi = new ImageStatusBarIcon(aux);
		
		if (bar!=null) {
			bar.setIcon(isbi);
			if (bar == topBar()){
				repaint();
			}
		}
	}
	/**
	 * Set sub icon of the status bar.
	 * 
	 * @param id status bar id, to indicate status bar type
	 * @param res_folder resources folder (usually IconResourceReader.IMG_SKIN_PATH)
	 * @param f_name icon file name
	 */
	public void setSubIcon(int id, String res_folder, String f_name) {
		
		StatusBarItem bar = getStatusBar(id);
		Image aux = ResourceReader.getResourceImage(res_folder, f_name);
		
		ImageStatusBarIcon isbsi = new ImageStatusBarIcon(aux);
		
		if (bar!=null) {
			bar.setSubIcon(isbsi);
			if (bar == topBar()){
				repaint();
			}
		}
	}

	/*
	private void setIcon(Image img) {
		icon = img;
	}
	*/
	
	/**
	 * Overriden paint method to draw the current icons and info text.
	 */
	public void paint(Graphics g) {
		
		int w = getWidth();
		
		// background strips
		for (int i=0; i<w; i = i + bgStripLength) {
			g.drawImage(bgStrip, i, 0, null);
		}

		StatusBarItem sbi = topBar();
		
		if (sbi == null) return;
		
		StatusBarIcon sbicon = sbi.getIcon(0);
		StatusBarIcon sbsicon = sbi.getIcon(1);
		String msg = sbi.getMessage();
		
		// icons
		
		int rightgap = 0;
		int ix = 0;
		
		
		if (sbicon != null) {
			rightgap = sbicon.getWidth();
			ix = w - rightgap;
			sbicon.paint(g, ix, 0);
			if (sbsicon != null)
				sbsicon.paint(g, ix, 2);
		}
		
		// other icons
		for (int i=2; i<StatusBarItem.MAX_NUMBER_ICONS; i++) {
			sbicon = sbi.getIcon(i);
			if (sbicon != null) {
				rightgap = rightgap + sbicon.getWidth();
				ix = w - rightgap;
				sbicon.paint(g, ix, 0);
			}
		}
		
		// message
		if ((msg != null) && (msg.length() > 0)) {
			g.setColor(textColor);
			g.setFont(DialogConstants.DIALOG_SMALL_FONT);
			g.drawString(
					msg,
					GvsigMobileGUIConstants.LABEL_LEFT_MARGIN,
					STATUS_BAR_HEIGHT - STATUS_BAR_TEXT_LOWER_GAP);
		}
	}

}
