package es.prodevelop.gvsig.mobile.gui.canvas;

import java.awt.Container;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.geom.Point2D;

import org.apache.log4j.Logger;
import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.listener.GPSRelevantEventsListener;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.core.acetate.AcetateTrail;
import es.prodevelop.gvsig.mobile.fmap.core.acetate.IAcetateElement;
import es.prodevelop.gvsig.mobile.fmap.core.acetate.MapCursor;
import es.prodevelop.gvsig.mobile.fmap.layer.FAcetateLayer;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.gui.bar.GPSIcons;
import es.prodevelop.gvsig.mobile.gui.bar.GPSStatusBarItem;
import es.prodevelop.gvsig.mobile.gui.bar.StatusBar;
import es.prodevelop.gvsig.mobile.icons.IconResourceReader;

public class MapCursorSupport implements GPSRelevantEventsListener {

	private static Logger logger = Logger.getLogger(MapCursorSupport.class);

	private MapCursor gps_cursor;
	// -------------------------------------- GPS
	// private GPSRelevantEventsListener gpsListener = null;
	private int oldGpsScreenX = 0;
	private int oldGpsScreenY = 0;
	private double gpsMapX = 0;
	private double gpsMapY = 0;
	private static int GPS_CURSOR_DIAMETER = 29
						* GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	private static int GPS_CURSOR_RADIUS = 0; // written by app
	private static GPSManager gpsManager;
	private boolean gpsPositionVisible = false;
	// --------------------------------------------

	private MapControl theMapControl;


	public MapCursorSupport(MapControl mc) {

		theMapControl = mc;
		gps_cursor = new MapCursor(IconResourceReader.GPS_CURSOR_ICON[0]);
		gps_cursor.setVisible(false);

		// ------------- gps
		gpsManager = GPSManager.getGPSManager();
		gpsManager.addGPSRelevantEventsListener(this);
//		GPS_CURSOR_RADIUS = 2 + GPS_CURSOR_DIAMETER / 2
		GPS_CURSOR_RADIUS = 1 + GPS_CURSOR_DIAMETER / 2;
	}

	public void drawCursor(Graphics g, int offx, int offy) {

		if (gps_cursor.isVisible()){
			int off_x = gps_cursor.getX();
			int off_y = gps_cursor.getY();
			g.drawImage(gps_cursor.getIcon(),
					off_x + offx, off_y + offy, null);
		}
	}

	public boolean getGpsPositionVisible() {
		return gpsPositionVisible;
	}

	public void addCursor(Container map_control) {

		if (gps_cursor != null) {
			map_control.add(gps_cursor);
		} else {
			logger.error("Cursor is NULL");
		}
	}

	/**
	 * Set GPS position visibility
	 *
	 */
	public void setViewGPSPosition(boolean b) {
		//gps_cursor.setVisible(!gps_cursor.isVisible());
		gpsPositionVisible = b;
		gps_cursor.setVisible(b && gpsManager.isShowIcon());
	}

	/**
	 * Switch activation of GPS position visibility
	 *
	 */
	public void switchViewGPSPosition(){
		//gps_cursor.setVisible(!gps_cursor.isVisible());
		gpsPositionVisible = !gpsPositionVisible;
		gps_cursor.setVisible(gpsPositionVisible && gpsManager.isShowIcon());
	}

	// ------------------

	/**
	 * New GPS event: new position (this is the main GPS event)
	 * @param longitude in EPSG:4326 (WGS84 datum)
	 * @param latitude in EPSG:4326 (WGS84 datum)
	 */
	public void gpsNewPosition(double longitude, double latitude) {

		gpsMapX = longitude;
		gpsMapY = latitude;

		ViewPort vp = theMapControl.getProject().getMapContext().getViewPort();
		IProjection the_projection = theMapControl.getProject().getProjection();

		String units = " " + the_projection.getUnitsAbbrev(); //$NON-NLS-1$
		int decimals = 2;

		if (GeoUtils.isLatLonProjection(the_projection.getAbrev())) {
			decimals = 5;
		} else {
			decimals = 2;
		}

		double[] coords = ConversionCoords.wgs842projection(
				longitude,
				latitude,
				the_projection);

		gpsMapX=coords[0];
		gpsMapY=coords[1];

		theMapControl.setStatusBarText(StatusBar.GPS_BAR,
				Utils.getFormattedDouble(gpsMapX,decimals) + units + ", " //$NON-NLS-1$
				+ Utils.getFormattedDouble(gpsMapY,decimals) + units);

		if (gpsPositionVisible) {
			Point2D scp;// = vp.fromMapPoint(oldGpsMapX, oldGpsMapY);

			int centerX = vp.getImageWidth()/2;
			int centerY = vp.getImageHeight()/2;

			scp = vp.fromMapPoint(gpsMapX, gpsMapY);
			int screen_x = (int) Math.round(scp.getX());
			int screen_y = (int) Math.round(scp.getY());

			if (gps_cursor.isVisible())
				theMapControl.repaint(
						oldGpsScreenX - GPS_CURSOR_RADIUS,
						oldGpsScreenY - GPS_CURSOR_RADIUS,
						GPS_CURSOR_DIAMETER + 4,
						GPS_CURSOR_DIAMETER + 4);

			if (gpsManager.isRecenter())
				if (Math.abs(screen_x - centerX)>20)
					if (Math.abs(screen_y - centerY)>30)
						theMapControl.recenter(screen_x,screen_y);

			scp = vp.fromMapPoint(gpsMapX, gpsMapY);
			screen_x = (int) Math.round(scp.getX());
			screen_y = (int) Math.round(scp.getY());
			if (gps_cursor.isVisible()){
				//if (screen_x!=screen_x_old || screen_y!=screen_y_old){
					// logger.debug("setting gps cursor position " + screen_x + ", " + screen_y); //$NON-NLS-1$ //$NON-NLS-2$
					gps_cursor.setBounds(screen_x - GPS_CURSOR_RADIUS,screen_y - GPS_CURSOR_RADIUS,GPS_CURSOR_DIAMETER,GPS_CURSOR_DIAMETER);
					theMapControl.repaint(screen_x - GPS_CURSOR_RADIUS,
							screen_y - GPS_CURSOR_RADIUS,
							GPS_CURSOR_DIAMETER + 4,
							GPS_CURSOR_DIAMETER + 4);
					oldGpsScreenX = screen_x;
					oldGpsScreenY = screen_y;
				//}
			}
		}

		FAcetateLayer ace_lyr = theMapControl.getMapContext().getAcetateLayer();

		if (ace_lyr != null) {
			IAcetateElement acet_ele = ace_lyr.getElement(FAcetateLayer.GPS_CURSOR_ACETATE_ID);

			if (acet_ele != null) {
				AcetateTrail trail = (AcetateTrail) acet_ele;
				trail.addVertex(gpsMapX, gpsMapY);
			} else {
				logger.error("No GPS cursor found in acetate layer");
			}
		} else {
			logger.error("No acetate layer found");
		}

		if (gps_cursor.isVisible()) {
			theMapControl.repaint();
		}

	}

	/**
	 * New GPS event: new speed data
	 * @param speed the speed computed by the GPS device
	 */
	public void gpsNewSpeed(double speed) {
		// TODO Auto-generated method stub

	}

	/**
	 * New GPS event : altitude
	 * @param altitude the new altitude computed by the GPS device
	 */
	public void gpsNewAltitude(double altitude) {
		// TODO Auto-generated method stub

	}

	/**
	 * New GPS event : satellites used
	 * @param satUsed the number of satellites currently detected and used by the GPS device
	 */
	public void gpsNewSatsUsed(int satUsed) {
		theMapControl.setStatusBarSubIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.getSubIcon(satUsed)); //$NON-NLS-1$

	}

	/**
	 * New GPS event : PDOP value
	 * @param pdop a value regarding the quality of the GPS data
	 */
	public void gpsNewPDOP(double pdop) {

		if (gpsStatus!=0){
			if (pdop > 10 || pdop==0.0){
				theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0); //$NON-NLS-1$
				return;
			}
			if (pdop > 8 ){
				theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco1); //$NON-NLS-1$
				return;
			}
			if (pdop > 6){
				theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco2); //$NON-NLS-1$
				return;
			}
			if (pdop > 4){
				theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco3); //$NON-NLS-1$
				return;
			}
			if (pdop > 2){
				theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco4); //$NON-NLS-1$
				return;
			}
			theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco5); //$NON-NLS-1$
		}

	}

	/**
	 * New GPS event : quality value
	 * @param quality a value regarding the quality of the GPS data
	 */
	public void gpsNewQuality(int quality) {
		// TODO Auto-generated method stub

	}

	protected int gpsStatus = 0;

	/**
	 * New GPS event : status value
	 * @param status a value regarding the state of the GPS device
	 */
	public void gpsNewStatus(int status) {
		gpsStatus = status;
		if (status==0)
			theMapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0); //$NON-NLS-1$
	}

	public void gpsNewHeading(double heading) {

		int ind = ResourceReader.getGpsIconIndexForAngle(heading, false);
		Image icon = IconResourceReader.GPS_CURSOR_ICON[ind];
		gps_cursor.setIcon(icon);
	}

	/**
	 * Returns a new value for the Starfire quality value
	 * @param starfire an integer between 0 and 26
	 */
	public void gpsNewStarfire(int starfire) {
		// logger.debug("Juan, pinta el icono del Starfire :P " + starfire);
		logger.debug("New Starfire quality level: " + starfire);
		theMapControl.setStatusBarDataValue(
				StatusBar.GPS_BAR,
				GPSStatusBarItem.NMEA_DATA_ID_NAVCOM_STARTFIRE_QUALITY,
				new Integer(starfire));
	}

	/**
	 * Returns a new value for the RMS standard deviation of
	 * the ranges inputs for the navigation solution
	 * @param rms a positive double value
	 */
	public void gpsNewRMS(double rms){
		logger.debug("New RMS value: " + rms);
		theMapControl.setStatusBarDataValue(
				StatusBar.GPS_BAR,
				GPSStatusBarItem.NMEA_DATA_ID_NAVCOM_RMS,
				new Double(rms));
	}

}
