/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;


/**
 * Customized file name filter. Simply needs to indicate the allowed extension in
 * the constructor. Does not work in J9. Does work in Phoneme.
 * 
 * @author jldominguez
 *
 */
public class CustomizedFilenameFilter implements FilenameFilter {
	
	private String ext = "";
	private ArrayList extensions = null;
	
	/**
	 * Constructor
	 * @param _ext allowed extension
	 */
	public CustomizedFilenameFilter(String _ext) {
		ext = _ext;
	}
	
	/**
	 * Adds an allowed extension
	 * @param _ext allowed extension
	 */
	public void addAllowExtension(String _ext){
		if (extensions == null){
			extensions = new ArrayList();
			if (ext!=null)
				extensions.add(ext);
		}
		extensions.add(_ext);
	}
	
	/**
	 * Method to find out whether a file is accepted by the filter.
	 * 
	 * @param f file object to be treated
	 * @param name file name
	 */
	public boolean accept(File f, String name) {
		//return arg0.getAbsolutePath().endsWith("." + ext);
		//System.out.println(arg1.endsWith("." + ext));
		if (extensions==null)
			return name.endsWith("." + ext);
		else{
			for(int i=0;i<extensions.size();i++){
				String sufix = (String) extensions.get(i);
				if (name.endsWith("."+sufix)) {
					return true;
				}
			}
		}
		return false;
	}

}
