/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Choice;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gpe.writer.ExportGpeWriter;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTable;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.settings.AppSettings;

/**
 * Only for vectorial layers.
 * This dialog is used to perform export vectorial shapes by a layer selection.
 * The user chooses a layer, an export format and a file name. The initials formats to export are 
 * GML v.2.x, GML v.3.x KML v.2.1 from Shapefiles, GML,KML, GPX...
 * 
 * When the user clicks on "OK", The file will be created. 
 *  
 * @author Carlos Snchez Perin
 *
 */
public class ExportDialog extends ExpansibleWindow implements MouseListener, ItemListener {
	
	private static final long serialVersionUID = 1L;
	private static Logger logger = Logger.getLogger(SearchDialog.class);
	
	private LayersTable toc = null;
	private Frame frame = null;
	
	private String saveDirectory = null;
	
	private Label titleLabel = null;
	private Choice lyrChoice = null;
	private Label lyrLabel;
	private Choice formatChoice = null;
	private Label formatLabel;
	private TextField fileNameField = null;
	private Label fileNameLabel;
	private TocButton pathButton;
	
	private TocButton okButton, cancelButton;

	/**
	 * Constructor. Needs the TOC to list all the available vector layers.
	 * 
	 * @param _toc the current toc, where the dialog will search for vector layers.
	 * @param p parent frame, needed for graphical purposes
	 */
	public ExportDialog(LayersTable _toc, Frame p) {
		super(p);
		frame = p;
		toc = _toc;
		if (isPhoneME()){
			setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		}else{
			setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		}
		init();
		setVisible(true);
	}
	
	private void init() {
		
		cancelButton = new TocButton(DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		cancelButton.addMouseListener(this);
		okButton = new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, "toc_ok_dis_small.png", 1, true, false); //$NON-NLS-1$
		okButton.addMouseListener(this);
		
		titleLabel = new Label(Messages.getText("ExportDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);

		lyrLabel = new Label(Messages.getText("ExportDialog.1")); //$NON-NLS-1$
		lyrLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		formatLabel = new Label(Messages.getText("ExportDialog.2")); //$NON-NLS-1$
		formatLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		fileNameLabel = new Label(Messages.getText("ExportDialog.3")); //$NON-NLS-1$
		fileNameLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

		lyrChoice = getLyrChoice(toc, okButton);
		lyrChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		formatChoice = getFormats();
		formatChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		fileNameField = getFile();
		fileNameField.setFont(DialogConstants.DIALOG_SMALL_FONT);
		pathButton = new TocButton("folder_transp.png",true);
        pathButton.addMouseListener(this);
        
   		titleLabel.setSize(DialogConstants.LABELS_WIDTH-2, DialogConstants.CONTROL_HEIGHT);
		lyrLabel.setSize(DialogConstants.LABELS_WIDTH-2, DialogConstants.TABLE_LABEL_HEIGHT);
		lyrChoice.setSize(DialogConstants.LABELS_WIDTH-2, DialogConstants.TABLE_LABEL_HEIGHT);
		formatLabel.setSize(DialogConstants.LABELS_WIDTH+45, DialogConstants.TABLE_LABEL_HEIGHT);
		formatChoice.setSize(DialogConstants.LABELS_WIDTH-2, DialogConstants.TABLE_LABEL_HEIGHT);
		fileNameLabel.setSize(DialogConstants.LABELS_WIDTH+45, DialogConstants.TABLE_LABEL_HEIGHT);
		fileNameField.setSize(DialogConstants.LABELS_WIDTH-2, DialogConstants.TABLE_LABEL_HEIGHT);
		
		// -------------------
		
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row00.addExtensible(titleLabel);
		add(row00);
		// --------------------
		ExpansibleRow row0 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row0.add(lyrLabel); row0.addExtensible(lyrChoice);
		add(row0);
		// --------------------
		ExpansibleRow row01 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row01.add(formatLabel); row01.addExtensible(formatChoice);
		add(row01);
		// --------------
		ExpansibleRow row2 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row2.add(fileNameLabel); row2.addExtensible(fileNameField);row2.add(pathButton);
		add(row2);
		// --------------
		ExpansibleRow row5 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		addExpansible(row5);
		// --------------
		ExpansibleRow row6 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row6.add(okButton); row6.add(cancelButton);
		add(row6);
		// --------------
		addEmptyRowIfPhoneME(this);
	}
	

	private Choice getLyrChoice(LayersTable t, TocButton b) {
		
		FLayer[] ll = t.getLayers();
		Choice resp = new Choice();
		
		for (int i=0; i<ll.length; i++) {
			FLayer lyr = ll[i];
			if (lyr instanceof FLyrVect) {
				resp.addItem(ll[i].getName());
			}
		}
		b.setEnabled(resp.getItemCount() > 0);
		resp.addMouseListener(this);
		resp.addItemListener(this);
		return resp;
	}

	private TextField getFile() {
		return new TextField();
	}

	private Choice getFormats() {
		Choice resp = new Choice();
		resp.addItem("GML 3.2 (SFP Simple Feature Profile)");
		resp.addItem("GML 2.x (Geographic Markup Language)");
		resp.addItem("KML 2.1 (KeyMap Markup Language)");
		resp.addItem("SHP");
		return resp;
	}

	private FLyrVect getSelectedLayer() {
		String lyrname = lyrChoice.getSelectedItem();
		
		if (lyrname == null) return null;

		FLayer[] ll = toc.getLayers();
		for (int i=0; i<ll.length; i++) {
			FLayer lyr = ll[i];
			if (lyr instanceof FLyrVect) {
				if (lyr.getName().compareToIgnoreCase(lyrname) == 0) {
					return (FLyrVect) lyr;
				}
			}
		}
		return null;
	}

	public void mousePressed(MouseEvent e) {
		
		Object src = e.getSource();
		
		if (src == pathButton) {
			
			FileDialog dlg = new FileDialog(frame, Messages.getText("ProjectPropertiesDialog.22"), FileDialog.LOAD); //$NON-NLS-1$
			
			if (saveDirectory==null){
				dlg.setDirectory(AppSettings.defaultFormsSavePath.getPath());
			} else {
				dlg.setDirectory(saveDirectory);
			}

			dlg.setModal(true);
			dlg.show();

			String getFile = dlg.getFile();
			if (getFile == null) return;
			
			saveDirectory = dlg.getDirectory();
			fileNameField.setText(dlg.getFile()); //$NON-NLS-1$
		}
		if (src == okButton) {
			hide();
			toc.getMapControl().repaint();
			executeExport();
		}

		if (src == cancelButton) {
			hide();
		}
	}

	private void executeExport() {
		FLyrVect vectLayer = getSelectedLayer();
			
		if (getSelectedLayer() == null) return;
		if (saveDirectory!=null){
			String writer = formatChoice.getSelectedItem();
			ExportGpeWriter gpeWriter = new ExportGpeWriter();
			try {
				String aux = fileNameField.getText();
				gpeWriter.write((FLyrVect) vectLayer, saveDirectory + File.separator + aux, writer);
			}catch (Exception e) {
				logger.error("While exporting vector layer: " + e.getMessage());
			}		
		}
		return;
	}

	public void mouseEntered(MouseEvent e) { }
	public void mouseExited(MouseEvent e) { }
	public void mouseClicked(MouseEvent e) { }
	public void mouseReleased(MouseEvent e) { }

	public void itemStateChanged(ItemEvent e) {
		
		Object src = e.getSource();
		
		if (src == lyrChoice) {
			//Reload Proposed Name
		}

	}

}
