/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;

/**
 * This dialog shows the info retrieved by the info by point tool from
 * various layers. It's a simple 2-column table. Layer names and field names to the left
 * and values to the right. 
 * 
 * @author jldominguez
 *
 */
public class InfoAttributesDialog extends ExpansibleWindow implements MouseListener {
	
	private static Logger logger = Logger.getLogger(InfoAttributesDialog.class);
	
	private Frame parent = null;
	
	private TocButton okButton;

	private int fld_values_col_w = 20;
	private int fld_names_col_w = 20;
	private String[] ff, vv;
	
	/**
	 * Constructor. For titles, the value of the values array is null.
	 * In that case, the content of the names array (title) will be
	 * shown in bold face.
	 * 
	 * @param p parent frame
	 * @param _ff array of left-side strings (generally, field names)
	 * @param _vv array of right-side strings 8generally, field values)
	 */
	public InfoAttributesDialog(Frame p, String[] _ff, String[] _vv) {
		
		super(p);
		parent = p;
		
		ff = _ff;
		vv = _vv;
		
		setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		// setModal(true);
		init();
		
		setVisible(true);
	}

	private void init() {
		
		Label title = new Label(Messages.getText("InfoAttributesDialog.0")); //$NON-NLS-1$
		title.setFont(DialogConstants.DIALOG_BOLD_FONT);
		title.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		title.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		title.setAlignment(1);
		
		okButton = new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, "toc_ok_dis_small.png", 1, true, false); //$NON-NLS-1$
		okButton.addMouseListener(this);

		ScrollPane scr = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scr.add(createPseudoTable(ff, vv, scr.getFontMetrics(DialogConstants.DIALOG_SMALL_FONT)));
		scr.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row00.addExtensible(title); add(row00);
		// --------------------
		 ExpansibleRow row0 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		 row0.addExtensible(scr);
		 addExpansible(row0);
		// --------------------
		ExpansibleRow row01 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row01.add(okButton);
		add(row01);
		
		// --------------
		addEmptyRowIfPhoneME(this);
		// -------------------
		scr.repaint();
	}
	
	public void mouseClicked(MouseEvent e) {}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {
		Object src = e.getSource();
		if (src == okButton) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("InfoAttDlg. A mysterious NP exception was caught: " + npe.getMessage()); //$NON-NLS-1$
			}
		}
	}
	public void mouseReleased(MouseEvent e) {}
	
	private int getMaxStringWidth(String[] ss, FontMetrics fm) {
		
		int resp = 0;
		
		for (int i=0; i<ss.length; i++) {
			if (ss[i] != null) {
				int aux = fm.stringWidth(ss[i]);
				if (aux > resp) resp = aux; 
			}
		}
		
		return resp;
	}
	
	
	private Panel createPseudoTable(
			String[] field_names,
			String[] field_values,
			FontMetrics fom) {
		
		Panel resp = new Panel();
		resp.setLayout(null);

		fld_names_col_w = 20 + getMaxStringWidth(field_names, fom);

		fld_values_col_w = 35 + getMaxStringWidth(field_values, fom);

		int cnt = field_names.length;

		resp.setBounds(0, 0, fld_names_col_w + fld_values_col_w, cnt * DialogConstants.TABLE_LABEL_HEIGHT);

		for (int i=0; i<cnt; i++) {
			Label f = getNewLabel(field_names[i], i, false);
			Label v = getNewLabel(field_values[i], i, true);
			
			if (v == null) {
				
				if (f == null) {
					// separator
					
				} else {
					// title
					// f.setBounds(0, i * ROW_HEIGHT, fld_names_col_w + fld_values_col_w, ROW_HEIGHT);
					// f.setAlignment(Label.CENTER);
					f.setFont(DialogConstants.DIALOG_BOLD_FONT);
					resp.add(f);
				}
			} else {
				// pair
				resp.add(f);
				resp.add(v);
			}
		}
		
		return resp;
	}
	
	private Label getNewLabel(String s, int index, boolean isvalue) {
		
		if (s == null) return null;
		
		Label resp = new Label(" " + s); //$NON-NLS-1$

		if (isvalue) {
			
			if (s.indexOf("http") == 0) { //$NON-NLS-1$
				resp = new UrlLabel(Messages.getText("InfoAttributesDialog.5"), s, parent); //$NON-NLS-1$
			}
			
			resp.setBounds(fld_names_col_w, index * DialogConstants.TABLE_LABEL_HEIGHT, fld_values_col_w, DialogConstants.CONTROL_HEIGHT);
		} else {
			resp.setBounds(0, index * DialogConstants.TABLE_LABEL_HEIGHT, fld_names_col_w, DialogConstants.CONTROL_HEIGHT);
		}
		resp.setFont(DialogConstants.DIALOG_SMALL_FONT);
		return resp;
	}

}
