/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.Checkbox;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrDefault;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.settings.AppSettings;

/**
 * This dialog shows the layer's properties (and allows edition of most of them).
 * It's simply an expansible dialog with the following components: layer name, layer's 
 * main source file (if there is one, non editable), visibility, infoability,
 * max/min scale and a comment.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.layer.FLayer
 * @see es.prodevelop.gvsig.mobile.gui.dialog.LayersManagerDialog
 *  
 * @author jldominguez
 *
 */
public class LayerPropertiesDialog extends ExpansibleWindow implements MouseListener {
	
	private static Logger logger = Logger.getLogger(LayerPropertiesDialog.class);
	
	private FLayer layer;
	private LayersManagerDialog parent;
	private Frame mFrame;
	private TocButton okButton, cancelButton;
	
	private boolean visible = false;
	private boolean infoable = false;
	private int w = 0;

	private Label titleLabel, nameLabel, fileNameLabel,
	visibleLabel, infoLabel, maxScaleLabel, minScaleLabel, customFormLabel, memoLabel;
	
	private TextField nameTextField, fileNameTextField;
	private TextArea memoArea;
	private Checkbox visibleCB, infoCB, customFormActiveCB;
	private Checkbox maxScaleActiveCB, minScaleActiveCB;
	private TextField maxScaleTF, minScaleTF, customFormTF;
	private TocButton customPathButton;
	
	private String customFormPath ;
	private String customFormDirectory;

	/**
	 * Constructor. 
	 * 
	 * @param lyr the layer whose properties are going to be shown
	 * @param p the parent dialog (TOC). Not only for graphic purposes. It has to be notified of
	 * changes in the layer properties.
	 * @param mf the parent frame
	 */
	public LayerPropertiesDialog(FLayer lyr, LayersManagerDialog p, Frame mf) {
		super(mf);
		
		mFrame = mf;
		parent = p;
		layer = lyr;
		w = parent.getWidth();

		setBounds(parent.getX(), parent.getY(), w, parent.getHeight());
		// setModal(true);
		init();
	}
	
	private void init() {
		
		cancelButton = new TocButton(DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		cancelButton.addMouseListener(this);
		okButton = new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 1, true, false);
		okButton.addMouseListener(this);

		// --------- get ------------
		visible = layer.isVisible();
		infoable = layer.isInfoable();

		// -------- set -----------
		titleLabel = new Label(Messages.getText("LayerPropertiesDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);
		
		nameLabel = new Label(Messages.getText("LayerPropertiesDialog.1")); //$NON-NLS-1$
		nameLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		fileNameLabel = new Label(Messages.getText("LayerPropertiesDialog.2")); //$NON-NLS-1$
		fileNameLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		memoLabel = new Label(Messages.getText("LayerPropertiesDialog.3")); //$NON-NLS-1$
		memoLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		visibleLabel = new Label(Messages.getText("LayerPropertiesDialog.4")); //$NON-NLS-1$
		visibleLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		infoLabel = new Label(Messages.getText("LayerPropertiesDialog.5")); //$NON-NLS-1$
		infoLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		maxScaleLabel = new Label(Messages.getText("LayerPropertiesDialog.6")); //$NON-NLS-1$
		maxScaleLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		minScaleLabel = new Label(Messages.getText("LayerPropertiesDialog.7")); //$NON-NLS-1$
		minScaleLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		customFormLabel = new Label(Messages.getText("LayerPropertiesDialog.8")); //$NON-NLS-1$
		customFormLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		customPathButton = new TocButton("folder_transp.png",true);
        customPathButton.addMouseListener(this);
		
		nameTextField = new TextField();
		nameTextField.setText(layer.getName());
		nameTextField.setFont(DialogConstants.DIALOG_SMALL_FONT);
		fileNameTextField = new TextField();

		String fname = getLayerFilePath(layer);
		if (fname == null) {
			fileNameTextField.setText(Messages.getText("LayerPropertiesDialog.18"));
		} else {
			File f = new File(fname);
			fileNameTextField.setText(f.getName());
		}
		
		fileNameTextField.setFont(DialogConstants.DIALOG_SMALL_FONT);
		
		memoArea = new TextArea();
		memoArea.setText(layer.getMemo());
		memoArea.setFont(DialogConstants.DIALOG_SMALL_FONT);
		
		visibleCB = new Checkbox(); //$NON-NLS-1$
		visibleCB.setState(visible);
		maxScaleActiveCB = new Checkbox(""); //$NON-NLS-1$
		minScaleActiveCB = new Checkbox(""); //$NON-NLS-1$
		//Custom form checkBox added
		customFormActiveCB = new Checkbox(""); //$NON-NLS-1$
		
		maxScaleTF = new TextField(""); //$NON-NLS-1$
		maxScaleTF.setFont(DialogConstants.DIALOG_SMALL_FONT);
		minScaleTF = new TextField(""); //$NON-NLS-1$
		minScaleTF.setFont(DialogConstants.DIALOG_SMALL_FONT);
		
		//Custom form text field to include form file name added
		if(getCustomFormPath(layer)==null){
			customFormTF = new TextField(Messages.getText("LayerPropertiesDialog.18"));
			customFormTF.setFont(DialogConstants.DIALOG_SMALL_FONT);
			customFormActiveCB.setState(false);
		}
		else{
			File FormName = new File(getCustomFormPath(layer));
			customFormTF = new TextField(FormName.getName()); //$NON-NLS-1$
			customFormTF.setFont(DialogConstants.DIALOG_SMALL_FONT);
		}
		if (layer.getMaxScale() == -1) {
			maxScaleActiveCB.setState(false);
		} else {
			maxScaleActiveCB.setState(true);
			maxScaleTF.setText("" + Math.round(layer.getMaxScale())); //$NON-NLS-1$
		}
		
		if (layer.getMinScale() == -1) {
			minScaleActiveCB.setState(false);
		} else {
			minScaleActiveCB.setState(true);
			minScaleTF.setText("" + Math.round(layer.getMinScale())); //$NON-NLS-1$
		}
		
		customFormPath = layer.getCustomForm();
		if (customFormPath == null) {
			customFormActiveCB.setState(false);
		} else {
			customFormActiveCB.setState(true);
			File FormName = new File(getCustomFormPath(layer));
			customFormTF.setText(FormName.getName()); //$NON-NLS-1$
		}
		
		infoCB = new Checkbox(""); //$NON-NLS-1$
		infoCB.setState(infoable);
		// -----------------------

		titleLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		
		nameLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		nameTextField.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		
		fileNameLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		fileNameTextField.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		fileNameTextField.setEditable(false);
		
		memoLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		memoArea.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);

		visibleLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		visibleCB.setSize(DialogConstants.SMALL_CONTROL_HEIGHT, DialogConstants.TABLE_LABEL_HEIGHT);
		
		customFormLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		customFormActiveCB.setSize(DialogConstants.SMALL_CONTROL_HEIGHT, DialogConstants.TABLE_LABEL_HEIGHT);
		customFormTF.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		
		infoLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		infoCB.setSize(DialogConstants.SMALL_CONTROL_HEIGHT, DialogConstants.TABLE_LABEL_HEIGHT);
		
		maxScaleLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		maxScaleActiveCB.setSize(DialogConstants.SMALL_CONTROL_HEIGHT, DialogConstants.TABLE_LABEL_HEIGHT);
		maxScaleTF.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);

		minScaleLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		minScaleActiveCB.setSize(DialogConstants.SMALL_CONTROL_HEIGHT, DialogConstants.TABLE_LABEL_HEIGHT);
		minScaleTF.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.TABLE_LABEL_HEIGHT);
		
		// --------------
		ExpansibleRow row00 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row00.addExtensible(titleLabel);
		add(row00);
		// --------------------
		ExpansibleRow row0 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row0.add(nameLabel); row0.addExtensible(nameTextField);
		add(row0);
		// --------------------
		ExpansibleRow row01 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row01.add(fileNameLabel); row01.addExtensible(fileNameTextField);
		add(row01);
		// --------------
		ExpansibleRow row1 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row1.add(visibleLabel); row1.add(visibleCB);
		add(row1);
		// --------------
		ExpansibleRow row2 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row2.add(infoLabel); row2.add(infoCB);
		add(row2);
		// --------------
		// --------------
		if(layer instanceof FLyrVect){
			ExpansibleRow row3 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
			row3.add(customFormLabel); row3.add(customFormActiveCB);
			row3.addExtensible(customFormTF);row3.add(customPathButton);
			add(row3);
		}
		// --------------
		ExpansibleRow row4 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row4.add(maxScaleLabel); row4.add(maxScaleActiveCB);
		row4.addExtensible(maxScaleTF);
		add(row4);
		// --------------
		ExpansibleRow row5 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row5.add(minScaleLabel); row5.add(minScaleActiveCB);
		row5.addExtensible(minScaleTF);
		add(row5);
		// -------------------
		ExpansibleRow row6 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row6.add(memoLabel); row6.addExtensible(memoArea);
		addExpansible(row6);
		// --------------
		ExpansibleRow row7 = new ExpansibleRow(DialogConstants.TABLE_LABEL_HEIGHT);
		row7.add(okButton); row7.add(cancelButton);
		add(row7);
		// --------------
		addEmptyRowIfPhoneME(this);
	}

	public void actionPerformed(ActionEvent arg0) {
		
	}

	private void setUserDataToLayer() {
		layer.setName(nameTextField.getText());
		layer.setMemo(memoArea.getText());
		layer.setVisible(visibleCB.getState());
		layer.setInfoable(infoCB.getState());
		layer.setCustomFormActive(customFormActiveCB.getState());
		if (customFormActiveCB.getState()){
			layer.setCustomForm(this.customFormPath);
		}else{
			layer.setCustomForm(null);
		}
		if (maxScaleActiveCB.getState()) {
			long sc = -1;
			try {
				sc = Long.parseLong(maxScaleTF.getText());
			} catch (Exception ex) {
				logger.debug("Not a valid scale, value set to -1 (no restriction)."); //$NON-NLS-1$
				sc = -1;
			}
			layer.setMaxScale(sc);
		} else {
			layer.setMaxScale(-1);
		}

		if (minScaleActiveCB.getState()) {
			long sc = -1;
			try {
				sc = Long.parseLong(minScaleTF.getText());
			} catch (Exception ex) {
				logger.debug("Not a valid scale, value set to -1 (no restriction)."); //$NON-NLS-1$
				sc = -1;
			}
			layer.setMinScale(sc);
		} else {
			layer.setMinScale(-1);
		}			
	}

	public void mousePressed(MouseEvent arg0) {
		Object src = arg0.getSource();
		
		if (src == customPathButton) {
			
			FileDialog dlg = new FileDialog(mFrame, Messages.getText("ProjectPropertiesDialog.22"), FileDialog.LOAD); //$NON-NLS-1$
			int pos = 0;
			if (customFormPath==null){
				dlg.setDirectory(AppSettings.defaultFormsSavePath.getPath());
			}
			else{
				pos=customFormPath.lastIndexOf(File.separator);
				if (pos != -1){
					customFormDirectory = customFormPath.substring(0, pos);
					dlg.setDirectory(customFormDirectory);
				}
				else{
					dlg.setDirectory(AppSettings.defaultFormsSavePath.getPath());
				}
			}
			dlg.setModal(true);
			dlg.show();

			String getFile = dlg.getFile();
			if (getFile == null) return;
			
			String absolute_path = dlg.getDirectory() + dlg.getFile();
			customFormPath=absolute_path;
			File FormName = new File(customFormPath);
			customFormTF.setText(FormName.getName()); //$NON-NLS-1$
		}
		
		if (src == okButton) {
			setUserDataToLayer();
			parent.updateLayer(layer);
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
			}
		}
		
		if (src == cancelButton) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
			}
		}
	}

	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) { }
	public void mouseClicked(MouseEvent arg0) { }
	
	private String getLayerFilePath(FLayer lyr) {
		
		String aux = Messages.getText("LayerPropertiesDialog.18"); //$NON-NLS-1$

		if (lyr instanceof FLayerFileVectorial) {
			aux = ((FLayerFileVectorial) lyr).getFilePath();
		}

		if (lyr instanceof FlyrGpeVect) {
			aux = ((FlyrGpeVect) lyr).getFilePath();
		}
		
		if (lyr instanceof FLayerGenericRaster) {
			aux = ((FLayerGenericRaster) lyr).getFilePath();
		}
		
		return aux;
	}
	
	private String getCustomFormPath(FLayer lyr) {	
		String aux = Messages.getText("LayerPropertiesDialog.18"); //$NON-NLS-1$	
		if ((lyr instanceof FlyrGpeVect)||(lyr instanceof FLayerFileVectorial)){
			if(((FLyrDefault)lyr).hasCustomForm())
				aux = ((FLyrDefault) lyr).getCustomForm();
		}
		return aux;
	}

}
