/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.Window;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTable;
import es.prodevelop.gvsig.mobile.gui.dialog.control.LayersTableItem;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;

/**
 * This dialog is the application's TOC (table of contents). It contains a list of the layers
 * currently contained in the view (that is, contained in the project). The user can modify
 * the order in which layers are drawn and access the layer properties. (some layer properties
 * are accessible directly from this dialog: visibility, infoability and scale limits status)
 *  
 *  @see es.prodevelop.gvsig.mobile.fmap.layer.FLayer
 *  @see es.prodevelop.gvsig.mobile.gui.dialog.LayerPropertiesDialog
 *  
 * @author jldominguez
 *
 */
public class LayersManagerDialog extends Window implements MouseListener {
	
	private static Logger logger = Logger.getLogger(LayersManagerDialog.class);

	private TocButton addVectButton, addRasterButton, addWmsButton,
	removeButton, propButton, upButton, downButton, symButton, okButton,
	zoomToLayerButton;
	
	private Panel centralPanel, centralButtonsPanel;
	private Label titleLabel;
	
	private MainWindow mFrame;
	private MapControl refreshable;
	private LayersTable layersTable;
	private ScrollPane tableScroll;
	private Panel scrolledPane;
	
	/**
	 * This method is called to automatically resize the dialog and make it
	 * expand to its optimal size.
	 * 
	 *
	 */
	public void expand(){
		//System.out.println("expanding");
		if (centralButtonsPanel.getHeight()>tableScroll.getViewportSize().height){
			layersTable.setSize(
					tableScroll.getViewportSize().width-centralButtonsPanel.getWidth()-20, 
					layersTable.getHeight());	
		} else {
			layersTable.setSize(
					tableScroll.getViewportSize().width-centralButtonsPanel.getWidth(), 
					layersTable.getHeight());
		}
		
		validate();
	}
	
	public void _refreshContent() {
		layersTable = refreshable.getToc();
		layersTable.storeStatus();
		tableScroll = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scrolledPane.add(layersTable,BorderLayout.CENTER);
		tableScroll.add(scrolledPane);
		centralPanel.removeAll();
		centralPanel.add(tableScroll, BorderLayout.CENTER);
		expand();
	}
	
	public void refreshContent() {
		
		layersTable = refreshable.getToc();
		layersTable.storeStatus();
		
		scrolledPane= new Panel();
		scrolledPane.setLayout(new BorderLayout());
		
		setBounds(mFrame.getX(), mFrame.getY(), mFrame.getWidth(), mFrame.getHeight());
		setLayout(new BorderLayout());
		
		if (Utils.USING_PDA) {
			// setUndecorated(true);
		}
		// WINDOW-DIALOG
		// setModal(true);
		
		// setLayout(new BorderLayout());
		// titleLabel = new Label(Messages.getText("LayersManagerDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		
		titleLabel.setAlignment(1);
		
		tableScroll = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scrolledPane.add(layersTable,BorderLayout.CENTER);
		tableScroll.add(scrolledPane);
		
		// addVectButton = new TocButton("toc_dlg_add_shp.png", 0, true, true); //$NON-NLS-1$
		// addRasterButton = new TocButton("toc_dlg_add_ecw.png", 1, true, true); //$NON-NLS-1$
		// addWmsButton = new TocButton("toc_dlg_add_wms.png", 2, true, true); //$NON-NLS-1$
		// removeButton = new TocButton("toc_dlg_delete.png", 3, true, true); //$NON-NLS-1$
		// upButton =  new TocButton("toc_dlg_up.png", 4, true, true); //$NON-NLS-1$
		// downButton = new TocButton("toc_dlg_down.png", 5, true, true); //$NON-NLS-1$
		// zoomToLayerButton = new TocButton("toc_dlg_zoom_layer.png", 6, true, true); //$NON-NLS-1$
		// propButton = new TocButton("toc_dlg_properties_2.png", 7, true, true); //$NON-NLS-1$
		
		// symButton = new TocButton("toc_dlg_symbology.png", 8, true, true); //$NON-NLS-1$
		// okButton = new TocButton(
		// 		DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 9, true, true);
		
		// upButton.addMouseListener(this);
		// downButton.addMouseListener(this);
		// zoomToLayerButton.addMouseListener(this);
		// addVectButton.addMouseListener(this);
		// addRasterButton.addMouseListener(this);
		// addWmsButton.addMouseListener(this);
		// removeButton.addMouseListener(this);
		// propButton.addMouseListener(this);
		// symButton.addMouseListener(this);
		// okButton.addMouseListener(this);

		centralButtonsPanel = new Panel();
		centralButtonsPanel.setBackground(Color.WHITE);
		
		
		
		centralButtonsPanel.setLayout(null);
		
		centralButtonsPanel.add(addVectButton);
		centralButtonsPanel.add(addRasterButton);
		centralButtonsPanel.add(addWmsButton);
		centralButtonsPanel.add(removeButton);
		
		centralButtonsPanel.add(upButton);
		centralButtonsPanel.add(downButton);
		centralButtonsPanel.add(zoomToLayerButton);

		centralButtonsPanel.add(propButton);
		
		centralButtonsPanel.add(symButton);
		centralButtonsPanel.add(okButton);

		centralButtonsPanel.setSize(
				GvsigMobileGUIConstants.ICON_SIZE_SMALL + 2 * TocButton.MARGIN,
				centralButtonsPanel.getComponentCount() * (GvsigMobileGUIConstants.ICON_SIZE_SMALL + TocButton.MARGIN)
				+ TocButton.MARGIN);
		
		centralPanel = new Panel();
		centralPanel.setLayout(new BorderLayout());
		centralPanel.add(tableScroll, BorderLayout.CENTER);
		
		//centralPanel.add(centralButtonsPanel, BorderLayout.EAST);
		scrolledPane.add(centralButtonsPanel,BorderLayout.EAST);

		removeAll();
		add(titleLabel, BorderLayout.NORTH);
		add(centralPanel, BorderLayout.CENTER);

		expand();

		layersTable.repaint();
		
		int h = getHeight();
		int mfh = mFrame.getHeight();
		if (h == mfh) {
			setSize(getWidth(),	h + 1);
		} else {
			setSize(getWidth(),	mfh);
		}
	}

	
	/**
	 * Constructor. Needs the MapControl to notify view refreshments when needed.
	 * 
	 * @param _frame parent frame
	 * @param mc The MapControl object to notify when changes are made
	 */
	public LayersManagerDialog(MainWindow _frame, MapControl mc) {
		
		super(_frame);
		
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				expand();
			}
		});
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				// dispose();
			}
		});
		
		mFrame = _frame;
		refreshable = mc;
		layersTable = mc.getToc();
		layersTable.storeStatus();
		
		scrolledPane= new Panel();
		scrolledPane.setLayout(new BorderLayout());
		
		setBounds(_frame.getX(), _frame.getY(), _frame.getWidth(), _frame.getHeight());
		setLayout(new BorderLayout());
		
		if (Utils.USING_PDA) {
			// setUndecorated(true);
		}
		// WINDOW-DIALOG
		// setModal(true);
		
		// setLayout(new BorderLayout());
		titleLabel = new Label(Messages.getText("LayersManagerDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		
		titleLabel.setAlignment(1);
		
		tableScroll = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		scrolledPane.add(layersTable,BorderLayout.CENTER);
		tableScroll.add(scrolledPane);
		
		addVectButton = new TocButton("toc_dlg_add_shp.png", 0, true, true); //$NON-NLS-1$
		addRasterButton = new TocButton("toc_dlg_add_ecw.png", 1, true, true); //$NON-NLS-1$
		addWmsButton = new TocButton("toc_dlg_add_wms.png", 2, true, true); //$NON-NLS-1$
		removeButton = new TocButton("toc_dlg_delete.png", 3, true, true); //$NON-NLS-1$
		upButton =  new TocButton("toc_dlg_up.png", 4, true, true); //$NON-NLS-1$
		downButton = new TocButton("toc_dlg_down.png", 5, true, true); //$NON-NLS-1$
		zoomToLayerButton = new TocButton("toc_dlg_zoom_layer.png", 6, true, true); //$NON-NLS-1$
		propButton = new TocButton("toc_dlg_properties_2.png", 7, true, true); //$NON-NLS-1$
		
		symButton = new TocButton("toc_dlg_symbology.png", 8, true, true); //$NON-NLS-1$
		okButton = new TocButton(
				DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 9, true, true);
		
		upButton.addMouseListener(this);
		downButton.addMouseListener(this);
		zoomToLayerButton.addMouseListener(this);
		addVectButton.addMouseListener(this);
		addRasterButton.addMouseListener(this);
		addWmsButton.addMouseListener(this);
		removeButton.addMouseListener(this);
		propButton.addMouseListener(this);
		symButton.addMouseListener(this);
		okButton.addMouseListener(this);

		centralButtonsPanel = new Panel();
		centralButtonsPanel.setBackground(Color.WHITE);
		
		
		
		centralButtonsPanel.setLayout(null);
		
		centralButtonsPanel.add(addVectButton);
		centralButtonsPanel.add(addRasterButton);
		centralButtonsPanel.add(addWmsButton);
		centralButtonsPanel.add(removeButton);
		
		centralButtonsPanel.add(upButton);
		centralButtonsPanel.add(downButton);
		centralButtonsPanel.add(zoomToLayerButton);

		centralButtonsPanel.add(propButton);
		
		centralButtonsPanel.add(symButton);
		centralButtonsPanel.add(okButton);
		
		centralButtonsPanel.setSize(
				GvsigMobileGUIConstants.ICON_SIZE_SMALL + 2 * TocButton.MARGIN,
				centralButtonsPanel.getComponentCount() * (GvsigMobileGUIConstants.ICON_SIZE_SMALL + TocButton.MARGIN)
				+ TocButton.MARGIN);

		centralPanel = new Panel();
		centralPanel.setLayout(new BorderLayout());
		centralPanel.add(tableScroll, BorderLayout.CENTER);
		
		//centralPanel.add(centralButtonsPanel, BorderLayout.EAST);
		scrolledPane.add(centralButtonsPanel,BorderLayout.EAST);

		add(titleLabel, BorderLayout.NORTH);
		add(centralPanel, BorderLayout.CENTER);
		
		expand();
	}
	
	/**
	 * This method updates the content of the row describing the provided layer.
	 * 
	 * @param lyr The layer whose description has to be updated.
	 */
	public void updateLayer(FLayer lyr) {
		layersTable.updateItem(lyr);
	}

	public void mousePressed(MouseEvent arg0) {
		Object src = arg0.getSource();
		
		if (src == okButton) {
			// dispose();
			try {
				
				hide();
			} catch (NullPointerException npe) {
				logger.warn("TOC dialog: A mysterious NP exception was caught: " + npe.getMessage()); //$NON-NLS-1$
			}
			
			if (!layersTable.sameAsLastStore()) {
				refreshable.redraw(true);
			}
			logger.warn("------------------OK---------------");
			refreshable.getParentFrame().setActiveEditionBar();
		}
		
		if (src == symButton) {
			
			int ind = layersTable.getSelectedIndex();
			if (ind != -1) {
				FLayer[] lyrs = layersTable.getLayers();
				FLayer lyr = lyrs[ind - 1];
				
				if (lyr instanceof FLyrVect) {
					FLyrVect lyrv = (FLyrVect) lyr;
					LayerSymbologyDialog dlg =
						new LayerSymbologyDialog(lyrv, this, mFrame);
					dlg.setVisible(true);
				}
				
			} else {
				AcceptCancelDialog.informUser(
						Messages.getText("LayersManagerDialog.11"),
						Messages.getText("LayersManagerDialog.12"),
						mFrame);
			}
		}

		if (src == addVectButton) {
			mFrame.addVectLayer(false);
		}
		
		if (src == addRasterButton) {
			mFrame.addRasterLayer(false);
		}
		
		if (src == addWmsButton) {
			mFrame.addWmsLayer(false);
		}

		if (src == removeButton) {
			int ind = layersTable.getSelectedIndex();
			if (ind != -1) {
				int remove = layersTable.removeItem(ind);

				if(remove==LayersTable.REMOVED_FAIL)
					
					AcceptCancelDialog.informUser(
							Messages.getText("LayersManagerDialog.23"),
							Messages.getText("LayersManagerDialog.24"),
							mFrame);

			} else {
				AcceptCancelDialog.informUser(
						Messages.getText("LayersManagerDialog.13"),
						Messages.getText("LayersManagerDialog.14"),
						mFrame);
			}
		}
		
		if (src == propButton) {
			
			int ind = layersTable.getSelectedIndex();
			if (ind != -1) {
				FLayer[] lyrs = layersTable.getLayers();

				LayerPropertiesDialog dlg =
					new LayerPropertiesDialog(lyrs[ind - 1], this, mFrame);
				dlg.setVisible(true);
			} else {
				AcceptCancelDialog.informUser(
						Messages.getText("LayersManagerDialog.15"),
						Messages.getText("LayersManagerDialog.16"),
						mFrame);
			}
		}
		
		if (src == downButton) {
			
			int ind = layersTable.getSelectedIndex();
			if (ind != -1) {
				layersTable.moveSelectedDown();
			} else {
				AcceptCancelDialog.informUser(
						Messages.getText("LayersManagerDialog.17"),
						Messages.getText("LayersManagerDialog.18"),
						mFrame);
			}
			
		}
		
		if (src == upButton) {
			int ind = layersTable.getSelectedIndex();
			if (ind != -1) {
				layersTable.moveSelectedUp();
			} else {
				AcceptCancelDialog.informUser(
						Messages.getText("LayersManagerDialog.19"),
						Messages.getText("LayersManagerDialog.20"),
						mFrame);
			}
		}
		
		if (src == zoomToLayerButton) {
			
			int ind = layersTable.getSelectedIndex();
			if (ind != -1) {
				try {
					hide();
				} catch (NullPointerException npe) {
					logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
				}
				layersTable.zoomToSelected();
			} else {
				AcceptCancelDialog.informUser(
						Messages.getText("LayersManagerDialog.21"),
						Messages.getText("LayersManagerDialog.22"),
						mFrame);
			}
		}
	}

	public void mouseClicked(MouseEvent arg0) {
	}

	public void mouseReleased(MouseEvent arg0) {
	}

	public void mouseEntered(MouseEvent arg0) {
	}

	public void mouseExited(MouseEvent arg0) {
	}

}
