/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Dialog;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.TextField;
import java.awt.Window;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Rectangle2D;
import java.net.URL;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.common.GvsigMobileException;
import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.BadWMSResponseException;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSLayerDescription;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSProtocolHandler;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;
import es.prodevelop.gvsig.mobile.gui.dialog.control.WMSLayerListItem;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.settings.AppSettings;

/**
 * 
 * This dialog is used to add a WMS layer to the view. When the user selectes or adds a new
 * URL to the combo, a request is sent to find out the server capabilities, then the other
 * components of the dialog are refreshed.
 * 
 * The available layers are presented in a tree-like structure.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.WMSLayerListItem
 * @see es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSLayerDescription
 *  
 * @author jldominguez
 *
 */
public class WMSDialog extends Window implements MouseListener,
		ItemListener {

	private static Logger logger = Logger.getLogger(WMSDialog.class);
	
	private MainWindow parent;

	private Panel northPanel, /*centralPanel, */ southPanel; // = new Panel();

	private Label titleLabel = null;

	private Choice serverChoice = null;

	private Label serverLabel;

	// -------------------------------------------------
	private ScrollPane availableLayersScroll = null;

	private Panel availableLayersPanel;

	private static int layerItemsPos = -1;

	private static int getNextLayerItemsPos() {
		layerItemsPos++;
		return layerItemsPos;
	}
	// -------------------------------------------------

	private Choice srsChoice = null;
	private Label srsLabel;
	private Choice formatChoice = null;
	private Label formatLabel;
	
	private Label userServerLabel;
	private TextField userServerText;
	private TocButton addUserServerButton;
	private boolean repaintAfterAdding = true;

	private TocButton okButton, cancelButton;
	
	private WMSProtocolHandler wms_handler;

	/**
	 * Constructor 
	 * @param p parent (main) window
	 * @param repaint whether the map has necessarily to be repainted after
	 * adding the new WMS layer. When the WMS layer is added from the TOC, we dont need
	 * a refresh. 
	 */
	public WMSDialog(MainWindow p, boolean repaint) {
		super(p);
		parent = p;
		repaintAfterAdding = repaint;
		setBounds(p.getX(), p.getY(), p.getWidth(), p.getHeight());
		// setUndecorated(true);
		// WINDOW-DIALOG
		wms_handler = new WMSProtocolHandler();
		init();
	}
	
	private void init() {
		
		setLayout(new BorderLayout());
		
		// -------------------------------------------
		// -------------------------------------------
		northPanel = new Panel();
		northPanel.setLayout(null);
		southPanel = new Panel();
		southPanel.setLayout(null);
		// -------------------------------------------
		// -------------------------------------------

		okButton = new TocButton(
				DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME,
				"toc_ok_dis_small.png", 0, true, false); //$NON-NLS-1$
		okButton.addMouseListener(this);
		okButton.setEnabled(false);
		
		cancelButton = new TocButton(
				DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 1,
				true, false);
		cancelButton.addMouseListener(this);
		
		// -------------------------------------------

		int _w = getWidth() - 2 * GvsigMobileGUIConstants.LABEL_LEFT_MARGIN;
		// getAvailableLayersScroll().add(getAvailableLayersPanel());

		// ------------ row 0 ----------------
		titleLabel = new Label(Messages.getText("WMSDialog.1")); //$NON-NLS-1$
		titleLabel.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN, 0, _w, DialogConstants.CONTROL_HEIGHT);
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);

		// ------------ row 1 ----------------
		serverLabel = new Label(Messages.getText("WMSDialog.2")); //$NON-NLS-1$
		serverLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		serverLabel.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN, DialogConstants.CONTROL_HEIGHT, DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		serverChoice = getServerChoice();
		serverChoice.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN + DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT, _w - DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		serverChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);
		
		// -------------- row 1.5
		userServerLabel = new Label(Messages.getText("WMSDialog.3")); //$NON-NLS-1$
		userServerLabel.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN, 2 * DialogConstants.CONTROL_HEIGHT, DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		userServerLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);

		userServerText = new TextField();
		
		
		int __w = _w - DialogConstants.LABELS_WIDTH - DialogConstants.SMALL_CONTROL_HEIGHT - 2; 
		userServerText.setBounds(
				GvsigMobileGUIConstants.LABEL_LEFT_MARGIN + DialogConstants.LABELS_WIDTH,
				2 * DialogConstants.CONTROL_HEIGHT,
				__w,
				DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		userServerText.setFont(DialogConstants.DIALOG_SMALL_FONT);
		
		addUserServerButton = new TocButton(
				DialogConstants.DIALOG_SMALL_ADD_BUTTON_IMAGE_FILE_NAME,
				DialogConstants.DIALOG_SMALL_ADD_BUTTON_IMAGE_FILE_NAME,
				0, true, false);
		
		addUserServerButton.setBounds(
				2 + GvsigMobileGUIConstants.LABEL_LEFT_MARGIN + DialogConstants.LABELS_WIDTH + __w,
				2 + 2 * DialogConstants.CONTROL_HEIGHT,
				DialogConstants.SMALL_CONTROL_HEIGHT,
				DialogConstants.SMALL_CONTROL_HEIGHT);
		addUserServerButton.addMouseListener(this);
		
		// private TocButton addUserServerButton
		
		// layerLabel = new Label("Layer:");
		// ------------ row 2 ----------------
		srsLabel = new Label(Messages.getText("WMSDialog.4")); //$NON-NLS-1$
		srsLabel.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN, 3 * DialogConstants.CONTROL_HEIGHT, DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		srsLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		srsChoice = getSrsChoice();
		srsChoice.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN + DialogConstants.LABELS_WIDTH, 3 * DialogConstants.CONTROL_HEIGHT, _w - DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		srsChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);

		// ------------ row 3 ----------------
		formatLabel = new Label(Messages.getText("WMSDialog.5")); //$NON-NLS-1$
		formatLabel.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN, 4 * DialogConstants.CONTROL_HEIGHT, DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		formatLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		formatChoice = getFormatChoice();
		formatChoice.setBounds(GvsigMobileGUIConstants.LABEL_LEFT_MARGIN + DialogConstants.LABELS_WIDTH, 4 * DialogConstants.CONTROL_HEIGHT, _w - DialogConstants.LABELS_WIDTH, DialogConstants.CHOICE_FRIENDLY_HEIGHT);
		formatChoice.setFont(DialogConstants.DIALOG_SMALL_FONT);

		// layerLabel.setSize(LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		northPanel.setSize(_w, 5 * DialogConstants.CONTROL_HEIGHT);
		northPanel.add(titleLabel);
		northPanel.add(serverLabel);
		northPanel.add(serverChoice);
		
		northPanel.add(userServerLabel);
		northPanel.add(userServerText);
		northPanel.add(addUserServerButton);
		
		northPanel.add(srsLabel);
		northPanel.add(srsChoice);
		northPanel.add(formatLabel);
		northPanel.add(formatChoice);

		// -------------------------------------------

		southPanel.setSize(_w, DialogConstants.CONTROL_HEIGHT);
		southPanel.add(okButton);
		southPanel.add(cancelButton);
		
		// -------------------------------------------
		add(northPanel, BorderLayout.NORTH);
		add(southPanel, BorderLayout.SOUTH);
		add(getAvailableLayersScroll(), BorderLayout.CENTER);
	}

	private Choice getFormatChoice() {
		if (formatChoice == null) {
			formatChoice = new Choice();
		}
		return formatChoice;
	}

	private Choice getSrsChoice() {
		if (srsChoice == null) {
			srsChoice = new Choice();
		}
		return srsChoice;
	}

	private ScrollPane getAvailableLayersScroll() {
		if (availableLayersScroll == null) {
			availableLayersScroll = new ScrollPane(
					ScrollPane.SCROLLBARS_AS_NEEDED);
		}
		return availableLayersScroll;
	}
	
	private Panel getAvailableLayersPanel() {
		if (availableLayersPanel == null) {
			availableLayersPanel = new Panel();
			availableLayersPanel.setLayout(new GridLayout(0, 1, 0, 0));
		}
		return availableLayersPanel;
	}
	

	private Choice getServerChoice() {

		if (serverChoice == null) {
			serverChoice = new Choice();
			loadFromPersistence(serverChoice);

			
			// serverChoice.add("http://atlas.gc.ca/cgi-bin/atlaswms_en?VERSION=1.1.1&request=Getcapabilities&service=wms");
			// serverChoice.add("http://onearth.jpl.nasa.gov/wms.cgi");
			
			// serverChoice.add("http://ovc.catastro.meh.es/Cartografia/WMS/ServidorWMS.aspx");
			// serverChoice.add("http://georef.application.equipement.gouv.fr/cartes/mapserv");
			// serverChoice.add("http://www.idee.es/wms/IDEE-Base/IDEE-Base");
			// serverChoice.add("http://orto.cth.gva.es/wmsconnector/com.esri.wms.Esrimap/wms_urbanismo_tematicos");
			// serverChoice.add("http://orto.cth.gva.es/wmsconnector/com.esri.wms.Esrimap/wms_patricova");
			serverChoice.addItemListener(this);
		}
		return serverChoice;
	}

	private void loadFromPersistence(Choice cho) {
		
		String[] wms = AppSettings.wmsServers;

		cho.removeAll();
		cho.add(""); //$NON-NLS-1$

		if (wms != null) {
			for (int i=0; i<wms.length; i++) {
				if (wms[i].length() > 0) cho.add(wms[i]);
			}
		}
	}

	public void mousePressed(MouseEvent e) {

		Object src = e.getSource();

		if (src == okButton) {
			if (!okButton.isEnabled()) return;
			setVisible(false);
			parent.doAddWmsLayer(
					this,
					wms_handler.getVersion(),
					wms_handler.getSymbol(),
					repaintAfterAdding);
			
			wms_handler.delete();
		}

		if (src == cancelButton) {
			if (!cancelButton.isEnabled()) return;
			setVisible(false);
			wms_handler.delete();
		}
		
		if (src == addUserServerButton) {
			
			String selected = userServerText.getText();
			if (selected.length() == 0) {
				return;
			}
			
			insertIfNotPresent(getServerChoice(), selected, 1);
			getServerChoice().select(selected);
			
			// persist wms servers
			// http://mapas.euitto.upm.es/cgi-bin/cnauticas
			AppSettings.wmsServers = getServerChoiceItems();
			try {
				AppSettings.store();
			} catch (GvsigMobileException e1) {
				logger.error("While storing persistence: " + e1.getMessage()); //$NON-NLS-1$
			}
			
			ItemEvent forced_event = new ItemEvent(getServerChoice(), 0, selected, 0);
			itemStateChanged(forced_event);
		}
	}

	private void insertIfNotPresent(Choice ch, String str, int j) {
		
		int cnt = ch.getItemCount();
		ArrayList list = new ArrayList();
		
		for (int i=0; i<cnt; i++) {
			if (ch.getItem(i).length() > 0) {
				list.add(ch.getItem(i));
			}
		}
		
		String[] current_list = (String[]) list.toArray(new String[0]);
		if (!isContained(str, current_list)) {
			ch.insert(str, j);
		}
	}

	private boolean isContained(String str, String[] arr) {
		
		for (int i=0; i<arr.length; i++) {
			if (arr[i].compareToIgnoreCase(str) == 0) {
				return true; 
			}
		}
		return false;
	}

	private String[] getServerChoiceItems() {
		
		int cnt = getServerChoice().getItemCount();
		
		String[] resp = new String[cnt];
		for (int i=0; i<cnt; i++) resp[i] = getServerChoice().getItem(i);
		return resp;
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}

	/**
	 * This event triggers a query to the server, to find out the server capabilities.
	 * The rest of the components are then updated, or a message error shows.
	 */
	public void itemStateChanged(ItemEvent e) {

		Object src = e.getSource();

		if (src == serverChoice) {

			String _host = serverChoice.getSelectedItem();
			if (_host.length() > 0) {
				
				URL host = null;
				try {
					host = new URL(_host);
				} catch (Exception ex) {
					logger.error("Bad url: " + _host);
					AcceptCancelDialog.informUser(
							Messages.getText("WMSDialog.8"), //$NON-NLS-1$
							Messages.getText("WMSDialog.9"), //$NON-NLS-1$
							parent);
					return;
				}
				

				disableWhileGettingCapabilities(false);

				String v = "";
				try {
					v = WMSProtocolHandler.getSuitableWMSVersion(host);
				} catch (BadWMSResponseException e1) {
					logger.error("WMS layer not created: " + e1.getMessage());
					clearLayersPanel();
					AcceptCancelDialog.informUser(
							Messages.getText("WMSDialog.8"), //$NON-NLS-1$
							Messages.getText("WMSDialog.9"), //$NON-NLS-1$
							parent);
					return;
				}
				
				wms_handler.setHost(_host);
				String sym = WMSProtocolHandler.getSymbol(_host);
				
				wms_handler.setVersion(v);
				wms_handler.setSymbol(sym);
				
				if (wms_handler.getCapabilities(v, sym)) {
					disableWhileGettingCapabilities(true);
				} else {
					AcceptCancelDialog.informUser(
							Messages.getText("WMSDialog.8"), //$NON-NLS-1$
							Messages.getText("WMSDialog.9"), //$NON-NLS-1$
							parent);
				}

			} else {
				logger.debug("Empty server selected, host = " + _host); //$NON-NLS-1$
			}
		}
		
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the url of the server selected by the user.
	 */
	public String getServerUrl() {
		return serverChoice.getSelectedItem();
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the SRS selected by the user.
	 */
	public String getSrs() {
		return getSrsChoice().getSelectedItem();
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the format of the images to be requested.
	 */
	public String getImageFormat() {
		return formatLabelToFormatParameter(getFormatChoice().getSelectedItem());
	}
	
	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the format in which the info by point will be received (not selectable by
	 * teh user. It's decided by the application after receiving available formats.)
	 */
	public ArrayList getInfoFormats() {
		return wms_handler.getInfoFormats();
	}

	private void disableWhileGettingCapabilities(boolean v) {
		
		if (!v) {

			getSrsChoice().removeAll();
			getFormatChoice().removeAll();

		} else {

			int c = wms_handler.getFormats().size();
			for (int i = 0; i < c; i++) {
				getFormatChoice().add((String) wms_handler.getFormats().get(i));
			}
			updateLayersPanel();
			loadComboWithSRS(getSrsChoice(), wms_handler.getRoot().getLayerId());
		}

		okButton.setEnabled(false);
		// cancelButton.setEnabled(v);
		// availableLayersPanel.setEnabled(v);
		
		getSrsChoice().setEnabled(v);
		getFormatChoice().setEnabled(v);
	}

	private void loadComboWithSRS(Choice ch, String lyrname) {

		ch.removeAll();

		WMSLayerDescription aux = findDescriptionWithName(
				wms_handler.getRoot(), lyrname);

		if (aux == null) {

		} else {
			String[] ss = aux.getAvailableSRS();
			for (int i = 0; i < ss.length; i++) {
				ch.add(ss[i]);
			}
		}
	}

	private WMSLayerDescription findDescriptionWithName(
			WMSLayerDescription desc, String name) {

		if (desc.getLayerId().compareToIgnoreCase(name) == 0) {
			return desc;
		} else {
			for (int i = 0; i < desc.getChildrenCount(); i++) {

				WMSLayerDescription resp = findDescriptionWithName(desc
						.getChild(i), name);
				if (resp != null) {
					return resp;
				}
			}
		}
		return null;
	}

	private String formatLabelToFormatParameter(String l) {

		if (true) {
			return StringUtilities.replace(l, " ", "%20"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		String label = l.toLowerCase();

		if (label.indexOf("png") != -1) { //$NON-NLS-1$
			if (label.indexOf("24") != -1) { //$NON-NLS-1$
				return "PNG24"; //$NON-NLS-1$
			} else {
				return "PNG"; //$NON-NLS-1$
			}
		} else {
			if ((label.indexOf("jpg") != -1) || (label.indexOf("jpeg") != -1)) { //$NON-NLS-1$ //$NON-NLS-2$
				return "JPEG"; //$NON-NLS-1$
			} else {
				if (label.indexOf("gif") != -1) { //$NON-NLS-1$
					return "GIF"; //$NON-NLS-1$
				} else {
					// default
					return "JPEG"; //$NON-NLS-1$
				}
			}
		}
	}

	/**
	 * This method is called after the dialog is closed. Gets the full extent
	 * of the layer to be added. This not fully functional. The dialog simply keeps
	 * a unique extent for each SRS, regardless of the layers.
	 * 
	 * TODO Keep the extent of each layer for each SRS.
	 * 
	 * @param name layer name
	 * @param srs SRS that will be used in the requests
	 * @param def default extent to be returned if no data is available
	 * @return the full extent of the layer
	 */
	public Rectangle2D getFullExtent(String name, String srs, Rectangle2D def) {

		WMSLayerDescription d = wms_handler.getRoot();

		if (d == null) {
			return def;
		}

		Rectangle2D resp = d.getExtentForSRS(srs);
		if (resp == null) {
			return def;
		} else {
			return resp;
		}
	}
	
	/**
	 * This method updates the layer checkboxes after one of them has been
	 * checked/unchecked. It has to be public because its called by another
	 * component.
	 * 
	 *  @see WMSLayerListItem
	 *
	 */
	public void updateCheckboxes() {

		Panel p = getAvailableLayersPanel();
		int cnt = p.getComponentCount();

		WMSLayerListItem item = null;
		for (int i=0; i<cnt; i++) {
			item = (WMSLayerListItem) p.getComponent(i);
			item.updateCheckboxInSilence();
		}
		
		int n_checked = 0;
		for (int i=0; i<cnt; i++) {
			item = (WMSLayerListItem) p.getComponent(i);
			if (item.isChecked()) n_checked++;
		}
		
		okButton.setEnabled(n_checked > 0);
		
	}
	
	private void clearLayersPanel() {
		getAvailableLayersPanel().removeAll();
		getAvailableLayersPanel().repaint();
	}
	

	private void updateLayersPanel() {

		// ------------- TODO improve this --------------
		setVisible(false);
		// ------------- TODO improve this --------------

		layerItemsPos = -1;
		getAvailableLayersPanel().removeAll();

		WMSLayerDescription rt = wms_handler.getRoot();
		int countitems = countItems(rt);
		int w = getWidth() - (DialogConstants.LABELS_WIDTH + 5);
		getAvailableLayersPanel().setSize(w, countitems
				* GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT);

		if (rt == null) return;

		loadAvailableLayersItems(getAvailableLayersPanel(), rt,
				getNextLayerItemsPos(), 2 * getWidth());

		getAvailableLayersScroll().removeAll();
		getAvailableLayersScroll().add(getAvailableLayersPanel());

		// ------------- TODO improve this --------------
		setVisible(true);
		// ------------- TODO improve this --------------
	}

	private void loadAvailableLayersItems(Panel p, WMSLayerDescription desc,
			int pos, int w) {

		WMSLayerListItem item = new WMSLayerListItem(desc, w, this);
		item.setSize(w, GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT);
		p.add(item);

		int cnt = desc.getChildrenCount();
		for (int i = 0; i < cnt; i++) {
			loadAvailableLayersItems(p, desc.getChild(i),
					getNextLayerItemsPos(), w);
		}
	}

	private int countItems(WMSLayerDescription desc) {

		if (desc == null)
			return 0;

		int resp = 1;
		for (int i = 0; i < desc.getChildrenCount(); i++) {
			resp = resp + countItems(desc.getChild(i));
		}
		return resp;
	}
	
	public String getCommaSeparatedSelectedLayers() {
		return concatenateUsefulCheckedLayers(); // concatenateCheckedLayers();
	}
	
	private String concatenateCheckedLayers() {

		String resp = ""; //$NON-NLS-1$
		Panel p = getAvailableLayersPanel();

		int cnt = p.getComponentCount();

		WMSLayerListItem item;
		for (int i=0; i<cnt; i++) {
			item = (WMSLayerListItem) p.getComponent(i);

			if (item.isChecked()) {
				String v = item.getLayerId().trim();
				if (v.length() > 0) {
					resp = resp + v + ","; //$NON-NLS-1$
				}
			}
		}
		
		return resp.substring(0, resp.length() - 1);
	}
	
	private String concatenateUsefulCheckedLayers() {
		
		WMSLayerDescription root = wms_handler.getRoot();
		String resp = root.appendUseful(""); //$NON-NLS-1$
		return resp.substring(1);
	}

	public String getCommaSeparatedSelectedInfoableLayers(String sel) {
		
		String resp = WMSProtocolHandler.leaveInfoable(sel, wms_handler.getRoot(), false);
		return resp;
	}

	public String getCommaSeparatedSelectedInfoableLayerNames(String sel) {
		
		String resp = WMSProtocolHandler.leaveInfoable(sel, wms_handler.getRoot(), true);
		return resp;
	}
	
	public String createLayerName(String sel) {
		String[] all_array = StringUtilities.split(sel, ",");
		String resp = WMSProtocolHandler.firstLayerName(all_array, wms_handler.getRoot());
		return resp;
	}
	
	
}
