/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Color;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Scrollbar;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;

/**
 * This class extends the Expansible dialog and allows the user to choose a color by
 * using three scroll bars (one for each channel : red green blue)
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooserPanel
 * 
 * @author jldominguez
 *
 */
public class ColorChooser extends ExpansibleWindow implements AdjustmentListener, MouseListener {
	
	private static Logger logger = Logger.getLogger(ColorChooser.class);
	
	private Color color = Color.BLACK;
	private Label title, redLabel, greenLabel, blueLabel;
	private Scrollbar redBar, greenBar, blueBar;
	private Panel sample;
	
	private TocButton okButton, cancelButton;
	private ColorChooserPanel changing = null;

	/**
	 * Constructor
	 * 
	 * @param parent parent frame, used also to use the same size
	 * @param init inital color
	 * @param c control clicked by the user to open this dialog. The new color will be
	 * applied to this control 
	 */
	public ColorChooser(Frame parent, Color init, ColorChooserPanel c) {
		
		super(parent);
		
		changing = c;
		int w = parent.getWidth();
		setBounds(parent.getX(), parent.getY(), w, parent.getHeight());

		color = init;
		
		title = new Label(Messages.getText("ColorChooser.0")); //$NON-NLS-1$
		title.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		title.setFont(DialogConstants.DIALOG_BOLD_FONT);
		title.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		title.setAlignment(1);

		sample = new Panel();
		sample.setBackground(color);
		sample.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		
		okButton = new TocButton(DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		okButton.addMouseListener(this);
		cancelButton = new TocButton(DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 1, true, false);
		cancelButton.addMouseListener(this);

		redLabel = new Label(Messages.getText("ColorChooser.1")); //$NON-NLS-1$
		redLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		redLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		redBar = new Scrollbar(Scrollbar.HORIZONTAL, init.getRed(), 5, 0, 255);
		redBar.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		redBar.addAdjustmentListener(this);
		
		greenLabel = new Label(Messages.getText("ColorChooser.2")); //$NON-NLS-1$
		greenLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		greenLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		greenBar = new Scrollbar(Scrollbar.HORIZONTAL, init.getGreen(), 5, 0, 255);
		greenBar.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		greenBar.addAdjustmentListener(this);
		
		blueLabel = new Label(Messages.getText("ColorChooser.3")); //$NON-NLS-1$
		blueLabel.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		blueLabel.setFont(DialogConstants.DIALOG_SMALL_FONT);
		blueBar = new Scrollbar(Scrollbar.HORIZONTAL, init.getBlue(), 5, 0, 255);
		blueBar.setSize(DialogConstants.LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		blueBar.addAdjustmentListener(this);
		
		// --------------
		ExpansibleRow row0 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row0.addExtensible(title);
		add(row0);
		// --------------
		ExpansibleRow row1 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row1.add(redLabel); row1.addExtensible(redBar);
		add(row1);
		// --------------
		ExpansibleRow row2 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row2.add(greenLabel); row2.addExtensible(greenBar);
		add(row2);
		// --------------
		ExpansibleRow row3 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row3.add(blueLabel); row3.addExtensible(blueBar);
		add(row3);
		
		// --------------
		ExpansibleRow row4 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row4.addExtensible(new Label("")); //$NON-NLS-1$
		add(row4);
		
		// --------------
		ExpansibleRow row5 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		Label leftLabel = new Label(""); Label rightLabel = new Label(""); //$NON-NLS-1$ //$NON-NLS-2$
		
		int w3 = getWidth() / 3;
		
		leftLabel.setSize(w3, 10); rightLabel.setSize(w3, 10);
		
		row5.add(leftLabel);
		row5.addExtensible(sample);
		row5.add(rightLabel);
		
		addExpansible(row5);
		
		// --------------
		ExpansibleRow row6 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row6.addExtensible(new Label("")); //$NON-NLS-1$
		add(row6);
		
		// --------------
		ExpansibleRow row7 = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		row7.add(okButton); row7.add(cancelButton);
		//ExpansibleRow row7 = new AceptCancelRow(DialogConstants.CONTROL_HEIGHT, this);
		add(row7);
		// --------------
		addEmptyRowIfPhoneME(this);
	}

	/**
	 * Method called when one of the scroll bars is moved.
	 * 
	 * @param arg0 event object 
	 */
	public void adjustmentValueChanged(AdjustmentEvent arg0) {
		
		Object src = arg0.getSource();
		
		if (src == redBar) {
			color = new Color(arg0.getValue(), color.getGreen(), color.getBlue());
		}
		
		if (src == greenBar) {
			color = new Color(color.getRed(), arg0.getValue(), color.getBlue());
		}
		
		if (src == blueBar) {
			color = new Color(color.getRed(), color.getGreen(), arg0.getValue());
		}
		
		sample.setBackground(color);
		sample.repaint();
	}

	/**
	 * Gets the color defined by the user
	 * @return the color defined by the user
	 */
	public Color getColor() {
		return color;
	}
	
	public void mousePressed(MouseEvent e) {
		
		Object src = e.getSource();
		
		if (src == okButton) {
			changing.setColor(color);
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("ColorChooser. A mysterious NP exception was caught: " + npe.getMessage()); //$NON-NLS-1$
			}
			close();
		}
		
		if (src == cancelButton) {
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("ColorChooser. A mysterious NP exception was caught: " + npe.getMessage()); //$NON-NLS-1$
			}
			close();
		}
	}

	public void mouseClicked(MouseEvent arg0) { }
	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) {
	}

}
