/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;

/**
 * This little component is used to show a customizable color.
 * When the user clicks on it, a new ColorChooser pops up.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ColorChooser
 * 
 * @author jldominguez
 *
 */
public class ColorChooserPanel extends Component implements MouseListener {
	
	private Color color = Color.BLACK;
	private Frame parent = null;
	private int size  = 0;
	private ColorChooser cc;
	
	/**
	 * Constructor
	 * 
	 * @param init initial color
	 * @param p parent frame
	 */
	public ColorChooserPanel(Color init, Frame p) {
		super();
		parent = p;
		size = DialogConstants.TABLE_LABEL_HEIGHT - 5;
		setSize(2 * size, DialogConstants.TABLE_LABEL_HEIGHT);
		color = init;
		addMouseListener(this);
	}
	
	/**
	 * Constructor with default initial color
	 * 
	 * @param p parent frame
	 */
	public ColorChooserPanel(Frame p) {
		super();
		parent = p;
		size = DialogConstants.TABLE_LABEL_HEIGHT - 5;
		setSize(2 * size, DialogConstants.TABLE_LABEL_HEIGHT);
		addMouseListener(this);
	}
	
	/**
	 * Overrriden paint method (draws a little square of the selected color)
	 */
	public void paint(Graphics g) {
		g.setColor(color);
		g.fillRect(0, 3, size, size);
		g.setColor(Color.BLACK);
		g.drawRect(0, 3, size, size);
	}
	
	/**
	 * Gets color
	 * @return the current color
	 */
	public Color getColor() {
		return color;
	}
	
	/**
	 * Sets the current color
	 * @param c the new color to be used
	 */
	public void setColor(Color c) {
		color = c;
	}

	public void mousePressed(MouseEvent arg0) {
		cc = new ColorChooser(parent, color, this);
		// cc.setModal(true);
		cc.setVisible(true);
		cc.toFront();
	}

	public void mouseClicked(MouseEvent arg0) { }
	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) {
	}
}
