/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Component;
import java.awt.Container;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;

/**
 * 
 * This is a component used by ExpansibleDialog. It consists of a row of components
 * that can be expanded horizontally.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog
 *  
 * @author jcarras
 *
 */
public class ExpansibleRow extends Container  {
	
	private static Logger logger = Logger.getLogger(ExpansibleRow.class);
	
	protected Component extComp = null;
	protected int maxWidth=-1;
	protected int minHeight=0;
	//private int currX = DialogConstants.XSPACE;
	
	public ExpansibleRow(int height){
		super();
		setMinHeight(height);
		setLayout(null);
		setSize(100,height);
	}

	public Component addExtensible(Component comp) {
		return addExtensible(comp, 0);
	}
	public Component addExtensible(Component comp, int maxWidth) {
		this.maxWidth = maxWidth;
		extComp = comp;
		return super.add(comp);
	}
	
	public void expand(int width){
		//Set the desired width
		setSize(width,getHeight());
		
		//Set the width of the expanded control
		if (extComp!=null){
			int newW = getMaxWidth();
			if (maxWidth > 0){
				newW = Math.min(newW,maxWidth);
			} 
			extComp.setSize(newW, extComp.getHeight());
			}

		//Put each control in his place
		int acX = DialogConstants.XSPACE;
		Component[] coms = getComponents();
		for (int i=0; i< coms.length; i++){
			coms[i].setLocation(acX,0);
			//System.out.print(acX + ", ");
			acX += coms[i].getWidth() + DialogConstants.XSPACE;
		}
		//System.out.println("");
	}
	
	protected int getMaxWidth(){
		Component[] coms = getComponents();
		int acwidth = 0;
		for (int i=0; i< coms.length; i++){
			acwidth += coms[i].getWidth();
			acwidth += DialogConstants.XSPACE;
		}
		acwidth += 2 * DialogConstants.XSPACE;
		
		if (extComp != null) 
			return getWidth() - acwidth + extComp.getWidth();
		return getWidth() - acwidth;
	}

	public void growHeight(){
		Component[] coms = getComponents();
		for (int i=0; i< coms.length; i++){
			coms[i].setSize(coms[i].getWidth(),getHeight());
		}
	}

	public int getMinHeight() {
		return minHeight;
	}

	public void setMinHeight(int minHeight) {
		this.minHeight = minHeight;
	}

	

}
