/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Panel;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;


/**
 * This class is used to show a sample of the current symbology by
 * drawing a little polygon/line/point with the current colors, width, etc.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol
 * 
 * @author jldominguez
 *
 */
public class LegendIcon extends Panel {
	
	private static Logger logger = Logger.getLogger(LegendIcon.class);
	
	public final static int LAYER_RASTER_TYPE = -10;
	public final static int RASTER_ICON_W = 20 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public final static int RASTER_ICON_H = 10 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	
	public final static int LEGEND_ICON_MARGIN_UP = 4 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;
	public final static int LEGEND_ICON_MARGIN_LEFT = 1 * GvsigMobileGUIConstants.GENERAL_SIZE_FACTOR;

	private int vectType = FShape.NULL;
	private FSymbol symbol = FSymbol.SELECTION_SYMBOL;
	private boolean isInterval = false;
	private static Image rasterIcon;
	
	/**
	 * Constructor
	 * @param shapeType vector shape type of the layer
	 * @param sym symbol to be represnted
	 */
	public LegendIcon(int shapeType, FSymbol sym) {
		vectType = shapeType;
		symbol = sym;
	}
	
	/**
	 * Constructor.
	 * @param type shape type of the vector layer. Default value will be used for symbol.
	 * 
	 */
	public LegendIcon(int type) {
		vectType = type;
	}
	
	/**
	 * Constructor. Default value will be used for symbol and shape type.
	 * 
	 */
	public LegendIcon() {
	}
	
	static {
		rasterIcon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "rastericon.png");
	}
	
	/**
	 * Overriden paint method. It will draw a little sample of the symbology
	 * depending on the shape type and the symbol.
	 * 
	 */
	public void paint(Graphics _g) {
		
		Graphics2D g = (Graphics2D) _g;
		
		int w = getWidth() - 1;
		int h = getHeight() - 1;
		int w2 = w / 2;
		int h2 = h / 2;
		
		switch (vectType % FShape.Z) {
		
		case FShape.MULTI:
			
			if (symbol.isFilled()) {
				g.setColor(symbol.getFillColor());
				g.fillRect(0, 0, w2, h);
			}
			g.setColor(symbol.getBorderColor());
			g.setStroke(new BasicStroke(symbol.getStrokeWidth()));
			g.drawRect(0, 0, w2 - 2, h);
			
			g.setColor(symbol.getBorderColor());
			g.setStroke(new BasicStroke(symbol.getStrokeWidth()));
			g.drawLine(w2, h2, 3 * w2 / 2, 0);
			g.drawLine(3 * w2 / 2, 0, 3 * w2 / 2, h2);
			g.drawLine(3 * w2 / 2, h2, w , 0);
			
			g.drawRect(5 + 3 * w2 / 2, 3 * h2 / 2, 3, 3);
			g.drawRect(-5 + 3 * w2 / 2, 3 * h2 / 2, 3, 3);
			return;
		
		
		case FShape.POLYGON:
			
			if (symbol.isFilled()) {
				g.setColor(symbol.getFillColor());
				g.fillRect(0, 0, w, h);
			}
			g.setColor(symbol.getBorderColor());
			g.setStroke(new BasicStroke(symbol.getStrokeWidth()));
			g.drawRect(0, 0, w, h);
			
			return;
			
		case FShape.LINE:
			g.setColor(symbol.getBorderColor());
			g.setStroke(new BasicStroke(symbol.getStrokeWidth()));
			g.drawLine(0, h, w2, 0);
			g.drawLine(w2, 0, w2, h);
			g.drawLine(w2, h, w, 0);
			return;
			
		case FShape.POINT:
			
			if(isInterval) {
				g.setColor(symbol.getFillColor());
				g.fillRect(w2 - 5, h2 - 5, 5, 5);
				g.setColor(symbol.getBorderColor());
				g.setStroke(new BasicStroke(symbol.getStrokeWidth()));
				g.fillRect(w2 , h2 , 5, 5);
			} else {
				g.setColor(symbol.getBorderColor());
				g.setStroke(new BasicStroke(symbol.getStrokeWidth()));
				g.drawRect(w2 -5, h2 -5, 10, 10);
				
			}
			
			return;
			
		case FShape.MULTIPOINT:
			int str_w = Math.min(3, symbol.getStrokeWidth());
			g.setColor(symbol.getBorderColor());
			g.setStroke(new BasicStroke(str_w));
			g.drawRect(w2 - 1 - (str_w + 2), h2 - 1, 3, 3);
			g.drawRect(w2 - 1 + (str_w + 1), h2 - 1, 3, 3);
			return;
			
		case LAYER_RASTER_TYPE:
			g.drawImage(rasterIcon, 0, 0, RASTER_ICON_W, RASTER_ICON_H, null);
			return;
			
		default:
			g.setColor(symbol.getBorderColor());
			g.drawString("?", 0, h);
			break;
		}
		
	}

	/**
	 * Gets shape type of the layer or FShape.NULL if it's not a vector layer.
	 * @return
	 */
	public int getVectType() {
		return vectType;
	}

	/**
	 * Sets the shape type of the layer.
	 * @param t
	 */
	public void setVectType(int t) {
		vectType = t;
	}

	/**
	 * gets the symbol that is being represented.
	 * @return the symbol that is being represented.
	 */
	public FSymbol getSymbol() {
		return symbol;
	}

	/**
	 * Sets the symbol to be represented
	 * @param s the symbol to be represented
	 */
	public void setSymbol(FSymbol s) {
		symbol = s;
	}

	public void setInterval(boolean isInterval) {
		this.isInterval = isInterval;
	}

	public boolean isInterval() {
		return isInterval;
	}

}
