/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Panel;

import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;


/**
 * This control is used to indicate whether the current scale
 * is inside the associated layer's scale limits
 * 
 * inside scale limits - colorful icon
 * outside scale limits - gray icon
 * no scale limits defined - no icon
 * 
 * @author jldominguez
 *
 */
public class ScaleVisibilityIcon extends Panel {
	
	/**
	 * The layer does not have values for scale limits, no iocn will be shown
	 */
	public static final int STATE_NOT_DEFINED = 0;
	/**
	 * The layer has scale limits, and the current scale is inside them. A colorful icon
	 * will show
	 */
	public static final int STATE_VISIBLE = 1;
	/**
	 * The layer has scale limits, and the current scale is not inside them.
	 * A gray, sad icon will show.
	 */
	public static final int STATE_NOT_VISIBLE = 2;
	
	private int state = STATE_NOT_DEFINED;
	
	private static Image inIcon =
		ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "toc_dlg_list_scale_on.png");
	private static Image outIcon =
		ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, "toc_dlg_list_scale_off.png");

	public ScaleVisibilityIcon() {
	}
	
	public void setState(int s) {
		state = s;
	}
	
	public void paint(Graphics g) {
		
		if (state == STATE_VISIBLE) {
			g.drawImage(inIcon, 0, 0, null);
		} else {
			if (state == STATE_NOT_VISIBLE) {
				g.drawImage(outIcon, 0, 0, null);
			}
		}
	}

}
