/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Panel;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;

/**
 * This class represents a button that will show only an icon and listen to mouse
 * events.
 * 
 * It is a square button, with two versions: Default and small. the sizes are:
 * 
 * GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH
 * GvsigMobileGUIConstants.ICON_SIZE_SMALL
 * 
 * @see es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants
 * @author jldominguez
 *
 */
public class TocButton extends Panel {
	
	private static Logger logger = Logger.getLogger(TocButton.class);
	
	private Image icon, dis_icon = null;
	private int offset;
	
	private boolean enabled = true;
	private boolean can_dis = false;
	
	/**
	 * If the button is not enabled, if wont listen to mouse events and
	 * the main icon will not show
	 */
	public void setEnabled(boolean e) {
		super.setEnabled(e);
		enabled = (e || (!can_dis));
		repaint();
	}
	
	public boolean isEnabled() {
		return enabled;
	}
	
	/**
	 * COnstructor.
	 * 
	 * @param image_name icon full path name
	 * @param ind index of this button, useful when there are several buttons in a row/column
	 * @param small whether it is a small icon (see class description)
	 * @param up_down true when the button is part of a column of buttons,
	 * false when it's part of a row of buttons.
	 */
	public TocButton(String image_name, int ind, boolean small, boolean up_down) {

		int size = GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH;
		if (small) {
			size = GvsigMobileGUIConstants.ICON_SIZE_SMALL;
		}
		offset = MARGIN + ind * (size + MARGIN) ;
		icon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, image_name);

		if (up_down) {
			setBounds(MARGIN, offset, size, size);
		} else {
			setBounds(offset, MARGIN, size, size);
		}
		
	}
	
	/**
	 * Constructor
	 * 
	 * @param image_name icon image full file path
	 * @param small whether it's a small button
	 */
	public TocButton(String image_name, boolean small) {
		
		int size = GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH;
		if (small) {
			size = GvsigMobileGUIConstants.ICON_SIZE_SMALL;
		}

		icon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, image_name);
		setSize(size, size);
	}

	/**
	 * Constructor.
	 * 
	 * @param image_name icon full path name
	 * @param dis_image_name icon full path name of the icon when button is disabled 
	 * @param ind index of this button, useful when there are several buttons in a row/column
	 * @param small whether it is a small icon (see class description)
	 * @param up_down true when the button is part of a column of buttons,
	 * false when it's part of a row of buttons.
	 */
	public TocButton(String image_name, String dis_image_name, int ind, boolean small, boolean up_down) {
		
		can_dis = true;
		
		int size = GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH;
		if (small) {
			size = GvsigMobileGUIConstants.ICON_SIZE_SMALL;
		}
		offset = MARGIN + ind * (size + MARGIN) ;
		icon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, image_name);
		dis_icon = ResourceReader.getResourceImage(ResourceReader.IMG_SKIN_PATH, dis_image_name);
		
		if (up_down) {
			setBounds(MARGIN, offset, size, size);
		} else {
			setBounds(offset, MARGIN, size, size);
		}
	}

	public static final int MARGIN = 3;

	/**
	 * Overriden method.
	 * 
	 * @param g The graphics object on which this tobject will be painted
	 */
	public void paint(Graphics g) {
		int y = 0; // (getHeight()-icon.getHeight(null))/2;

		int x=0;
		if (enabled) {
			g.drawImage(icon, x, y, null);
		} else {
			if (dis_icon != null) {
				g.drawImage(dis_icon, x, y, null);
			}
		}
		
	}

}
