/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.dialog.control;

import java.awt.Checkbox;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSLayerDescription;
import es.prodevelop.gvsig.mobile.gui.dialog.WMSDialog;

/**
 * This class represents a row in the list of available layers when accessing
 * a WMS server. The left margin depend on the "depth" of the layer
 * (in a tree strcuture).
 * 
 * Behavior:
 * 
 * - when an unchecked checkbox is checked, all its descendats are checked
 * - when a checked checkbox is unchecked, all its direct ancestors are unchecked
 * to keep coherency
 * 
 * This class keeps a reference to the WMSLayerDescription of the layer.
 * 
 * It keeps a reference to the parent dialog, since it needs to be refreshed when one of the
 * checkboxes changes.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WMSLayerDescription
 * @see es.prodevelop.gvsig.mobile.gui.dialog.WMSDialog
 *  
 * @author jldominguez
 *
 */
public class WMSLayerListItem extends Panel implements ItemListener {
	
	private static Logger logger = Logger.getLogger(WMSLayerListItem.class);	
	
	private boolean selected = false;
	private WMSLayerDescription desc = null;
	
	private Label leftLabel = new Label("");
	private Checkbox chk = new Checkbox();
	private WmsLayerNameLabel nameLabel = null;
	private int maxw = 100;
	private String layerid = "";
	private WMSDialog mainDialog;
	private boolean listening = true;
	
	/**
	 * @return whether the checkbox is checked or not
	 */
	public boolean isChecked() {
		return chk.getState();
	}

	public WMSLayerDescription getLayerDescription() {
		return desc;
	}
	
	/**
	 * Utility method to manage the behavior of the check boxes.
	 *
	 */
	public void updateCheckboxInSilence() {
		
		listening = false;
		chk.setState(desc.isSelected());
		listening = true;
	}
	
	public String getLayerId() {
		return desc.getLayerId();
	}

	/**
	 * Constructor
	 * 
	 * @param d WMS layer description
	 * @param max_w max width (in pixels) of the row
	 * @param dlg dialog that contains this row.
	 */
	public WMSLayerListItem(WMSLayerDescription d, int max_w, WMSDialog dlg) {
		
		mainDialog = dlg;
		maxw = max_w;
		desc = d;
		
		setLayout(null);
		
		int leftm = GvsigMobileGUIConstants.WMS_LAYER_ITEM_LEVEL_SKIP * desc.getLevel();
		
		leftLabel.setBounds(0, 0, leftm, GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT);
		chk.setBounds(leftm, 0, GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT, GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT);
		chk.addItemListener(this);
		int aux = leftm + GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT;
		nameLabel = new WmsLayerNameLabel(d.getLayerName(), d.getLayerId());
		nameLabel.setBounds(aux, 0, maxw - aux, GvsigMobileGUIConstants.WMS_LAYER_ITEM_HEIGHT);
		
		add(leftLabel);
		add(chk);
		add(nameLabel);
	}
	
	/**
	 * This methis will force a refresh of the other checkboxes.
	 */
	public void itemStateChanged(ItemEvent e) {
		
		if (! listening) return;

		boolean s = chk.getState();
		desc.setSelected(s, true);
		mainDialog.updateCheckboxes();
	}

}
