/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tab;

import java.awt.Component;
import java.awt.ScrollPane;

import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;

/**
 * 
 * This class is used to contain a scroll pane that has to fit a certain
 * size dynamically.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog
 * 
 * @author jcarras
 *
 */
public class ExpansibleScrolledPanel extends ScrollPane{

	private ExpansiblePanel panel;
	
	/**
	 * Constructor with default size.
	 *
	 */
	public ExpansibleScrolledPanel() {
		super();
		setSize(100, 20);
		initialize();
	}
	
	/**
	 * Constructor with specific size.
	 * 
	 * @param w initial width
	 * @param h initial height
	 *
	 */
	public ExpansibleScrolledPanel(int w, int h) {
		super();
		setSize(w, h);
		initialize();
	}
	
	private void initialize(){
		
		panel = new ExpansiblePanel(this.getWidth(),this.getHeight());
		super.add(panel);
	}
	
	/**
	 * This method is called when the panel components have been added
	 * and this component has to adopt a sensible size.
	 *
	 */
	public void expand(){
		panel.setSize(this.getWidth()-4, this.getHeight()-4);
		panel.expand();
		doLayout();
		if (panel.getWidth()>this.getViewportSize().width){ //
			panel.setSize(this.getViewportSize().width-4, getHeight()-4);
			panel.expand();
			doLayout();
		}

		//System.out.println("expansiblePanel Height = " + panel.getHeight());
	}
	
	/**
	 * Adds a new component to this container.
	 * 
	 * @param com the new component to be added.
	 * 
	 */
	public Component add(Component com){
		return panel.add(com);
	}
	
	/**
	 * Adds a new expansible component to this container
	 * 
	 * @param row the new component to be added
	 */
	public void addExpansible(ExpansibleRow row){
		panel.addExpansible(row);
	}
	
	/**
	 * Gets the maximum height that this component can have.
	 * 
	 * @return the maximum height of this component.
	 */
	public int getMaxHeight(){
		return panel.getMaxHeight();
	}
}
