/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tab;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Window;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;

/**
 * This class implements a tabbed pane using the available AWT classes.
 * 
 * It consists of a title, a central <code>TabbedPanel</code> to contain the tabs
 * and a ok-cancel bottom row.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tab.TabbedPanel
 * 
 * @author jldominguez
 *
 */
public class TabbedDialog extends Window implements MouseListener {
	
	private static Logger logger = Logger.getLogger(TabbedDialog.class);
	
	private Frame parent = null;
	
	private Label titleLabel;
	private TabbedPanel tabPanel;
	protected TocButton okButton, cancelButton;
	private String title;
	private boolean ok_Pressed = false;
	
	private Panel northp, southp;
	
	/**
	 * Gets whether the user has closed this dialog by clicking [ok] or not
	 * @return whether the user has closed this dialog by clicking [ok] or not
	 */
	public boolean isOkPressed() {
		return ok_Pressed;
	}
	
	/**
	 * Constructor.
	 * @param parentFrame parent frame, needed for graphical purposes
	 * @param t The dialog title.
	 */
	public TabbedDialog(Frame parentFrame, String t) {
		super(parentFrame);
		parent = parentFrame;
		title = t;
		initialize();
		// setModal(true);
	}
	
	/**
	 * Reorders the dialog components after they have been added to adopt a sensible
	 * layout.
	 *
	 */
	public void reorder() {
		northp.setSize(getWidth()-4, DialogConstants.CONTROL_HEIGHT);
		southp.setSize(getWidth()-4, DialogConstants.CONTROL_HEIGHT);
		doLayout();
		tabPanel.reorder(
				tabPanel.getWidth()-4,
				tabPanel.getHeight()-4);
				//getHeight()-4-2*DialogConstants.CONTROL_HEIGHT);
		doLayout();
	}
	
	/**
	 * Adds a new tab to this dialog.
	 * 
	 * @param title the title of the new tab.
	 * @param tabComponent the component associated with the new tab. Normally, a panel.
	 * @return the new tab count.
	 */
	public int addTab(String title, Component tabComponent) {
		int resp = tabPanel.addTab(title, tabComponent);
		
		return resp;
	}
	
	/**
	 * Closes this dialog and tries to destroy it.
	 * 
	 */
	public void close() {
		dispose();
	}
	
	
	private void initialize() {
		
		int x=0;
		int y=0;
		if (Utils.USING_PDA) {
			// WINDOW-DIALOG
			// setUndecorated(true);
		} else {
			y=20;
		}
		
		setBounds(
				parent.getX(),
				parent.getY(),
				parent.getWidth(),
				parent.getHeight()
				);		
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				reorder();
			}
		});
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				close();
			}
		});
		
		titleLabel = new Label(title);
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);
		titleLabel.setSize(
				DialogConstants.LABELS_WIDTH,
				DialogConstants.CONTROL_HEIGHT);
		
		okButton = new TocButton(
				DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		okButton.addMouseListener(this);
		cancelButton = new TocButton(
				DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 1, true, false);
		cancelButton.addMouseListener(this);
		
		setLayout(new BorderLayout());
		setBounds(parent.getX(), parent.getY(), parent.getWidth(), parent.getHeight());
		
		northp = new Panel();
		northp.setSize(titleLabel.getWidth(), DialogConstants.CONTROL_HEIGHT);
		northp.add(titleLabel);
		
		southp = new Panel();
		southp.setSize(getWidth(), GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT);
		southp.setLayout(null);
		southp.add(okButton);
		southp.add(cancelButton);
		
		tabPanel = new TabbedPanel(
				getWidth(),
				2 * DialogConstants.TABLE_LABEL_HEIGHT);

		add(northp, BorderLayout.NORTH);
		add(tabPanel, BorderLayout.CENTER);
		add(southp, BorderLayout.SOUTH);
		
		reorder();
	}
	
	public void mousePressed(MouseEvent e) {
		//System.out.print(this.getClass().getName());
		//System.out.println(e);
		Object src = e.getSource();
		
		if (src == okButton) {
			ok_Pressed = true;
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("Tabbed dialog. A mysterious NP exception was caught: " + npe.getMessage());
			}
		}
		
		if (src == cancelButton) {
			ok_Pressed = false;
			try {
				hide();
			} catch (NullPointerException npe) {
				logger.warn("Tabbed dialog. A mysterious NP exception was caught: " + npe.getMessage());
			}
		}
	}

	public void mouseClicked(MouseEvent arg0) { }
	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) { }
}
