/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tab;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Panel;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.Set;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;

/**
 * This class is used inside the <code>TabbedDialog</code> to contain the tabs.
 * 
 * The strings are placed horizontally in a strip at the top
 * (<code>TabbedPanelTopPanel</code>). Components
 * are placed in the middle with a <code>CardLayout</code>
 * 
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tab.TabbedDialog
 * @see java.awt.CardLayout
 * @see es.prodevelop.gvsig.mobile.gui.tab.TabbedPanelTopPanel
 * 
 * @author jldominguez
 *
 */
public class TabbedPanel extends Panel implements MouseListener {
	
	private static Logger logger = Logger.getLogger(TabbedPanel.class);
	
	private int width = 10, componentsHeight = 10;
	private TabbedPanelTopPanel tabsPanel;
	private Panel  mainPanel;
	private CardLayout cardLayout;

	private HashMap tabToComponent = new HashMap();
	//private Container conSelected = null;
	
	/**
	 * Reorders the components to the new overall size.
	 * 
	 * @param w the new width
	 * @param h the new height 
	 */
	public void reorder(int w, int h) {
		// logger.debug("TabbedPanel, reorder, w = " + w);
		// logger.debug("TabbedPanel, reorder, h = " + h);
		
		setSize(w, h);
		//System.out.println("tabbedPanel Height = " + h);
		width = w;
		componentsHeight = h - DialogConstants.TABLE_LABEL_HEIGHT;
		
		tabsPanel.setSize(width, DialogConstants.TABLE_LABEL_HEIGHT);
		tabsPanel.rewidth();
		
		mainPanel.setSize(w, componentsHeight);
		//System.out.println("mainPanel Height = " + componentsHeight);
		//mainPanel.setSize(w, h);
		Component[] com = mainPanel.getComponents(); 
		for(int i=0;i<com.length;i++){
			if (com[i] instanceof ExpansibleScrolledPanel){
			//if (conSelected!=null && conSelected instanceof ExpansibleScrolledPanel){
				ExpansibleScrolledPanel comi = (ExpansibleScrolledPanel)com[i];
				comi.setSize(w, componentsHeight);
				comi.expand();
				//System.out.println("scrollPanel Height = " + comi.getHeight());
			}
			
		}
	}	

	/**
	 * COnstructor.
	 * 
	 * @param _w the initial width in pixels
	 * @param _h the initial height in pixels
	 */
	public TabbedPanel(int _w, int _h) {
		
		// setSize(_w, _h);
		
		width = _w;
		
		componentsHeight = _h - DialogConstants.TABLE_LABEL_HEIGHT;
		// logger.debug("TabbedPanel, componentsHeight = " + componentsHeight);
		
		tabsPanel = new TabbedPanelTopPanel(width);
		tabsPanel.setSize(width, DialogConstants.TABLE_LABEL_HEIGHT);

		cardLayout = new CardLayout(DialogConstants.INSIDE_MARGIN, DialogConstants.INSIDE_MARGIN);
		
		mainPanel = new Panel(cardLayout);
		mainPanel.setSize(width, componentsHeight);
		
		setLayout(new BorderLayout());
		add(tabsPanel, BorderLayout.NORTH);
		add(mainPanel, BorderLayout.CENTER);
	}
	
	/**
	 * Adds a new tab to this panel.
	 * 
	 * @param title the title of the new tab
	 * @param c the component associated to the new tab
	 * @return the new tab count
	 */
	public int addTab(String title, Component c) {
		
		int size = tabToComponent.size();
		
		TabLabel newTabLabel = new TabLabel(title, this);
		tabToComponent.put(newTabLabel, c);
		
		mainPanel.add(title, c);
		
		c.setSize(mainPanel.getWidth(), mainPanel.getHeight());
		
		tabsPanel.addTabLabel(newTabLabel);
		
		// ---------------- moving to first after adding:
		Component first = tabsPanel.getFirst();
		if (first != null) {
			/*MouseEvent update_event =
				new MouseEvent(first, 0, 0, 0, 0, 0, 0, false);
			mousePressed(update_event);*/
			setSelected((TabLabel)first);
		}
		// ---------------- moving to first after adding.
		
		return (size + 1);
	}

	public void mousePressed(MouseEvent arg0) {
		
		Object src = arg0.getSource();
		
		if (src instanceof TabLabel) {
			TabLabel l = (TabLabel) src;
			setSelected(l);			
			
		}
	}
	private int kk=0;
	private void setSelected(TabLabel tl) {
		tabsPanel.repaint();
		
		//reorder(getWidth(), getHeight());
		cardLayout.show(mainPanel, tl.getText());
		//validate();
		
		Set keys = tabToComponent.keySet();
		Object[] keys_array = keys.toArray();
		
		tl.setSelected(true);
		for (int i=0; i<keys_array.length; i++) {
			TabLabel item = (TabLabel) keys_array[i];
			if (item != tl) {
				item.setSelected(false);
				// logger.debug("another one...");
			} else {
				// logger.debug("found it!");
			}
			item.repaint();
		}
		if (kk>1){
			Component[] com = mainPanel.getComponents(); 
			for(int i=0;i<com.length;i++){
				if (com[i] instanceof ExpansibleScrolledPanel){
				//if (conSelected!=null && conSelected instanceof ExpansibleScrolledPanel){
					ExpansibleScrolledPanel comi = (ExpansibleScrolledPanel)com[i];
					comi.setSize(getWidth(), componentsHeight);
					comi.expand();
					//System.out.println("scrollPanel Height = " + comi.getHeight());
				}
			}
		}
		kk+=1;
		//doLayout();
	}

	public void mouseClicked(MouseEvent arg0) { }
	public void mouseReleased(MouseEvent arg0) { }
	public void mouseEntered(MouseEvent arg0) { }
	public void mouseExited(MouseEvent arg0) { }



}
