/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseListener;

/**
 * A tool bar button without state. 
 * When clicked an action is performed at once.
 * <p>These buttons don't affect other selected buttons in a Tool Bar</p>
 *  
 * @author mmontesinos
 *
 */
public class TBActionButton extends TBButton {
	
	/**
	 * Creates an instance of a TBActionButton button by calling to the
	 * superclass constructor given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface,e.g.
	 * <code>ToolBar</code>), the icons and the listener of the events for this
	 * new instance
	 * @param _owner Parent of the TBActionButton (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 */
	public TBActionButton(IToolContainer _owner,
			 String _icon,
			 String _selectedIcon,
			 String _helpText,
			 MouseListener _mouseListener) {
		super(_owner, _icon, _selectedIcon, _helpText, _mouseListener);
		
	}
	
	/**
	 * Constructs a new instance of this class given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface,e.g.
	 * <code>ToolBar</code>), the icons, the listener of the events for this
	 * new instance and the position of the new button in the ToolBar
	 * @param _owner Parent of the TBActionButton (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 * @param _index Position of the new button in the <code>ToolBar</code>.
	 * 				 It must be an <code>int</code> between 0 and the actual
	 * 				 size of the array.
	 */
	public TBActionButton(IToolContainer _owner,
						 String _icon,
						 String _selectedIcon,
						 String _helpText,
						 MouseListener _mouseListener,
						 int _index){
		super(_owner, _icon, _selectedIcon, _helpText, _mouseListener, _index);
	}

	/**
	 * Creates an instance of a TBActionButton button by calling to the
	 * superclass constructor given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface,e.g.
	 * <code>ToolBar</code>, <code>TBMenu</code>,...),the icons and the
	 * listener of the events for this new instance
	 * @param _owner Parent of the TBActionButton (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 */
	public TBActionButton(IToolContainer _owner,
			 Image _icon,
			 Image _selectedIcon,
			 String _helpText,
			 MouseListener _mouseListener) {
		super(_owner, _icon, _selectedIcon, _helpText, _mouseListener);
		
	}
	

	/**
	 * Creates an instance of a TBActionButton button by calling to the
	 * superclass constructor given the icons and the
	 * listener of the events for this new instance
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 */
	public TBActionButton(String _icon,
			 String _selectedIcon,
			 String _helpText,
			 MouseListener _mouseListener) {
		super(_icon, _selectedIcon, _helpText, _mouseListener);

		
	}
	
	/**
	 * Forces the paint of the object. This method should be called when the 
	 * content of the TBActionButton needs to be painted.
	 * This method overrides the paint method of the <code>TBWidget</code> parent
	 * by painting the selected image when the state is <i>selected</i> and then
	 * repaints the unselected image and sets the state to <i>unselected</i>
	 */
	public void paint(Graphics g) {
		if (selected) {
			if (selIcon != null) g.drawImage(selIcon, 0, 0, null);
		} 
		if (icon != null) g.drawImage(icon, 0, 0, null);
		
		this.setSelected(false);
	}

	/**
	 * Changes the <i>selected/unselected</i> state of the button.
	 * If it's <i>selected</i>, the new status will be <i>selected</i>
	 * and vice versa 
	 *
	 */
	public void switchButton() {
		selected = !selected;
	}


}
