/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool;

import java.awt.Image;
import java.awt.event.MouseListener;

import es.prodevelop.gvsig.mobile.common.GvsigMobileException;
import es.prodevelop.gvsig.mobile.common.GvsigMobileGUIConstants;

/**
 * Abstract class for toolbar Buttons. All types of buttons which launch a
 * command inheritate from TBButton
 * @author mmontesinos
 * 
 * @see TBActionButton
 * @see TBSelectableButton
 * @see TBComboButton
 * 
 */
public abstract class TBButton extends TBWidget implements Cloneable {
	
	private String helpText = "";
	private int width = GvsigMobileGUIConstants.DEFAULT_BUTTON_WIDTH;
	private int height = GvsigMobileGUIConstants.DEFAULT_BUTTON_HEIGHT;
	
	/**
	 * StatusBar Associated with this button
	 */
	protected int statusBarId = -1; //No statusbar
	
	/*
	 * To be used for storing and possibly later adding default buttons to a TBComboButton
	 */
	protected MouseListener myListener; 
	

	/**
	 * Constructs a new instance of this class given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface, 
	 * the icons, the help text and the listener of the events for
	 * this new instance
	 * @param _owner Parent of the TBButton (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 */
	public TBButton(IToolContainer _owner,
						 String _icon,
						 String _selectedIcon,
						 String _helpText,
						 MouseListener _mouseListener) {
		super(_owner, _icon, _selectedIcon, _mouseListener);
		this.setVisible(false); //Invisible after creation
		this.helpText = _helpText;
		this.myListener = _mouseListener;
		try{
			_owner.addButton(this, _owner.getButtonCount());
		}
		catch (GvsigMobileException e){
			logger.error(e.getMessage());
			// System.err.println("Error: " + e.getMessage());
			// e.printStackTrace();
		}
	}

	/**
	 * Constructs a new instance of this class given the owner where it should
	 * be placed (which must accomplish a <code>IToolContainer</code> interface, 
	 * the icons, the help text and the listener of the events for
	 * this new instance and the position of the new button in the container
	 * @param _owner Parent of the TBButton (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 * @param _index Position of the new button in the <code>ToolBar</code>.
	 * 				 It must be an <code>int</code> between 0 and the actual
	 * 				 size of the array.
	 */
	public TBButton(IToolContainer _owner,
						 String _icon,
						 String _selectedIcon,
						 String _helpText,
						 MouseListener _mouseListener,
						 int _index){
		super(_owner, _icon, _selectedIcon, _mouseListener);
		this.setVisible(false); //Invisible after creation
		this.helpText = _helpText;
		this.myListener = _mouseListener;
		try{
			_owner.addButton(this, _index);
		}
		catch (GvsigMobileException e){
			logger.error(e.getMessage());
			// System.err.println("Error: " + e.getMessage());
			// e.printStackTrace();
		}
	}
	

	/**
	 * Constructs a new instance of this class given the owner where it should
	 * be placed (which must be a <code>ToolBar</code>, the icons <code>Image</code>s, the help 
	 * text and the listener of the events for this new instance
	 * @param _owner Parent of the TBButton (<code>IToolContainer</code> compliant,
	 * 				 e.g. a <code>ToolBar</code>) where the button is to be added
	 * @param _icon Image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Image to be loaded as <i>selected</i>
	 * 			 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 */
	public TBButton(IToolContainer _owner,
						 Image _icon,
						 Image _selectedIcon,
						 String _helpText,
						 MouseListener _mouseListener) {
		super(_owner, _icon, _selectedIcon, _mouseListener);
		this.setVisible(false); //Invisible after creation
		this.helpText = _helpText;
		this.myListener = _mouseListener;
		try{
			_owner.addButton(this, _owner.getButtonCount());
		}
		catch (GvsigMobileException e){
			logger.error(e.getMessage());
			// System.err.println("Error: " + e.getMessage());
			// e.printStackTrace();
		}
	}
	
	/**
	 * Constructs a new instance of this class not given yet the owner where it should
	 * be placed, but given the icons , the help 
	 * text and the listener of the events for this new instance
	 * @param _icon Name of the image to be loaded as <i>normal</i> image 
	 * @param _selectedIcon Name of the image to be loaded as <i>selected</i>
	 * 				 image. Used when the button is selected
	 * @param _helpText String with a short help description of the button.
	 * 	   			 it shouldn't be longer than 2 words.
	 * @param _mouseListener Listener of Mouse Events, which must be a
	 * 				 <code>MousListener</code>
	 */
	public TBButton(String _icon,
						 String _selectedIcon,
						 String _helpText,
						 MouseListener _mouseListener) {
		super(_icon, _selectedIcon, _mouseListener);
		this.setVisible(false); //Invisible after creation
		this.helpText = _helpText;
		this.myListener = _mouseListener;
	}

	/**
	 * @return Returns the height.
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param height The height to set in pixels.
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * @return Returns the width in pixels.
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @param width The width to set in pixels.
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	/**
	 * @return Returns the helpText. A string with a short help description
	 * of the button. It shouldn't be longer than 2 words.
	 */
	public String getHelpText() {
		return helpText;
	}

	/**
	 * @return Returns the MouseListener of the button.
	 * @see TBComboButton
	 */
	protected MouseListener getMyListener() {
		return myListener;
	}
	
	/**
	 * "pseudo-override" visilibity and return typeof <code>java.lang.Object.clone()</code> for being used
	 * when copying array of TBButtons
	 * @return <code>TBButton</code> cloned
	 */
	public TBButton buttonClone(){
		try {
			return ((TBButton) this.clone());
		} catch (CloneNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
	
	public boolean equals(Object obj){
		
		TBButton button = null;
		try{
			button = (TBButton) obj;
			//Let's check main attributes
			if ((this.helpText == button.helpText)
				&& (this.icon == button.icon)
				&& (this.selIcon == button.selIcon)
				&& (this.height == button.height)
				&& (this.width == button.width)
				&& (this.myListener == button.myListener)){
			
				return true;
			} else{ 
				return false;
			}
		}
		catch (Exception e) {
			return false;
		}
		
	}

	public int getStatusBarId() {
		return statusBarId;
	}

	public void setStatusBarId(int statusBarId) {
		this.statusBarId = statusBarId;
	}



}
