package es.prodevelop.gvsig.mobile.gui.tool.behavior;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.EditionLineAreaToolListener;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

public class EditionLineBehaviour extends Behavior{
	private Logger logger = Logger.getLogger(EditionPointBehaviour.class);
	private EditionLineAreaToolListener listener = null;

	/**
	 * Gets the current listener, which will be notified when point events occur.
	 * @return
	 */
	public PointToolListener getListener() {
		return listener;
	}

	/**
	 * Constructor.
	 *
	 * @param l the listener, which will be notified when point events occur.
	 */
	public EditionLineBehaviour(PointToolListener l) {
		if(l instanceof EditionLineAreaToolListener)
			listener = (EditionLineAreaToolListener) l;
	}

	/**
	 * Overriden method, in case the listener needs to set another cursor while
	 * its tool is active.
	 *
	 * @return the cursor to be used when the associated tool is active.
	 */
	public Cursor getCursor() {
		return listener.getCursor();
	}

	/**
	 * This behavior class creates a relevant event for point events and
	 * notifies the current listener.
	 *
	 * @param e the mouse event generated by the user on the map
	 */
	public void mousePressed(MouseEvent e) {
		logger.info("Capturo el evento");
		PointEvent pe = new PointEvent(new Point(e.getX(), e.getY()), e);

		listener.point(pe);
	}

	/**
	 * This behavior class creates a relevant event for point events and
	 * notifies the current listener.
	 *
	 * @param e the mouse event generated by the user on the map
	 */
	public void mouseReleased(MouseEvent e) {
		logger.info("Capturo el evento");
		PointEvent pe = new PointEvent(new Point(e.getX(), e.getY()), e);

		listener.point(pe);
	}

	/**
	 * This behavior class creates a relevant event for point events and
	 * notifies the current listener.
	 *
	 * @param e the mouse event generated by the user on the map
	 */
	public void mouseDragged(MouseEvent e) {
		PointEvent pe = new PointEvent(new Point(e.getX(), e.getY()), e);
		listener.point(pe);
	}
}
