/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.behavior;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.event.RectangleEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.RectangleToolListener;

/**
 * This behavior class is used when the tool needs to know the rectangle
 * defined with a drag and drop process. So it applies to tools like zoom to rectangle
 * or select by rectangle.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.RectangleToolListener
 * 
 * @author jldominguez
 *
 */
public class RectangleBehavior extends Behavior {

	private static Logger logger = Logger.getLogger(RectangleBehavior.class);
	private RectangleToolListener listener = null;
	
	private int start_x = 0;
	private int start_y = 0;
	
	/**
	 * COnstructor.
	 * 
	 * @param l the listener which will be notified with rectangle-like
	 * events.
	 * 
	 */
	public RectangleBehavior(RectangleToolListener l) {
		listener = l;
	}
	
	/**
	 * Overrides ancestor method in case another cursor has to be used
	 * when the asscodiated tool is in use.
	 * 
	 */
	public Cursor getCursor() {
		return listener.getCursor();
	}

	/**
	 * This behavior class has to remember the starting point of drag and drop processes.
	 * 
	 * @param e the mouse event that could be the start of a drag and drop process.
	 */
	public void mousePressed(MouseEvent e) {
		start_x = e.getX();
		start_y = e.getY();
		
		PointEvent pev = new PointEvent(new Point(e.getX(), e.getY()), e);

		listener.point(pev);
	}

	public void mouseClicked(MouseEvent e) {
	}

	/**
	 * This behavior class has to remember the starting point of drag and drop processes.
	 * When the mouse-up event happens, a rectangle-like event is created and notified
	 * to its listener.
	 * 
	 * @param e the mouse event that could be the end of a drag and drop process.
	 */
	public void mouseReleased(MouseEvent e) {
		RectangleEvent rev = new RectangleEvent(
				new Point(start_x, start_y),
				new Point(e.getX(), e.getY()), e);

		listener.rectangle(rev);
	}

	/**
	 * This behavior class has to remember the starting point of drag and drop processes.
	 * When the mouse-up event happens, a rectangle-like event is created and notified
	 * to its listener. this drag event is in the middle of the process and could be 
	 * useful for the listener.
	 * 
	 * @param e a mouse event in the middle of a drag and drop process.
	 */
	public void mouseDragged(MouseEvent e) {
		PointEvent pev = new PointEvent(new Point(e.getX(), e.getY()), e);

		listener.point(pev);
	}



}
