/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.event;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

/**
 * Utility class to encapsulate mouse events, used to send info between
 * behavior classes and tool listeners.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.RectangleBehavior
 * @see es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.RectangleToolListener
 * 
 * @author jldominguez
 *
 */
public class RectangleEvent {
	
	private Point start_p;
	private Point end_p;
	private MouseEvent e;

	/**
	 * Constructor.
	 * 
	 * @param p1 the starting point on which the user has clicked 
	 * @param p2 the final point, when the user has lifted the mouse 
	 * @param e the original java mouse event.
	 */
	public RectangleEvent(Point p1, Point p2, MouseEvent e) {
		this.start_p = p1;
		this.end_p = p2;
		this.e = e;
	}

	/**
	 * Gets the rectangle defined by the start and end points.
	 * 
	 * @return the rectangle defined by the start and end points.
	 */
	public Rectangle getRectangle() {
		int x = Math.min(start_p.x, end_p.x);
		int y = Math.min(start_p.y, end_p.y);
		int w = Math.abs(start_p.x - end_p.x);
		int h = Math.abs(start_p.y - end_p.y);
		return new Rectangle(x, y, w, h);
	}

	/**
	 * Sets the initial point.
	 * 
	 * @param p the initial point where the user has clicked.
	 */
	public void setStartPoint(Point p) {
		this.start_p = p;
	}

	/**
	 * Sets the final point.
	 * 
	 * @param p the final point where the user has lifted the mouse.
	 */
	public void setEndPoint(Point p) {
		this.end_p = p;
	}

	/**
	 * Gets the original mouse event.
	 * 
	 * @return the original mouse event.
	 */
	public MouseEvent getEvent() {
		return e;
	}
}