package es.prodevelop.gvsig.mobile.gui.tool.listener;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.listener.GPSRelevantEventsListener;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.core.DefaultFeature;
import es.prodevelop.gvsig.mobile.fmap.core.GeneralPathX;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.edition.EditionEvent;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.canvas.TransparentCanvas;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

public class EditionLineAreaToolListener implements PointToolListener, GPSRelevantEventsListener{

	private MapControl mapControl = null;
	private TransparentCanvas transparente = null;
	private static Logger logger = Logger.getLogger(EditionLineAreaToolListener.class);
	private FLyrVect lyr = null;
	private boolean isArea = false;
	private boolean emptyGpx = true;
	private GeneralPathX gpx = new GeneralPathX();
	private int count = 0;
	private IGeometry ig = null;
	private Point2D firstPoint = null;

	private boolean isGPSAutomatic, newArea = false;
	private GPSManager gpsManager = GPSManager.getGPSManager();
	private double gpsMapX = 0;
	private double gpsMapY = 0;
	private double gpsPreviousX = 0;
	private double gpsPreviousY = 0;
	private double toleranceX = 0;
	private double toleranceY = 0;
	private ArrayList areas = new ArrayList();

	private IGeometry previousPol = null;

	private IGeometry previousLine = null;

	public EditionLineAreaToolListener(boolean area){
		isArea = area;
		transparente = TransparentCanvas.getInstance();
	}


	public void point(PointEvent event) {

		if (event.getEvent().getID() == MouseEvent.MOUSE_PRESSED) {
			Point point = event.getPoint();
			ViewPort vp = mapControl.getViewPort();
			Point2D pointMap = vp.toMapPoint(new Point2D.Double(point.x, point.y));

			Rectangle2D gpsRect =null;
			double[] coords = null;

			if(gpsManager.isConnected()){
				if(mapControl.getMapCursorSupport().getGpsPositionVisible()){

					double longitud = gpsManager.lastGPSFix.getLongitude();
					double latitud = gpsManager.lastGPSFix.getLatitude();

					coords = ConversionCoords.wgs842projection(
							longitud,
							latitud,
							mapControl.getProjection());

					logger.info("Calculo la posicion y tama�o del icono");
					gpsRect = new Rectangle2D.Double(
							coords[0]-50/2,
							coords[1]-50/2,
							50,50);
				}
			}

			if(gpsRect!=null){
				Rectangle2D rectPointMap = new Rectangle2D.Double(
						pointMap.getX()-29/2,
						pointMap.getY()-29/2,
						29,29);

				logger.info("Calculo interseccion");
				boolean intersects = Utils.rectanglesItersect(rectPointMap,gpsRect);

				if(intersects){
					logger.info("Intersecta con el icono del GPS");
					drawPoint(new Point2D.Double(coords[0],coords[1]));
				}else{
					drawPoint(pointMap);
				}
			}

			else{
				drawPoint(pointMap);
			}
		}

		if(event.getEvent().getClickCount()==2 && event.getEvent().getID() == MouseEvent.MOUSE_PRESSED){
			stopAreaGeometry();
		}

		if (event.getEvent().getID() == MouseEvent.MOUSE_DRAGGED) {
			return;
		}

		if (event.getEvent().getID() == MouseEvent.MOUSE_RELEASED) {
			return;
		}

	}

	private void addGeometry(IGeometry geom){
		if(geom!=null){
			logger.info("Entro en addGeometry");
			int countLyrs = mapControl.getLayerCount();
			lyr = null;
			for (int i = 0; i < countLyrs; i++) {
				FLayer lyraux = mapControl.getLayer(i);
				if (lyraux.isEditing() && lyraux instanceof FLyrVect) {
					lyr = (FLyrVect) lyraux;
				}
			}
			logger.info("Obtengo la capa");
			if (lyr != null) {
				logger.info("Capa distinto de null");

				if(!lyr.isAvailable())
					logger.info("Capa no disponible");
				VectorialEditableAdapter vea = (VectorialEditableAdapter) lyr
				.getSource();
				logger.info("Obtengo el vea y el s�mbolo de la capa");
				// Deber�amos comprobar que lo que escribimos es correcto:
				// Lo hacemos en el VectorialAdapter, justo antes de
				// a�adir, borrar o modificar una feature

				try {

					int numAttr = vea.getRecordset().getFieldCount();
					Value[] values = new Value[numAttr];
					for (int i = 0; i < numAttr; i++) {
						values[i] = ValueFactory.createNullValue();
					}
					logger.info("Creo los value");
					String newFID;

					newFID = vea.getNewFID();
					DefaultFeature df = new DefaultFeature(geom.cloneGeometry(),
							values, newFID);
					vea.addRow(df, getName(), EditionEvent.GRAPHIC);

				} catch (IOException e) {
					logger.info("While adding a row: " + e.getMessage());
					return;
				} catch (FMapDriverException e) {
					logger.info("While loading the data source from the layer: "
							+ e.getMessage());
					return;
				} catch (DriverException e) {
					logger.info("While loading the data source from the layer: "
							+ e.getMessage());
					return;
				}
				logger.info("Termino addGeometry");
			}
		}
	}

	private void drawPoint(Point2D p) {

		if (emptyGpx) {
			emptyGpx = false;
			gpx.moveTo(p.getX(), p.getY());
			ig = ShapeFactory.createPoint2D(p.getX(), p.getY());
			transparente.addEditionGeometry(ig);
			firstPoint=p;
			mapControl.refresh(true);
			newArea=true;


		} else {
			gpx.lineTo(p.getX(), p.getY());
			if (isArea) {
				GeneralPathX _gpx = (GeneralPathX) gpx.clone();
				_gpx.lineTo(firstPoint.getX(),firstPoint.getY());
//				_gpx.closePath();
				ig = ShapeFactory.createPolygon2D(_gpx);
				if (previousPol != null && !newArea){
					areas.remove(previousPol);
					transparente.removeEditionGeometry(previousPol);

				}
				newArea=false;
				previousPol = ig;
				areas.add(previousPol);

			} else {
				ig = ShapeFactory.createPolyline2D(gpx);
//				addGeometry(ig);
//				start();
			}
			transparente.addEditionGeometry(ig);
			mapControl.refresh(true);
		}

	}


	public Cursor getCursor() {
		return null;
	}

	public boolean cancelDrawing() {
		return false;
	}

	public void setMapControl(MapControl mc) {
		mapControl = mc;

	}

	private String getName() {
		return "line";
	}

	public void setFirstPoint(){
		emptyGpx = true;
		newArea = true;
	}

	public void start() {
		emptyGpx = true;
		areas.clear();
		ig=null;
		newArea=true;
		gpx = new GeneralPathX();
	}


	double oldX = -600;
	double oldY = -600;
	boolean locked = false;
	/**
	 * Returns a new position calculated
	 * @param longitude latitude in degrees
	 * @param latitude latitude in degrees
	 */
	public void gpsNewPosition(double longitude, double latitude){

		if (locked) return;

		//Fixed tolerance to reduce the computational cost
		if (Math.abs(longitude - oldX) + Math.abs(latitude - oldY) < 0.0000002){
			return;
		}
		locked=true;

		gpsMapX = longitude;
		gpsMapY = latitude;
		oldX = longitude;
		oldY = latitude;


		logger.debug("-----------------------------------> Entro en el evento del GPS para capturar punto.");
		IProjection the_projection = mapControl.getProject().getProjection();

		String units = " " + the_projection.getUnitsAbbrev(); //$NON-NLS-1$
		int decimals = 2;

		if (GeoUtils.isLatLonProjection(the_projection.getAbrev())) {
			decimals = 5;
		} else {
			decimals = 2;
		}

		//TODO 	Tener en cuenta la proyeccion en la que hay que pedir
		//		las coordenadas alGPS

		double[] coords = ConversionCoords.wgs842projection(
				longitude,
				latitude,
				the_projection);

		gpsMapX=coords[0];
		gpsMapY=coords[1];

		double difX = 0;
		double difY = 0;

		if(gpsMapX > gpsPreviousX)difX = gpsMapX - gpsPreviousX;
		else  difX = gpsPreviousX - gpsMapX;

		if(gpsMapY > gpsPreviousY)difY = gpsMapY - gpsPreviousY;
		else  difY = gpsPreviousY - gpsMapY;


		if(difX > toleranceX || difY > toleranceY ){

			gpsPreviousX = gpsMapX;
			gpsPreviousY = gpsMapY;

			logger.error("X: " + gpsMapX);
			logger.error("Y: " + gpsMapY);

			Point2D point = new Point2D.Double(gpsMapX, gpsMapY);

			drawPoint(point);

			logger.info("-----------------------------------> Termino point");
		}
		locked=false;
	}

	/**
	 * Returns a new altitude
	 * @param altitude altitude in meters
	 */
	public void gpsNewAltitude(double altitude){

	}

	/**
	 * Returns a new heading
	 * @param heading heading in degrees
	 */
	public void gpsNewHeading(double heading){

	}

	/**
	 * Returns a new value for the number of satellites used in the fix
	 * @param satUsed number of satellites used
	 */
	public void gpsNewSatsUsed(int satUsed){

	}

	/**
	 * Returns a new value for the position dillution of precision estimated for the fix
	 * @param pdop position dillution of precision
	 */
	public void gpsNewPDOP(double pdop){

	}

	/**
	 * Returns a new value for the quality of the fix
	 * @param quality one of the following values:
	 *   - FIX_QUALITY_INVALID if the fix is invalid
	 * 	 - FIX_QUALITY_GPS if the fix is an autonomous GPS fix
	 *   - FIX_QUALITY_DGPS if the fix is a differential GPS fix
	 */
	public void gpsNewQuality(int quality){

	}

	/**
	 * Returns a new value for the status of the GPS
	 * @param status one of the following values:
	 * 	 - FIX_DIMENSIONS_INVALID if the GPS has not a valid fix (less than 3 valid satellites)
	 *   - FIX_DIMENSIONS_2D if the GPS is in 2D mode (less than 4 valid satellites)
	 *   - FIX_DIMENSIONS_3D if the GPS is in 3D mode
	 */
	public void gpsNewStatus(int status){

	}

	public boolean isGPSAutomatic(){
		return isGPSAutomatic;
	}


	public void setGPSAutomatic(boolean isGPSAutomatic) {
		this.isGPSAutomatic = isGPSAutomatic;

		if(this.isGPSAutomatic == true){
			gpsManager.addGPSRelevantEventsListener(this);
		}
		else{
			gpsManager.removeGPSRelevantEventsListener(this);
		}
	}

	public void addGPSPoint(Point2D point){
		drawPoint(point);
	}

	public void stopAreaGeometry(){
		for(int i=0; i<areas.size(); i++){
			IGeometry geom = (IGeometry)areas.get(i);
			int length =geom.getHandlers(IGeometry.SELECTHANDLER).length;
			if(geom.getHandlers(IGeometry.SELECTHANDLER)[0].getPoint().equals(
					geom.getHandlers(IGeometry.SELECTHANDLER)[length-1].getPoint()))
				length--;
			if(length >= 3){
				addGeometry(geom);
				setFirstPoint();
				start();
				mapControl.repaintEdition( 0,0,0,0);
				mapControl.repaint();
				mapControl.draw(false);
			}
		}
		areas.clear();
	}

	public void stopPolyLine(){

		addGeometry(ig);
		start();
		mapControl.repaintEdition( 0,0,0,0);
		mapControl.repaint();
		mapControl.draw(false);
		areas.clear();
	}

	/**
	 * Returns a new value for the Starfire quality value
	 * @param starfire an integer between 0 and 26
	 */
	public void gpsNewStarfire(int starfire) {
	}

	/**
	 * Returns a new value for the RMS standard deviation of
	 * the ranges inputs for the navigation solution
	 * @param rms a positive double value
	 */
	public void gpsNewRMS(double rms){
	}

}
