package es.prodevelop.gvsig.mobile.gui.tool.listener;

import org.apache.log4j.Logger;
import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.listener.GPSRelevantEventsListener;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.bar.GPSIcons;
import es.prodevelop.gvsig.mobile.gui.bar.StatusBar;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;

public class EditionPointGPSListener extends EditionPointToolListener implements GPSRelevantEventsListener{

	private MapControl mapControl = null;

	private static GPSManager gpsManager = null;

	private double gpsMapX = 0;

	private double gpsMapY = 0;
	
	private double gpsPreviousX = 0;
	
	private double gpsPreviousY = 0;
	
	private double toleranceX = 0;
	
	private double toleranceY = 0;

	protected int gpsStatus = 0;

	private static Logger logger = Logger
	.getLogger(EditionPointGPSListener.class);



	public EditionPointGPSListener(MapControl mc){
		super.setMapControl(mc);
		mapControl = mc;
		gpsManager = GPSManager.getGPSManager();

		gpsManager.addGPSRelevantEventsListener(this);
		logger.debug("Creating editing gps listener...");
		
		toleranceX = gpsManager.getToleranceX();
		toleranceY = gpsManager.getToleranceY();

	}




	public void gpsNewPosition(double longitude, double latitude) {

		gpsMapX = longitude;
		gpsMapY = latitude;

		logger.error("-----------------------------------> Entro en el evento del GPS para capturar punto.");
		IProjection the_projection = mapControl.getProject().getProjection();

		String units = " " + the_projection.getUnitsAbbrev(); //$NON-NLS-1$
		int decimals = 2;

		if (GeoUtils.isLatLonProjection(the_projection.getAbrev())) {
			decimals = 5;
		} else {
			decimals = 2;
		}

		//TODO 	Tener en cuenta la proyeccion en la que hay que pedir
		//		las coordenadas alGPS

		double[] coords = ConversionCoords.wgs842projection(
				longitude,
				latitude,
				the_projection);

		gpsMapX=coords[0];
		gpsMapY=coords[1];

		double difX = 0;
		double difY = 0;
		
		if(gpsMapX > gpsPreviousX)difX = gpsMapX - gpsPreviousX;
		else  difX = gpsPreviousX - gpsMapX;
		
		if(gpsMapY > gpsPreviousY)difY = gpsMapY - gpsPreviousY;
		else  difY = gpsPreviousY - gpsMapY;
		
		
		if(difX > toleranceX || difY > toleranceY ){
			
			gpsPreviousX = gpsMapX;
			gpsPreviousY = gpsMapY;
			
			logger.error("X: " + gpsMapX);
			logger.error("Y: " + gpsMapY);
			
			IGeometry geom =ShapeFactory.createPoint2D(gpsMapX, gpsMapY);
			addGeometry(geom);
			
			logger.info("-----------------------------------> Termino point");
		}
	}

	public void gpsNewAltitude(double altitude) {}

	public void gpsNewHeading(double heading) {}

	public void gpsNewQuality(int quality) {}

	public void gpsNewSatsUsed(int satUsed) {
		mapControl.setStatusBarSubIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.getSubIcon(satUsed)); //$NON-NLS-1$
	}

	public void gpsNewPDOP(double pdop) {
		if (gpsStatus!=0){
			if (pdop > 10 || pdop==0.0){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0); //$NON-NLS-1$
				return;
			}
			if (pdop > 8 ){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco1); //$NON-NLS-1$
				return;
			}
			if (pdop > 6){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco2); //$NON-NLS-1$
				return;
			}
			if (pdop > 4){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco3); //$NON-NLS-1$
				return;
			}
			if (pdop > 2){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco4); //$NON-NLS-1$
				return;
			}
			mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco5); //$NON-NLS-1$
		}
	}

	public void gpsNewStatus(int status) {
		gpsStatus = status;
		if (status==0)
			mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0); //$NON-NLS-1$
	}
	
}
