package es.prodevelop.gvsig.mobile.gui.tool.listener;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.gvsig.mobile.location.GPSManager;
import org.gvsig.mobile.location.listener.GPSRelevantEventsListener;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.core.DefaultFeature;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.edition.EditionEvent;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.Graph2DUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.gui.bar.GPSIcons;
import es.prodevelop.gvsig.mobile.gui.bar.StatusBar;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;

public class EditionPointToolListener implements
		es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener, GPSRelevantEventsListener {

	private static Logger logger = Logger
			.getLogger(EditionPointToolListener.class);

	private MapControl mapControl = null;

	private FLyrVect lyr = null;

	private static FSymbol handlerSymbol = new FSymbol(Color.ORANGE.darker(),
			Color.ORANGE, false, 1);

	private static GPSManager gpsManager = null;

	private double gpsMapX = 0;

	private double gpsMapY = 0;

	private double gpsPreviousX = 0;

	private double gpsPreviousY = 0;

	private double toleranceX = 0;

	private double toleranceY = 0;

	protected int gpsStatus = 0;

	private boolean isGPSAutomatic = false;


	public EditionPointToolListener(){
		gpsManager = GPSManager.getGPSManager();

		logger.debug("Creating editing point listener...");

		toleranceX = gpsManager.getToleranceX();
		toleranceY = gpsManager.getToleranceY();
	}


	public void point(PointEvent event) {
		logger.info("Entro en point");
		if (event.getEvent().getID() != MouseEvent.MOUSE_PRESSED) {
			return;
		}
		Point point = event.getPoint();
		ViewPort vp = mapControl.getViewPort();
		Point2D pointMap = vp.toMapPoint(new Point2D.Double(point.x, point.y));



		Rectangle2D gpsRect =null;
		double[] coords = null;

		if(gpsManager.isConnected()){
			if(mapControl.getMapCursorSupport().getGpsPositionVisible()){

				double longitud = gpsManager.lastGPSFix.getLongitude();
				double latitud = gpsManager.lastGPSFix.getLatitude();

				coords = ConversionCoords.wgs842projection(
						longitud,
						latitud,
						mapControl.getProjection());

				gpsRect = new Rectangle2D.Double(
						coords[0]-29/2,
						coords[1]-29/2,
						29,29);
			}
		}

		if(gpsRect!=null){
			Rectangle2D rectPointMap = new Rectangle2D.Double(
					pointMap.getX()-29/2,
					pointMap.getY()-29/2,
					29,29);
			boolean intersects = Utils.rectanglesItersect(rectPointMap,gpsRect);
			if(intersects){
				addGeometry(ShapeFactory
						.createPoint2D(coords[0], coords[1]));
			}else{
				addGeometry(ShapeFactory
						.createPoint2D(pointMap.getX(), pointMap.getY()));
				logger.info("Termino point");
			}
		}

		else{
			addGeometry(ShapeFactory
					.createPoint2D(pointMap.getX(), pointMap.getY()));
			logger.info("Termino point");
		}

	}

	public Cursor getCursor() {
		return Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	}

	public boolean cancelDrawing() {
		return false;
	}

	public void setMapControl(MapControl mc) {
		mapControl = mc;

	}

	protected void addGeometry(IGeometry geom) {
		logger.info("Entro en addGeometry");
		int countLyrs = mapControl.getLayerCount();
		logger.info("CountLyrs: "+ countLyrs);
		lyr = null;
		for (int i = 0; i < countLyrs; i++) {
			FLayer lyraux = mapControl.getLayer(i);
			if (lyraux.isEditing() && lyraux instanceof FLyrVect) {
				lyr = (FLyrVect) lyraux;
			}
		}
		logger.info("Obtengo la capa");
		if (lyr != null) {
			logger.info("Capa distinto de null");

			if(!lyr.isAvailable())
				logger.info("Capa no disponible");
			VectorialEditableAdapter vea = (VectorialEditableAdapter) lyr
					.getSource();
			logger.info("Obtengo el vea y el s�mbolo de la capa");
			// Deber�amos comprobar que lo que escribimos es correcto:
			// Lo hacemos en el VectorialAdapter, justo antes de
			// a�adir, borrar o modificar una feature
			Graphics2D gs=null;
			ViewPort vp =null;
			try {

				int numAttr = vea.getRecordset().getFieldCount();
				Value[] values = new Value[numAttr];
				for (int i = 0; i < numAttr; i++) {
					values[i] = ValueFactory.createNullValue();
				}
				logger.info("Creo los value");
				String newFID;

				newFID = vea.getNewFID();
				DefaultFeature df = new DefaultFeature(geom.cloneGeometry(),
						values, newFID);
				int index = vea.addRow(df, getName(), EditionEvent.GRAPHIC);

				 vp = mapControl.getViewPort();

				logger.info("creo una feature y la a�ado");
				BufferedImage selectionImage = Utils.createBufferedImage(vp
						.getImageWidth(), vp.getImageHeight());

				 gs = selectionImage.createGraphics();

				logger.info("creo la imagen");

				Rectangle2D bb = geom.getBounds2D();

				if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {

					AffineTransform _at = vp.getAffineTransform();
					geom.transform(_at);
					Shape shp = geom.getInternalShape();

					Graphics2D mapGraph = (Graphics2D)mapControl.getCurrentImage().getGraphics();
					mapControl.setProcessing(true);
					FSymbol sym = lyr.getLegend().getDefaultSymbol();
					mapGraph.setColor(sym.getBorderColor());
					mapGraph.setStroke(new BasicStroke(sym.getStrokeWidth()));
					Graph2DUtilities.drawPoint(mapGraph, (FPoint2D) shp);
					mapControl.setProcessing(false);
					mapControl.refresh(false);

					logger.info("Pinto la nueva ");
				}


			} catch (IOException e) {
				logger.info("While adding a row: " + e.getMessage());
				return;
			} catch (FMapDriverException e) {
				logger.info("While loading the data source from the layer: "
						+ e.getMessage());
				return;
			} catch (DriverException e) {
				logger.info("While loading the data source from the layer: "
						+ e.getMessage());
				return;
			}



			logger.info("Termino addGeometry");
		}

	}



	private String getName() {
		return "point";
	}

	public void gpsNewPosition(double longitude, double latitude) {

		gpsMapX = longitude;
		gpsMapY = latitude;

		logger.error("-----------------------------------> Entro en el evento del GPS para capturar punto.");
		IProjection the_projection = mapControl.getProject().getProjection();

		String units = " " + the_projection.getUnitsAbbrev(); //$NON-NLS-1$
		int decimals = 2;

		if (GeoUtils.isLatLonProjection(the_projection.getAbrev())) {
			decimals = 5;
		} else {
			decimals = 2;
		}

		//TODO 	Tener en cuenta la proyeccion en la que hay que pedir
		//		las coordenadas alGPS

		double[] coords = ConversionCoords.wgs842projection(
				longitude,
				latitude,
				the_projection);

		gpsMapX=coords[0];
		gpsMapY=coords[1];

		double difX = 0;
		double difY = 0;

		if(gpsMapX > gpsPreviousX)difX = gpsMapX - gpsPreviousX;
		else  difX = gpsPreviousX - gpsMapX;

		if(gpsMapY > gpsPreviousY)difY = gpsMapY - gpsPreviousY;
		else  difY = gpsPreviousY - gpsMapY;


		if(difX > toleranceX || difY > toleranceY ){

			gpsPreviousX = gpsMapX;
			gpsPreviousY = gpsMapY;

			logger.error("X: " + gpsMapX);
			logger.error("Y: " + gpsMapY);

			IGeometry geom =ShapeFactory.createPoint2D(gpsMapX, gpsMapY);
			addGeometry(geom);

			logger.info("-----------------------------------> Termino point");
		}

	}

	public void gpsNewAltitude(double altitude) {
	}

	public void gpsNewHeading(double heading) {

	}

	public void gpsNewSatsUsed(int satUsed) {
		mapControl.setStatusBarSubIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.getSubIcon(satUsed)); //$NON-NLS-1$
	}

	public void gpsNewPDOP(double pdop) {
		if (gpsStatus!=0){
			if (pdop > 10 || pdop==0.0){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0); //$NON-NLS-1$
				return;
			}
			if (pdop > 8 ){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco1); //$NON-NLS-1$
				return;
			}
			if (pdop > 6){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco2); //$NON-NLS-1$
				return;
			}
			if (pdop > 4){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco3); //$NON-NLS-1$
				return;
			}
			if (pdop > 2){
				mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco4); //$NON-NLS-1$
				return;
			}
			mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco5); //$NON-NLS-1$
		}
	}

	public void gpsNewQuality(int quality) {
	}

	public void gpsNewStatus(int status) {
		gpsStatus = status;
		if (status==0)
			mapControl.setStatusBarIcon(StatusBar.GPS_BAR, ResourceReader.IMG_SKIN_PATH, GPSIcons.gpsIco0); //$NON-NLS-1$
	}

	public void setGPSAutomatic(boolean isGPSAutomatic) {
		this.isGPSAutomatic = isGPSAutomatic;

		if(this.isGPSAutomatic == true){
			gpsManager.addGPSRelevantEventsListener(this);
		}
		else{
			gpsManager.removeGPSRelevantEventsListener(this);
		}
	}

	public void addGPSPoint(Point2D point){
		this.addGeometry(ShapeFactory.createPoint2D(point.getX(), point.getY()));
	}

	/**
	 * Returns a new value for the Starfire quality value
	 * @param starfire an integer between 0 and 26
	 */
	public void gpsNewStarfire(int starfire) {
	}

	/**
	 * Returns a new value for the RMS standard deviation of
	 * the ranges inputs for the navigation solution
	 * @param rms a positive double value
	 */
	public void gpsNewRMS(double rms){
	}

}
