/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package es.prodevelop.gvsig.mobile.gui.tool.listener;

import java.awt.Cursor;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.gvsig.forms.model.FeatureTemp;
import org.gvsig.forms.view.AlphanumericalData;
import org.gvsig.forms.view.CustomForm;
import org.gvsig.forms.view.FeatureForm;
import org.gvsig.forms.view.GenericForm;
import org.gvsig.i18n.Messages;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IRow;
import es.prodevelop.gvsig.mobile.fmap.driver.BoundedShapes;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.edition.EditionEvent;
import es.prodevelop.gvsig.mobile.fmap.edition.IRowEdited;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.InfoByPoint;
import es.prodevelop.gvsig.mobile.fmap.layer.SelectableDataSource;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.cancel.Cancellable;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.dialog.AcceptCancelDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.InfoAttributesDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.PointToolListener;

/**
 * This listener is associated with the info-by-point tool. It will iterate over the
 * queriable layers and show a dialog with a simple table showing
 * the results.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.PointBehavior
 * 
 * @author jldominguez
 */
public class InfoPointListener implements PointToolListener {
	
	private static Logger logger = Logger.getLogger(InfoPointListener.class);

	private MapControl mapCtrl;
	private Frame parentFrame;
	
	/**
	 * Constructor.
	 * 
	 * @param infoFrame main frame, needed for graphical purposes, since
	 * every dialog needs a frame to be shown.
	 */
	public InfoPointListener(Frame infoFrame) {
		parentFrame = infoFrame;
	}
	
	private InfoPointListener() {
	}
	
	private void addPair(ArrayList al, String a, String b) { al.add(a); al.add(b); }

	public FeatureTemp infoByPoint(FLyrVect layer,Point p, double tolerance, Cancellable cancel, String srs){
		
		Point2D pReal = mapCtrl.getProject().getMapContext().getViewPort().toMapPoint(
				new Point2D.Double(p.x, p.y)
				);

		Object vd=layer.getSource().getDriver();
		if (layer.getSource() instanceof VectorialEditableAdapter){
			vd=layer.getSource();
		}

		int[] intersect_indices = null;
		if (vd instanceof BoundedShapes) {
			
			Rectangle2D click_rect = new Rectangle2D.Double(
					pReal.getX() - 0.5 * tolerance,
					pReal.getY() - 0.5 * tolerance,
					tolerance, tolerance);
			
			try {
				intersect_indices = ((BoundedShapes) vd).getIntersectingIndices(click_rect, 1);
			} catch (IOException e) {
				logger.error("While computing intersecting indices: " + e.getMessage());
			}
			
			if (intersect_indices == null) return null;

			
			try {
				SelectableDataSource sds = layer.getRecordset();
				String[] f_names = sds.getFieldNames();
				int[] types = new int[f_names.length];
				sds.start();
				for (int i=0;i<f_names.length;i++){
					types[i]=sds.getFieldType(i);
				}
				sds.stop();
				Value[] atts=null;
				if (layer.getSource() instanceof VectorialEditableAdapter){
					try{
					atts = ((IRow)((VectorialEditableAdapter)layer.getSource()).getRow(intersect_indices[intersect_indices.length-1]).getLinkedRow()).getAttributes();
					}catch(IOException e){
						logger.error(e.getMessage());
					}
				}else{
					atts = sds.getRow(intersect_indices[intersect_indices.length-1]);
				}

				FeatureTemp fea = new FeatureTemp();
				fea.setRowIndex(intersect_indices[intersect_indices.length-1]);
				fea.setFieldNames(f_names);
				fea.setFieldTypes(types);
				fea.setValues(atts);
				return fea;

			} catch (DriverException dex) {
				dex.printStackTrace();
			} catch (FMapDriverException fex){
				fex.printStackTrace();
			}
		}
		return null;
	}

	/**
	 * Point event that indicates the screen coordinates where the user has clicked
	 * to query for info. This method will show a dialog with the information available
	 * from each layer.
	 *
	 * @param event the point event that indicates the point of interest.
	 */
	public void point_old(PointEvent event, FLayer item_layer) {
		
		if (event.getEvent().getID() != MouseEvent.MOUSE_PRESSED) {
			return;
		}
		

		int x = event.getPoint().x;
		int y = event.getPoint().y;
		
		Point2D wp = mapCtrl.getViewPort().toMapPoint(x, y);
		
		ArrayList labelValueList = new ArrayList();
		
		addPair(labelValueList, "Coordinates", null);
		
		addPair(labelValueList,
				"x = " + Utils.getFormattedDouble(wp.getX(), 3, false),
				"y = " + Utils.getFormattedDouble(wp.getY(), 3, false));
		
		addPair(labelValueList, null, null);
		
		//addPair(labelValueList, "Queried layers:", "" + info_lyrs.length);
		
		addPair(labelValueList, null, null);

			
			if (item_layer instanceof InfoByPoint) {
				
				addPair(labelValueList, "Name:", item_layer.getName());
				
				double tol = mapCtrl.getViewPort().toMapDistance(5);
				try {
					labelValueList.addAll(
							((InfoByPoint) item_layer).getInfo(event.getPoint(),
									tol, null, mapCtrl.project.getProjection().getAbrev())
					);
				} catch (Exception e) {
					labelValueList.add("Error:");
					labelValueList.add(e.getMessage().substring(0, 20) + "...");
				}
			} else {
				addPair(labelValueList, "Name:", item_layer.getName());
				addPair(labelValueList, "[Not queryable]", null);
			}

			addPair(labelValueList, null, null);
		
		String[] ff = getEvenIndexStrings(labelValueList);
		String[] vv = getOddIndexStrings(labelValueList);
		
		new InfoAttributesDialog(parentFrame, ff, vv);

	}
	
	/**
	 * Point event that indicates the screen coordinates where the user has clicked
	 * to query for info. This method will show a dialog with the information available
	 * from each layer.
	 * 
	 * @param event the point event that indicates the point of interest.
	 */
	public void point(PointEvent event) {

		if (event.getEvent().getID() != MouseEvent.MOUSE_PRESSED) {
			return;
		}

		// -------------------------- starts working
		if (mapCtrl.isProcessing()) {
			return;
		} else {
			mapCtrl.setProcessing(true);
		}

		int x = event.getPoint().x;
		int y = event.getPoint().y;

		Point2D wp = mapCtrl.getViewPort().toMapPoint(x, y);
		//FLayer[] info_lyrs = mapCtrl.getInfoableLayers(false);
		//FLayer item_layer;
		FLayer[] info_lyrs = mapCtrl.getInfoableLayers(false);
		FLyrVect item_layer=null;
		if (info_lyrs.length==0){
			mapCtrl.setProcessing(false);
			mapCtrl.repaint();
			return;
		}
		FeatureTemp fea =null;

		for (int i=info_lyrs.length-1;i>=0;i--){
			if (! (info_lyrs[i] instanceof FLyrVect)){
				point_old(event, info_lyrs[i]);
				mapCtrl.setProcessing(false);
				mapCtrl.repaint();
				return;
			}
			item_layer=(FLyrVect)info_lyrs[i];

			double tol = mapCtrl.getViewPort().toMapDistance(5);
			fea = infoByPoint(item_layer, event.getPoint(),
					tol, null, mapCtrl.project.getProjection().getAbrev());
			if (fea!=null) {
				break;
			}
		}
		if (fea==null) {
			mapCtrl.setProcessing(false);
			mapCtrl.repaint();
			return;
		}
		FeatureForm form;
		String pathForm=item_layer.getCustomForm();
		if (pathForm==null || pathForm.equals("")){
			form = new GenericForm(parentFrame);
		} else {
			File fForm = new File(pathForm);
			if (!fForm.exists()){
				form = new GenericForm(parentFrame);
			} else {
				try{
					form = new CustomForm(parentFrame, pathForm);
				} catch (Exception frmex){
					AcceptCancelDialog.informUser(
							Messages.getText("form_error"), //$NON-NLS-1$
							Messages.getText("custom_form_has_errors") + ": " + frmex.getMessage(),
							parentFrame);
					mapCtrl.setProcessing(false);
					mapCtrl.repaint();
					return;
				}
			}
		}

		if (item_layer.isEditing()){
			if (AlphanumericalData.update(form, fea)){

				VectorialEditableAdapter vea = null;
				vea = (VectorialEditableAdapter)item_layer.getSource();
				try{
					IFeature featAux =  ((IFeature)((IRowEdited)vea.getRow(fea.getRowIndex())).getLinkedRow());
					featAux.setAttributes(fea.getValues());
					vea.modifyRow(fea.getRowIndex(),featAux,"", EditionEvent.ALPHANUMERIC);
				}
				catch( IOException e){
					AcceptCancelDialog.informUser(
							Messages.getText("form_error"), //$NON-NLS-1$
							Messages.getText("error_while_savung_data") + ": " + e.getMessage(),
							parentFrame);
					mapCtrl.setProcessing(false);
					mapCtrl.repaint();
					return;
				}

			}
		} else {
			AlphanumericalData.visualize(form, fea);
		}

		mapCtrl.setProcessing(false);
		mapCtrl.repaint();
		/*
		//ArrayList labelValueList = new ArrayList();

		//addPair(labelValueList, "Coordinates", null);

//		addPair(labelValueList,
//				"x = " + Utils.getFormattedDouble(wp.getX(), 3),
//				"y = " + Utils.getFormattedDouble(wp.getY(), 3));
//
//		addPair(labelValueList, null, null);
//
//		addPair(labelValueList, "Queried layers:", "" + info_lyrs.length);
//
//		addPair(labelValueList, null, null);

		for (int i=0; i<info_lyrs.length; i++) {

			item_layer = info_lyrs[i];
			addPair(labelValueList, "Layer # " + (i+1), null);

			if (item_layer instanceof InfoByPoint) {

				addPair(labelValueList, "Name:", item_layer.getName());

				double tol = mapCtrl.getViewPort().toMapDistance(5);
				try {
					labelValueList.addAll(
							((InfoByPoint) item_layer).getInfo(event.getPoint(),
									tol, null, mapCtrl.project.getProjection().getAbrev())
					);
				} catch (Exception e) {
					labelValueList.add("Error:");
					labelValueList.add(e.getMessage().substring(0, 20) + "...");
				}
			} else {
				addPair(labelValueList, "Name:", item_layer.getName());
				addPair(labelValueList, "[Not queryable]", null);
			}

			addPair(labelValueList, null, null);
		}

		String[] ff = getEvenIndexStrings(labelValueList);
		String[] vv = getOddIndexStrings(labelValueList);

		new InfoAttributesDialog(parentFrame, ff, vv);

		mapCtrl.setProcessing(false);
		mapCtrl.repaint();
		*/
	}

	private String[] getOddIndexStrings(ArrayList l) {
		int res_sz = l.size() / 2;
		String[] resp = new String[res_sz];

		for (int i=0; i<res_sz; i++) {
			resp[i] = (String) l.get(2 * i + 1);
		}
		return resp;
	}

	private String[] getEvenIndexStrings(ArrayList l) {
		int res_sz = (l.size() + 1) / 2;
		String[] resp = new String[res_sz];

		for (int i=0; i<res_sz; i++) {
			resp[i] = (String) l.get(2 * i);
		}
		return resp;
	}

	/**
	 * Gets the cursor that should be used while this tool is active.
	 *
	 * @return the cursor that should be used while this tool is active.
	 */
	public Cursor getCursor() {
		return Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	}

	/**
	 * This method asks the listener to cancel the current drawing process
	 * (currently unsued)
	 *
	 * @return whether it was possible to cancel the drawing
	 */
	public boolean cancelDrawing() {
		return false;
	}

	/**
	 * Sets the <code>MapControl</code> object that will be updated by
	 * this listener.
	 */
	public void setMapControl(MapControl mc) {
		mapCtrl = mc;
	}


}
