/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.listener;

import java.awt.Cursor;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.gui.canvas.MapControl;
import es.prodevelop.gvsig.mobile.gui.tool.event.PointEvent;
import es.prodevelop.gvsig.mobile.gui.tool.event.RectangleEvent;
import es.prodevelop.gvsig.mobile.gui.tool.listener.imterfaces.RectangleToolListener;

/**
 * This listener is associated with the zoom-by-rectangle tool.
 * It will zoom the map to the indicated rectangle.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.behavior.RectangleBehavior
 * @see es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort
 * 
 * @author jldominguez
 *
 */
public class ZoomInRectListener implements RectangleToolListener {
	
	public final static double ZOOM_OUT_FACTOR = 0.5;
	private final static int MIN_SIDE = 8;
	
	private static Logger logger = Logger.getLogger(ZoomInRectListener.class);
	
	private MapControl mapCtrl;
	private boolean firstp = true;
	private int start_x = 0;
	private int start_y = 0;
	private int prev_w = 0;
	private int prev_h = 0;
	
	private void fixedZoom(int x, int y, double factor) {
		Point2D geop = mapCtrl.getViewPort().toMapPoint(x, y);

		Rectangle2D oldext = mapCtrl.getViewPort().getAdjustedExtent();
		double half_w = 0.5 * oldext.getWidth() / factor;
		double half_h = 0.5 * oldext.getHeight() / factor;
		Rectangle2D.Double newext = new Rectangle2D.Double(
				geop.getX() - half_w, geop.getY() - half_h, 2 * half_w, 2 * half_h);
		mapCtrl.getViewPort().setExtent(newext, true);
	}
	

	/**
	 * This method will be called  when the rectangle eventy happens after
	 * a drag-amd-drop process.
	 * 
	 * @param event the rectangle event that indicates the zone of interest
	 */
	public void rectangle(RectangleEvent event) {
		
		logger.debug(Utils.time() + "ZoomInRectListener, rectangle starts");
		
		// -------------------------- starts working
		if (mapCtrl.isProcessing()) {
			return;
		} else {
			mapCtrl.setProcessing(true);
		}
		

		int src_min_x = event.getRectangle().x;
		int src_min_y = event.getRectangle().y;
		int src_w = event.getRectangle().width;
		int src_h = event.getRectangle().height;
		
		if ((src_w <= 1) && (src_h <= 1)) {
			firstp = true;
			fixedZoom(src_min_x, src_min_y, 1.0 / ZOOM_OUT_FACTOR);
			return;
		}
		
		src_w = Math.max(src_w, MIN_SIDE);
		src_h = Math.max(src_h, MIN_SIDE);

		Rectangle2D screen_rect = new Rectangle2D.Double(
				src_min_x,
				src_min_y,
				src_w,
				src_h);
		
		Rectangle2D map_rect = mapCtrl.getViewPort().toMapRectangle(screen_rect);

		mapCtrl.getViewPort().setExtent(map_rect, true);
		
		firstp = true;
	}

	/**
	 * Gets the cursor that should be used while this tool is active.
	 * 
	 * @return the cursor that should be used while this tool is active.
	 */
	public Cursor getCursor() {
		return null;
	}

	/**
	 * This method asks the listener to cancel the current drawing process
	 * (currently unsued)
	 * 
	 * @return whether it was possible to cancel the drawing
	 */
	public boolean cancelDrawing() {
		return false;
	}

	/**
	 * Sets the <code>MapControl</code> object that will be updated by
	 * this listener.
	 */
	public void setMapControl(MapControl mc) {
		mapCtrl = mc;
	}

	/**
	 * This method is called  when a drag event happens.
	 * The listener tells the map to update
	 * the borders of the rectangle that is being drawn to indicated the current
	 * zone of interest.
	 * 
	 * @param event the point event that indicates a drag event
	 */
	public void point(PointEvent event) {
		
		if (firstp) {
			start_x = event.getPoint().x;
			start_y = event.getPoint().y;
			firstp = false;
			
			prev_w = 0;
			prev_h = 0;
			return;
		}
		
		int currw = event.getPoint().x - start_x;
		int currh = event.getPoint().y - start_y;

		mapCtrl.xorAndRepaint(start_x, start_y, prev_w, 1);
		mapCtrl.xorAndRepaint(start_x, start_y + prev_h, prev_w, 1);
		mapCtrl.xorAndRepaint(start_x, start_y, 1, prev_h);
		mapCtrl.xorAndRepaint(start_x + prev_w, start_y, 1, prev_h);
		mapCtrl.xorAndRepaint(start_x, start_y, currw, 1);
		mapCtrl.xorAndRepaint(start_x, start_y + currh, currw, 1);
		mapCtrl.xorAndRepaint(start_x, start_y, 1, currh);
		mapCtrl.xorAndRepaint(start_x + currw, start_y, 1, currh);
		prev_w = currw;
		prev_h = currh;
	}



}
