/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.gui.tool.util;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.log4j.Logger;

/**
 * This class performs the animation of the group button bar (to the left
 * of the main window), using tasks and delay timers.
 * 
 * @see es.prodevelop.gvsig.mobile.gui.tool.util.Animable
 * @author jcarras
 *
 */
public class Animator {
	private static final int milisecondsStep  = 100;
	ArrayList subscribers;
	Timer timr;
	private static Logger logger = Logger.getLogger(Animator.class);
	
	
	/**
	 * Forces the complete cancellation of the refresh process. It is called when 
	 * the application terminates and this functionality is no longer needed.
	 *
	 */
	public void terminate(){
		timr.cancel();
	}
	
	/**
	 * Constructor. It initializes the components needed to perform a refresh
	 * of the tool group bar.
	 *
	 */
	public Animator(){
		 subscribers = new ArrayList();
		 timr = new Timer();
		 Temporizer executor = new Temporizer(this);
		 timr.scheduleAtFixedRate(executor,milisecondsStep,milisecondsStep);
		 
	}
	
	/**
	 * This method adds a new animable object to the list of objects that will receive
	 * an animation notification.
	 * 
	 * @param pobj the object that has to be animated
	 * @param pquantity offset in pixels between one position and the following.
	 * @param pmiliseconds duration of the animation in miliseconds. Each step
	 * of the animation will substract a certain amount of miliseconds. When
	 * it reaches zero, the naimation will end.
	 */
	public void subscribe(Animable pobj, int pquantity, int pmiliseconds){
		Subscriptor subs = new Subscriptor(pobj, pquantity, pmiliseconds);
		//subs.doStep();
		subscribers.add(subs);
	}
	
	/**
	 * This method notifies a new step in the animation to all the animable
	 * objects subscribed to it.
	 *
	 */
	public void doStep(){
		int i = subscribers.size() - 1;
		while (i >= 0){
			Subscriptor subs = (Subscriptor)subscribers.get(i);
			subs.doStep();
			if (subs.miliseconds <=0 || subs.quantity <=0) {
				subscribers.remove(i);
				//Automatic desubscription
			}
			i--;
		}
	}
	
	
	/**
	 * 
	 * This is a autility class used to perform the animation.
	 * 
	 * 
	 * @author jcarras
	 *
	 */
    public class Temporizer extends TimerTask{
    	Animator father;
        public void run(){
        	//logger.debug("thread: " + this.hashCode() + " milis: " + System.currentTimeMillis());
        	father.doStep();
        }
        public Temporizer(Animator father){
        	super();
        	this.father=father;
        }
    }
    
    /**
     * 
     * This is a autility class used to perform the animation.
     * 
     * @author jcarras
     *
     */
	public class Subscriptor{
		Animable subscriber;
		int quantity;
		int miliseconds;
		int nullsteps = 1;
		double inc;
		
		public Subscriptor(Animable obj, int quan, int mili){
			subscriber = obj;
			quantity = quan;
			miliseconds = mili;
			double steps = ((double)miliseconds) / (double)milisecondsStep;
			inc = ((double)quantity / steps);
		}
		
		public void doStep(){
			// logger.debug("miliseconds: " + miliseconds);
			// logger.debug("quantity: " + quantity);
			
			miliseconds -= milisecondsStep;
			
			int thisStepInt = (int)(inc * nullsteps + 0.5);
			if (thisStepInt == 0){
				nullsteps++;
			} else {
				nullsteps=1;
				subscriber.step(Math.min(thisStepInt,quantity) );
				quantity -= thisStepInt;
				double steps = ((double)miliseconds) / (double)milisecondsStep;
				inc = ((double)quantity / steps);
			}
			
			// logger.debug("thisStepInt: " + thisStepInt);
		}
		
	}
	


    
    
	
	
}
