/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.project;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.gvsig.i18n.Messages;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

import es.prodevelop.gvsig.mobile.app.GvsigMobileApp;
import es.prodevelop.gvsig.mobile.common.XmlProjectTags;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.legend.SingleSymbolLegend;
import es.prodevelop.gvsig.mobile.fmap.proj.CRSFactory;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;
import es.prodevelop.gvsig.mobile.gui.dialog.MessageDialog;
import es.prodevelop.gvsig.mobile.gui.mainframe.MainWindow;
import es.prodevelop.gvsig.mobile.settings.AppSettings;

/**
 * This class instantiates a full project from a XML file.
 * 
 * @see es.prodevelop.gvsig.mobile.project.Project
 * @see es.prodevelop.gvsig.mobile.project.ProjectSerializer
 * 
 * @author jcarrasco
 *
 */
public class ProjectDeserializer {
	
	private static Logger logger = Logger.getLogger(ProjectDeserializer.class); 
	
	/**
	 * This method instantiates a project from a given XML file and a canvas size
	 * 
	 * @param f XML file where the project is stored
	 * @param d canvas size where the project will be opened 
	 * @return the project object fully instantiated.
	 */
	public static Project deSerializeProject(File f, Dimension d) {
    	Project prj = new Project();
    	prj.setPath(f.getParentFile().getAbsolutePath());
    	FileReader reader = null;
    	String encoding = "UTF-8";
    	BufferedReader br = null;
    	BufferedReader br1 = null;
    	try
    	{
    		reader = new FileReader(f);
    		br = new BufferedReader(reader);
    		char[] buffer = new char[100];
    		br.read(buffer);
    		String instr = new String(buffer);
    		StringBuffer st = new StringBuffer(instr);
    		String searchText = "encoding=\"";
    		int index = st.indexOf(searchText);
    		if (index>-1) {
    			st.delete(0, index+searchText.length());
    			encoding = st.substring(0, st.indexOf("\""));
    		}
    	} catch(Exception ex)	{
    		ex.printStackTrace();
		}
    	logger.debug("encoding: " + encoding);
    	KXmlParser kxmlParser = null;
    	kxmlParser = new KXmlParser();
    	logger.debug("KXMLParser created");
    	try
    	{
    		reader = new FileReader(f);
    		br1 = new BufferedReader(reader);

    		kxmlParser.setInput(new FileInputStream(f), encoding);
            
    		kxmlParser.nextTag();
    		logger.debug("starting deserialize ");
    		kxmlParser.require(KXmlParser.START_TAG, null, XmlProjectTags.PROJECT);
    		prj = deSerializeProject(kxmlParser,d, prj);
    		kxmlParser.next();
    		kxmlParser.require(KXmlParser.END_DOCUMENT, null, null);
    		logger.debug("end deserialize ");
    	}
    	catch(XmlPullParserException parser_ex){
     		parser_ex.printStackTrace();
    	}
   		catch (IOException ioe) {
   			ioe.printStackTrace();
		} finally {
			try {reader.close();} catch (IOException e) {}
	    	try {br.close();} catch (IOException e) {}
	    	try {br1.close();} catch (IOException e) {}
        }
		
   		return prj;
    }
	
	/**
	 * Updates teh provided project with the given canvas size and the data found
	 * by the provided parser.
	 * 
	 * @param parser the parser that will search for more project data
	 * @param d the size of the canvas where the project has to be opened
	 * @param prj the project taht is being updated
	 * @return the updated project
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static Project deSerializeProject(KXmlParser parser, Dimension d, Project prj)throws IOException, XmlPullParserException{
		prj.setNew(false);
		Color bgcolor = null;
		String comments = "";
		Date creationDate = null;
		Date lastModDate = null;
		
		logger.debug("deSerializeProject - 0, d = " + d);

		prj.getMapContext().getViewPort().setImageSize(d);
		Rectangle2D bounds = null;
		ArrayList lyrs = null;
		String name = null;
		String srs = null;
		//TODO: Projection
		//parser.require(KXmlParser.START_TAG, null, XmlProjectTags.PROJECT);
		logger.debug("<project> ");
		int tag = parser.nextTag();
		while (true){
			
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.PROJECT)==0) {
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.B_BOX)==0){
				bounds = deSerializeBounds(parser,prj);
			} else if (parser.getName().compareTo(XmlProjectTags.NAME)==0){
				 name = parser.nextText();
				//System.out.println(prj.getName());
			} else if(parser.getName().compareTo(XmlProjectTags.LAYERS)==0) {
				lyrs = deSerializeLayers(parser,prj);
			} else if(parser.getName().compareTo(XmlProjectTags.PRJCOMMENTS)==0) {
				comments = parser.nextText();
			} else if(parser.getName().compareTo(XmlProjectTags.CREATIONDATE)==0) {
				try {
					creationDate = getDate(parser);
				} catch (Exception e){
					logger.error("Incorrect date format");
					creationDate = new Date();
				}
			} else if(parser.getName().compareTo(XmlProjectTags.LASTMODDATE)==0) {
				try {
					lastModDate = getDate(parser);
				} catch (Exception e){
					logger.error("Incorrect date format");
					creationDate = new Date();
				}
			} else if(parser.getName().compareTo(XmlProjectTags.CRS)==0) {
				srs = parser.nextText();
			} else if(parser.getName().compareTo(XmlProjectTags.BGCOLOR)==0) {
				bgcolor = getcolor(parser);
			} else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
		logger.debug("</project> ");
		prj.setName(name);
		prj.setDescription(comments, true);
		prj.setCreationDate(creationDate);
		prj.setLastModified(lastModDate, true);
		prj.setProjection(CRSFactory.getCRS(srs));
		
		try{
			prj.getMapContext().setViewPort(new ViewPort(CRSFactory.getCRS(srs)));
		} catch (Exception e){
			prj.getMapContext().setViewPort(new ViewPort(CRSFactory.getCRS(AppSettings.defaultSRS)));
		}
		
		ViewPort vp = prj.getMapContext().getViewPort();

		vp.setImageSize(d);
		Iterator it = lyrs.iterator();
		vp.setExtent(bounds, false);
		while (it.hasNext()){
			prj.getMapContext().addLayer((FLayer)it.next(),false);
		}		
		vp.setExtent(bounds, true);
		
		return prj;
	}
	
	/**
	 * Extracts the data from the parser to instantiate the layers contained in it.
	 * 
	 * @param parser the parser that is reading the project file
	 * @param prj  the project that is in the process of being deserialized.
	 * @return an an array list containing the layers found in the layers section
	 * of the project file
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static ArrayList deSerializeLayers(KXmlParser parser, Project prj) throws IOException, XmlPullParserException{
		ArrayList layers = new ArrayList();
		int tag = parser.nextTag();
		logger.debug("<layers> ");
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.LAYERS)==0) {
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.LAYER)==0){

				FLayer item_lyr = deSerializeLayer(parser,prj);
				if (item_lyr != null) {
					layers.add(item_lyr); 
				}

			}
			tag = parser.nextTag();
		}
		logger.debug("</layers> ");
		return layers;
	}
	
	/**
	 * Gets the new available data from the parser as a <code>double</code> 
	 * @param parser the parser where the data comes from
	 * @return the new available data from the parser as a <code>double</code>
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static double getdbl(KXmlParser parser)throws IOException, XmlPullParserException{
		String svalue = parser.nextText();
		return Double.valueOf(svalue).doubleValue();
	}
	
	/**
	 * 
	 * Gets the new available data from the parser as a <code>int</code> 
	 * @param parser the parser where the data comes from
	 * @return the new available data from the parser as a <code>int</code>
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static int getint(KXmlParser parser)throws IOException, XmlPullParserException{
		String svalue = parser.nextText();
		return Integer.valueOf(svalue).intValue();
	}
	/**
	 * Gets the new available data from the parser as a <code>double</code> 
	 * @param parser the parser where the data comes from
	 * @param deflt the default value in case there is an error
	 * @return the new available data from the parser as a <code>double</code>
	 * or the default value if there is an error
	 */
	private static double getdouble(KXmlParser parser, double deflt) {
		String svalue;
		try {
			svalue = parser.nextText();
			return Double.parseDouble(svalue);
		} catch (Exception e) {
			logger.error("Unable to read double: " + e.getMessage() + " - Returned default: " + deflt);
			return deflt;
		}
	}

	/**
	 * 
	 * Gets the new available data from the parser as a <code>boolean</code> 
	 * @param parser the parser where the data comes from
	 * @return the new available data from the parser as a <code>boolean</code> 
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static boolean getbool(KXmlParser parser)throws IOException, XmlPullParserException{
		String svalue = parser.nextText();
		if (svalue.equals("1") || svalue.equalsIgnoreCase("TRUE")){
			return true;
		} 
		return false;
		
	}
	
	/**
	 * 
	 * Gets the new available data from the parser as a <code>Date</code> 
	 * @param parser the parser where the data comes from
	 * @return the new available data from the parser as a <code>Date</code> 
	 * @throws IOException
	 * @throws XmlPullParserException
	 * @throws ParseException
	 */
	public static Date getDate(KXmlParser parser)throws IOException, XmlPullParserException, ParseException{
		return Utils.dateFromString(parser.nextText());
	}
	
	/**
	 * 
	 * Gets the new available data from the parser as a <code>Color</code> 
	 * @param parser the parser where the data comes from
	 * @return the new available data from the parser as a <code>Color</code> 
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static Color getcolor(KXmlParser parser)throws IOException, XmlPullParserException{
		String svalue = parser.nextText();
		String[] tokens = StringUtilities.split(svalue,",");
		logger.debug("tokens: " + StringUtilities.getComaSeparated(tokens));
		try{
			int r = Integer.valueOf(tokens[0]).intValue();
			int g = Integer.valueOf(tokens[1]).intValue();
			int b = Integer.valueOf(tokens[2]).intValue();
			int a = Integer.valueOf(tokens[3]).intValue();
			return new Color(r,g,b); //,a);
		} catch (Exception e){
			return null;
		}
		
	}	
	
	/**
	 * 
	 * Gets the new available data from the parser as a <code>Rectangle2D</code> 
	 * @param parser the parser where the data comes from
	 * @param prj the project that is in the process of being deserialized
	 * @return the new available data from the parser as a <code>Rectangle2D</code> 
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static Rectangle2D deSerializeBounds(KXmlParser parser, Project prj)throws IOException, XmlPullParserException{
		Rectangle2D rec =  new Rectangle2D.Double();
		double xmin = 0;
		double xmax = 0;
		double ymin = 0;
		double ymax = 0;
		logger.debug("<bounds> ");
		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.B_BOX)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.X_MIN)==0){
				xmin = getdbl(parser);
			} else if (parser.getName().compareTo(XmlProjectTags.X_MAX)==0){
				xmax = getdbl(parser);
			} else if (parser.getName().compareTo(XmlProjectTags.Y_MIN)==0){
				ymin = getdbl(parser);
			} else if (parser.getName().compareTo(XmlProjectTags.Y_MAX)==0){
				ymax = getdbl(parser);
			} else {
				parser.skipSubTree();
			}

			tag = parser.nextTag();
		}
		logger.debug("</bounds> ");
		rec.setFrame(xmin, ymin, xmax - xmin, ymax - ymin);
		return rec;
	}
	
	/**
	 * 
	 * Gets the new available data from the parser as a <code>FSymbol</code> 
	 * @param parser the parser where the data comes from
	 * @return the new available data from the parser as a <code>FSymbol</code> 
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static FSymbol deSerializeSymbol(KXmlParser parser)throws IOException, XmlPullParserException{
		FSymbol simb = new FSymbol();
		logger.debug("<symbol> ");
		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.STYLE)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.POLYGON_SYMBOL)==0){
				deSerializePolygonSymbol(parser,simb);
			} else if (parser.getName().compareTo(XmlProjectTags.LINE_SYMBOL)==0){
				deSerializeLineSymbol(parser,simb);
			} else if (parser.getName().compareTo(XmlProjectTags.POINT_SYMBOL)==0){
				deSerializePointSymbol(parser,simb);
			} else if (parser.getName().compareTo(XmlProjectTags.GEOMETRY_SYMBOL)==0){
				deSerializeGeometrySymbol(parser,simb);
			} else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
		logger.debug("</symbol> ");
		return simb;
	}
	
	/**
	 * Updates a symbol with the data deserialized from a project file.
	 * 
	 * @param parser the parser where the data comes from
	 * @param simb the symbol that is being updated
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 */
	private static void deSerializePolygonSymbol(KXmlParser parser,FSymbol simb) throws XmlPullParserException, IOException {
		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.POLYGON_SYMBOL)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.FILL_COLOR)==0){
				Color fcol = getcolor(parser);
				simb.setFillColor(fcol);
			} else if (parser.getName().compareTo(XmlProjectTags.FILL)==0){
				simb.setFilled(getbool(parser));
			} else if (parser.getName().compareTo(XmlProjectTags.LINE_SYMBOL)==0){
				deSerializeLineSymbol(parser, simb);
			} else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
		
	}

	/**
	 * Updates a symbol with the data deserialized from a project file.
	 * 
	 * @param parser the parser where the data comes from
	 * @param simb the symbol that is being updated
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static void deSerializeGeometrySymbol(KXmlParser parser, FSymbol simb)throws IOException, XmlPullParserException{
		/*
		 * simb is an input / output parameter
		 */

		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.GEOMETRY_SYMBOL)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.FILL_COLOR)==0){
				Color fcol = getcolor(parser);
				simb.setFillColor(fcol);
			} else if (parser.getName().compareTo(XmlProjectTags.FILL)==0){
				simb.setFilled(getbool(parser));
			} else if (parser.getName().compareTo(XmlProjectTags.COLOR)==0){
				Color fcol = getcolor(parser);
				simb.setBorderColor(fcol);
			} else if (parser.getName().compareTo(XmlProjectTags.SIZE)==0){
				int size = getint(parser);
				simb.setStrokeWidth(size);
			} else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
	}
	
	/**
	 * 
	 * Updates a symbol with the data deserialized from a project file.
	 * 
	 * @param parser the parser where the data comes from
	 * @param simb the symbol that is being updated
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static void deSerializeLineSymbol(KXmlParser parser, FSymbol simb)throws IOException, XmlPullParserException{
		/*
		 * simb is an input / output parameter
		 */
		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.LINE_SYMBOL)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.COLOR)==0){
				Color fcol = getcolor(parser);
				simb.setBorderColor(fcol);
			} else if (parser.getName().compareTo(XmlProjectTags.SIZE)==0){
				int size = getint(parser);
				simb.setStrokeWidth(size);
			}else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
	}

	/**
	 * 
	 * Updates a symbol with the data deserialized from a project file.
	 * 
	 * @param parser the parser where the data comes from
	 * @param simb the symbol that is being updated
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static void deSerializePointSymbol(KXmlParser parser, FSymbol simb)throws IOException, XmlPullParserException{
		/*
		 * simb is an input / output parameter
		 */
		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.POINT_SYMBOL)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.COLOR)==0){
				Color fcol = getcolor(parser);
				simb.setBorderColor(fcol);
			} else if (parser.getName().compareTo(XmlProjectTags.SIZE)==0){
				int size = getint(parser);
				simb.setStrokeWidth(size);
			} else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
	}
	
	/**
	 * Deserializes (instantiates) a single layer from a project file.
	 * 
	 * @param parser the parser where the data comes from
	 * @param prj the project that is in the process of being deserialized
	 * @return the next layer found in the project file
	 * @throws IOException
	 * @throws XmlPullParserException
	 */
	public static FLayer deSerializeLayer(KXmlParser parser, Project prj)throws IOException, XmlPullParserException{
		FLayer lyr =  null;
		String customform = null;
		String type = null;
		String path = null;
		String url = null;
		String wmslayrs = null;
		String wmsstyles = null;
		String srs = null;
		String name = null;
		String format = null;
		int transparency = 0;
		int visible = 1;
		double maxScale = -1.0, minScale = -1.0;
		boolean infoable = true;
		SingleSymbolLegend leg ;
		FSymbol simb = new FSymbol(); 
		logger.debug("<layer> ");		
		int tag = parser.nextTag();
		while (true){
			if (tag == KXmlParser.END_TAG && parser.getName().compareTo(XmlProjectTags.LAYER)==0){
				break;
			} else if (parser.getName().compareTo(XmlProjectTags.TYPE)==0){
				type = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.PATH)==0){
				path = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.NAME)==0){
				name = parser.nextText();				
			} else if (parser.getName().compareTo(XmlProjectTags.STYLE)==0){
				simb = deSerializeSymbol(parser);
			} else if (parser.getName().compareTo(XmlProjectTags.URL)==0){
				url = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.WMSLAYERS)==0){
				wmslayrs = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.WMSSTYLES)==0){
				wmsstyles = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.CRS)==0){
				srs = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.IMAGE_FORMAT)==0){
				format = parser.nextText();
			} else if (parser.getName().compareTo(XmlProjectTags.TRANSPARENCY)==0){
				transparency = getint(parser);
			} else if (parser.getName().compareTo(XmlProjectTags.VISIBLE)==0){
				visible = getint(parser);
			} else if (parser.getName().compareTo(XmlProjectTags.SCALE_MAX)==0){
				maxScale = getdouble(parser, maxScale);
			} else if (parser.getName().compareTo(XmlProjectTags.SCALE_MIN)==0){
				minScale = getdouble(parser, minScale);
			} else if (parser.getName().compareTo(XmlProjectTags.INFOABLE)==0){
				infoable = (getint(parser)==1);
			} else if (parser.getName().compareTo(XmlProjectTags.CUSTOM_FORM)==0){
				customform = parser.nextText();
			} else {
				parser.skipSubTree();
			}
			tag = parser.nextTag();
		}
		File prjDir = new File(prj.getPath());
		if (type.equals(XmlProjectTags.SHP)){
			File f = getFile(prjDir, path);	
			if (!f.exists()){
				String err_msg = Utils.replaceAllString(
						Messages.getText("LayerNotFound"), 
						"LAYER_NAME",name); 
				err_msg=Utils.replaceAllString(
						err_msg, "FILE_PATH",f.getAbsolutePath());
				MessageDialog._showModalErrorMessageDialog(
						Messages.getText("LayersManagerDialog.23"), 
						err_msg, GvsigMobileApp.getTheWindow());
				return null;
			}
			lyr = FLayerFileVectorial.createLayer(f,f.getName(),prj.getMapContext());
			if (lyr!=null){
				leg = new SingleSymbolLegend(simb);
				((FLayerFileVectorial)lyr).setLegend(leg);
				if (customform!=null){
					File cf = getFile(prjDir, customform);
					lyr.setActive(true);
					lyr.setCustomForm(cf.getAbsolutePath());
				}
			}
		}else if (type.equals(XmlProjectTags.GPE)){
			File f = getFile(prjDir, path);	
			if (!f.exists()){
				String err_msg = Utils.replaceAllString(
						Messages.getText("LayerNotFound"), 
						"LAYER_NAME",name); 
				err_msg=Utils.replaceAllString(
						err_msg, "FILE_PATH",f.getAbsolutePath());
				MessageDialog._showModalErrorMessageDialog(
						Messages.getText("LayersManagerDialog.23"), 
						err_msg, GvsigMobileApp.getTheWindow());
				return null;
			}
			try {
				lyr = FlyrGpeVect.createGpeLayer(f,f.getName(),prj.getMapContext());
			} catch (FMapDriverException e) {
				logger.error("While creating GPE layer: " + e.getMessage());
			}
			if (lyr!=null){
				leg = new SingleSymbolLegend(simb);
				((FlyrGpeVect)lyr).setLegend(leg);
				if (customform!=null){
					File cf = getFile(prjDir, customform);
					lyr.setActive(true);
					lyr.setCustomForm(cf.getAbsolutePath());
				}
			}
		}else if (type.equals(XmlProjectTags.RASTER)){
			logger.debug("<path>" + path + "</path>");
			File f = getFile(prjDir, path);
			if (!f.exists()){
				String err_msg = Utils.replaceAllString(
						Messages.getText("LayerNotFound"), 
						"LAYER_NAME",name); 
				err_msg=Utils.replaceAllString(
						err_msg, "FILE_PATH",f.getAbsolutePath());
				MessageDialog._showModalErrorMessageDialog(
						Messages.getText("LayersManagerDialog.23"), 
						err_msg, GvsigMobileApp.getTheWindow());
				return null;
			}
			String fname = f.getName();
			String ext = f.getName().substring(fname.lastIndexOf(".")+1);
			logger.debug("<ext>" + ext + "</ext>");
			if (ext.equalsIgnoreCase("ECW")){
				lyr = FLayerGenericRaster.createEcwLayer(f,f.getName(),prj.getMapContext());
			} else {
				lyr = FLayerGenericRaster.createWKFRasterLayer(f,f.getName(),prj.getMapContext());
			}
		}else if (type.equals(XmlProjectTags.WMS)){
			
			lyr = FLayerGenericRaster.createWMSLayer(url,srs,wmslayrs,format,"WMSLayer",prj.getMapContext());
		}

		if (lyr != null){
			lyr.setVisible(visible == 1);
			lyr.setMaxScale(maxScale);
			lyr.setMinScale(minScale);
			lyr.setInfoable(infoable);
			if (name!=null)
				lyr.setName(name);
		}
		
		logger.debug("</layer> ");
		return lyr;
	}
	

	/**
	 * gets the full path name of a file. If it is a relative path, this method will
	 * find out the full path by reading the project properties.
	 * 
	 * @param projectFile the project file where the file path is cited
	 * @param filePath the possibly relative file path
	 * @return the full path name of a file
	 */
	public static File getFile(File projectFile, String filePath){
		//If filePath is relative, we join it with project path
		
		File resp;
		
		if (isPathRelative(filePath)){
			String path = projectFile.getAbsolutePath();
			if (path.endsWith(File.separator)){
				path += filePath;
			} else {
				path += File.separator + filePath;
			}
			resp = new File(path);	
		} else{
			resp = new File(filePath);
		}
		
		String respaux;
		try {
			respaux = resp.getCanonicalPath();
		} catch (IOException e) {
			logger.error("While getting canon path: " + e.getMessage());
			return resp;
		}
		return new File(respaux);
		
	}
	
	/**
	 * 
	 * Returns true if the indicated path is relative and
	 * false if it is absolute
	 */
	public static boolean isPathRelative(String path){
		path = path.replace("/".charAt(0),File.separatorChar);
		//The path is only the file name
		if (path.indexOf(File.separator)<0)
			return true;
		//The first directory is a root dir
		File[] roots = File.listRoots();
		for (int i=0;i<roots.length;i++){
			if (path.startsWith(roots[i].getAbsolutePath()))
					return false;
		}
		//The other cases
		return true;
	}


}
