/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package es.prodevelop.gvsig.mobile.project;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Date;
import java.util.Iterator;

import es.prodevelop.gvsig.mobile.common.XmlBuilder;
import es.prodevelop.gvsig.mobile.common.XmlProjectTags;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialFileDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.wms.WmsRasterDriver;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerFileVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayerGenericRaster;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.legend.SingleSymbolLegend;
import es.prodevelop.gvsig.mobile.fmap.legend.VectorialLegend;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;
import es.prodevelop.gvsig.mobile.gpe.fmap.VectorialGpeDriver;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;

/**
 * This class stores a project object into a XML text file.
 * 
 * @see es.prodevelop.gvsig.mobile.project.Project
 * @see es.prodevelop.gvsig.mobile.project.ProjectDeserializer
 * 
 * @author jcarras
 *
 */
public abstract class ProjectSerializer {
	
	/**
	 * Stores the project in the file indiceted by the project irself
	 * 
	 * @param prj the project to be stored.
	 * 
	 */
	public static void storeProject(Project prj){
		String sDir = prj.getPath();
		//sDir += File.separator + prj.getName();
		//sDir = sDir.replaceAll(File.separator + File.separator, File.separator);
		//sDir = sDir.replaceAll(File.separator + File.separator, File.separator);
		
		File f = new File(sDir + File.separator + prj.getName() + ".gvm");
		File dir = new File(sDir);
		
		XmlBuilder xml = new XmlBuilder();
		try {
			projectSerialize(xml, prj);
			if (!f.exists()) {
				dir.mkdirs();
				//f.createNewFile();
			}
			OutputStream fout = new FileOutputStream(f);
			OutputStream bout = new BufferedOutputStream(fout);
			OutputStreamWriter out = new OutputStreamWriter(bout,"UTF-8");
			BufferedWriter bw = new BufferedWriter(out);
			bw.write(xml.getXML());
			bw.close();
			bw = null;
			out.close();
			out=null;
			bout.close();
			bout=null;
			fout.close();
			fout=null;
			prj.setModified(false);
		} catch (IOException e) {
			e.printStackTrace();
		} 
	}
	
	/**
	 * Serializes the given project into the existing xml builder.
	 * 
	 * @param xml an existing XML builder where the project has to be serialized. 
	 * @param prj the project to be serialized.
	 */
	public static void projectSerialize(XmlBuilder xml, Project prj){
		
		xml.writeHeader();
		//<Project>
		xml.openTag(XmlProjectTags.PROJECT);
			
			//<Name>
			xml.writeTag(XmlProjectTags.NAME, prj.getName());
			//<Name>
			
			
			xml.writeTag(XmlProjectTags.PRJCOMMENTS, prj.getDescription());
			if (prj.getCreationDate()==null)
				prj.setCreationDate(new Date());
			xml.writeTag(XmlProjectTags.CREATIONDATE, Utils.dateToString(prj.getCreationDate()));
			if (prj.getLastModified()==null)
				prj.setLastModified(new Date(), true);
			xml.writeTag(XmlProjectTags.LASTMODDATE, Utils.dateToString(prj.getLastModified()));
			xml.writeTag(XmlProjectTags.CRS, prj.getProjection().getAbrev());
			xml.writeTag(XmlProjectTags.BGCOLOR, StringUtilities.color2String(prj.getBgColor()));
			
//			<BoundingBox>
			boundSerializer(xml, prj.getMapContext().getViewPort().getExtent());
		
			//</BoundingBox>
			
			//<Layers>
			xml.openTag(XmlProjectTags.LAYERS);
			//layerList.setXML(xml);
			Iterator it= prj.getMapContext().getLayers().iterator();
			while (it.hasNext()){
				layerSerialize(xml, (FLayer)it.next(), prj.getPath());
			}
			//layerList.createXML(xml);
			xml.closeTag();
			//</Layers>
			
			
		xml.closeTag();
		//</Project>
	
	}
	
	/**
	 * This method serializes the layer properties into the existing XML builder.
	 * @param xml an existing XML builder where the layer properties have to be serialized. 
	 * @param lyr the layer whose properties have to be serialized.
	 */
	public static void styleSerialize(XmlBuilder xml, FLayer lyr){
		

		String color = null;
		int size = 1;
		String colorFill = null;		
		String is_filled = "0";
		
		if(lyr instanceof FLyrVect){
			xml.openTag(XmlProjectTags.STYLE);			
			FLyrVect layerV = (FLyrVect) lyr;
			try {
				VectorialLegend vleg = (VectorialLegend)layerV.getLegend();
				FSymbol symbol = (FSymbol)((SingleSymbolLegend)vleg).getDefaultSymbol();
				
				//We take the values we need from the symbol
				size = symbol.getStrokeWidth();
				Color col = symbol.getBorderColor();
				color = StringUtilities.color2String(col);
				is_filled = (symbol.isFilled() ? "1" : "0");
				
				if((layerV.getShapeType() == FShape.POLYGON)||(layerV.getShapeType() == FShape.MULTI)){
					Color colFill=symbol.getFillColor();
					colorFill = StringUtilities.color2String(colFill);
					
					//TODO:
					/*int symboltype = symbol.get;
					FConstant f = null;
					if(symboltype == f.SYMBOL_TYPE_FILL)
						this.fill=1;
					else
						this.fill=0;*/	
				}
				
				//And write the values in the XML
				if(layerV.getShapeType() == FShape.POINT){
					xml.openTag(XmlProjectTags.POINT_SYMBOL);
					xml.writeTag(XmlProjectTags.COLOR, color);
					xml.writeTag(XmlProjectTags.SIZE, java.lang.Integer.toString(size));
					xml.closeTag();
				}
				if(layerV.getShapeType() == FShape.LINE){
					xml.openTag(XmlProjectTags.LINE_SYMBOL);
					xml.writeTag(XmlProjectTags.COLOR, color);
					xml.writeTag(XmlProjectTags.SIZE, java.lang.Integer.toString(size));
					xml.closeTag();
				}
				if(layerV.getShapeType() == FShape.POLYGON){
					xml.openTag(XmlProjectTags.POLYGON_SYMBOL);
					xml.openTag(XmlProjectTags.LINE_SYMBOL);
						xml.writeTag(XmlProjectTags.COLOR, color);
						xml.writeTag(XmlProjectTags.SIZE, java.lang.Integer.toString(size));
					xml.closeTag();
					xml.writeTag(XmlProjectTags.FILL_COLOR, colorFill);
					//TODO:
					xml.writeTag(XmlProjectTags.FILL, is_filled);
					xml.closeTag();
				}
				if(layerV.getShapeType() == FShape.MULTI){
					xml.openTag(XmlProjectTags.GEOMETRY_SYMBOL);
						xml.writeTag(XmlProjectTags.COLOR, color);
						xml.writeTag(XmlProjectTags.SIZE, java.lang.Integer.toString(size));
						xml.writeTag(XmlProjectTags.FILL_COLOR, colorFill);
						xml.writeTag(XmlProjectTags.FILL, is_filled);
					xml.closeTag();
				}
			}  catch (Exception e){
				e.printStackTrace();
			}
		}
		xml.closeTag();
	}
	
	/**
	 * This method serializes the given rectangle into an existing XMl builder object.
	 * @param xml an existing XML builder object where the rectangle has to be serialized
	 * @param bounds the rectangle to be serialized
	 */
	public static void boundSerializer(XmlBuilder xml, Rectangle2D bounds){
		
		//<Bounds>
		xml.openTag(XmlProjectTags.B_BOX);
		
			//<xMin>
			xml.writeTag(XmlProjectTags.X_MIN,java.lang.Double.toString(bounds.getMinX()));
			//</xMin>
			
			//<xMax>
			xml.writeTag(XmlProjectTags.X_MAX,java.lang.Double.toString(bounds.getMaxX()));
			//</xMax>
			
			//<yMin>
			xml.writeTag(XmlProjectTags.Y_MIN,java.lang.Double.toString(bounds.getMinY()));
			//</yMin>
			
			//<yMax>
			xml.writeTag(XmlProjectTags.Y_MAX,java.lang.Double.toString(bounds.getMaxY()));
			//</yMax>
			
		xml.closeTag();
		//</Bounds>
	}
	

	/**
	 * Writes the state of a layer lyr in the xml document
	 * 
	 * @param xml an existing XML builder where the layer has to be serialized 
	 * @param lyr the layer which has to be serialized
	 * @param projPath the project path 
	 */
	public static void layerSerialize(XmlBuilder xml, FLayer lyr, String projPath){

		xml.openTag(XmlProjectTags.LAYER);

		 String path;
		 path=lyr.getName();
		 if (!lyr.isVisible()){
			 xml.writeTag(XmlProjectTags.VISIBLE, "0");
		 } else {
			 xml.writeTag(XmlProjectTags.VISIBLE, "1");
		 }
		 
		 xml.writeTag(XmlProjectTags.SCALE_MAX, "" + Double.toString(lyr.getMaxScale()));
		 xml.writeTag(XmlProjectTags.SCALE_MIN, "" + Double.toString(lyr.getMinScale()));
		 
		 if (!lyr.isInfoable()){
			 xml.writeTag(XmlProjectTags.INFOABLE, "0");
		 } else {
			 xml.writeTag(XmlProjectTags.INFOABLE, "1");
		 }
		 
		 xml.writeTag(XmlProjectTags.NAME, lyr.getName());
		 
		if(lyr instanceof FLyrVect){
	
			try {
				
				if (lyr instanceof FLayerFileVectorial) {
					xml.writeTag(XmlProjectTags.TYPE, XmlProjectTags.SHP);
					FLayerFileVectorial lyrV = (FLayerFileVectorial) lyr;
					VectorialFileDriver drvV = (VectorialFileDriver)lyrV.getDriver();
					path = drvV.getFilePath();
					path = Utils.relPath(projPath, path);
					if (path==null)
						path=drvV.getFilePath();
				}
				if (lyr instanceof FlyrGpeVect) {
					xml.writeTag(XmlProjectTags.TYPE, XmlProjectTags.GPE);
					FlyrGpeVect lyrV = (FlyrGpeVect) lyr;
					VectorialGpeDriver drvV = (VectorialGpeDriver)lyrV.getDriver();
					path = drvV.getFilePath();
					path = Utils.relPath(projPath, path);
					if (path==null)
						path=drvV.getFilePath();
				}
				xml.writeTag(XmlProjectTags.PATH, path);
				if (lyr.getCustomForm()!= null){
					path = lyr.getCustomForm();
					path = Utils.relPath(projPath, path);
					if (path==null)
						path=lyr.getCustomForm();
					xml.writeTag(XmlProjectTags.CUSTOM_FORM,path);
				}
				styleSerialize(xml,lyr);
				
			} catch (IndexOutOfBoundsException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			
		}else {
			if (lyr instanceof FLayerGenericRaster) {
				FLayerGenericRaster lyrR = (FLayerGenericRaster)lyr;
				
				if (lyrR.getSource().getDriver() instanceof WmsRasterDriver) {
					WmsRasterDriver drv = (WmsRasterDriver)lyrR.getSource().getDriver();
					xml.writeTag(XmlProjectTags.TYPE, XmlProjectTags.WMS);
					//xml.writeTag(XmlProjectTags.NAME, lyrR.getName());
					xml.writeTag(XmlProjectTags.URL, drv.getServerUrl().toString());
					xml.writeTag(XmlProjectTags.WMSLAYERS, drv.getRequestedLayers());
					xml.writeTag(XmlProjectTags.WMSSTYLES,drv.getRequestedStyles());
					xml.writeTag(XmlProjectTags.CRS, drv.getSrs());
					xml.writeTag(XmlProjectTags.IMAGE_FORMAT, drv.getImageFormat());
					xml.writeTag(XmlProjectTags.TRANSPARENCY,"" + drv.getTransparency());
					
				} else {
					xml.writeTag(XmlProjectTags.TYPE, XmlProjectTags.RASTER);
					path = lyrR.getFilePath();
					System.out.println("Before rel: " + path); // quitar
					path = Utils.relPath(projPath, path);
					System.out.println("After rel: " + path);
					xml.writeTag(XmlProjectTags.PATH, path);
				}
			}
		}
		/*if(lyr instanceof FLyrWMS){
			
			this.type = WMS;
			FLyrWMS lyrWMS = (FLyrWMS) lyr;
			
			XMLwms xmlwms = new XMLwms();
			xmlwms.setxml(lyrWMS, xml);
			xmlwms.createXML();
			
			
		}*/
		
		xml.closeTag();
		//</Layer>
	}
}
