/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package es.uv.robotica.gvsig.mobile.gps.gui;

import java.awt.Checkbox;
import java.awt.Frame;
import java.awt.Label;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.gvsig.i18n.Messages;

import es.prodevelop.gvsig.mobile.gui.dialog.DialogConstants;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleDialog;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleRow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.ExpansibleWindow;
import es.prodevelop.gvsig.mobile.gui.dialog.control.TocButton;

/**
 * This dialog enables the selection of the NMEA messages to be processed
 * @author juangui
 *
 */
public class GPSNmeaPrefsDialog extends ExpansibleWindow implements MouseListener {

	// fixed width for labels that are on the left side of the dialog
	private static final int LABELS_WIDTH = 65;
	private static final int BUTTONS_WIDTH = 22;
	private boolean okPressed = false;

	// label for title
	private Label titleLabel;

	// row 1
	private Checkbox dbtChkbox;
	private Checkbox ggaChkbox;

	// row 2
	private Checkbox gllChkbox;
	private Checkbox gsvChkbox;

	// row 3
	private Checkbox hdgChkbox;
	private Checkbox hdtChkbox;

	// row 4
	private Checkbox rmcChkbox;
	private Checkbox vtgChkbox;

	// row 5 is empty and vertically expandable
	private TocButton okButton, cancelButton;

	/**
	 * Constructor
	 */
	public GPSNmeaPrefsDialog(Frame parent_frame) {
		super(parent_frame);
		setBounds(
				parent_frame.getX(),
				parent_frame.getY(),
				parent_frame.getWidth(),
				parent_frame.getHeight());

		init();
	}

	/**
	 * Fills the dialog with its controls
	 *
	 */
	private void init() {

		// --------- create controls: ----------------

		// title (row 0)
		titleLabel = new Label(Messages.getText("GPSNmeaPrefsDialog.0")); //$NON-NLS-1$
		titleLabel.setFont(DialogConstants.DIALOG_BOLD_FONT);
		titleLabel.setForeground(DialogConstants.DIALOG_TITLE_COLOR);
		titleLabel.setAlignment(1);
		titleLabel.setSize(LABELS_WIDTH, DialogConstants.CONTROL_HEIGHT);

		// row 1
		dbtChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.1"), true); //$NON-NLS-1$
		dbtChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		dbtChkbox.addMouseListener(this);
		dbtChkbox.setEnabled(false);
		ggaChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.2"), true); //$NON-NLS-1$
		ggaChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		ggaChkbox.addMouseListener(this);
		ggaChkbox.setEnabled(false);

		// row 2
		gllChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.3"), true); //$NON-NLS-1$
		gllChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		gllChkbox.addMouseListener(this);
		gllChkbox.setEnabled(false);
		gsvChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.4"), true); //$NON-NLS-1$
		gsvChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		gsvChkbox.addMouseListener(this);
		gsvChkbox.setEnabled(false);

		// row 3
		hdtChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.5"), true); //$NON-NLS-1$
		hdtChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		hdtChkbox.addMouseListener(this);
		hdtChkbox.setEnabled(false);
		hdgChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.6"), true); //$NON-NLS-1$
		hdgChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		hdgChkbox.addMouseListener(this);
		hdgChkbox.setEnabled(false);

		// row 4
		rmcChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.7"), true); //$NON-NLS-1$
		rmcChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		rmcChkbox.addMouseListener(this);
		rmcChkbox.setEnabled(false);
		vtgChkbox = new Checkbox(Messages.getText("GPSNmeaPrefsDialog.8"), true); //$NON-NLS-1$
		vtgChkbox.setSize(110, DialogConstants.CONTROL_HEIGHT);
		vtgChkbox.addMouseListener(this);
		vtgChkbox.setEnabled(false);

		// row 6
		okButton = new TocButton(
				DialogConstants.DIALOG_SMALL_OK_BUTTON_IMAGE_FILE_NAME, 0, true, false);
		okButton.addMouseListener(this);
//		okButton.setSize(BUTTONS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		okButton.addMouseListener(this);
		cancelButton = new TocButton(
				DialogConstants.DIALOG_SMALL_CANCEL_BUTTON_IMAGE_FILE_NAME, 1, true, false);
//		cancelButton.setSize(BUTTONS_WIDTH, DialogConstants.CONTROL_HEIGHT);
		cancelButton.addMouseListener(this);
		// -----------------------

		// put controls in its place: -----------
		// title --------------
		ExpansibleRow row;
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add horizontally expandable control to row
		row.addExtensible(titleLabel);
		// add row to dialog, not expandable
		add(row);
		// row 1 --------------
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add controls to row
		row.add(dbtChkbox);
		row.add(ggaChkbox);
		// add row to dialog, not expandable
		add(row);
		// row 2 --------------
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add controls to row
		row.add(gllChkbox);
		row.add(gsvChkbox);
		// add row to dialog, not expandable
		add(row);
		// row 3 --------------
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add controls to row
		row.add(hdgChkbox);
		row.add(hdtChkbox);
		// add row to dialog, not expandable
		add(row);
		// row 4 --------------
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add controls to row
		row.add(rmcChkbox);
		row.add(vtgChkbox);
		// add row to dialog, not expandable
		add(row);
		// row 5 --------------
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add expandable empty row to dialog
		addExpansible(row);
		// row 6 --------------
		row = new ExpansibleRow(DialogConstants.CONTROL_HEIGHT);
		// add controls to row
		row.add(okButton);
		row.add(cancelButton);
		// add row to dialog, not expandable
		add(row);
		// --------------
		addEmptyRowIfPhoneME(this);
	}

	/**
	 * This method can be called when the dialog is closed to test
	 * wether it was closed by pressing the ok button or not
	 * @return true if the dialog was closed by pressing the ok button
	 */
	public boolean isOkPressed() {
		return okPressed;
	}

	/**
	 * Process mouse click event in the dialog
	 * When closing the dialog checks wether the ok button or
	 * the cancel button was pressed
	 *
	 * Implements java.awt.event.MouseListener
	 */
	public void mousePressed(MouseEvent e) {
		Object src = e.getSource();

		if (src == okButton) {
			okPressed = true;
			try {
				hide();
			} catch (NullPointerException npe) {
				// logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
			}
		}

		if (src == cancelButton) {
			okPressed = false;
			try {
				hide();
			} catch (NullPointerException npe) {
				// logger.warn("Dialog hidden: A mysterious NP exception was caught: " + npe.getMessage());
			}
		}

	}

	/**
	 * This method is just included to comply with MouseListener interface
	 * Implements java.awt.event.MouseListener
	 */
	public void mouseReleased(MouseEvent arg0) { }

	/**
	 * This method is just included to comply with MouseListener interface
	 * Implements java.awt.event.MouseListener
	 */
	public void mouseEntered(MouseEvent arg0) { }

	/**
	 * This method is just included to comply with MouseListener interface
	 * Implements java.awt.event.MouseListener
	 */
	public void mouseExited(MouseEvent arg0) { }

	/**
	 * This method is just included to comply with MouseListener interface
	 * Implements java.awt.event.MouseListener
	 */
	public void mouseClicked(MouseEvent arg0) {	}

}
