/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import org.apache.log4j.Logger;

/**
 * Implementation of the ExtentHistory interface
 * 
 * @author jldominguez
 */
public class ExtentHistoryImpl implements  ExtentHistory {
	
	private static Logger logger = Logger.getLogger(ExtentHistoryImpl.class);
	
	private int recordSize = 10;
	private ArrayList backward;
	private ArrayList forward;

	/**
	 * Creates a new ExtentsHistory object.
	 */
	public ExtentHistoryImpl() {
		backward = new ArrayList();
		forward = new ArrayList();
	}

	/**
	 * Creates a new ExtentsHistory object.
	 *
	 * @param numEntries Numero de entradas que se guardan en el historico de
	 * 		  rectngulos, por defecto 20
	 */
	public ExtentHistoryImpl(int numEntries) {
		recordSize = numEntries;
		backward = new ArrayList();
		forward = new ArrayList();
	}

	/**
	 * Pone un nuevo rectngulo al final del array
	 *
	 * @param ext Rectngulo que se aade al histrico
	 */
	public void put(Rectangle2D ext) {
		if (ext != null) {
			if (backward.size() == recordSize) {
				backward.remove(0);
			}
			backward.add(ext);
			forward.clear();
		}
	}

	/**
	 * Devuelve true si hay algn rectngulo en el histrico
	 * @return true o false en caso de que haya o no haya rectngulos
	 */
	public boolean hasPrevious() {
		return (backward.size() > 0);
	}

	/**
	 * Devuelve true si hay algn rectngulo en el histrico
	 * @return true o false en caso de que haya o no haya rectngulos
	 */
	public boolean hasNext() {
		return (forward.size() > 0);
	}

	/**
	 * Obtiene el ltimo rectngulo que se aadi al histrico
	 *
	 * @return Ultimo rectngulo aadido
	 */
	public Rectangle2D getPrevious(Rectangle2D r) {
		int last = backward.size() - 1;
		Rectangle2D resp = (Rectangle2D) backward.get(last);
		backward.remove(last);
		
		if (forward.size() == recordSize) {
			forward.remove(0);
		}
		forward.add(0, r);
		return resp;
	}
	
	/**
	 * Obtiene el ltimo rectngulo que se aadi al histrico
	 *
	 * @return Ultimo rectngulo aadido
	 */
	public Rectangle2D getNext(Rectangle2D curr) {
		
		logger.debug("getNext(Rectangle2D curr): inicio");
		logger.debug("getNext(Rectangle2D curr): curr = " + curr);
		logger.debug("getNext(Rectangle2D curr): forward size = " + forward.size());
		
		Rectangle2D resp = (Rectangle2D) forward.get(0);
		
		logger.debug("getNext(Rectangle2D curr): resp = " + resp);
		
		forward.remove(0);
		logger.debug("getNext(Rectangle2D curr): 2 forward size = " + forward.size());
		
		if (backward.size() == recordSize) {
			backward.remove(0);
		}
		
		logger.debug("getNext(Rectangle2D curr): 1 backward size = " + backward.size());
		backward.add(curr);
		logger.debug("getNext(Rectangle2D curr): 2 backward size = " + backward.size());
		
		logger.debug("getNext(Rectangle2D curr): fin resp = " + resp);
		return resp;
	}
	
	public int getCapacity() {
		return recordSize;
	}

}
