/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.core;

import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.Value;


/**
 * 
 * This class encapsulates a row of alphanumerical attributes.
 * 
 *
 * @author Vicente Caballero Navarro
 */
public class DefaultRow implements IRow {
    private Value[] attributes;
    private String id;

    

    /**
     * FROM GEOTOOLS: Creates an ID from a hashcode. Uses current time.
     *
     * @return an id for the feature.
     */
    String defaultID() {
        return "fid-" + System.currentTimeMillis();
    }
    
    /**
     * Constructor. Uses a default ID
     *
     * @param att array of attributes 
     */
    public DefaultRow(Value[] att) {
        this.attributes = att;
        this.id = defaultID();
    }

    /**
     * Constructor.
     *
     * @param att array of attributes
     * @param id feature ID
     */
    public DefaultRow(Value[] att, String id) {
        this.attributes = att;
        this.id = id;
    }

    /**
     * 
     *
     * @return gets the ID
     */
    public String getID() {
        return id;
    }

    /**
     * 
     *
     * @param fieldIndex index of the requested field 
     *
     * @return the attribute of the given index
     */
    public Value getAttribute(int fieldIndex) {
    	if (attributes == null)
    		return new NullValue();
    	if (fieldIndex >= attributes.length)
    		return new NullValue();
        return attributes[fieldIndex];
    }

    /**
     * 
     * Gets the array of attribute
     * @return the array of attribute 
     */
    public Value[] getAttributes() {
        return attributes;
    }

    /**
     * Cones the current object
     *
     * @return anothe rinstance of this object with the same attributes and ID
     */
    public IRow cloneRow() {
        Value[] values = new Value[attributes.length];

        if (attributes != null) {
            values = (Value[]) attributes.clone();
        }

        DefaultRow dr = new DefaultRow(values, id);

        return dr;
    }
    
    /**
     * Sets the ID.
     * @param ID the new ID
     */
	public void setID(String ID) {
		id = ID;
	}

	/**
	 * Sets the array of attributes
	 * @param att the new array of attributes
	 * 
	 */
	public void setAttributes(Value[] att) {
		this.attributes = att;
	}
    
}
