/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.core.acetate;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.core.acetate.IAcetateElement;

/**
 * 
 * This class contains the GPS cursor used to indicate the position delivered by the
 * GPS device.
 * 
 * 
 * @author jcarras
 *
 */
public class MapCursor extends Component implements IAcetateElement {
	
	private static Logger logger = Logger.getLogger(MapCursor.class);

	private boolean visible = false;
	private Image icon;
	private double rotationDegrees = 0;
	
	private Point2D geoPoint = new Point2D.Double();
	private int radius_x = 0;
	private int radius_y = 0;

	/**
	 * Constructor
	 * @param i the icon image to be used. Preferable, a square image
	 * with odd size: w = h = 2n + 1 (n is a natural number).
	 * 
	 */
	public MapCursor(Image i){
		icon=i;

		radius_x = icon.getWidth(null) / 2;
		radius_y = icon.getHeight(null) / 2;;
	}
	
	public void paint(Graphics g) {
		if (icon != null && visible) {
			g.drawImage(icon, 0, 0, null);
			logger.debug("MapCursor Icon painted");
		}
	}
	
	public void setXY(double x, double y) {
		geoPoint.setLocation(x, y);
	}
	
	/**
	 * @return the icon image
	 */
	public Image getIcon() {
		return icon;
	}
	
	/**
	 * Sets the icon image.
	 */
	public void setIcon(Image i) {
		icon = i;
		radius_x = icon.getWidth(null) / 2;
		radius_y = icon.getHeight(null) / 2;
	}

	public double getRotation() {
		return rotationDegrees;
	}

	public void setRotation(double r) {
		rotationDegrees = r;
	}

	public void draw(Graphics2D g, AffineTransform at) {
		
		Point2D aux = new Point2D.Double(0, 0);
		at.transform(geoPoint, aux);
		int screenx = (int) Math.round(aux.getX()) - radius_x;
		int screeny = (int) Math.round(aux.getY()) - radius_y;
		g.drawImage(icon, screenx, screeny, null);
	}
	
}
