/*
 * Created on 20-dic-2004
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.driver.raster;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

public class RasterAdapter {
	private boolean driverInitialized = false;
	private RasterDriver driver;
	
    /**
     * Establece el driver sobre el que acta el adaptador 
     */
    public void setDriver(RasterDriver drv) {
    	this.driver = drv;
    }

	/**
     * Obtiene una referencia al objeto que implementa la interfaz vectorial con
     *  el fin de que las Strategy puedan optimizar en funcin del driver.
     */
    public RasterDriver getDriver(){
    	return driver;
    }

    /**
	 * Obtiene el valor de transparencia
	 * @return entero que representa el valor de transparencia
	 */
	public int getTransparency() {
		return ((RasterDriver) driver).getTransparency();
	}
	
	/**
	 * Asigna el valor de transparencia
	 * @param trans	entero que representa el valor de transparencia
	 */
	public void setTransparency(int trans) {
		((RasterDriver) driver).setTransparency(trans);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.LayerOperations#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, ISymbol)
	 */
	public void draw(Graphics2D g, ViewPort vp) throws FMapDriverException {
		((RasterDriver) driver).draw(g, vp);
	}
	
	/**
	 * Obtiene la lista de atributos de raster
	 * @return	lista de atributos. Cada elmento de la lista es un array Object
	 * con dos elementos. El primero el nombre del atributo y el segundo el valor
	 * del mismo.
	 */
	
	/**
	 * Obtiene el nmero de bandas de la imagen
	 * @return nmero de bandas
	 */
	public int getNumBands(){
		return ((RasterDriver)driver).getNumBands();
	}
	
	/**
	 * Obtiene el extent de la vista
	 * @return extent
	 */
	public Rectangle2D getFullExtent(){
		return ((RasterDriver)driver).getFullExtent();
	}
	
	public boolean isGeoreferenced() {
		return ((GeorreferencedRasterDriver)driver).isGeoreferenced();
	}
	
}
