/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Random;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.GeorreferencedRasterDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.RasterDriver;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.download.Downloader;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

/**
 * This is the ECW driver based on JNI access to Ermapper libraries.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw.EcwReader
 * @see java.awt.geom.AffineTransform
 * 
 * @author jldominguez
 *
 */
public class EcwRasterDriver implements GeorreferencedRasterDriver {
	
	private static Logger logger = Logger.getLogger(EcwRasterDriver.class);
	

	private IProjection proj = null;
	private int trans = 0;
	private AffineTransform at = AffineTransform.getTranslateInstance(0, 0);
	
	/**
	 * Raster units constants
	 */
	public final static int UNITS_UNKNOWN = 0;
	/**
	 * Raster units constants
	 */
	public final static int UNITS_METERS = 1;
	/**
	 * Raster units constants
	 */
	public final static int UNITS_DEGREES = 2;
	/**
	 * Raster units constants
	 */
	public final static int UNITS_FEET = 3;

	private String path = "";
	private long handler = 0;
	private double min_geo_x = 0;
	private double min_geo_y = 0;
	private double geo_w = 0;
	private double geo_h = 0;
	private int w = 0;
	private int h = 0;
	private int nbands = 0;
	private int band1 = -1;
	private int band2 = -1;
	private int band3 = -1;
	private int units = UNITS_UNKNOWN;
	// ---------------------------------------------
	// IMAGE
	// private BufferedImage image = null;
	// private int image_offset_x = 0;
	// private int image_offset_y = 0;
	// ---------------------------------------------
	
	private ViewPort viewPort = null;
	
	/**
	 * Constructor
	 * 
	 * @param vp a reference to the viewport to perform geographic computations
	 */
	public EcwRasterDriver(ViewPort vp) {
		viewPort = vp;
	}
	
	private void setBands(int nb) {

		band1 = 0;
		if (nb > 1) band2 = 1;
		if (nb > 2) band3 = 2;
	}
	
	/**
	 * Draws layer.
	 * 
	 * @param mingeox refers to zone of interest
	 * @param mingeoy refers to zone of interest
	 * @param geow refers to zone of interest
	 * @param geoh refers to zone of interest
	 * @param pixelxoffset
	 * @param pixelyoffset
	 * @param pixelw refers to zone of interest
	 * @param pixelh refers to zone of interest
	 * @param g
	 * @param repeat
	 * @param ishalf
	 * 
	 * @return
	 */
	private boolean draw(
			double mingeox, double mingeoy,
			double geow, double geoh,
			int pixelxoffset, int pixelyoffset,
			int pixelw, int pixelh,
			Graphics2D g, int repeat) {
		
		if (repeat == 0) {
			// logger.error(GeoUtils.time() + "@@@ REPEAT = 0. Renunciamos a pintarla :(");
			return false;
		} else {
			// logger.debug(GeoUtils.time() + "@@@ REPEAT = " + repeat + ". Intento: " + (4 - repeat));
		}
		
		if (handler == 0) {
			return false;
		}

		/*
		logger.debug(GeoUtils.time() + "Llamamos a getEcwView() con estos parametros: \n"
				+ "HANDLER = " + handler
				+ ", ASK_MINGEOX = " + mingeox
				+ ", ASK_MINGEOY = " + mingeoy
				+ ", ASK_GEOW = " + geow
				+ ", ASK_GEOH = " + geoh
				+ ", FILE_MINGEOX = " + min_geo_x
				+ ", FILE_MINGEOY = " + min_geo_y
				+ ", FILE_PIX_W = " + (geo_w / w)
				+ ", FILE_PIX_H = " + (geo_h / h)
				+ ", FILE_W = " + w
				+ ", FILE_H = " + h
				+ ", OUT_W_EN_PIXELS = " + pixelw
				+ ", OUT_H_EN_PIXELS = " + pixelh
				+ ", B1 = " + band1
				+ ", B2 = " + band2
				+ ", B3 = " + band3);
		*/
		
		int[] pixels = null;

		try {
			
			// pixels = getTestArray((int) (System.currentTimeMillis() % 1000));
			
			pixels = EcwReader.getEcwView(
					handler,
					mingeox,
					mingeoy,
					geow,
					geoh,
					min_geo_x,
					min_geo_y,
					geo_w / w,
					geo_h / h,
					w,
					h,
					pixelw,
					pixelh,
					band1,
					band2,
					band3);
		} catch (Throwable th) {
			logger.error(Utils.time() + "While getting image: " + th.getMessage());
			logger.debug("Trying to reopen...");
			reopen();
			logger.info(Utils.time() + "### Recursive call...");
			return draw(mingeox, mingeoy,
					Utils.smallerDouble(geow, 0.92, 0), // asked geo size
					Utils.smallerDouble(geoh, 0.92, 0),
					pixelxoffset, pixelyoffset,
					Utils.smallerInteger(pixelw, 0.92, 1), // asked pix size
					Utils.smallerInteger(pixelh, 0.92, 1),
					g, repeat - 1);
		}

		if (pixels == null) {
			logger.debug(Utils.time() + "Apparently Image does not intersect view...");
			return false;
		}

		if (pixels.length == 6) {
			logger.error(Utils.time() + "JNI-ECW returned ERROR. "
					+ "Watch this array to debug:");
			for (int i=0; i<pixels.length; i++) {
				logger.error("ERROR ARRAY[" + i + "] = " + pixels[i]);
			}
			return false;
		}
		
		logger.debug(Utils.time() + "DLL returned pixels: " + (pixels.length - 6));
		
		int paste_w =     pixels[0];
		int paste_h =     pixels[1];
		int paste_off_x = pixels[2];
		int paste_off_y = pixels[3];
		int truly_ret_w = pixels[4];
		int truly_ret_h = pixels[5];
		
		// logger.debug(Utils.time() + "TRULY RET W = " + truly_ret_w);
		// logger.debug(Utils.time() + "TRULY RET H = " + truly_ret_h);

		Image bim = null;
		
		try {
			bim = getTheImage(pixels, truly_ret_w, truly_ret_h, 6);
		} catch (Throwable th) {
			logger.error(Utils.time() + "::: PNG error: " + th.getMessage());
		}
		
		if (bim == null) {
			logger.debug(Utils.time() + "Image is NULL (PNG issue). Let's reopen...");
			reopen();
			return draw(mingeox, mingeoy,
					Utils.smallerDouble(geow, 0.92, 0), // asked geo size
					Utils.smallerDouble(geoh, 0.92, 0),
					pixelxoffset, pixelyoffset,
					Utils.smallerInteger(pixelw, 0.92, 1), // asked pix size
					Utils.smallerInteger(pixelh, 0.92, 1),
					g, repeat - 1);
			
		}

		if (paste_w != truly_ret_w) {
			logger.debug(Utils.time() + "Image being rescaled...");
			bim = ResourceReader.getScaled(bim, paste_w, paste_h);

			if (bim == null) {
				logger.error("Unable to resample image, got a NULL image.");
				return false;
			}
		}

		try {
			g.drawImage(bim, pixelxoffset + paste_off_x, pixelyoffset + paste_off_y, null);
			/*
			Random rnd = new Random(System.currentTimeMillis());
			
			Color c = new Color(
					128 + rnd.nextInt(128),
					128 + rnd.nextInt(128),
					128 + rnd.nextInt(128));
			
			g.setColor(c);
			g.drawRect(
					pixelxoffset + paste_off_x,
					pixelyoffset + paste_off_y,
					bim.getWidth(null) - 4,
					bim.getHeight(null) - 4);
			*/
		} catch (Throwable th) {
			logger.error(Utils.time() + "While drawing image: " + th.getMessage());
			return false;
		}
		
		// logger.debug(Utils.time() + "Vamos a hacer drawImage... hecho");
		
		bim = null;
		pixels = null;
		return true;
	}
	
	private int[] getTestArray(int s) {
		
		double seed = s * 0.001;
		
		int size = 240 * 268 + 6;
		int[] resp = new int[size];
		resp[0] = 240;
		resp[1] = 268;
		resp[2] = 0;
		resp[3] = 0;
		resp[4] = 240;
		resp[5] = 268;
		
		int pixel = 0; 
		
		for (int i=6; i<size; i++) {
			pixel =
				65536 * Math.round((float) (254 * Math.abs(Math.cos(seed + i * 0.003)))) +
				256 * Math.round((float) (254 * Math.abs(Math.cos(seed + i * 0.004)))) +
				65536 * Math.round((float) (254 * Math.abs(Math.cos(seed + i * 0.005))));
			resp[i] = pixel;
		}
		return resp;
	}

	private int getPixelFromGeoCoordinates(double wcx, double wcy, double tol) {
		
		/*
		logger.debug(GeoUtils.time() + "Llamamos a EcwReader.getEcwView(...) con: " +
		handler + ", " +
		wcx + ", " +
		wcy + ", " +
		tol + ", " +
		tol + ", " +
		min_geo_x + ", " +
		min_geo_y + ", " +
		(geo_w / w) + ", " +
		(geo_h / h) + ", " +
		w + ", " +
		h + ", " +
		1 + ", " +
		1 + ", " +
		band1 + ", " +
		band2 + ", " +
		band3);
		*/

		int[] pixels = EcwReader.getEcwView(
				handler,
				wcx,
				wcy,
				tol,
				tol,
				min_geo_x,
				min_geo_y,
				geo_w / w,
				geo_h / h,
				w,
				h,
				1,
				1,
				band1,
				band2,
				band3);
		

		if (pixels == null) {
			return RasterDriver.ONE_PIXEL_NO_DATA;
		}

		if (pixels.length == 6) {
			logger.error(Utils.time() + "JNI-ECW (1 pixel) returned ERROR "
					+ "Watch this array to debug:");
			for (int i=0; i<pixels.length; i++) {
				logger.error("ARRAY OF ERROR[" + i + "] = " + pixels[i]);
			}
			return RasterDriver.ONE_PIXEL_IMAGE_ERROR;
		}
		
		if (pixels.length != 7) {
			logger.error(Utils.time() + "JNI-ECW (1 pixel) returned wrong size: " + pixels.length);
			for (int i=0; i<Math.min(pixels.length, 10); i++) {
				logger.error("ARRAY DE ERROR[" + i + "] = " + pixels[i]);
			}
			logger.error("...");
			return RasterDriver.ONE_PIXEL_IMAGE_ERROR;
		}

		return pixels[6];
	}
	
	private void printMetadata(double[] d) {
		logger.debug("---------------------------");
		for (int i=0; i<d.length; i++) {
			logger.debug("metadato[" + i + "] = " + d[i]);
		}
		logger.debug("---------------------------");
	}
	
	/**
	 * Opens the raster file. This method has to be used after the file has been set, of
	 * course.
	 * 
	 * @return whether the fiel could be opened succesfully
	 */
	public boolean open() {
		
		try {
			// handler = EcwReader.openEcwFile(path.getBytes());
			handler = EcwReader.openEcwFile(path);
		} catch (Throwable th) {
			logger.error("EcwReader.openEcwFile(path): " + th.getMessage());
		}
		
		logger.info("After openEcwFile. HANDLER = " + handler);

		if (handler != 0) {
			double[] metadata = EcwReader.getEcwImageInfo(handler);
			// printMetadata(metadata);

			double incr_x = metadata[2];
			double incr_y = metadata[3];
			
			geo_w = Math.abs(metadata[2]);
			geo_h = Math.abs(metadata[3]);
			
			min_geo_x = Math.min(metadata[0], metadata[0] + incr_x);
			min_geo_y = Math.min(metadata[1], metadata[1] + incr_y);

			w = (int) Math.round(metadata[4]);
			h = (int) Math.round(metadata[5]);
			nbands = (int) Math.round(metadata[6]);
			
			setBands(nbands);
			
			units = (int) Math.round(metadata[7]);
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 
	 * @return raster height in map units
	 */
	public double getGeoHeight() {
		return geo_h;
	}

	/**
	 * 
	 * @return raster width in map units
	 */
	public double getGeoWidth() {
		return geo_w;
	}

	/**
	 * 
	 * @return raster height in pixels
	 */
	public int getHeight() {
		return h;
	}

	/**
	 * 
	 * @return file handler (a long integer). if the file could not be opened
	 * this will probably return 0.
	 */
	public long getHandler() {
		return handler;
	}

	/**
	 * 
	 * @return minimum x value in map units
	 */
	public double getMinGeoX() {
		return min_geo_x;
	}

	/**
	 * 
	 * @return minimum y value in map units
	 */
	public double getMinGeoY() {
		return min_geo_y;
	}

	/**
	 * 
	 * @return number of bands in raster
	 */
	public int getBandCount() {
		return nbands;
	}

	/**
	 * 
	 * @return full raster file path
	 */
	public String getPath() {
		return path;
	}

	/**
	 * 
	 * @return a constant indicating the map units 
	 */
	public int getUnits() {
		return units;
	}

	/**
	 * 
	 * @return the raster width in pixels
	 */
	public int getWidth() {
		return w;
	}

	/**
	 * Initializes the driver, simply sets the projection.
	 * 
	 * @param p the raster projection
	 */
	public void initialize(IProjection p) throws IOException {
		proj = p;
	}

	/**
	 * @return the raster projection
	 */
	public IProjection getProjection() {
		return proj;
	}

	/**
	 * Sets the projection.
	 * 
	 * @param p the raster projection
	 */
	public void setProjection(IProjection p) {
		proj = p;
	}

	/**
	 * @return the raster full extent in map units
	 */
	public Rectangle2D getFullExtent() {
		Rectangle2D resp = new Rectangle2D.Double(
				min_geo_x, min_geo_y, geo_w, geo_h);
		return resp;
	}

	/**
	 * @return whether this is a georreferenced raster. Returns true.
	 */
	public boolean isGeoreferenced() {
		return true;
	}

	/**
	 * Sets the current affine transform
	 * @param t the new affine transform
	 */
	public void setAffineTransform(AffineTransform t) {
		at = t;
	}

	/**
	 * Gets the current affine transform
	 * @return the current affine transform
	 */
	public AffineTransform getAffineTransform() {
		return at;
	}

	/**
	 * Tells whether the provided file is accepted as a ECW file.
	 * Simply checks the extension.
	 * 
	 * @param fName file object of interest
	 * @return whether its an accepted ECW file or not
	 */
	public boolean fileAccepted(File fName) {
		try {
			String fpath = fName.getCanonicalPath();
			int ind_ext = fpath.lastIndexOf(".");
			if (ind_ext == -1) {
				return false;
			}
			String ext = fpath.substring(ind_ext + 1, fpath.length());
			return (ext.compareToIgnoreCase("ecw") == 0);
		} catch (IOException e) {
			return false;
		}
	}

	/**
	 * This method opens the provided file.
	 * 
	 * @param f the ECW file to be opened
	 */
	public void open(File f) throws IOException {
		path = f.getCanonicalPath();
		if (open()) {
		} else {
			throw new IOException("Unable to open file: " + path);
		}
	}

	/**
	 * Initialize method, not used right now.
	 */
	public void initialize() throws IOException {
	}

	/**
	 * Thios method tries to close the current ECW file.
	 */
	public void close() throws IOException {
		
		logger.debug(Utils.time() + "Empieza close() del driver ecw.");
		if (EcwReader.closeEcwFile(handler)) {
			
		} else {
			throw new IOException("While closing ecw file: " + path);
		}
	}

	/**
	 * @return the number of bands of thsi raster
	 */
	public int getNumBands() {
		return nbands;
	}

	/**
	 * @return the transparency value of this raster
	 */
	public int getTransparency() {
		return trans;
	}

	/**
	 * Sets the transparency value of this raster
	 * @param t the new transparency value of this raster
	 */
	public void setTransparency(int t) {
		trans = t;
	}


	/**
	 * draws the image on the provided Graphics object
	 * 
	 * @param g the graphics object on which the image must be drawn
	 * @param vp the view port
	 */
	public void draw(Graphics2D g, ViewPort vp) throws FMapDriverException {
		draw(g, vp.getAdjustedExtent(), vp.getImageWidth(), vp.getImageHeight(), 0, 0);
	}
	
	/**
	 * draws the image on the provided Graphics object
	 * 
	 * @param g the graphics object on which the image must be drawn
	 * @param extent
	 * @param imgw
	 * @param imgh
	 */
	public void draw(Graphics2D g, Rectangle2D extent, int imgw, int imgh, int offx, int offy) throws FMapDriverException {
		
		if ((imgw < 250) || (! Utils.USING_PDA)) {
			// nomally
			draw(extent.getMinX(),
					extent.getMinY(),
					extent.getWidth(),
					extent.getHeight(),
					offx, offy,
					imgw, imgh, g, 3);
			
		} else {
			// prevent very large byte arrays when screen size = 480 x 640
			SimpleVP wholevp = new SimpleVP(extent, imgw, imgh); 
			SimpleVP[] quad = SimpleVP.getQuad(wholevp);
			int[] wwhh = SimpleVP.getWidthsHeights(wholevp);
			draw(g, quad[0].getExtent(), quad[0].getImageWidth(), quad[0].getImageHeight(),
					offx, offy);
			draw(g, quad[1].getExtent(), quad[1].getImageWidth(), quad[1].getImageHeight(),
					offx + wwhh[0], offy);
			draw(g, quad[2].getExtent(), quad[2].getImageWidth(), quad[2].getImageHeight(),
					offx, offy + wwhh[2]);
			draw(g, quad[3].getExtent(), quad[3].getImageWidth(), quad[3].getImageHeight(),
					offx + wwhh[0], offy + wwhh[2]);
		}
	}

	/**
	 * @return the raster data type of this raster.
	 * 
	 */
	public int getRasterDataType() {
		return DataBuffer.TYPE_BYTE;
	}

	/**
	 * Gets the data associated with the given coordinates and band index.
	 * 
	 * @param x x coordinate of the POI
	 * @param y y coordinate of the POI
	 * @param band band index
	 * @return the data associated to the given coordinates and band 
	 */
	public Object getData(int x, int y, int band) {
		return null;
	}

	/**
	 * Gets the RGB values of the pixel of interest
	 * 
	 * @param wcx x coordinate in map units
	 * @param wcy y coordinate in map units
	 * 
	 * @return array of RGB values of the pixel of interest
	 */
	public int[] getPixel(double wcx, double wcy) {
		
		double tol = viewPort.toMapDistance(1);
		
		int p = getPixelFromGeoCoordinates(wcx, wcy, tol);
		int[] resp = null;
		
		if ((p == RasterDriver.ONE_PIXEL_NO_DATA)
				|| (p == RasterDriver.ONE_PIXEL_IMAGE_ERROR)) {
			
			resp = new int[1];
			resp[0] = p;
		} else {
			resp = new int[3];
			resp[0] = (p >> 16) & 0xff;
			resp[1] = (p >> 8) & 0xff;
			resp[2] = p & 0xff;
		}
		return resp;
	}

	/**
	 * @return the driver name
	 */
	public String getName() {
		// TODO Auto-generated method stub
		return "ECW raster driver";
	}


    private Image getTheImage(int[] bitmap, int w, int h, int offset) {

    	if (Utils.USING_PDA) {
    		

    		
        	byte[] test = EcwReader.paintArray(bitmap, w, h, offset, false);
        	if (test == null) {
        		logger.error(Utils.time() + "EcwReader.paintArray(...) returned NULL");
        		return null;
        	}
        	
        	// logger.debug("--- WRITE PNG FILE ---");
        	// writeBytesToFile(test, "" + (System.currentTimeMillis() % 10000));
        	// logger.debug("--- WRITE PNG FILE DONE ---");
        	
        	Image img = ResourceReader.getImage(test);
        	
        	// Image img = Toolkit.getDefaultToolkit().createImage(test); // -.getImage(f.getAbsolutePath());
        	if (img.getWidth(null) == -1) {
        		return null;
        	} else {
        		return img;
        	}
        	
    	} else { // PC
    		BufferedImage resp = new BufferedImage(w, h, 1);
    		resp.setRGB(0, 0, w, h, bitmap, offset, w);
    		return resp;
    	}
    }

	private void writeBytesToFile(byte[] test, String str) {
		
		try {
			File outf;
			
			if (Utils.USING_PDA) {
				outf = new File(
						File.separator
						+ "apps"
						+ File.separator
						+ "27mar2007"
						+ File.separator
						+ "debug."
						+ str + ".png");
			} else {
				outf = new File(
						"c:"
						+ File.separator
						+ "windows"
						+ File.separator
						+ "temp"
						+ File.separator + "debug." + str + ".png");
			}
			FileOutputStream fos = new FileOutputStream(outf);
			fos.write(test);
			fos.flush();
			fos.close();
		} catch (Exception ex) {
			logger.error("While writing PNG file: " + ex.getMessage());
		}
	}

	private Image createImage(int[] bitmap, int w, int h, int offset) {

		 BufferedImage _bim = Utils.createBufferedImage(w, h);
		 _bim.setRGB(0, 0, w, h, bitmap, offset, w);
		 return _bim;
		}	
	
	private void reopen() {
		try {
			close();
		} catch (IOException e) {
			logger.error(Utils.time() + "While closing file: " + e.getMessage());
		}
		logger.error(Utils.time() + "Reopen: " + path);
		try {
			handler = EcwReader.openEcwFile(path);
		} catch (Throwable e) {
			handler = 0;
			logger.error(Utils.time() + "New handler: " + handler + " !!");
			return;
		}
		logger.debug(Utils.time() + "New handler: " + handler);
	}

	/**
	 * @return the full file path
	 */
	public String getFilePath() {
		return path;
	}
	
	


}
