/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.raster.osm;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.GeorreferencedRasterDriver;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.util.ResourceReader;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.download.DownloadException;
import es.prodevelop.gvsig.mobile.fmap.util.download.DownloadTask;
import es.prodevelop.gvsig.mobile.fmap.util.download.Downloader;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

public class OpenStreetMapDriver implements GeorreferencedRasterDriver {
	
	private static Logger logger = Logger.getLogger(OpenStreetMapDriver.class);

	private int layerType = OpenStreetMapUtils.OSM_LAYER_OSMARENDER;
	private boolean resizeDown = true;
	
	private OpenStreetMapDriver() { }
	
	public OpenStreetMapDriver(int lyr, boolean st) {
		resizeDown = st;
		layerType = lyr;
	}
	
	public void initialize(IProjection proj) throws IOException {
		// TODO Auto-generated method stub

	}

	public IProjection getProjection() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setProjection(IProjection proj) {
		// TODO Auto-generated method stub

	}

	public Rectangle2D getFullExtent() {
		
		double lat_factor = 0.56; 
		double lon_factor = 0.98;
		return new Rectangle2D.Double(
				- lon_factor * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH,
				- lat_factor * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH,
				2 * lon_factor * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH,
				2 * lat_factor * OpenStreetMapUtils.SEMI_EQUATOR_LENGTH);
	}

	public boolean isGeoreferenced() {
		// TODO Auto-generated method stub
		return true;
	}

	public void setAffineTransform(AffineTransform t) {
		// TODO Auto-generated method stub

	}

	public AffineTransform getAffineTransform() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean fileAccepted(File fName) {
		// TODO Auto-generated method stub
		return false;
	}

	public void open(File f) throws IOException {
		// TODO Auto-generated method stub

	}

	public void initialize() throws IOException {
		// TODO Auto-generated method stub

	}

	public void close() throws IOException {
		// TODO Auto-generated method stub

	}

	public String getFilePath() {
		// TODO Auto-generated method stub
		return null;
	}

	public int getNumBands() {
		// TODO Auto-generated method stub
		return 0;
	}

	public int getTransparency() {
		// TODO Auto-generated method stub
		return 0;
	}

	public void setTransparency(int trans) {
		// TODO Auto-generated method stub

	}

	public void draw(Graphics2D g, ViewPort viewPort)
			throws FMapDriverException {
		
		int[] d_w =
			OpenStreetMapUtils.getDepthAndTileSize(
					viewPort.getAdjustedExtent().getWidth(),
					viewPort.getImageWidth(),
					resizeDown);
		
		int d = d_w[0];
		int w = d_w[1];

		// System.out.println("depth = " + d);
		// System.out.println("width = " + w);
		
		int vp_img_w = viewPort.getImageWidth();
		int vp_img_h = viewPort.getImageHeight();
		
		// System.out.println("vp_img_w = " + vp_img_w);
		// System.out.println("vp_img_h = " + vp_img_h);
		
		int n_tiles_hor = 2 + vp_img_w / w;
		int n_tiles_ver = 2 + vp_img_h / w;
		
		// System.out.println("n_tiles_hor = " + n_tiles_hor);
		// System.out.println("n_tiles_ver = " + n_tiles_ver);

		Point2D nor_vp_top_left = OpenStreetMapUtils.normalize(
				new Point2D.Double(
				viewPort.getAdjustedExtent().getMinX(),
				viewPort.getAdjustedExtent().getMaxY()));
		
		// double vp_max_y = viewPort.getAdjustedExtent().getMaxY();
		
		// System.out.println("vp_min_x = " + vp_min_x);
		// System.out.println("vp_max_y = " + vp_max_y);

		// Point2D vp_top_left = new Point2D.Double(vp_min_x, vp_max_y);

		OpenStreetMapTile first_tile =
			OpenStreetMapUtils.getTileFor(nor_vp_top_left, d, layerType);
		Point2D fi_tile_top_left = OpenStreetMapUtils.normalize(
				new Point2D.Double(
						first_tile.getExtent().getMinX(),
						first_tile.getExtent().getMaxY()));

		// System.out.println("first_tile, w = " + first_tile_w);
		
		// double first_ve_world = first_tile.getVerWorld();
		// System.out.println("first_tile, ve_world = " + first_ve_world);
		// double first_ho_world = first_tile.getHorWorld();
		// System.out.println("first_tile, ho_world = " + first_ho_world);
		
		double first_x_offset = fi_tile_top_left.getX() - nor_vp_top_left.getX();
		double first_y_offset = nor_vp_top_left.getY() - fi_tile_top_left.getY();
		
		first_x_offset = OpenStreetMapUtils.normalizeOffset(first_x_offset);
		first_y_offset = OpenStreetMapUtils.normalizeOffset(first_y_offset);

		// System.out.println("first_tile, first_x_offset = " + first_x_offset);
		// System.out.println("first_tile, first_y_offset = " + first_y_offset);

		int first_x_offset_pix = viewPort.fromMapDistance(first_x_offset);
		int first_y_offset_pix = viewPort.fromMapDistance(first_y_offset);
		
		// System.out.println("first_tile, first_x_offset_pix = " + first_x_offset_pix);
		// System.out.println("first_tile, first_y_offset_pix = " + first_y_offset_pix);

		int screen_x_item = 0;
		int screen_y_item = 0;

		OpenStreetMapTile top_tile = first_tile;
		
		for (screen_x_item = 0; screen_x_item < n_tiles_hor; screen_x_item++) {
			
			if (screen_x_item > 0) top_tile = top_tile.getTileToTheRight();
			OpenStreetMapTile item_tile = (OpenStreetMapTile) top_tile.clone();

			for (screen_y_item = 0; screen_y_item < n_tiles_ver; screen_y_item++) {
				
				if (screen_y_item > 0) item_tile = item_tile.getTileUnder();
				
				URL u = item_tile.getURL();
				
				// System.out.println("TILE URL = " + u.toString());

				File img_file = null;
				try {
					img_file = (File) Downloader.downloadObjectCancel(
							u, "png", DownloadTask.FILE, "osm",
							item_tile.getCacheFilePath(), false);
					
				} catch (DownloadException e) {

					logger.error("While downloading: " + e.getMessage());
					g.setFont(Utils.TILE_PROBLEM_FONT);
					if (e.getMessage().indexOf("403") != -1) {
						g.setColor(Utils.TILE_PROBLEM_403_COLOR);
						g.drawString(Utils.TILE_PROBLEM_MSG_403,
								Utils.TILE_PROBLEM_X_OFFSET + screen_x_item * w + first_x_offset_pix,
								Utils.TILE_PROBLEM_Y_OFFSET + screen_y_item * w + first_y_offset_pix);
					} else {
						logger.error("While downloading: " + e.getMessage());
						g.setColor(Utils.TILE_PROBLEM_COLOR);
						g.drawString(Utils.TILE_PROBLEM_MSG_ERROR,
								Utils.TILE_PROBLEM_X_OFFSET + screen_x_item * w + first_x_offset_pix,
								Utils.TILE_PROBLEM_Y_OFFSET + screen_y_item * w + first_y_offset_pix);
					}
					continue;
				}
				
				logger.debug("Instantiate Image...");
				
				Image img = ResourceReader.getImage(img_file);
				
				// /*
				if ((img == null) || (img.getWidth(null) == -1)) {
					g.setFont(Utils.TILE_PROBLEM_FONT);
					g.setColor(Utils.TILE_PROBLEM_COLOR);
					g.drawString(Utils.TILE_PROBLEM_MSG_NODATA,
							Utils.TILE_PROBLEM_X_OFFSET + screen_x_item * w + first_x_offset_pix,
							Utils.TILE_PROBLEM_Y_OFFSET + screen_y_item * w + first_y_offset_pix);
					continue;
				}
				// */
				
				// logger.debug("Instantiate buffered Image...");

				if (Utils.USING_PDA) {
					img = ResourceReader.getScaled(img, w, w);
				} else {
					BufferedImage bim = Utils.createBufferedImage(256, 256);
					bim.getGraphics().drawImage(img, 0, 0, null);
					img = ResourceReader.getScaled(bim, w, w);
				}

				g.drawImage(
						img,
						screen_x_item * w + first_x_offset_pix,
						screen_y_item * w + first_y_offset_pix,
						null);
				
				// resize_img = null;
				img = null;
			}
		}
		
		System.gc();
	}

	public int getRasterDataType() {
		// TODO Auto-generated method stub
		return 0;
	}

	public Object getData(int x, int y, int band) {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] getPixel(double wcx, double wcy) {
		// TODO Auto-generated method stub
		return null;
	}

	public String getName() {
		// TODO Auto-generated method stub
		return "OpenStreetMap Raster Driver";
	}

}
