package es.prodevelop.gvsig.mobile.fmap.driver.raster.tile;

import java.awt.geom.Rectangle2D;
import java.net.MalformedURLException;
import java.net.URL;

public class QuadTreeTile extends RemoteTile {
	
	private String endUrlSection = null;
	private String baseUrl = "";
	
	private String quadTreeNWTag = "";
	private String quadTreeNETag = "";
	private String quadTreeSWTag = "";
	private String quadTreeSETag = "";
	
	private int[] quadPath;
	
	public QuadTreeTile(
			String base_url,
			String nw,
			String ne,
			String sw,
			String se,
			String endSection) {

		quadPath = new int[1];
		quadPath[0] = 0;
		
		baseUrl = base_url;

		quadTreeNWTag = nw;
		quadTreeNETag = ne;
		quadTreeSWTag = sw;
		quadTreeSETag = se;
		
		endUrlSection = endSection;
	}

	public URL getURL() {
		String u = baseUrl;
		for (int i=0; i<quadPath.length; i++) u = u + letter(quadPath[i]);
		URL resp = null;
		try {
			resp = new URL(u);
		} catch (MalformedURLException e) {
			logger.error("Bad url: " + u);
		}
		return resp;
	}
	
	private String letter(int i) {
		
		if (i == 0) return quadTreeNWTag;
		if (i == 1) return quadTreeNETag;
		if (i == 2) return quadTreeSETag;
		if (i == 3) return quadTreeSWTag;
		return "";
	}
	

	public RemoteTile getParentTile() {
		if (quadPath.length == 1) return this;
		int[] new_path = new int[quadPath.length - 1];
		for (int i=0; i<(quadPath.length - 1); i++) new_path[i] = quadPath[i];
		return new 	QuadTreeTile(
				new_path,
				baseUrl,
				quadTreeNWTag,
				quadTreeNETag,
				quadTreeSWTag,
				quadTreeSETag,
				endUrlSection);
	}

	public RemoteTile getTileToTheEast() {
		if (quadPath.length == 1) return this;
		int lastq = quadPath[quadPath.length - 1];
		
		if (lastq == 0) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			parent.goToQ(1);
			return parent;
		}
		
		if (lastq == 3) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			parent.goToQ(2);
			return parent;
		}
		
		if (lastq == 1) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			QuadTreeTile parent_right = (QuadTreeTile) parent.getTileToTheEast();
			parent_right.goToQ(0);
			return parent_right;
		}
		
		if (lastq == 2) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			QuadTreeTile parent_right = (QuadTreeTile) parent.getTileToTheEast();
			parent_right.goToQ(3);
			return parent_right;
		}
		
		logger.error("Bad path item: " + lastq);
		return this;
	}

	public RemoteTile getTileToTheSouth() {
		if (quadPath.length == 1) return this;
		int lastq = quadPath[quadPath.length - 1];
		
		if (lastq == 2) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			parent.goToQ(1);
			return parent;
		}
		
		if (lastq == 3) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			parent.goToQ(0);
			return parent;
		}
		
		if (lastq == 0) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			QuadTreeTile parent_under = (QuadTreeTile) parent.getTileToTheSouth();
			parent_under.goToQ(3);
			return parent_under;
		}
		
		if (lastq == 1) {
			QuadTreeTile parent = (QuadTreeTile) getParentTile();
			QuadTreeTile parent_under = (QuadTreeTile) parent.getTileToTheSouth();
			parent_under.goToQ(2);
			return parent_under;
		}
		
		logger.error("Bad path item: " + lastq);
		return this;
	}

	public Object clone() {
		return new 	QuadTreeTile(
				quadPath,
				baseUrl,
				quadTreeNWTag,
				quadTreeNETag,
				quadTreeSWTag,
				quadTreeSETag,
				endUrlSection);
	}
	
	public int[] getQuadPath() {
		return quadPath;
	}
	
	public void goToQ(int q) {
		
		if ((q < 0) || (q > 3)) return;
		
		int[] aux = new int[quadPath.length + 1];
		for (int i=0; i<quadPath.length; i++) aux[i] = quadPath[i];
		aux[quadPath.length] = q;
		quadPath = new int[aux.length];
		for (int i=0; i<aux.length; i++) quadPath[i] = aux[i];
		
		double new_x = extent.getMinX();
		double new_y = extent.getMinY();
		double new_w = 0.5 * extent.getWidth();
		if (q == 1) { new_x = extent.getCenterX(); }
		if (q == 2) { new_x = extent.getCenterX(); new_y = extent.getCenterY(); }
		if (q == 3) { new_y = extent.getCenterY(); }
		extent = new Rectangle2D.Double(new_x, new_y, new_w, new_w);
		
	}
	
	private QuadTreeTile(
			int[] path,
			String base_url,
			String nw,
			String ne,
			String sw,
			String se,
			String end_sect) {
		
		
		extent = new Rectangle2D.Double(
				- Util.SEMI_EQUATOR_LENGTH,
				- Util.SEMI_EQUATOR_LENGTH,
				2 * Util.SEMI_EQUATOR_LENGTH,
				2 * Util.SEMI_EQUATOR_LENGTH);
		
		quadPath = new int[quadPath.length];
		
		for (int i=0; i<quadPath.length; i++) {
			
			int q = quadPath[i];
			if ((q < 0) || (q > 3)) break;
			
			quadPath[i] = q;
			
			if (i == 0) continue;
			
			double new_x = extent.getMinX();
			double new_y = extent.getMinY();
			double new_w = 0.5 * extent.getWidth();
			if (q == 1) { new_x = extent.getCenterX(); }
			if (q == 2) { new_x = extent.getCenterX(); new_y = extent.getCenterY(); }
			if (q == 3) { new_y = extent.getCenterY(); }
			extent = new Rectangle2D.Double(new_x, new_y, new_w, new_w);
		}
		
		baseUrl = base_url;

		quadTreeNWTag = nw;
		quadTreeNETag = ne;
		quadTreeSWTag = sw;
		quadTreeSETag = se;
		
		endUrlSection = end_sect;
		
	}
	
	


}
