/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.vect.dbf;

import java.io.File;
import java.io.IOException;
import java.sql.Types;

import org.apache.log4j.Logger;

import com.hardcode.driverManager.Driver;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.IDataSourceListener;
import com.hardcode.gdbms.engine.data.SourceInfo;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.data.persistence.Memento;
import com.hardcode.gdbms.engine.data.persistence.MementoException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueCollection;
import com.hardcode.gdbms.engine.values.ValueFactory;

/**
 * This class implements a DataSource which is based on
 * a DBF file fully loaded into memory.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.driver.vect.dbf.DbaseFileNIO
 * 
 * @author jldominguez
 *
 */
public class DbfMemoryDataSource implements DataSource {
	
	private static Logger logger = Logger.getLogger(DbfMemoryDataSource.class);
	private File dbFile = null;
	private DbaseFileNIO dbFileAccess = null;
	
	private int[] fieldTypes = null;

	/**
	 * Constructor
	 * @param _dbFile the DBF file to read
	 */
	public DbfMemoryDataSource(File _dbFile) {
		dbFile = _dbFile;
		dbFileAccess = new DbaseFileNIO();
	}

	/**
	 * Initializes the dfata source (reads the file into memory)
	 */
	public void start() throws DriverException {
		
		try {

			dbFileAccess.open(dbFile);
			
			int fcnt = dbFileAccess.getFieldCount();
			fieldTypes = new int[fcnt];
			for (int i=0; i<fcnt; i++) {
				fieldTypes[i] = dbFileAccess.getFieldIntType(i);
			}
			
		} catch (IOException e) {
			logger.error("Error while opening db file " + dbFile.getAbsolutePath() + ": " + e.getMessage());
			throw new DriverException(e.getMessage());
		}
	}

	/**
	 * Stops the datasource. Tries to close the DBF file.
	 */
	public void stop() throws DriverException {
		try {
			dbFileAccess.close();
		} catch (IOException e) {
			logger.error("Error while closing db file " + dbFile.getAbsolutePath() + ": " + e.getMessage());
			throw new DriverException(e.getMessage());
		}
	}

	/**
	 * @return the data source generic name
	 */
	public String getName() {
		return "DBF Data Source";
	}

	/**
	 * @return an array representing the WHERE filter. Currently unused.  
	 */
	public long[] getWhereFilter() throws IOException {
		logger.warn("Method getWhereFilter() was called, returned new long[0]");
		return new long[0];
	}

	/**
	 * @return the associated datasource factory. Currently unused.
	 */
	public DataSourceFactory getDataSourceFactory() {
		logger.warn("Method getDataSourceFactory() was called, returned NULL");
		return null;
	}

	/**
	 * @return the memento. Currently unused.
	 */
	public Memento getMemento() throws MementoException {
		logger.warn("Method getMemento() was called, returned NULL");
		return null;
	}

	/**
	 * Sets the datasource factory. Currently  unused.
	 * 
	 * @param arg0 the new data source factory
	 */
	public void setDataSourceFactory(DataSourceFactory arg0) {
		logger.warn("Method setDataSourceFactory(...) was called, nothing done.");
	}

	/**
	 * Sets the source info. Currently unused.
	 * @param arg0 the new source info
	 */
	public void setSourceInfo(SourceInfo arg0) {
		logger.warn("Method setSourceInfo(...) was called, nothing done.");
	}

	/**
	 * @return the source info. Currently unused.
	 */
	public SourceInfo getSourceInfo() {
		logger.warn("Method getSourceInfo() was called, returned NULL");
		return null;
	}

	/**
	 * @return a string representing the datasource. Unused. Simply redirects to
	 * <code>getName()</code>
	 */
	public String getAsString() throws DriverException {
		logger.warn("Method getAsString() was called, returned getName()");
		return getName();
	}

	/**
	 * Called when the layer is removed. Tries to prevent memory leaks.
	 *  
	 */
	public void remove() throws DriverException {
		logger.warn("Method remove() was called, nothing done.");
	}

	/**
	 * @return an array with the indices of the primary keys.
	 */
	public int[] getPrimaryKeys() throws DriverException {
		logger.warn("Method getPrimaryKeys() was called, returned all indices.");
		int size = dbFileAccess.getFieldCount();
		int[] resp = new int[size];
		for (int i=0; i<size; i++) resp[i] = i;
		return resp;
	}

	/**
	 * Gets the primary key value for a row
	 * @param arg0 the row of interest
	 * @return a collection representing the primary key
	 */
	public ValueCollection getPKValue(long arg0) throws DriverException {
		logger.warn("Method getPKValue(" + arg0 + ") was called, returned NULL");
		return null;
	}

	/**
	 * Gets the name of one of the fields that compose the PK.
	 * @param arg0 the index of the PK field of interest
	 * @return the name of the field of interest
	 */
	public String getPKName(int arg0) throws DriverException {
		logger.warn("Method getPKName(" + arg0 + ") was called, returned NULL");
		return null;
	}

	/**
	 * @return the names of the fields that compose the PK.
	 * 
	 */
	public String[] getPKNames() throws DriverException {
		logger.warn("Method getPKNames() was called, returned NULL");
		return null;
	}

	/**
	 * Gets the type of one of the fields that compose the PK.
	 * 
	 * @param arg0 the index of the field of interest
	 * @return the type of the field of interest
	 */
	public int getPKType(int arg0) throws DriverException {
		logger.warn("Method getPKType(" + arg0 + ") was called, returned getFieldType(...)");
		return dbFileAccess.getFieldType(arg0);
	}

	/**
	 * @return the cumber of fields that compose the PK.
	 */
	public int getPKCardinality() throws DriverException {
		return dbFileAccess.getFieldCount();
	}
	
	private Value getValueOfItsType(int fieldindex, int row) {
		
		Value resp = ValueFactory.createNullValue();
		
		int type = dbFileAccess.getFieldIntType(fieldindex);
		switch (type) {

		case Types.DOUBLE:
			resp = ValueFactory.createValue(
					dbFileAccess.getNumberFieldValue(
							row, fieldindex
							).doubleValue());
			break;
		case Types.VARCHAR:
			resp = ValueFactory.createValue(
					dbFileAccess.getStringFieldValue(
							row, fieldindex
							));
			break;
		case Types.DATE:
			resp = ValueFactory.createValue(
					dbFileAccess.getStringFieldValue(
							row, fieldindex
					));
			break;
		case Types.BOOLEAN:
			resp = ValueFactory.createValue(
					dbFileAccess.getBooleanFieldValue(
							row, fieldindex
							));
			break;
		}
		return resp;
	}

	/**
	 * Gets a row of this data source.
	 * @param rowind the index of the row of interest
	 * @return the row as an array of values
	 */
	public Value[] getRow(long rowind) throws DriverException {
		
		int size = dbFileAccess.getFieldCount();
		Value[] resp = new Value[size];
		for (int i=0; i<size; i++) {
			resp[i] = getValueOfItsType(i, (int) rowind);
		}
		return resp;
	}

	/**
	 * @return an array with all the field names
	 */
	public String[] getFieldNames() throws DriverException {
		
		int size = dbFileAccess.getFieldCount();
		String[] fnames = new String[size];
		for (int i=0; i<size; i++) {
			fnames[i] = dbFileAccess.getFieldName(i);
		}
		return fnames;
	}

	/**
	 * Gets the idnex of a field, provided its name.
	 * @param arg0 the name of the field of interest
	 * @return the index of the field of interest
	 */
	public int getFieldIndexByName(String arg0) throws DriverException {
		int size = dbFileAccess.getFieldCount();
		for (int i=0; i<size; i++) {
			if (arg0.compareToIgnoreCase(dbFileAccess.getFieldName(i)) == 0) return i;
		}
		logger.error("Field name not found: " + arg0);
		return -1;
	}

	/**
	 * Gets the data ware associated with the given index.
	 * Currently unused.
	 * 
	 * @param arg0 the index of interest
	 * @return the associated data ware
	 */
	public DataWare getDataWare(int arg0) throws DriverException {
		logger.warn("Method getDataWare(" + arg0 + ") returned NULL.");
		return null;
	}

	/**
	 * Gets whether the referenced field is virtual or not.
	 * 
	 * @param arg0 the index of the field of interest
	 * @return whether it is a virtual field.
	 * 
	 */
	public boolean isVirtualField(int arg0) throws DriverException {
		// logger.warn("Method isVirtualField(" + arg0 + ") returned FALSE.");
		return false;
	}

	/**
	 * @return the associated driver, currently unused.
	 */
	public Driver getDriver() {
		logger.warn("Method getDriver() returned NULL.");
		return null;
	}

	/**
	 * Reloads the data source (stops and starts)
	 */
	public void reload() throws DriverException, IOException {
		stop();
		dbFileAccess = new DbaseFileNIO(); 
		start();
	}

	/**
	 * Adds a datasource listener. Currently unused.
	 * @param arg0 the listener to be added.
	 */
	public void addDataSourceListener(IDataSourceListener arg0) {
		logger.warn("Method addDataSourceListener(), nothing done.");
	}

	/**
	 * Removes a datasource listener. Currently unused.
	 * @param arg0 the listener to be removed.
	 */
	public void removeDataSourceListener(IDataSourceListener arg0) {
		logger.warn("Method removeDataSourceListener(), nothing done.");
	}

	/**
	 * Gets a value from thsi data source.
	 * @param arg0 the row of interest
	 * @param arg1 the index of the field of interest
	 * 
	 * @return the value of the given row and field
	 */
	public Value getFieldValue(long arg0, int arg1) throws DriverException {
		return getValueOfItsType(arg1, (int) arg0);
	}

	/**
	 * @return the number of fields in this datasource
	 */
	public int getFieldCount() throws DriverException {
		return dbFileAccess.getFieldCount();
	}

	/**
	 * Gets the name of the field with the given index.
	 * @param arg0 the index of the field of interest
	 * @return the name of the field
	 */
	public String getFieldName(int arg0) throws DriverException {
		return dbFileAccess.getFieldName(arg0);
	}

	/**
	 * @return the number of rows in this data source
	 */
	public long getRowCount() throws DriverException {
		return dbFileAccess.getRecordCount();
	}

	/**
	 * Gets the type of the field with the given index.
	 * @param arg0 the index of the field of interest
	 * @return teh field type
	 */
	public int getFieldType(int arg0) throws DriverException {
		return dbFileAccess.getFieldIntType(arg0);
	}

	/**
	 * Gets the width of the field with the given index.
	 * @param arg0 the index of the field of interest
	 * @return teh field width
	 */
	public int getFieldWidth(int arg0) throws DriverException {
		return dbFileAccess.getFieldLength(arg0);
	}
	
	public File getFile() {
		return dbFile;
	}

}
