/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.vect.shp;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import es.prodevelop.gvsig.mobile.fmap.core.FMapFastGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.GeneralPathX;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.driver.ITableDefinition;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;

/**
 * SHP driver via JNI when the file does not have to be loaded into memory.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.driver.vect.shp.ShpReader
 * 
 * @author jldominguez
 */
public class DiskShpDriver extends MemoryShpDriver {
	
	private static Logger logger = Logger.getLogger(DiskShpDriver.class.getName());

	private long handler = 0;
	
	private int type;
	private int numReg;
	private Rectangle2D extent;
	// private File fileShp = null;
	
	private HashMap shapesBoxes;

	/**
	 * Closes the file
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#close()
	 */
	public void close() throws IOException {
		// ShpReader.closeShpFile(handler);
	}

	/**
	 * Opens the file with through JNI functions
	 * 
	 * @param f the file to be opened
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {
		fileShp = f;
		handler = ShpReader.openShpFile(f.getCanonicalPath(), false);
	}
	/**
	 * Gets the geometry for an index
	 *
	 * @param index index of the geometry of interest
	 *
	 * @return the geometry 
	 *
	 * @throws IOException 
	 */
	public IGeometry getShape(int index) throws IOException {

		double[] shp = ShpReader.getShapeWithBox(handler, index);
		
		if (shp == null) return null;
		addBoxToHash(index, shp[0], shp[1], shp[2], shp[3]);
		
		FMapFastGeometry resp = null;
		int _type = Math.round((float) shp[4]);
		int nparts = Math.round((float) shp[5]);
		int i;
		double[] coords;

		switch (_type) {

		case SHP.SHPT_POLYGON:
			
			resp = new FMapFastGeometry(2, true);
			for (i=0; i<nparts; i++) {
				coords = getPartSkipBox(shp, i, nparts, 2);
				resp.addPart(coords);
			}
			break;

			// -------------------------------------------------------------
			// -------------------------------------------------------------

		case SHP.SHPT_ARC:
			
			resp = new FMapFastGeometry(2, false);
			for (i=0; i<nparts; i++) {
					coords = getPartSkipBox(shp, i, nparts, 2);
					resp.addPart(coords);
			}
			break;

				// -------------------------------------------------------------
				// -------------------------------------------------------------
		
		case SHP.SHPT_ARCZ:
		case SHP.SHPT_ARCM:
			resp = new FMapFastGeometry(3, false);

				for (i = 0; i < nparts; i++) {
					coords = getPartSkipBox(shp, i, nparts, 3);
					resp.addPart(coords);
				}
				break;
				
				// -------------------------------------------------------------
				// -------------------------------------------------------------

		case SHP.SHPT_POLYGONZ:
		case SHP.SHPT_POLYGONM:
			
			resp = new FMapFastGeometry(3, true);

				for (i = 0; i < nparts; i++) {
					coords = getPartSkipBox(shp, i, nparts, 3);
					resp.addPart(coords);
				}
				break;
			
				// -------------------------------------------------------------
				// -------------------------------------------------------------

		case SHP.SHPT_POINT:
			resp = new FMapFastGeometry(2, false);
				coords = getPartSkipBox(shp, 0, 1, 2);
				resp.addPart(coords);
				break;

		case SHP.SHPT_MULTIPOINT:
			resp = new FMapFastGeometry(2, false);

				for (i=0; i<nparts; i++) {
					coords = getPartSkipBox(shp, i, nparts, 2);
					resp.addPart(coords);
				}
				break;
				
				// -------------------------------------------------------------
				// -------------------------------------------------------------
				
		case SHP.SHPT_POINTZ:
		case SHP.SHPT_POINTM:
			resp = new FMapFastGeometry(3, false);
				coords = getPartSkipBox(shp, 0, 1, 3);
				resp.addPart(coords);
				break;
				
				// -------------------------------------------------------------
				// -------------------------------------------------------------

		case SHP.SHPT_MULTIPOINTZ:
		case SHP.SHPT_MULTIPOINTM:
			resp = new FMapFastGeometry(3, false);

				for (i=0; i<nparts; i++) {
					coords = getPartSkipBox(shp, i, nparts, 3);
					resp.addPart(coords);
			}
				break;
		}

		try {
			return fastGeometryToFMapGeometry(resp, _type);
		} catch (Exception ex) {
			logger.debug("fastGeometryToFMapGeometry(resp, _type) DIO EXCEPCION, NULL");
			return null;
		}
	}

	private void addBoxToHash(int index, double minx, double miny, double w, double h) {
		Rectangle2D rect = new Rectangle2D.Double(minx, miny, w, h);
		shapesBoxes.put(new Integer(index), rect);
	}

	/**
	 * @return the number of geometries in this driver
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShapeCount()
	 */
	public int getShapeCount() {
		return numReg;
	}

	/**
	 * @return the shape type of the geometries of this driver
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		int auxType = 0;

		switch (type) {
			case SHP.SHPT_POINT:
			case SHP.SHPT_POINTZ:
			case SHP.SHPT_POINTM:
				auxType = auxType | FShape.POINT;
				break;

			case SHP.SHPT_ARC:
			case SHP.SHPT_ARCZ:
			case SHP.SHPT_ARCM:
				auxType = auxType | FShape.LINE;
				break;

			case SHP.SHPT_POLYGON:
			case SHP.SHPT_POLYGONZ:
			case SHP.SHPT_POLYGONM:
				auxType = auxType | FShape.POLYGON;
				break;

			case SHP.SHPT_MULTIPOINT:
			case SHP.SHPT_MULTIPOINTZ:
			case SHP.SHPT_MULTIPOINTM:
				auxType = auxType | FShape.MULTIPOINT;

				break;
		}

		return auxType;
	}


	/**
	 * Initializes the driver
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#initialize()
	 */
	public void initialize() throws IOException {
		
		double[] meta = ShpReader.getShpMetadata(handler);
		
		shapesBoxes = new HashMap();
		// [tipo, ext_min_x, ext_min_y, ext_w, ext_h, numReg]
		
		if (meta == null) {
			// error
			logger.error(Utils.time() + "No se pudo obtener metadatos de archivo SHP: " + fileShp.getName());
			type = SHP.SHPT_NULL;
			extent = new Rectangle2D.Double(0, 0, 1, 1);
			numReg = 0;
		} else {
			// ok
			type = Math.round((float) meta[0]);
			extent = new Rectangle2D.Double(meta[1], meta[2], meta[3], meta[4]);
			numReg = Math.round((float) meta[5]);
		}
	}


	/**
	 * Gets the extent of the geometries of this driver
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws IOException {
		return extent;
	}


	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#accept(java.io.File)
	 */
	public boolean accept(File f) {
		return (f.getName().toUpperCase().endsWith("SHP"));
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getDataFile(java.io.File)
	 */
	public File getDataFile(File f) {
		String str = f.getAbsolutePath();
		return new File(str.substring(0, str.length() - 3) + "dbf");
	}

	/**
	 * Gets the SHX associated with the given SHP file
	 * 
	 * @param f a file objects that points to a SHP file
	 * @return the associated SHX file
	 *   
	 */
    public File getShxFile(File f) {
        String str = f.getAbsolutePath();

        return new File(str.substring(0, str.length() - 3) + "shx");
    }


	/**
	 * Gets the shape type of the geometry of interest. Currently redirects to
	 * <code>getShapeType()</code> 
	 * 
	 * @param index the index of the geometry of interest
	 * @return the type of the geometry
	 */
	public int getShapeType(int index) {
		return getShapeType();
	}

    /**
     * 
     * @return the main SHP file
     */
	public File getFile() {
		return fileShp;
	}

	/**
	 * Reloads this driver (reopens and initializes)
	 */
	public void reload() throws IOException {
		open(fileShp);
		initialize();
	}

	/**
	 * Initializes the driver with the given table definition. Currently unused.
	 * 
	 * @param layerDef the table definitiion to be used
	 */
	public void initialize(ITableDefinition layerDef) {
	}

	/**
	 * @return Whether the driver is writable. Currently unused. 
	 *  
	 */
	public boolean isWritable() {
		return fileShp.canWrite();
	}

	/**
	 * @return whether the associated table can be altered. Returns false. Currently
	 * unused.
	 */
	public boolean canAlterTable() {
		return false;
	}

	/**
	 * Currently unused.
	 */
	public boolean isWriteAll() {
		return true;
	}

	/**
	 * 
	 * @return the main SHP file handler
	 */
	public long getHandler() {
		return handler;
	}

	/**
	 * Sets the main file handler
	 * @param h the new handler
	 */
	public void setHandler(long h) {
		handler = h;
	}

	/**
	 * Gets the extent of one geometry of this driver.
	 * 
	 * @param index the index of the geometry of interest
	 * @return the bounding box of the geometry
	 */
	public Rectangle2D getShapeBounds(int index) throws IOException {
		
		Integer _index = new Integer(index);
		Rectangle2D rect = (Rectangle2D) shapesBoxes.get(_index);
		if (rect == null) {
			double[] bounds = ShpReader.getShapeBounds(handler, index);
			rect = new Rectangle2D.Double(bounds[0], bounds[1], bounds[2], bounds[3]);
			shapesBoxes.put(_index, rect);
		}
		return rect;
	}

	private double[] getPartSkipBox(double[] shp, int part, int n_parts, int dim) {
		
		int part_start = dim * Math.round((float) shp[part + 6]);
		int part_size = dim * Math.round((float) shp[part + 7]) - part_start;
		
		double[] resp = new double[part_size];
		
		for (int i=0; i<part_size; i++) {
			resp[i] = shp[7 + n_parts + part_start + i];
		}
		return resp;
	}
	
	private IGeometry fastGeometryToFMapGeometry(FMapFastGeometry fg, int type) throws Exception {
		
		if (fg == null) return null;
		IGeometry ig = null;
		
		GeneralPathX elShape;
		int numParts, i, v;
		double[] part;
		double[] zc;
		ArrayList zcoords = new ArrayList();
		double[] pointx;
		double[] pointy;
		double[] pointz;
		
		switch (type) {

		case SHP.SHPT_POLYGON:
			numParts = fg.getNumberOfParts();
			elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, fg.getPart(0).length);
			
			for (i = 0; i < numParts; i++) {
				part = fg.getPart(i);
				int part_size = part.length / 2;
				elShape.moveTo(part[0], part[1]);
				for (v = 1; v < part_size; v++) {
					elShape.lineTo(part[2 * v], part[2 * v + 1]);
				}
			}
			zc = getDoubleArray(zcoords);
			ig = ShapeFactory.createPolygon2D(elShape);
			break;

			// -------------------------------------------------------------
			// -------------------------------------------------------------

		case SHP.SHPT_ARC:
			numParts = fg.getNumberOfParts();
			elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, fg.getPart(0).length);
			
			for (i = 0; i < numParts; i++) {
				part = fg.getPart(i);
				int part_size = part.length / 2;
				elShape.moveTo(part[0], part[1]);
				for (v = 1; v < part_size; v++) {
					elShape.lineTo(part[2 * v], part[2 * v + 1]);
				}
			}
			ig = ShapeFactory.createPolyline2D(elShape);
			break;

				// -------------------------------------------------------------
				// -------------------------------------------------------------
		
		case SHP.SHPT_ARCZ:
		case SHP.SHPT_ARCM:
			numParts = fg.getNumberOfParts();
			elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, fg.getPart(0).length);
			
			for (i = 0; i < numParts; i++) {
				part = fg.getPart(i);
				int part_size = part.length / 3;
				elShape.moveTo(part[0], part[1]);
				zcoords.add(new Double(part[2]));
				for (v = 1; v < part_size; v++) {
					elShape.lineTo(part[3 * v], part[3 * v + 1]);
					zcoords.add(new Double(part[3 * v + 2]));
				}
			}
			zc = getDoubleArray(zcoords);
			ig = ShapeFactory.createPolyline3D(elShape, zc);
			break;
				
				// -------------------------------------------------------------
				// -------------------------------------------------------------

		case SHP.SHPT_POLYGONZ:
		case SHP.SHPT_POLYGONM:
			numParts = fg.getNumberOfParts();
			elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, fg.getPart(0).length);
			
			for (i = 0; i < numParts; i++) {
				part = fg.getPart(i);
				int part_size = part.length / 3;
				elShape.moveTo(part[0], part[1]);
				zcoords.add(new Double(part[2]));
				for (v = 1; v < part_size; v++) {
					elShape.lineTo(part[3 * v], part[3 * v + 1]);
					zcoords.add(new Double(part[3 * v + 2]));
				}
			}
			zc = getDoubleArray(zcoords);
			ig = ShapeFactory.createPolygon3D(elShape, zc);
			break;
			
				// -------------------------------------------------------------
				// -------------------------------------------------------------

		case SHP.SHPT_POINT:
			part = fg.getPart(0);
			ig = ShapeFactory.createPoint2D(part[0], part[1]);
			break;
			
			// -------------------------------------------------------------
			// -------------------------------------------------------------

		case SHP.SHPT_MULTIPOINT:
			numParts = fg.getNumberOfParts();
			pointx = new double[numParts];
			pointy = new double[numParts];
			
			for (i = 0; i < numParts; i++) {
				part = fg.getPart(i);
				pointx[i] = part[0];
				pointy[i] = part[1];
			}
			ig = ShapeFactory.createMultipoint2D(pointx, pointy);
			break;
				
				// -------------------------------------------------------------
				// -------------------------------------------------------------
				
		case SHP.SHPT_POINTZ:
		case SHP.SHPT_POINTM:
			part = fg.getPart(0);
			ig = ShapeFactory.createPoint3D(part[0], part[1], part[2]);
			break;
				
				// -------------------------------------------------------------
				// -------------------------------------------------------------

		case SHP.SHPT_MULTIPOINTZ:
		case SHP.SHPT_MULTIPOINTM:
			numParts = fg.getNumberOfParts();
			pointx = new double[numParts];
			pointy = new double[numParts];
			pointz = new double[numParts];
			
			for (i = 0; i < numParts; i++) {
				part = fg.getPart(i);
				pointx[i] = part[0];
				pointy[i] = part[1];
				pointz[i] = part[2];
			}
			ig = ShapeFactory.createMultipoint3D(pointx, pointy, pointz);
			break;
		}

		return ig;
	}

	private double[] getDoubleArray(ArrayList zcoords) {
		int size = zcoords.size();
		double[] resp = new double[size];
		for (int i=0; i<size; i++) resp[i] = ((Double) zcoords.get(i)).doubleValue();
		return resp;
	}
	
	/**
	 * Gets the indices of the geometries intersecting the given rectangle
	 * 
	 * @param rect the rectangle of interest
	 * @param max the maximum number of indices to be returned
	 */
	public int[] getIntersectingIndices(Rectangle2D rect, int max) throws IOException {
		
		ArrayList _resp = new ArrayList();
		for (int i=0; i<numReg; i++) {
			Integer ind_obj = new Integer(i);
			Rectangle2D item_bb = (Rectangle2D) shapesBoxes.get(ind_obj); 
			if (Utils.rectanglesItersect(item_bb, rect)) {
				IGeometry ig = getShape(i);
				if (ig.getGeometryType() == FShape.POINT || ig.intersects(rect)) {
					_resp.add(new Integer(i));
					if (_resp.size() == max) {
						break;
					}
				}
			}
		}
		int size = _resp.size();
		int[] resp = new int[size];
		for (int i=0; i<size; i++) resp[i] = ((Integer) _resp.get(i)).intValue();
		return resp;
	}
	

}
