/* gvSIG. Sistema de Informain Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.driver.vect.shp;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.index.strtree.STRtree;

import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.GeneralPathX;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.driver.BoundedShapes;
import es.prodevelop.gvsig.mobile.fmap.driver.ExternalData;
import es.prodevelop.gvsig.mobile.fmap.driver.ITableDefinition;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialFileDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.dbf.DbfMemoryDataSource;
import es.prodevelop.gvsig.mobile.fmap.edition.IWriteable;
import es.prodevelop.gvsig.mobile.fmap.edition.IWriter;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialFileAdapter;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.bytebuffer.FalseByteBuffer;

/**
 * SHP driver for a file that is completely loaded into memory.
 * 
 * @see es.prodevelop.gvsig.mobile.fmap.util.bytebuffer.FalseByteBuffer
 * 
 * @author jldominguez
 */
public class MemoryShpDriver implements VectorialFileDriver, BoundedShapes,
	ExternalData, IWriteable {
	
	private static Logger logger = Logger.getLogger(MemoryShpDriver.class.getName());
	private static String tempDirectoryPath = System.getProperty("java.io.tmpdir");
	protected File fileShp;
	private FalseByteBuffer bb;
	private FileInputStream fin;
	private int type;
	// private long[] m_posShapes;
	private int numReg = 0;
	private Rectangle2D extent;

    // private File shxFile;
    private FalseByteBuffer bbShx;
    private FileInputStream finShx;
    
    // point bound box tol:
    public static final double POINT_TOL = 0.000001;
    
//    private HashMap index = new HashMap();
//    private STRtree spIndex = new STRtree();
    
    /**
     * @return gets the fiel path
     */
    public String getFilePath(){
    	return fileShp.getAbsolutePath();
    }
    
	/**
	 * Closes the file an dtries to preven memory leaks
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#close()
	 */
	public void close() throws IOException {
		
		try {
			fin.close();
		} catch (IOException e1) {
			logger.error("While closing file input stream: " + e1.getMessage());
		}
		bb = null;
		bbShx = null;
	}

	/**
	 * Opens the file with using standard Java I/O system and loads it completely into memory
	 * 
	 * @param f the file to be opened
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#open(java.io.File)
	 */
	public void open(File f) throws IOException {

		fileShp = f;

		fin = new FileInputStream(f);

		// Open the file and then get a channel from the stream
		// channel = fin.getChannel();
		// channel = FileChannelImpl.open(fin.getFD(), true, false, fin);

		// long size = channel.size();

		// Get the file's size and then map it into memory
		// bb = channel.map(FileChannel.MapMode.READ_ONLY, 0, size);
        // bb = ByteBuffer.allocate((int) f.length());
        bb = Utils.loadFileInputStream(fin, (int) f.length());
        // new BigByteBuffer2(channel, FileChannel.MapMode.READ_ONLY);

        File shxf = getShxFile(f);
        finShx = new FileInputStream(shxf);

        // Open the file and then get a channel from the stream
        // channelShx = finShx.getChannel();
        // channelShx = FileChannelImpl.open(finShx.getFD(), true, false, finShx);

        long sizeShx =  shxf.length();

        // Get the file's size and then map it into memory
        // bb = channel.map(FileChannel.MapMode.READ_ONLY, 0, size);
        // bbShx = channelShx.map(FileChannel.MapMode.READ_ONLY, 0, sizeShx);

        // bbShx = ByteBuffer.allocate((int) sizeShx); //2(channelShx, FileChannel.MapMode.READ_ONLY);
        
        bbShx = Utils.loadFileInputStream(finShx, (int) shxf.length());
        bbShx.setByteOrder(ByteOrder.BIG_ENDIAN);
        
	}

	/**
	 * 
	 *
	 * @param index 
	 *
	 * @return 
	 *
	 * @throws IOException
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShape(int)
	 */

	/* public FShape getShapeByID(int ID) {
	   Point2D.Double p = new Point2D.Double();
	   Point2D.Double pAnt = null;
	   int numParts;
	   int numPoints;
	   int i;
	   int j;
	   int numReg;
	   int numeroPuntos;
	   int hasta;
	   int desde;
	   Rectangle2D.Double BoundingBox = new Rectangle2D.Double();

	           SHPShape shapeShp=null;
	   FShape resulShape = null;
	   try {
	       bb.position(m_posShapes[ID]);
	       bb.order(ByteOrder.LITTLE_ENDIAN);
	       int tipoShape = bb.getInt();
	       m_shapeType = tipoShape;
	       // retrieve that shape.
	       // tempRecord.setShape(readShape(tempShapeType, tempContentLength, in));
	       if (tipoShape == FConstant.SHAPE_TYPE_POINT) {
	           p = readPoint(bb);
	           resulShape = new FShape(new FPoint(p.getX(),p.getY()),FConstant.SHAPE_TYPE_POINT);
	           //Comprobacin punto.
	           //System.err.println("p.x = "+p.x);
	           //System.err.println("p.y = "+p.y);

	       } else if ((tipoShape == FConstant.SHAPE_TYPE_POLYLINE) ||
	               (tipoShape == FConstant.SHAPE_TYPE_POLYGON)) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numParts = bb.getInt();
	           numPoints = bb.getInt();
	           // part indexes.
	           // Geometry geom = GeometryFactory.toGeometryArray();
	           GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
	                   numPoints);
	           int[] tempParts = new int[numParts];
	           for (i = 0; i < numParts; i++) {
	               tempParts[i] = bb.getInt();
	           }
	           j = 0;
	           ///Line2D.Double line2D;
	           FPoint[] points=new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	                   p=readPoint(bb);
	               points[i] = new FPoint(p.x,p.y);
	               // System.out.println("x= " + p.x + " y=" + p.y);
	               // System.out.println("x= " + (float) p.x + " y=" + (float) p.y);
	               if (i == tempParts[j]) {
	                   elShape.moveTo(p.x, p.y);
	                   if (j < (numParts - 1)) {
	                       j++;
	                   }
	               } else {
	                   elShape.lineTo(p.x, p.y);
	               }
	           }
	           //FGeometry pol=new FPolyLine(points,tempParts,BoundingBox);

	           resulShape = new FShape(tipoShape,elShape);
	       } else if (tipoShape == FConstant.SHAPE_TYPE_MULTIPOINT) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numPoints = bb.getInt();
	           FPoint[] tempPoints = new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	                   Point2D p2=readPoint(bb);
	               tempPoints[i] = new FPoint(p2.getX(),p2.getY(),0);
	           }
	           FMultiPoint multipoint = new FMultiPoint(tempPoints,BoundingBox);
	           resulShape = new FShape(multipoint,tipoShape);
	       } else if (tipoShape == FConstant.SHAPE_TYPE_POINTZ) {
	           FPoint p3d = new FPoint();
	           p3d.read(bb);
	           resulShape = new FShape(p3d,tipoShape);
	       } else if ((tipoShape == FConstant.SHAPE_TYPE_POLYLINEZ) ||
	               (tipoShape == FConstant.SHAPE_TYPE_POLYGONZ)) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numParts = bb.getInt();
	           numPoints = bb.getInt();
	           // part indexes.
	           // Geometry geom = GeometryFactory.toGeometryArray();

	           GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
	                   numPoints);
	           int[] tempParts = new int[numParts];
	           for (i = 0; i < numParts; i++) {
	               tempParts[i] = bb.getInt();
	           }
	           j = 0;
	          //Line2D.Double line2D;
	           FPoint[] points=new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	               p = readPoint(bb);
	               points[i]=new FPoint(p.x,p.y);

	               if (i == tempParts[j]) {
	                   elShape.moveTo(p.x, p.y);
	                   if (j < (numParts - 1)) {
	                       j++;
	                   }
	               } else {
	                   elShape.lineTo(p.x, p.y);
	               }

	           }

	           double[] boxZ = new double[2];
	           boxZ[0] = bb.getDouble();
	           boxZ[1] = bb.getDouble();
	           double[] pZ = new double[numPoints];
	           for (i = 0; i < numPoints; i++) {
	               pZ[i] = bb.getDouble();
	           }
	           //FGeometry pol=new FPolyLine(points,tempParts,BoundingBox);
	           resulShape = new FShape(tipoShape, elShape, pZ);
	       } else if (tipoShape == FConstant.SHAPE_TYPE_MULTIPOINTZ) {
	           // BoundingBox
	           BoundingBox = readRectangle(bb);
	           numPoints = bb.getInt();
	           FPoint[] tempPoints3D = new FPoint[numPoints];
	           for (i = 0; i < numPoints; i++) {
	               tempPoints3D[i] = new FPoint();
	               tempPoints3D[i].read(bb);
	           }
	           FMultiPoint multipoint3D = new FMultiPoint(tempPoints3D,BoundingBox);
	           resulShape = new FShape(multipoint3D,tipoShape);
	       }
	   } catch (Exception e) {
	       System.err.println("Fallo en getShapeByID. ID=" + ID +
	           " m_posShapes[ID]=" + m_posShapes[ID]);
	       System.err.println("getShapeByID: " + e.getMessage());
	       e.printStackTrace();
	   }
	   return resulShape;
	   }
	 */
	
	/**
	 * Gets the geometry for an index
	 *
	 * @param index index of the geometry of interest
	 *
	 * @return the geometry 
	 *
	 * @throws IOException 
	 */
	public IGeometry getShape(int index) throws IOException {
		
		Point2D.Double p = new Point2D.Double();
		int numParts;
		int numPoints;
		int i;
		int j;
		int shapeType;

		//Rectangle2D.Double BoundingBox;
        // if (m_posShapes[index] == 0)

		// bb.position(m_posShapes[index]);
        bb.position(getPositionForRecord(index));
		bb.setByteOrder(ByteOrder.LITTLE_ENDIAN);

		///bb.position(bb.position()+4);
		shapeType = bb.getInt();
		//el shape tal con tema tal y nmro tal es null
		if (shapeType==SHP.SHPT_NULL){
			logger.debug("El shape ="+index+ " de la capa ="+this.toString()+" es null");
			return null;
		}

		// retrieve that shape.
		// tempRecord.setShape(readShape(tempShapeType, tempContentLength, in));
		switch (type) {
		
		case SHP.SHPT_POINT:
				p = readPoint(bb);

				return ShapeFactory.createPoint2D(p.getX(), p.getY());

		case SHP.SHPT_ARC:

				//BoundingBox = readRectangle(bb);
				//bb.getDouble();
				//bb.getDouble();
				//bb.getDouble();
				//bb.getDouble();
				bb.position(bb.getPosition() + 32);
				numParts = bb.getInt();
				numPoints = bb.getInt();

				// part indexes.
				// Geometry geom = GeometryFactory.toGeometryArray();
				GeneralPathX elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD,
						numPoints);

				int[] tempParts = new int[numParts];

				for (i = 0; i < numParts; i++) {
					tempParts[i] = bb.getInt();
				}

				j = 0;

				for (i = 0; i < numPoints; i++) {
					p = readPoint(bb);

					if (i == tempParts[j]) {
						elShape.moveTo(p.x, p.y);

						if (j < (numParts - 1)) {
							j++;
						}
					} else {
						elShape.lineTo(p.x, p.y);
					}
				}

				return ShapeFactory.createPolyline2D(elShape);

			case SHP.SHPT_POLYGON:

				//	            	BoundingBox = readRectangle(bb);
				bb.getDouble();
				bb.getDouble();
				bb.getDouble();
				bb.getDouble();

				numParts = bb.getInt();

				numPoints = bb.getInt();

				// part indexes.
				// Geometry geom = GeometryFactory.toGeometryArray();
				elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, numPoints);

				tempParts = new int[numParts];

				for (i = 0; i < numParts; i++) {
					tempParts[i] = bb.getInt();
				}

				j = 0;

				for (i = 0; i < numPoints; i++) {
					p = readPoint(bb);

					if (i == tempParts[j]) {
						elShape.moveTo(p.x, p.y);

						if (j < (numParts - 1)) {
							j++;
						}
					} else {
						elShape.lineTo(p.x, p.y);
					}
				}

				return ShapeFactory.createPolygon2D(elShape);

			case SHP.SHPT_POINTZ:
			case SHP.SHPT_POINTM:

				double x = bb.getDouble();
				double y = bb.getDouble();
				double z = bb.getDouble();

				return ShapeFactory.createPoint3D(x, y, z);

			case SHP.SHPT_ARCZ:
			case SHP.SHPT_ARCM:
				bb.position(bb.getPosition() + 32);
				numParts = bb.getInt();
				numPoints = bb.getInt();
				elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, numPoints);
				tempParts = new int[numParts];

				for (i = 0; i < numParts; i++) {
					tempParts[i] = bb.getInt();
				}

				j = 0;

				for (i = 0; i < numPoints; i++) {
					p = readPoint(bb);

					if (i == tempParts[j]) {
						elShape.moveTo(p.x, p.y);

						if (j < (numParts - 1)) {
							j++;
						}
					} else {
						elShape.lineTo(p.x, p.y);
					}
				}

				double[] boxZ = new double[2];
				boxZ[0] = bb.getDouble();
				boxZ[1] = bb.getDouble();

				double[] pZ = new double[numPoints];

				for (i = 0; i < numPoints; i++) {
					pZ[i] = bb.getDouble();
				}

				return ShapeFactory.createPolyline3D(elShape, pZ);
				
			case SHP.SHPT_POLYGONZ:
			case SHP.SHPT_POLYGONM:
				
			bb.position(bb.getPosition() + 32);
			numParts = bb.getInt();
			numPoints = bb.getInt();
			elShape = new GeneralPathX(GeneralPathX.WIND_EVEN_ODD, numPoints);
			tempParts = new int[numParts];

			for (i = 0; i < numParts; i++) {
				tempParts[i] = bb.getInt();
			}

			j = 0;

			for (i = 0; i < numPoints; i++) {
				p = readPoint(bb);

				if (i == tempParts[j]) {
					elShape.moveTo(p.x, p.y);

					if (j < (numParts - 1)) {
						j++;
					}
				} else {
					elShape.lineTo(p.x, p.y);
				}
			}

			double[] boxpoZ = new double[2];
			boxpoZ[0] = bb.getDouble();
			boxpoZ[1] = bb.getDouble();

			double[] poZ = new double[numPoints];

			for (i = 0; i < numPoints; i++) {
				poZ[i] = bb.getDouble();
			}

			return ShapeFactory.createPolygon3D(elShape, poZ);

			case SHP.SHPT_MULTIPOINT:
				bb.position(bb.getPosition() + 32);
				numPoints = bb.getInt();

				double[] tempX = new double[numPoints];
				double[] tempY = new double[numPoints];

				for (i = 0; i < numPoints; i++) {
					tempX[i] = bb.getDouble();
					tempY[i] = bb.getDouble();
				}

				return ShapeFactory.createMultipoint2D(tempX, tempY);

			case SHP.SHPT_MULTIPOINTZ:
			case SHP.SHPT_MULTIPOINTM:
				bb.position(bb.getPosition() + 32);
				numPoints = bb.getInt();

				double[] temX = new double[numPoints];
				double[] temY = new double[numPoints];
				double[] temZ = new double[numPoints];

				for (i = 0; i < numPoints; i++) {
					temX[i] = bb.getDouble();
					temY[i] = bb.getDouble();
					//temZ[i] = bb.getDouble();
				}

				for (i = 0; i < numPoints; i++) {
					temZ[i] = bb.getDouble();
				}
				return ShapeFactory.createMultipoint3D(temX, temY, temZ);
		}

		return null;
	}

//	public IGeometry getShape(int index) throws IOException {
//		return ((IGeometry)this.index.get(new Integer(index)));//.cloneGeometry();
//	}
	/**
	 * @return the number of geometries in this driver
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getShapeCount()
	 */
	public int getShapeCount() {
		return numReg;
	}

	/**
	 * @return the shape type of the geometries of this driver
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getShapeType()
	 */
	public int getShapeType() {
		
		int auxType = 0;

		switch (type) {
		case SHP.SHPT_POINT:
			auxType = FShape.POINT;
			break;
			
		case SHP.SHPT_POINTZ:
		case SHP.SHPT_POINTM:
			auxType = FShape.Z | FShape.POINT;
			break;

		case SHP.SHPT_ARC:
			auxType = FShape.LINE;
			break;

		case SHP.SHPT_ARCZ:
		case SHP.SHPT_ARCM:
			auxType = FShape.Z | FShape.LINE;
			break;

		case SHP.SHPT_POLYGON:
			auxType = FShape.POLYGON;
			break;

		case SHP.SHPT_POLYGONZ:
		case SHP.SHPT_POLYGONM:
			auxType = FShape.Z | FShape.POLYGON;
			break;

		case SHP.SHPT_MULTIPOINT:
			auxType = FShape.MULTIPOINT;
			break;

		case SHP.SHPT_MULTIPOINTZ:
		case SHP.SHPT_MULTIPOINTM:
			auxType = FShape.Z | FShape.MULTIPOINT;
			break;
				
		}
		return auxType;
	}


	/**
	 * Initializes the driver
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#initialize()
	 */
	public void initialize() throws IOException {
		logger.debug(Utils.time() + "Empieza initialize() del driver shp.");

		ShapeFileHeader myHeader = new ShapeFileHeader();
		bb.position(0);

		// read the header
		logger.debug(Utils.time() + "Leer header:");
		myHeader.readHeader(bb);
		
		logger.debug(Utils.time() + "Leer header, done");

		extent = new Rectangle2D.Double(myHeader.myXmin, myHeader.myYmin,
				myHeader.myXmax - myHeader.myXmin,
				myHeader.myYmax - myHeader.myYmin);

		type = myHeader.myShapeType;

		double x = myHeader.myXmin;
		double y = myHeader.myYmin;
		double w = myHeader.myXmax - myHeader.myXmin;
		double h = myHeader.myYmax - myHeader.myYmin;

		if (w == 0) {
			x -= 0.1;
			w = 0.2;
		}

		if (h == 0) {
			y -= 0.1;
			h = 0.2;
		}
		
		boolean done = false;

		File dbf_file = getDataFile(fileShp);
		DbfMemoryDataSource dbf_ds = new DbfMemoryDataSource(dbf_file);
		
		try {
			dbf_ds.start();
			numReg = (int) dbf_ds.getRowCount();
			dbf_ds.stop();
			
			done = true;
		} catch (Exception ex) {
			logger.error("While getting row count: " + ex.getMessage());
		}
		
		if (done) {
			logger.debug("NUMREG del DBF: " + numReg);
		} else {
			logger.debug("No se ha obtenido NUMREG, vale 0");
			numReg = 0;
		}

		/*
		logger.debug(Utils.time() + "Intentando: ShpReader.openShpFile()");
		
		long aux_handler = ShpReader.openShpFile(fileShp.getAbsolutePath(), false);
		
		logger.debug(Utils.time()
				+ "Intentando: ShpReader.openShpFile(), aux_handler = " + aux_handler);
		
		try {
			double[] meta = ShpReader.getShpMetadata(aux_handler);
			numReg = Math.round((float) meta[5]);
			ShpReader.closeShpFile(aux_handler);
			done = true;
		} catch (Throwable th) {
			logger.error(
					Utils.time()
					+ "NUMREG, openShpFile(...), getShpMetadata(...) HA FALLADO: "
					+ th.getMessage());
		}
		
		if (done) {
			logger.debug("Se ha obtenido NUMREG = " + numReg);
			return;
		}
		
		String dbfpath = getDataFilePath(fileShp);
		
		try {
			aux_handler = ShpReader.openDbfFile(dbfpath, false);
			numReg = ShpReader.getDbfFileRowCount(aux_handler);
			ShpReader.closeDbfFile(aux_handler);
			done = true;
		} catch (Throwable th) {
			logger.error(
					Utils.time() + "NUMREG, openDbfFile(...), TAMBIEN HA FALLADO: "
					+ th.getMessage());
		}
		*/
		
//		for (int i=0;i<numReg;i++){
//        	IGeometry geom = getShape_(i);
//        	Rectangle2D rec = geom.getBounds2D();
//        	Envelope env = new Envelope(rec.getMinX(),rec.getMaxX(),rec.getMinY(),rec.getMaxY());
//        	//System.out.println(env);
//        	spIndex.insert(env, geom);
//        	index.put(new Integer(i), geom);
//        }
	}

	/**
	 * Reads a Point from the shape file.
	 *
	 * @param in ByteBuffer.
	 *
	 * @return Point2D.
	 */
	private synchronized Point2D.Double readPoint(FalseByteBuffer in) {
		// create a new point
		Point2D.Double tempPoint = new Point2D.Double();

		// bytes 1 to 4 are the type and have already been read.
		// bytes 4 to 12 are the X coordinate
		in.setByteOrder(ByteOrder.LITTLE_ENDIAN);
		tempPoint.setLocation(in.getDouble(), in.getDouble());

		return tempPoint;
	}

	/**
	 * Lee un rectngulo del fichero.
	 *
	 * @param in ByteBuffer.
	 *
	 * @return Rectngulo.
	 *
	 * @throws IOException
	 */
	private synchronized Rectangle2D.Double readRectangle(FalseByteBuffer in)
		throws IOException {
		Rectangle2D.Double tempRect = new Rectangle2D.Double();
		in.setByteOrder(ByteOrder.LITTLE_ENDIAN);
		tempRect.x = in.getDouble();
		tempRect.y = in.getDouble();

		tempRect.width = in.getDouble() - tempRect.x;

		if (tempRect.width == 0) {
			tempRect.width = 0.2;
			tempRect.x -= 0.1;
		}

		tempRect.height = in.getDouble() - tempRect.y;

		if (tempRect.height == 0) {
			tempRect.height = 0.2;
			tempRect.y -= 0.1;
		}

		return tempRect;
	}

	/**
	 * Gets the extent of the geometries of this driver
	 * 
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws IOException {
		return extent;
	}

	/**
	 * Gets the extent of one geometry of this driver.
	 * 
	 * @param index the index of the geometry of interest
	 * @return the bounding box of the geometry
	 */
	public Rectangle2D getShapeBounds(int index) throws IOException {
		Point2D p = new Point2D.Double();
		Rectangle2D BoundingBox = new Rectangle2D.Double();
		bb.position(getPositionForRecord(index));
		bb.setByteOrder(ByteOrder.LITTLE_ENDIAN);

		int tipoShape = bb.getInt();

		//AZABALA: si tipoShape viene con valores erroneos deja de funcionar
		//el metodo getShape(i)
//		if (tipoShape != SHP.NULL) {
//			type = tipoShape;
//
//		}

		// retrieve that shape.
		// tempRecord.setShape(readShape(tempShapeType, tempContentLength, in));
		switch (tipoShape) {
		
			case SHP.SHPT_POINT:
			case SHP.SHPT_POINTZ:
			case SHP.SHPT_POINTM:
				p = readPoint(bb);
				BoundingBox = new Rectangle2D.Double(p.getX() - POINT_TOL,
						p.getY() - POINT_TOL, 2 * POINT_TOL, 2 * POINT_TOL);

				break;

			case SHP.SHPT_ARC:
			case SHP.SHPT_ARCZ:
			case SHP.SHPT_ARCM:
			case SHP.SHPT_MULTIPOINT:
			case SHP.SHPT_MULTIPOINTZ:
			case SHP.SHPT_MULTIPOINTM:
			case SHP.SHPT_POLYGON:
			case SHP.SHPT_POLYGONZ:
			case SHP.SHPT_POLYGONM:
			default:
				// BoundingBox
				try {
					BoundingBox = readRectangle(bb);
				} catch (IOException e) {
					logger.error("Al calcular BB: " + e.getMessage());
					BoundingBox = null;
				}

				break;
			
		}

		return BoundingBox;
	}

	/**
	 * Says whether a file is a SHP file.
	 * 
	 * @param f the file of interest
	 * @return whether it is a valid SHP file
	 */
	public boolean accept(File f) {
		return (f.getName().toUpperCase().endsWith("SHP"));
	}

	/**
	 * Gets the driver name
	 */
	public String getName() {
		return "gvSIG SHP driver";
	}

	/**
	 * @return the name of the associated data driver (DBF in this case)
	 */
	public String getDataDriverName() {
		return VectorialFileAdapter.DBF_FILE_DATASOURCE;
	}

	/**
	 * Gets the data file (attributes file, in this case a DBF file)
	 * that corresponds to the given SHP file
	 */
	public File getDataFile(File f) {
		String str = f.getAbsolutePath();
		return new File(str.substring(0, str.length() - 3) + "dbf");
	}

	/**
	 * Gets the associated SHX file
	 * 
	 * @param f a certain SHP file
	 * @return the associated SHX file
	 */
    public File getShxFile(File f) {
        String str = f.getAbsolutePath();
        return new File(str.substring(0, str.length() - 3) + "shx");
    }


    /**
     * Gets the shape type of the geometry of interest. Redirects to
     * <code>getShapeType()</code> 
     * 
     * @param index the index of the geometry of interest
     * @return the type of the geometry of interest.
     * 
     */
	public int getShapeType(int index) {
		// Por ahora todos los fichero .shp contienen
		// entidades del mismo tipo. Si trabajamos con
		// alguno mixto, tendremos que cambiar esta funcin.
		return getShapeType();
	}

    private synchronized int getPositionForRecord(int numRec)
    {
        // shx file has a 100 bytes header, then, records
        // 8 bytes length, one for each entity.
        // first 4 bytes are the offset
        // next 4 bytes are length

        int posIndex = 100 + (numRec * 8);
        // bbShx.position(posIndex);
        int pos = 8 + 2*bbShx.getInt(posIndex);

        return pos;
    }

    /**
     * Gets the main SHP file object.
     * @return the main file object
     */
	public File getFile() {
		return fileShp;
	}

	/**
	 * Reloads the driver (reopens file and initializes)
	 */
	public void reload() throws IOException {
		open(fileShp);
		initialize();
	}

	/**
	 * Initializes the driver 
	 * 
	 * @param layerDef the table definition to initialize the driver
	 * 
	 */
	public void initialize(ITableDefinition layerDef) {
	}


	/**
	 * @return whether this driver is writable.
	 */
	public boolean isWritable() {
		return fileShp.canWrite();
	}

	/**
	 * @return whether this driver's table can be altered 
	 *
	 */
	public boolean canAlterTable() {
		return false;
	}

	

	/**
	 * Unused.
	 */
	public boolean isWriteAll() {
		return true;
	}

	
	/**
	 * Gets the indices of the geometries intersecting the given rectangle
	 * 
	 * @param rect the rectangle of interest
	 * @param max the maximum number of indices to be returned
	 */
	public int[] getIntersectingIndices(Rectangle2D rect, int max) throws IOException {
		
		ArrayList _resp = new ArrayList();
		for (int i=0; i<numReg; i++) {
			if (Utils.rectanglesItersect(getShapeBounds(i), rect)) {
				IGeometry ig = getShape(i);
				if (ig.getGeometryType() == FShape.POINT || ig.intersects(rect)) {
					_resp.add(new Integer(i));
					// logger.debug("Ha intersectado uno!");
					if (_resp.size() == max) {
						break;
					}
				}
			}
		}
		int size = _resp.size();
		
		if (size == 0) return null;

		int[] resp = new int[size];
		for (int i=0; i<size; i++) resp[i] = ((Integer) _resp.get(i)).intValue();
		return resp;
	}

	public IWriter getWriter() {
		// TODO Auto-generated method stub
		return null;
	}

	public int getSHPShapeType() {
		return type;
	}
	
//	public List getShapes(Rectangle2D rect){
//    	Envelope env = new Envelope(rect.getMinX(),rect.getMaxX(),rect.getMinY(),rect.getMaxY());
//		return spIndex.query(env);
//	}
}
