/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package es.prodevelop.gvsig.mobile.fmap.driver.vect.write;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.ByteOrder;

/**
 *
 * This class is used to write a binary file (SHP, SHX) using different byte orders.
 * 
 * @author Juan Lucas Dominguez Rubio, jldominguez@prodevelop.es
 *
 */
public class OrderFileOutputStream extends BufferedOutputStream {
	
	private ByteOrder order = ByteOrder.LITTLE_ENDIAN;

	public OrderFileOutputStream(File file) throws FileNotFoundException {
		super(file);
	}
	
	public void order(ByteOrder o) {
		order = o;
	}

	public void putInt(int v) throws IOException {
		byte[] data = integerToByteArray(v);
		putNumber(data);
	}

	public void putShort(short s) throws IOException {
		byte[] data = shortToByteArray(s);
		putNumber(data);
	}
	
	public void putDouble(double v) throws IOException {
		byte[] data = doubleToByteArray(v);
		putNumber(data);
	}

	public void put(byte b) throws IOException {
		byte[] aux = new byte[1];
		aux[0] = b;
		write(aux);
	}

	public void put(byte[] bytes) throws IOException {
		write(bytes);
	}

	private byte[] shortToByteArray(short s) {
		byte[] resp = new byte[2];
		resp[0] = (byte) ((s >> 8) & 0xff);
		resp[1] = (byte) (s & 0xff);
		return resp;
	}
	
	private byte[] integerToByteArray(int i) {
		byte[] resp = new byte[4];
		resp[0] = (byte) ((i >> 24) & 0xff);
		resp[1] = (byte) ((i >> 16) & 0xff);
		resp[2] = (byte) ((i >> 8) & 0xff);
		resp[3] = (byte) (i & 0xff);
		return resp;
	}
	
	private byte[] doubleToByteArray(double d) {
		
		long l = Double.doubleToRawLongBits(d);
		byte[] resp = new byte[8];
		int p = 0;
		
		for (int i=0; i<8; i++) {
			p = 56 - (i * 8);
			resp[i] = (byte) ((l >> p) & 0xff);
		}
		return resp;
	}
	
	
	private byte[] reverseBytes(byte[] bb) {
		
		int len = bb.length;
		byte[] resp = new byte[len];
		for (int i=0; i<len; i++) {
			resp[i] = bb[len - 1 - i];
		}
		return resp;
	}
	

	private void putNumber(byte[] data) throws IOException {
		
        if (order == ByteOrder.LITTLE_ENDIAN) {
        	write(reverseBytes(data));
        } else {
        	write(data);
        }
    }	

}
