/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package es.prodevelop.gvsig.mobile.fmap.driver.vect.write.export;

import java.io.File;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.dbf.DbaseFileHeaderNIO;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.DbaseFileWriterNIO;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.OrderFileOutputStream;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;


/**
 * This class creates a DBF file from a given ReadableVectorial object. Used to export
 * vector layers
 *
 * @author Juan Lucas Dominguez Rubio, jldominguez@prodevelop.es
 * @author Vicente Caballero Navarro
 */
public class DBFFromReadVect {

	private static Logger logger = Logger.getLogger(DBFFromReadVect.class);
	private String dbfPath;

	private DbaseFileWriterNIO dbfWrite;
	private Value[] enteros;
	private Object[] record;
	private ReadableVectorial theRV;
	private int count = 0;
	private int fullCount = 0;
	
	private int theType = 0;
	private boolean useType = false;

	private OrderFileOutputStream outStream;

	public DBFFromReadVect(
			ReadableVectorial rv,
			File f,
			int shptype,
			boolean use_type,
			int _count) {
		
		theRV = rv;
		theType = shptype;
		useType = use_type;
		
		setFile(f);
		try {
			fullCount = theRV.getShapeCount();
			if (use_type) {
				count = _count;
			} else {
				count = fullCount;
			}
			outStream = new OrderFileOutputStream(f);
		} catch (Exception e) {
			logger.error("While creating DBF writer: " + e.getMessage());
		}
	}

	/**
	 * Inserta el fichero.
	 *
	 * @param f Fichero.
	 */
	private void setFile(File f) {
		String strFichDbf = StringUtilities.replace(f.getAbsolutePath(), ".shp", ".dbf");
		dbfPath = StringUtilities.replace(strFichDbf, ".SHP", ".DBF");
	}

	/**
	 * Inicializa.
	 *
	 * @param sds Capa.
	 */
	public void create() {

		try {

			DbaseFileHeaderNIO myHeader;
			myHeader = DbaseFileHeaderNIO.createDbaseHeader(theRV.getRecordset());
			myHeader.setNumRecords(count);
			dbfWrite = new DbaseFileWriterNIO(myHeader, outStream);
			int fcnt = theRV.getRecordset().getFieldCount();
			record = new Object[fcnt];
			createdbf();
			logger.debug("DBF management finished");
		} catch (Exception e) {
			logger.error("While creating dbf: " + e.getMessage());
		}
	}

	/**
	 * Rellena los registros del dbf.
	 *
	 * @param sds DOCUMENT ME!
	 */
	private synchronized void  createdbf() {
		try {
			
			if (useType) {
				
				for (int j = 0; j < fullCount; j++) {
					if (theRV.getShape(j).getGeometryType() == theType) {
						record = theRV.getRecordset().getRow(j);
						dbfWrite.write(record);
					}
				}
				
			} else {
				
				for (int j = 0; j < count; j++) {
					record = theRV.getRecordset().getRow(j);
					dbfWrite.write(record);
				}
			}
				
			dbfWrite.close();
		} catch (Exception e) {
			logger.error("While writing DBF: " + e.getMessage());
		}
	}

}
