/*
 * Created on 12-ene-2006 by fjp
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package es.prodevelop.gvsig.mobile.fmap.edition;

import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.core.DefaultFeature;
import es.prodevelop.gvsig.mobile.fmap.core.FNullGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.IRow;
import es.prodevelop.gvsig.mobile.fmap.driver.BoundedShapes;
import es.prodevelop.gvsig.mobile.fmap.driver.Driver;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.IFeatureIterator;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.driver.featureiterators.AttrQueryFeatureIterator;
import es.prodevelop.gvsig.mobile.fmap.driver.featureiterators.DefaultFeatureIterator;
import es.prodevelop.gvsig.mobile.fmap.driver.featureiterators.SpatialQueryFeatureIterator;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.VectorialDefaultAdapter;
import es.prodevelop.gvsig.mobile.fmap.proj.ICoordTrans;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.fmap.spatialindex.ISpatialIndex;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.FBitSet;

/**
 * @author fjp
 *
 */
public class VectorialEditableAdapter extends EditableAdapter implements
		 BoundedShapes, ReadableVectorial {
	protected ReadableVectorial ova;

	protected Rectangle2D fullExtent;

	protected Image selectionImage;

	protected BufferedImage handlersImage;

	private static Logger logger = Logger.getLogger(VectorialEditableAdapter.class);

	/*
	 * azo; ReadableVectorial implementations need a reference to
	 * IProjection (of the layer) and to ISpatialIndex (of the layer)
	 * to allow iteration with different criteria
	 * */
	protected IProjection projection;

	//private double flatness=0.8;
	/*
	 * private class MyFeatureIterator implements IFeatureIterator { int numReg =
	 * 0; Rectangle2D rect; String epsg; IFeatureIterator origFeatIt; boolean
	 * bHasNext = true;
	 *
	 * public MyFeatureIterator(Rectangle2D r, String strEPSG) throws
	 * DriverException { rect = r; epsg = strEPSG; origFeatIt =
	 * ova.getFeatureIterator(r, epsg); } public boolean hasNext() throws
	 * DriverException { return bHasNext; }
	 *
	 * public IFeature next() throws DriverException { IFeature aux =
	 * origFeatIt.next(); return null; }
	 *
	 * public void closeIterator() throws DriverException {
	 *  }
	 *  }
	 */



	public VectorialEditableAdapter() {
		super();
	}

	public void setOriginalVectorialAdapter(ReadableVectorial rv) throws IOException, FMapDriverException {
		ova = rv;
		setOriginalDataSource(rv.getRecordset());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#start()
	 */
	public void start() throws DriverException {
		ova.start();
	}

	public IWriter getWriter() {
		if (ova.getDriver() instanceof IWriteable)
		{
			IWriter writer = ((IWriteable) ova.getDriver()).getWriter();
			if (writer instanceof ISpatialWriter)
				return writer;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#stop()
	 */
	public void stop() throws  DriverException {
		ova.stop();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShape(int)
	 */
	public IGeometry getShape(int rowIndex) throws  FMapDriverException {
		// Si no est en el fichero de expansin
		int calculatedIndex = getCalculatedIndex(rowIndex);
		Integer integer = new Integer(calculatedIndex);
		if (!relations.containsKey(integer)) {
			// Si ha sido eliminada
			/*
			 * if (delRows.get(integer.intValue())) { return null; } else {
			 */
			return ova.getShape(calculatedIndex);
			// }
		}
		int num = ((Integer) relations.get(integer)).intValue();
		DefaultRowEdited feat = null;
		try {
			feat = (DefaultRowEdited) expansionFile.getRow(num);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return ((IFeature) feat.getLinkedRow()).getGeometry()
					.cloneGeometry();// getGeometry();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeType()
	 */
	public int getShapeType() throws FMapDriverException  {
		return ova.getShapeType();
	}

	public ReadableVectorial getOriginalAdapter() {
		return ova;
	}

	public VectorialDriver getDriver() {
		return ova.getDriver();
	}

	public void setDriver(VectorialDriver driver) {
		this.ova.setDriver(driver);
	}

//	public DriverAttributes getDriverAttributes() {
//		return ova.getDriverAttributes();
//	}

	/**
	 * DOCUMENT ME!
	 * @throws StartEditionLayerException
	 * @throws StartWriterVisitorException
	 *
	 * @throws EditionException
	 *             DOCUMENT ME!
	 */
	public void startEdition(int sourceType) throws IOException,
			FMapDriverException {

		super.startEdition(sourceType);
		Driver drv = (Driver) ova.getDriver();

		if (drv == null) {
			logger.error("DRIVER is null!");
		}
		if (drv instanceof IWriteable) {
			setWriter(((IWriteable) drv).getWriter());
		}

		try {
			expansionFile.open();

			int count = ova.getShapeCount();
			boolean boundshapes = (ova.getDriver() instanceof BoundedShapes);

			for (int i = 0; i < count; i++) {
				
				Rectangle2D r = null;
				
				if (boundshapes) {
					r = ((BoundedShapes) ova.getDriver()).getShapeBounds(i);
				} else {
					IGeometry g = null;
					g = ((DefaultFeature) ova.getFeature(i)).getGeometry();
					if (g == null) {
						continue;
					}
					r = g.getBounds2D();
				}
				
				if (fullExtent == null) {
					fullExtent = r;
				} else {
					fullExtent = fullExtent.createUnion(r);
				}
			}
		} catch (IOException e) {
			logger.error("IOException" + e.getMessage());
			throw new IOException(writer.getName());
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#getRow(int)
	 */
	public IRowEdited getRow(int index) throws IOException {
		try {
			start();
		} catch (DriverException e) {
			e.printStackTrace();
		}
		int calculatedIndex = getCalculatedIndex(index);
		Integer integer = new Integer(calculatedIndex);
		// Si no est en el fichero de expansin
		DefaultRowEdited edRow = null;
		if (!relations.containsKey(integer)) {
			try {
				edRow = new DefaultRowEdited(ova.getFeature(calculatedIndex),
						IRowEdited.STATUS_ORIGINAL, index);
			} catch (FMapDriverException e1) {
				e1.printStackTrace();
			}
			try {
				stop();
			} catch (DriverException e) {
				e.printStackTrace();
			}
			return createExternalRow(edRow, 0);
		}
		int num = ((Integer) relations.get(integer)).intValue();
		IRowEdited aux = expansionFile.getRow(num);
		edRow = new DefaultRowEdited(aux.getLinkedRow().cloneRow(), aux
				.getStatus(), index);
		try {
			stop();
		} catch (DriverException e) {
			e.printStackTrace();
		}
		return edRow;
	}

	/**
	 * Elimina una geometria. Si es una geometra original de la capa en edicin
	 * se marca como eliminada (haya sido modificada o no). Si es una geometra
	 * aadida posteriormente se invalida en el fichero de expansin, para que
	 * una futura compactacin termine con ella.
	 *
	 * @param index
	 *            ndice de la geometra.
	 * @throws ReadDriverException
	 * @throws ExpansionFileReadException
	 *
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public IRow doRemoveRow(int index, int sourceType) throws IOException{
		boolean cancel = fireBeforeRemoveRow(index, sourceType);
		if (cancel)
			return null;
		// Llega el calculatedIndex
		Integer integer = new Integer(index);

		IFeature feat = null;
		delRows.set(index, true);
		// Si la geometra no ha sido modificada
		if (!relations.containsKey(integer)) {
			try {
				feat = (DefaultFeature) (ova.getFeature(index));
			} catch (FMapDriverException e) {
				e.printStackTrace();
			}
		} else {
			int num = ((Integer) relations.get(integer)).intValue();
			feat = (IFeature) expansionFile.getRow(num).getLinkedRow();
			// expansionFile.invalidateRow(num);
		}
		System.err.println("Elimina una Row en la posicin: " + index);
		// Se actualiza el ndice
		if (feat != null) {
			Rectangle2D r = feat.getGeometry().getBounds2D();
//			this.fmapSpatialIndex.delete(r,
//					new Integer(index).intValue());
//			System.out.println("Est borrado : " + borrado);
//			System.out.println("Index.lenght : " + this.index.size());
			isFullExtentDirty = true;
		}
		try {
			setSelection(new FBitSet());
		} catch (FMapDriverException e) {
			e.printStackTrace();
		}
		fireAfterRemoveRow(index, sourceType);
		return feat;
	}

	/**
	 * Si se intenta modificar una geometra original de la capa en edicin se
	 * aade al fichero de expansin y se registra la posicin en la que se
	 * aadi. Si se intenta modificar una geometria que se encuentra en el
	 * fichero de expansin (por ser nueva o original pero modificada) se invoca
	 * el mtodo modifyGeometry y se actualiza el ndice de la geometria en el
	 * fichero.
	 *
	 * @param calculatedIndex
	 *            DOCUMENT ME!
	 * @param feat
	 *            DOCUMENT ME!
	 *
	 * @return position inside ExpansionFile
	 * @throws ReadDriverException
	 * @throws ExpansionFileWriteException
	 * @throws ExpansionFileReadException
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public int doModifyRow(int calculatedIndex, IRow feat, int sourceType) {
		boolean cancel = false;
		try {
			cancel = fireBeforeModifyRow(feat, calculatedIndex, sourceType);
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (cancel)
			return -1;
		int posAnteriorInExpansionFile = -1;
		Integer integer = new Integer(calculatedIndex);

		IFeature featAnt = null;
//		System.err.println("Modifica una Row en la posicin: "
//				+ calculatedIndex);
		// Si la geometra no ha sido modificada
		if (!relations.containsKey(integer)) {
			int newPosition = 0;
			try {
				newPosition = expansionFile.addRow(feat,
						IRowEdited.STATUS_MODIFIED, actualIndexFields);
			} catch (IOException e) {
				e.printStackTrace();
			}
			relations.put(integer, new Integer(newPosition));

			if (sourceType == EditionEvent.GRAPHIC) {
				// Se actualiza el ndice espacial
				try {
					featAnt = (DefaultFeature) (ova.getFeature(calculatedIndex));
				} catch (FMapDriverException e) {
					e.printStackTrace();
				}
				IGeometry g = featAnt.getGeometry();
				Rectangle2D rAnt = g.getBounds2D();
				Rectangle2D r = ((IFeature) feat).getGeometry().getBounds2D();
//				this.fmapSpatialIndex.delete(rAnt, new Integer(calculatedIndex).intValue());
//				this.fmapSpatialIndex.insert(r, new Integer(calculatedIndex).intValue()
//	);
			}
		} else {
			// Obtenemos el ndice en el fichero de expansin
			int num = ((Integer) relations.get(integer)).intValue();
			posAnteriorInExpansionFile = num;

			// Obtenemos la geometra para actualiza el ndice
			// espacialposteriormente
			try {
				featAnt = (IFeature) expansionFile.getRow(num).getLinkedRow();
			} catch (IOException e) {
				e.printStackTrace();
			}

			/*
			 * Se modifica la geometra y nos guardamos el ndice dentro del
			 * fichero de expansin en el que se encuentra la geometra
			 * modificada
			 */
			try {
				num = expansionFile.modifyRow(num, feat, actualIndexFields);
			} catch (IOException e) {
				e.printStackTrace();
			}

			/*
			 * Actualiza la relacin del ndice de la geometra al ndice en el
			 * fichero de expansin.
			 */
			relations.put(integer, new Integer(num));
			if (sourceType == EditionEvent.GRAPHIC) {
				// Se modifica el ndice espacial
				Rectangle2D rAnt = featAnt.getGeometry().getBounds2D();
				Rectangle2D r = ((IFeature) feat).getGeometry().getBounds2D();
//				this.fmapSpatialIndex.delete(rAnt, new Integer(calculatedIndex).intValue());
//				this.fmapSpatialIndex.insert(r, new Integer(calculatedIndex).intValue());
			}
		}
		isFullExtentDirty = true;
		fireAfterModifyRow(calculatedIndex, sourceType);
		return posAnteriorInExpansionFile;
	}

	/**
	 * Actualiza en el mapa de ndices, la posicin en la que estaba la
	 * geometra antes de ser modificada. Se marca como vlida, en caso de que
	 * fuera una modificacin de una geometra que estuviese en el fichero de
	 * expansin antes de ser modificada y se pone el puntero de escritura del
	 * expansion file a justo despues de la penultima geometra
	 *
	 * @param calculatedIndex
	 *            ndice de la geometra que se quiere deshacer su modificacin
	 * @param previousExpansionFileIndex
	 *            ndice que tena antes la geometra en el expansionFile. Si
	 *            vale -1 quiere decir que es una modificacin de una geometra
	 *            original y por tanto no hay que actualizar el mapa de indices
	 *            sino eliminar su entrada.
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public void undoModifyRow(int calculatedIndex,
			int previousExpansionFileIndex, int sourceType)throws IOException {
		try {
			// Llega el CalculatedIndex
			/*
			 * Si la accin de modificar se realiz sobre una geometra original
			 */
			if (previousExpansionFileIndex == -1) {

				// Se obtiene la geometra para actualizar el ndice
				// IGeometry g = ((DefaultFeature)
				// getRow(calculatedIndex).getLinkedRow()).getGeometry();
				int inverse = getInversedIndex(calculatedIndex);
				DefaultFeature df;

				df = (DefaultFeature) getRow(inverse).getLinkedRow();

				boolean cancel = fireBeforeModifyRow(df, calculatedIndex,
						sourceType);
				if (cancel)
					return;
				IGeometry g = df.getGeometry();
				// IGeometry g = ova.getShape(calculatedIndex);
				Rectangle2D r = g.getBounds2D();

				// Se elimina de las relaciones y del fichero de expansin
				relations.remove(new Integer(calculatedIndex));
				expansionFile.deleteLastRow();
				if (sourceType == EditionEvent.GRAPHIC) {
					// Se actualizan los ndices
					IGeometry gAnt = null;
					try {
						gAnt = ova.getShape(calculatedIndex);
					} catch (FMapDriverException e) {
						e.printStackTrace();
					}
					/*
					 * IGeometry gAnt = ((DefaultFeature) getRow(calculatedIndex)
					 * .getLinkedRow()).getGeometry();
					 */
					Rectangle2D rAnt = gAnt.getBounds2D();
//					this.fmapSpatialIndex.delete(r, new Integer(calculatedIndex).intValue());
//					this.fmapSpatialIndex.insert(rAnt, new Integer(calculatedIndex).intValue());
				}
			} else {
				// Se obtiene la geometra para actualizar el ndice
				IGeometry g = null;
				int inverse = getInversedIndex(calculatedIndex);
				DefaultFeature df = (DefaultFeature) getRow(inverse).getLinkedRow();
				boolean cancel = fireBeforeModifyRow(df, calculatedIndex,
						sourceType);
				if (cancel)
					return;
				if (sourceType == EditionEvent.GRAPHIC) {
					g = df.getGeometry();
					System.out.println("Actual: " + g.toString());

					Rectangle2D r = g.getBounds2D();
//					this.fmapSpatialIndex.delete(r, new Integer(calculatedIndex).intValue());

					// Se recupera la geometra
					// expansionFile.validateRow(previousExpansionFileIndex);

					// Se actualizan los ndices
					// g = ((IFeature)
					// (expansionFile.getRow(previousExpansionFileIndex).getLinkedRow())).getGeometry();
					// System.out.println("Anterior a la que volvemos : " +
					// g.toString());
					g = ((DefaultFeature) getRow(inverse).getLinkedRow())
					.getGeometry();
					r = g.getBounds2D();
//					this.fmapSpatialIndex.insert(r, new Integer(calculatedIndex).intValue());
				}
				// Se actualiza la relacin de ndices
				// Integer integer = new Integer(geometryIndex);
				relations.put(new Integer(calculatedIndex), new Integer(
						previousExpansionFileIndex));

			}
		} catch (IOException e) {
			throw new IOException(writer.getName());
		}
		//fireAfterModifyRow(calculatedIndex, sourceType);
	}

	/**
	 * Aade una geometria al fichero de expansin y guarda la correspondencia
	 * en la tabla relations.
	 *
	 * @param feat
	 *            geometra a guardar.
	 *
	 * @return calculatedIndex
	 * @throws ExpansionFileWriteException
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public int doAddRow(IRow feat, int sourceType) {
		int calculatedIndex = -1;
		try {
			calculatedIndex = super.doAddRow(feat, sourceType);
		} catch (IOException e) {
			e.printStackTrace();
		}
		// Actualiza el ndice espacial
		IGeometry g = ((IFeature) feat).getGeometry();
		Rectangle2D r = g.getBounds2D();
//		fmapSpatialIndex.insert(r, new Integer(calculatedIndex).intValue());
		isFullExtentDirty = true;
		return calculatedIndex;
	}

	/**
	 * Se desmarca como invalidada en el fichero de expansion o como eliminada
	 * en el fichero original
	 *
	 * @param index
	 *            DOCUMENT ME!
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public void undoRemoveRow(int index, int sourceType) throws IOException{
		super.undoRemoveRow(index, sourceType);

		IGeometry g = null;
		try {
			g = ((IFeature) getRow(getInversedIndex(index)).getLinkedRow()).getGeometry();
		} catch (IOException e) {
			throw new IOException(writer.getName());
		}

		Rectangle2D r = g.getBounds2D();
//		this.fmapSpatialIndex.insert(r, new Integer(index).intValue());
	}

	/**
	 * Se elimina del final del fichero de expansin poniendo el puntero de
	 * escritura apuntando al final de la penltima geometra. Deber quitar la
	 * relacin del mapa de relaciones
	 *
	 * @param fmapSpatialIndex
	 *            ndice de la geometra que se aadi
	 * @throws EditionCommandException
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public void undoAddRow(int calculatedIndex, int sourceType)throws IOException {
		int inverse = getInversedIndex(calculatedIndex);
		IGeometry g;
		try {
			g = ((IFeature) getRow(inverse).getLinkedRow()).getGeometry();
			Rectangle2D r = g.getBounds2D();
//			this.fmapSpatialIndex.delete(r, new Integer(calculatedIndex).intValue());

			super.undoAddRow(calculatedIndex, sourceType);
			try {
				setSelection(new FBitSet());
			} catch (FMapDriverException e) {
				e.printStackTrace();
			}
		} catch (IOException e) {
			throw new IOException(writer.getName());
		}
	}

	/**
	 * Obtiene las geometras que se encuentran en el rectngulo que se pasa
	 * como parmetro haciendo uso del ndice espacial
	 *
	 * @param r
	 *            Rectngulo indicando la porcin del espacio para el cual se
	 *            quiere saber los ndices de las geometras que se encuentran
	 *            dentro de l
	 *
	 * @return Array de ndices para su uso con getGeometry, removeGeometry, ...
	 */
	/*
	 * public int[] getRowsIndexes_OL(Rectangle2D r) { Envelope e = new
	 * Envelope(r.getX(), r.getX() + r.getWidth(), r.getY(), r.getY() +
	 * r.getHeight()); List l = index.query(e); int[] indexes = new
	 * int[l.size()];
	 *
	 * for (int index = 0; index < l.size(); index++) { Integer i = (Integer)
	 * l.get(index); indexes[index] = getInversedIndex(i.intValue()); }
	 *
	 * return indexes; }
	 */
	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeCount()
	 */
	public int getShapeCount() throws FMapDriverException{
		int count = 0;
		try {
			count = getRowCount();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return count;
	}

	/**
	 * @throws ExpansionFileReadException
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws FMapDriverException{
		if (fullExtent == null) {
			fullExtent = ova.getFullExtent();
		}

		if (isFullExtentDirty) {
			try {
				fullExtent = reCalculateFullExtent();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (FMapDriverException e) {
				e.printStackTrace();
			}
			isFullExtentDirty = false;
		}
		return fullExtent;
	}

	/**
	 * Use it BEFORE writing to a file.
	 *
	 * @return real full extent.
	 * @throws ExpansionFileReadException
	 * @throws ReadDriverException
	 * @throws DriverIOException
	 */
	public Rectangle2D reCalculateFullExtent() throws IOException, FMapDriverException{
		if (getShapeCount() > 0) {
			fullExtent = getShape(0).getBounds2D();
			for (int i = 1; i < getShapeCount(); i++) {
				IGeometry geom=getShape(i);
				if (fullExtent==null) {
					fullExtent=geom.getBounds2D();
					continue;
				}
				if (!(geom instanceof FNullGeometry)) {
					fullExtent.add(geom.getBounds2D());
				}
			}
		} else {
			fullExtent = ova.getFullExtent();
		}
		return fullExtent;
	}

	/**
	 * En la implementacin por defecto podemos hacer que cada feature tenga ID =
	 * numero de registro. En el DBAdapter podramos "overrride" este mtodo y
	 * poner como ID de la Feature el campo nico escogido en la base de datos.
	 *
	 * @param numReg
	 * @return
	 * @throws ExpansionFileReadException
	 */
	public IFeature getFeature(int numReg) throws  FMapDriverException{
		try{
			IGeometry geom;
			IFeature feat = null;
			geom = getShape(numReg);
			DataSource rs = getRecordset();
			Value[] regAtt = new Value[rs.getFieldCount()];
			for (int fieldId = 0; fieldId < rs.getFieldCount(); fieldId++) {
				regAtt[fieldId] = rs.getFieldValue(numReg, fieldId);
			}
			feat = new DefaultFeature(geom, regAtt, numReg + "");
		return feat;
		}catch (DriverException e){
			throw new FMapDriverException(e.getMessage());
		}
	}

	public void stopEdition(IWriter writer, int sourceType) throws IOException{
//		ISpatialWriter spatialWriter = (ISpatialWriter) writer;
//		spatialWriter.setFlatness(FConverter.flatness);
		super.stopEdition(writer, sourceType);
		isEditing = false;
		fireStopEditionEvent(sourceType);
		try {
			ova.getDriver().reload();
		} catch (DriverException e) {
			throw new IOException(writer.getName());
		}
//		fmapSpatialIndex=null;
		writer=null;
	}

	public Rectangle2D getShapeBounds(int index) throws IOException{
		// Solo se utiliza cuando el driver es BoundedShapes
		// Si no est en el fichero de expansin
		Integer integer = new Integer(index);
		if (!relations.containsKey(integer)) {
			if (ova.getDriver() instanceof BoundedShapes) {
				BoundedShapes bs = (BoundedShapes) ova.getDriver();
				return bs.getShapeBounds(index);
			}
			return ova.getDriver().getShape(index).getBounds2D();

		}
		int num = ((Integer) relations.get(integer)).intValue();
		DefaultRowEdited feat;
		feat = (DefaultRowEdited) expansionFile.getRow(num);
		if (feat.getStatus() == IRowEdited.STATUS_DELETED)
			return null;
		IGeometry geom = ((IFeature) feat.getLinkedRow()).getGeometry();
		return geom.getBounds2D();// getGeometry();

	}

	public int getShapeType(int index) {
		try {
			return ova.getShapeType();
		} catch (FMapDriverException e) {
			e.printStackTrace();
			return -1;
		}
	}

	/**
	 * Usar solo cuando ests seguro de que puedes gastar memoria. Nosotros lo
	 * usamos para las bsquedas por ndice espacial con el handle. La idea es
	 * usarlo una vez, guardar las geometras que necesitas en ese extent y
	 * trabajar con ellas hasta el siguiente cambio de extent.
	 *
	 * @param r
	 * @param strEPSG
	 * @return
	 * @throws ExpansionFileReadException
	 * @throws ReadDriverException
	 * @throws DriverException
	 */
	public IRowEdited[] getFeatures(Rectangle2D r, String strEPSG) throws IOException{
		// En esta clase suponemos random access.
		// Luego tendremos otra clase que sea VectorialEditableDBAdapter
		// que reescribir este mtodo.
//		Envelope e = FConverter.convertRectangle2DtoEnvelope(r);
//		List l = fmapSpatialIndex.query(r);
//		IRowEdited[] feats = new IRowEdited[l.size()];
		ArrayList feats=new ArrayList();
		for (int index = 0; index < getRowCount(); index++) {
			try {
				if (r.intersects(getShape(index).getBounds2D())){
//			Integer i = (Integer) l.get(index);
				int inverse = getInversedIndex(index);
				feats.add(getRow(inverse));
				}
			} catch (FMapDriverException e) {
				throw new IOException();
			}
		}
		return (IRowEdited[])feats.toArray(new IRowEdited[0]);
	}

//	public void setSpatialIndex(SpatialIndex spatialIndex) {
//		index = (Quadtree) spatialIndex;
//		QuadtreeJts fmapidx = new QuadtreeJts(index);
//		setSpatialIndex(fmapidx);
//
//	}

	public void setFullExtent(Rectangle2D fullExtent2) {
		fullExtent = fullExtent2;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Image getSelectionImage() {
		return selectionImage;
	}

	public Image getHandlersImage() {
		return handlersImage;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param i
	 *            DOCUMENT ME!
	 */
	public void setSelectionImage(Image i) {
		selectionImage = i;
	}

	public void setHandlersImage(BufferedImage handlersImage) {
		this.handlersImage = handlersImage;
	}

	public IFeatureIterator getFeatureIterator()throws DriverException,FMapDriverException {
		return new DefaultFeatureIterator(this, projection, null, null);
	}


	public IFeatureIterator getFeatureIterator(String[] fields, IProjection newProjection)
		throws DriverException,FMapDriverException
	{
		return new DefaultFeatureIterator(this, projection, newProjection, fields);
	}

	/**
	* Return a feature iterator from a given sql statement.
	* <br>
	* In this case, the statement will have the "projection" operator
	* (select campo1, campo2, ...etc) and the "selection" operator (where ....)
	* @param sql statement which define a filter
	* @return feature iterator
	* */
	public IFeatureIterator getFeatureIterator(String sql,
								IProjection newProjection)
		throws DriverException,FMapDriverException
	{

		return new AttrQueryFeatureIterator(this, projection, newProjection, sql);
	}


	/**
	* Makes an spatial query returning a feature iterator over the features which intersects
	* or are contained in the rectangle query. Applies a restriction to the alphanumeric fields
	* returned by the iterator.
	* @param rect
	* @param fields
	* @return
	 * @throws ReadDriverException
	*/
	public IFeatureIterator getFeatureIterator(Rectangle2D rect, String[] fields,
									IProjection newProjection, boolean fastIteration)
		throws DriverException,FMapDriverException {
			return new SpatialQueryFeatureIterator(this, projection, newProjection, fields, rect, fastIteration);
//		}else{
//			return new IndexedSptQueryFeatureIterator(this, projection, newProjection, fields, rect, fmapSpatialIndex, fastIteration);
//		}
	}

//	public ISpatialIndex getSpatialIndex() {
//		return fmapSpatialIndex;
//	}

	public void setSpatialIndex(ISpatialIndex spatialIndex) {
//		this.fmapSpatialIndex = spatialIndex;
	}

	public void setProjection(IProjection projection) {
		this.projection = projection;
	}

	public IProjection getProjection() {
		return projection;
	}

	public void cancelEdition(int sourceType) throws IOException{
		super.cancelEdition(sourceType);
//		fmapSpatialIndex=null;
		setWriter(null);
		System.gc();
	}

	/**
	 * Inserta las coordenadas de transformacin.
	 *
	 * @param ct Coordenadas de transformacin.
	 */
//	public void setCoordTrans(ICoordTrans ct) {
//		this.ct=ct;
//	}

	public int[] getIntersectingIndices(Rectangle2D rect, int max_count) throws IOException {

		ArrayList _resp = new ArrayList();
		try {
			for (int i=0; i<getShapeCount(); i++) {
				if (Utils.rectanglesItersect(getShapeBounds(i), rect)) {
					IGeometry ig = getShape(i);
					if (ig.getGeometryType() == FShape.POINT || ig.intersects(rect)) {
						_resp.add(new Integer(i));
						// logger.debug("Ha intersectado uno!");
						if (_resp.size() == max_count) {
							break;
						}
					}
				}
			}
		} catch (FMapDriverException e) {
			e.printStackTrace();
		}
		int size = _resp.size();

		if (size == 0) return null;

		int[] resp = new int[size];
		for (int i=0; i<size; i++) resp[i] = ((Integer) _resp.get(i)).intValue();
		return resp;
	}


	public HashMap getAddRowInHM(){
		return relations;
	}
}
