/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.layer;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.BitSet;

import org.apache.log4j.Logger;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.core.FDot;
import es.prodevelop.gvsig.mobile.fmap.core.FMultiPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.BoundedShapes;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;
import es.prodevelop.gvsig.mobile.fmap.edition.EditionEvent;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.legend.IntervalLegend;
import es.prodevelop.gvsig.mobile.fmap.legend.Legend;
import es.prodevelop.gvsig.mobile.fmap.legend.LegendFactory;
import es.prodevelop.gvsig.mobile.fmap.legend.VectorialLegend;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.cancel.Cancellable;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.FBitSet;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.Graph2DUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

/**
 * Capa bsica Vectorial.
 * 
 * @author Fernando Gonzlez Corts
 */

// TODO Cuando no sea para pruebas debe no ser public
public class FLyrVect extends FLyrDefault implements ClassifiableVectorial,
		SingleLayer, AlphanumericData, InfoByPoint {

	protected static final int MAX_REQUESTED_IN_INFO = 5;

	private static Logger logger = Logger.getLogger(FLyrVect.class.getName());
	private VectorialLegend legend;
	private IntervalLegend intervalLegend;
	private int typeShape = -1;
	private ReadableVectorial source;
	private SelectableDataSource sds;
	private SelectionSupport selectionSupport = new SelectionSupport();
	private boolean spatialCacheEnabled = false;
	private boolean useInterval = false;

	// Added to edition
	private SpatialCache spatialCache = new SpatialCache();

	public FLyrVect() {
		id = System.currentTimeMillis();
	}

	public ReadableVectorial getSource() {
		if (!this.isAvailable())
			return null;
		return source;
	}

	public void setSource(ReadableVectorial rv) {
		source = rv;
	}

	public Rectangle2D getFullExtent() throws FMapDriverException {

		Rectangle2D rAux;

		try {
			source.start();
			rAux = (Rectangle2D) source.getFullExtent().clone();
			source.stop();
		} catch (DriverException e) {
			throw new FMapDriverException(e.getMessage());
		}

		// Si existe reproyeccin, reproyectar el extent
		// ICoordTrans ct = getCoordTrans();

		// if (ct != null) {
		// Point2D pt1 = new Point2D.Double(rAux.getMinX(), rAux.getMinY());
		// Point2D pt2 = new Point2D.Double(rAux.getMaxX(), rAux.getMaxY());
		// pt1 = ct.convert(pt1, null);
		// pt2 = ct.convert(pt2, null);
		// rAux = new Rectangle2D.Double();
		// rAux.setFrameFromDiagonal(pt1, pt2);
		// }

		// Esto es para cuando se crea una capa nueva con el fullExtent de ancho
		// y alto 0.
		if (rAux.getWidth() == 0 && rAux.getHeight() == 0) {
			rAux = new Rectangle2D.Double(0, 0, 100, 100);
		}

		return rAux;
	}

	public void drawSelected(Graphics2D g, ViewPort _vp, FSymbol sym)
			throws FMapDriverException {

		IGeometry ig = null;
		Rectangle2D bb = null;

		BitSet sel = selectionSupport.getSelection();
		AffineTransform aft = _vp.getAffineTransform();

		try {
			g.setStroke(new BasicStroke(sym.getStrokeWidth()));
			g.setColor(FSymbol.SELECTION_SYMBOL.getBorderColor());

			for (int i = sel.nextSetBit(0); i >= 0; i = sel.nextSetBit(i + 1)) {
				bb = getSource().getShapeBounds(i);
				if (Utils.rectanglesItersect(bb, _vp.getAdjustedExtent())) {

					ig = getSource().getShape(i);
					if (ig == null) {
						logger.debug("ig es NULL: " + i);
						continue;
					}

					if (ig.getGeometryType() == FShape.POINT
							|| ig.intersects(_vp.getAdjustedExtent())) {
						ig.transform(aft);
						Shape shp = ig.getInternalShape();
						Graph2DUtilities.drawShape(g, shp,
								FSymbol.SELECTION_SYMBOL);
					}
				}
			}
		} catch (Exception ex) {
			throw new FMapDriverException(ex.getMessage());
		}
	}

	protected double angle = Math.PI / 2;
	
	private boolean updateColorAndStroke(Graphics2D g, int i) throws DriverException, FMapDriverException {
		if (useInterval && intervalLegend != null) {
			Value v = getSource().getRecordset().getFieldValue(
					i,
					getSource().getRecordset().getFieldIndexByName(
							intervalLegend.getFieldName()));

			if (v != null) {
				try {
					Color color = intervalLegend
							.getSymbolForValue(new Double(
									v.toString()).doubleValue());
//					g.setBackground(temp.getFillColor());
					g.setColor(color);
					g.setStroke(new BasicStroke(intervalLegend.getDefaultSymbol()
							.getStrokeWidth()));
//					System.out.println("Value: " + Double.valueOf(
//							v.toString()).doubleValue());					

				} catch (NumberFormatException e) {
					logger.debug(e.getStackTrace());
					return false;
				}
			}	
		} else {
			return false;
		}
		return true;
	}

	protected void drawPoints(Graphics2D g, ViewPort vp)
			throws FMapDriverException {
		AffineTransform _at = vp.getAffineTransform();
		int count = getSource().getShapeCount();

		for (int i = 0; i < count; i++) {
			try {
				if (selectionSupport.isSelected(i)) {
					g.setColor(FSymbol.SELECTION_SYMBOL.getBorderColor());
				}

				Rectangle2D bb = getSource().getShapeBounds(i);

				if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
					IGeometry ig = getSource().getShape(i);

					ig.transform(_at);
					Shape shp = ig.getInternalShape();
					// Graph2DUtilities.drawShape(g, shp, sym);
					// Graph2DUtilities.drawPoint(g, (FPoint2D) shp);

					updateColorAndStroke(g, i);
//						FSymbol temp = intervalLegend.getSymbolForValue(i);
//						g.setColor(temp.getFillColor());
//						g.setStroke(new BasicStroke(temp.getStrokeWidth()));
					

					Graph2DUtilities.drawPoint(g, (FPoint2D) shp);
					// Rectangle2D r2 = (Rectangle2D)ig.getBounds();
					if (ig == null)
						logger.error("No hay bbox");
					// RotationUtils.drawRotatedText(g,"Hola",Color.red,(int)r2.
					// getMinX(),(int)r2.getMinY(),0.5);
					/*
					 * RotationUtils.drawRotatedText(g, "Hola", Color.red,
					 * (int)((FPoint2D) shp).getX(), (int)((FPoint2D)
					 * shp).getY(), angle); angle-=0.1;
					 */
				}
				if (selectionSupport.isSelected(i)) {
					g.setColor(legend.getDefaultSymbol().getBorderColor());
				}
			} catch (Exception e) {
				logger.error("drawPoints: ", e);
			}
		}
	}

	protected void drawMultiPoints(Graphics2D g, ViewPort vp)
			throws FMapDriverException {		
		AffineTransform _at = vp.getAffineTransform();
		int count = getSource().getShapeCount();

		for (int i = 0; i < count; i++) {
			try {
				if (selectionSupport.isSelected(i)) {

					g.setColor(FSymbol.SELECTION_SYMBOL.getBorderColor());
				}

				Rectangle2D bb = getSource().getShapeBounds(i);

				if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
					IGeometry ig = getSource().getShape(i);
					ig.transform(_at);
					Shape shp = ig.getInternalShape();
					updateColorAndStroke(g, i);
					// Graph2DUtilities.drawShape(g, shp, sym);
					// Graph2DUtilities.drawPoint(g, (FPoint2D) shp);
					Graph2DUtilities.drawMultiPoint(g, (FMultiPoint2D) shp);
				}

				if (selectionSupport.isSelected(i)) {
					g.setColor(legend.getDefaultSymbol().getBorderColor());
				}
			} catch (Exception e) {
				logger.error("drawPoints: ", e);
			}
		}
	}

	protected void drawLines(Graphics2D g, ViewPort vp)
			throws FMapDriverException {
		AffineTransform _at = vp.getAffineTransform();
		int count = getSource().getShapeCount();
		int lastbbx = Integer.MIN_VALUE;
		int lastbby = Integer.MIN_VALUE;

		Rectangle2D bbox = vp.getAdjustedExtent();
		double minWX = (double) bbox.getMinX();
		double minWY = (double) bbox.getMaxY();
		double pixSize = (double) bbox.getWidth() / vp.getImageSize().width;
		double diffSize = pixSize;
		double pixsize2 = pixSize * 12;

		for (int i = 0; i < count; i++) {
			try {
				if (selectionSupport.isSelected(i)) {
					g.setColor(FSymbol.SELECTION_SYMBOL.getBorderColor());
				}
				//	
				Rectangle2D bb = getSource().getShapeBounds(i);
				double mx = bb.getMinX();
				double my = bb.getMinY();
				double bw = bb.getWidth();
				if (bw == 0.2) {
					bw = 0;
					mx += 0.1;
				}
				double bh = bb.getHeight();
				if (bh == 0.2) {
					bh = 0;
					my += 0.1;
				}

				if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {

					if (bw <= pixSize && bh <= pixSize) {
						// If shape size is only a pixel size we paint only a
						// pixel

						FPoint2D gbb = new FPoint2D(mx, my);
						gbb.transform(_at);

						int bbx = (int) (gbb.getX() + 0.5);
						int bby = (int) (gbb.getY() + 0.5);

						if (lastbbx != bbx || lastbby != bby) {
							FDot pt = new FDot(bbx, bby);
							updateColorAndStroke(g, i);
							Graph2DUtilities.drawDot(g, pt);
						} else {
							lastbbx = bbx;
							lastbby = bby;
						}

					} else if (bw <= pixSize || bh <= pixSize) {
						int bbx = (int) ((mx - minWX) / pixSize - 0.5);
						int bby = (int) ((minWY - my) / pixSize - 0.5);
						int bbx1 = (int) ((mx + bw - minWX) / pixSize - 0.5);
						int bby1 = (int) ((minWY - my - bh) / pixSize - 0.5);
						//System.out.println("height: " + bh);
						updateColorAndStroke(g, i);
						g.drawLine(bbx, bby, bbx1, bby1);
					} else {
						IGeometry ig = getSource().getShape(i);
						if (ig == null) {
							logger.debug("ig es NULL: " + i);
							continue;
						}

						Shape shp = ig.getInternalShape();
						updateColorAndStroke(g, i);
						if (bb.getWidth() <= pixsize2
								&& bb.getHeight() <= pixsize2)
							Graph2DUtilities.drawPolyLine(g, (FPolyline2D) shp,
									minWX, minWY, pixSize, diffSize, true);
						else
							Graph2DUtilities.drawPolyLine(g, (FPolyline2D) shp,
									minWX, minWY, pixSize, diffSize, false);
					}
				} else {
					// logger.debug("No intersecto");
				}
				if (selectionSupport.isSelected(i)) {
					g.setColor(legend.getDefaultSymbol().getBorderColor());
				}
			} catch (Exception e) {
				logger.error(e);
			}
		}

	}

	// protected void drawLines(Graphics2D g, ViewPort vp) throws
	// FMapDriverException{
	// AffineTransform _at = vp.getAffineTransform();
	// int count = getSource().getShapeCount();
	// FSymbol sym = legend.getDefaultSymbol();
	// for (int i=0; i<count; i++) {
	// try{
	// if (selectionSupport.isSelected(i)) {
	// continue; //sym = FSymbol.SELECTION_SYMBOL;
	// }
	//	
	// Rectangle2D bb = getSource().getShapeBounds(i);
	//	
	// if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
	//					
	// IGeometry ig = getSource().getShape(i);
	// if (ig == null) {
	// //logger.debug("ig es NULL: " + i);
	// continue;
	// }
	//					
	// double[] xs = {bb.getMinX(),bb.getMaxX()};
	// double[] ys = {bb.getMinY(),bb.getMaxY()};
	// FMultiPoint2D gbb = new FMultiPoint2D(xs,ys);
	// gbb.transform(_at);
	// Rectangle2D bb2 = gbb.getBounds2D();
	//	
	// if(bb2.getWidth() < 2 && bb2.getHeight()<2){
	// //If shape size is only a pixel size we paint only a pixel
	// FDot pt = new FDot(bb2.getMinX(),bb2.getMinY());
	// Graph2DUtilities.drawDot(g, pt);
	// } /*else if(bb2.getWidth() < 2 || bb2.getHeight()<2){
	// g.drawLine(
	// (int)Math.round(bb2.getMinX()),
	// (int)Math.round(bb2.getMinY()),
	// (int)Math.round(bb2.getMaxX()),
	// (int)Math.round(bb2.getMaxY()));
	// }*/ else if (ig.intersects(vp.getAdjustedExtent())) {
	// ig.transform(_at);
	// Shape shp = ig.getInternalShape();
	// //Graph2DUtilities.drawShape(g, shp, sym);
	// Graph2DUtilities.drawPolyLine(g, (FPolyline2D)shp);
	// }
	// }
	// } catch (Exception e){
	// logger.error(e);
	// }
	// }
	// }

	// protected void drawPolygons(Graphics2D g, ViewPort vp) throws
	// FMapDriverException{
	// AffineTransform _at = vp.getAffineTransform();
	// int count = getSource().getShapeCount();
	// FSymbol sym = legend.getDefaultSymbol();
	// int lastbbx = Integer.MIN_VALUE;
	// int lastbby = Integer.MIN_VALUE;
	//		
	// Rectangle2D bbox = vp.getAdjustedExtent();
	// float minWX = (float)bbox.getMinX();
	// float minWY = (float)bbox.getMaxY();
	// float pixSize = (float)bbox.getWidth()/vp.getImageSize().width;
	// float diffSize = pixSize;
	// float pixsize2 = pixSize*2;
	//		
	// VectorialFileAdapter drv = (VectorialFileAdapter)getSource();
	// List shapes = drv.getshapes(bbox);
	//		
	// Iterator it = shapes.iterator();
	// while (it.hasNext()){
	// IGeometry ig = (IGeometry)it.next();
	// Rectangle2D bb = ig.getBounds2D();
	// if(bb.getWidth() <= pixSize && bb.getHeight() <= pixSize){
	// //If shape size is only a pixel size we paint only a pixel
	//
	// FPoint2D gbb = new FPoint2D(bb.getMinX(),bb.getMinY());
	// gbb.transform(_at);
	//						
	// int bbx = (int)(gbb.getX()+0.5);
	// int bby = (int)(gbb.getY()+0.5);
	//						
	// if (lastbbx!=bbx||lastbby!=bby){
	// FDot pt = new FDot(bbx,bby);
	// Graph2DUtilities.drawDot(g, pt);
	// } else {
	// lastbbx=bbx;
	// lastbby=bby;
	// }
	//						
	//						
	// }else if(bb.getWidth() <= pixsize2 || bb.getHeight() <= pixsize2){
	// double[] xs = {bb.getMinX(),bb.getMaxX()};
	// double[] ys = {bb.getMinY(),bb.getMaxY()};
	// FMultiPoint2D gbb = new FMultiPoint2D(xs,ys);
	// gbb.transform(_at);
	// Rectangle2D bb2 = gbb.getBounds2D();
	//						
	// int bbx = (int)(bb2.getMinX()+0.5);
	// int bby = (int)(bb2.getMinY()+0.5);
	// int bbw = (int)(bb2.getWidth()+0.5);
	// int bbh = (int)(bb2.getHeight()+0.5);
	// if (sym.isFilled())
	// g.fillRect(
	// bbx,
	// bby,
	// bbw,
	// bbh);
	// else
	// g.drawRect(
	// bbx,
	// bby,
	// bbw,
	// bbh);
	// } else {
	// Shape shp = ig.getInternalShape();
	//							
	//							
	// if (bb.getWidth() <= pixSize*12 && bb.getHeight() <= pixSize*12 )
	// Graph2DUtilities.drawPolygon(g, (FPolygon2D)shp, sym,
	// minWX,minWY,pixSize,diffSize,true);
	// else
	// Graph2DUtilities.drawPolygon(g, (FPolygon2D)shp, sym,
	// minWX,minWY,pixSize,diffSize,false);
	// }
	//				
	// }
	//		
	// }

	protected void drawPolygons(Graphics2D g, ViewPort vp)
			throws FMapDriverException {
		AffineTransform _at = vp.getAffineTransform();
		int count = getSource().getShapeCount();
		FSymbol sym = legend.getDefaultSymbol();
		int lastbbx = Integer.MIN_VALUE;
		int lastbby = Integer.MIN_VALUE;
		boolean fill = sym.isFilled();

		Rectangle2D bbox = vp.getAdjustedExtent();
		double minWX = bbox.getMinX();
		double minWY = bbox.getMaxY();
		double pixSize = bbox.getWidth() / vp.getImageSize().width;
		double diffSize = pixSize;
		double pixsize2 = pixSize * 2;

		for (int i = 0; i < count; i++) {
			try {
				if (selectionSupport.isSelected(i)) {
					sym = FSymbol.SELECTION_SYMBOL;
				}
				//	
				Rectangle2D bb = getSource().getShapeBounds(i);

				if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {

					if (bb.getWidth() <= pixSize && bb.getHeight() <= pixSize) {
						// If shape size is only a pixel size we paint only a
						// pixel

						FPoint2D gbb = new FPoint2D(bb.getMinX(), bb.getMinY());
						gbb.transform(_at);

						int bbx = (int) (gbb.getX() + 0.5);
						int bby = (int) (gbb.getY() + 0.5);						

						if (lastbbx != bbx || lastbby != bby) {
							updateColorAndStroke(g, i);
							FDot pt = new FDot(bbx, bby);
							Graph2DUtilities.drawDot(g, pt);
						} else {
							lastbbx = bbx;
							lastbby = bby;
						}

					} else if (bb.getWidth() <= pixsize2
							|| bb.getHeight() <= pixsize2) {
						double[] xs = { bb.getMinX(), bb.getMaxX() };
						double[] ys = { bb.getMinY(), bb.getMaxY() };
						FMultiPoint2D gbb = new FMultiPoint2D(xs, ys);
						gbb.transform(_at);
						Rectangle2D bb2 = gbb.getBounds2D();

						int bbx = (int) (bb2.getMinX() + 0.5);
						int bby = (int) (bb2.getMinY() + 0.5);
						int bbw = (int) (bb2.getWidth() + 0.5);
						int bbh = (int) (bb2.getHeight() + 0.5);
						updateColorAndStroke(g, i);
						if (fill)
							g.fillRect(bbx, bby, bbw, bbh);
						else
							g.drawRect(bbx, bby, bbw, bbh);
					} else {
						IGeometry ig = getSource().getShape(i);
						if (ig == null) {
							logger.debug("ig es NULL: " + i);
							continue;
						}

						Shape shp = ig.getInternalShape();						
						
						if (updateColorAndStroke(g, i) && !selectionSupport.isSelected(i)) {
							sym.setFillColor(g.getColor());
						}
//						
						if (bb.getWidth() <= pixSize * 12
								&& bb.getHeight() <= pixSize * 12)
							Graph2DUtilities.drawPolygon(g, (FPolygon2D) shp,
									sym, minWX, minWY, pixSize, diffSize, true);
						else
							Graph2DUtilities
									.drawPolygon(g, (FPolygon2D) shp, sym,
											minWX, minWY, pixSize, diffSize,
											false);
					}

				} else {
					// logger.debug("No intersecto");
				}
				if (selectionSupport.isSelected(i)) {
					sym = legend.getDefaultSymbol();
				}
			} catch (Exception e) {
				logger.error(e);
			}
		}

	}

	public void draw(Graphics2D g, ViewPort vp) throws FMapDriverException {

		double scale = vp.getScale();

		if (isWithinScale(scale)) {

			// AffineTransform _at = vp.getAffineTransform();
			// int count = getSource().getShapeCount();

			try {
				FSymbol sym = legend.getDefaultSymbol();
				// IGeometry ig = null;
				g.setColor(sym.getBorderColor());
				g.setStroke(new BasicStroke(sym.getStrokeWidth()));

				int lyrType = getShapeType();

				switch (lyrType % FShape.Z) {
				case FShape.POINT:
					drawPoints(g, vp);
					break;
				case FShape.MULTIPOINT:
					drawMultiPoints(g, vp);
					break;
				case FShape.LINE:
					drawLines(g, vp);
					break;
				case FShape.POLYGON:
					drawPolygons(g, vp);
					break;

				}
				/*
				 * for (int i=0; i<count; i++) {
				 * 
				 * //logger.debug("Init: " + Utils.time()); if
				 * (selectionSupport.isSelected(i)) { continue; //sym =
				 * FSymbol.SELECTION_SYMBOL; } //logger.debug("1: " +
				 * Utils.time());
				 * 
				 * Rectangle2D bb = getSource().getShapeBounds(i);
				 * //logger.debug("2: " + Utils.time()); if
				 * (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
				 * //logger.debug("3: " + Utils.time()); ig =
				 * getSource().getShape(i); //logger.debug("4: " +
				 * Utils.time()); if (ig == null) {
				 * //logger.debug("ig es NULL: " + i); continue; }
				 * //logger.debug("5: " + Utils.time());
				 * 
				 * //[jcarras] Performance modifications
				 * 
				 * boolean fill = sym.isFilled();
				 * 
				 * double[] xs = {bb.getMinX(),bb.getMaxX()}; double[] ys =
				 * {bb.getMinY(),bb.getMaxY()}; //logger.debug("6: " +
				 * Utils.time()); FMultiPoint2D gbb = new
				 * FMultiPoint2D(xs,ys);//15 //logger.debug("7: " +
				 * Utils.time()); gbb.transform(_at); //logger.debug("8: " +
				 * Utils.time()); Rectangle2D bb2 = gbb.getBounds2D();//47
				 * //logger.debug("9: " + Utils.time());
				 * 
				 * if (fill && (bb2.getWidth() < 2 || bb2.getHeight()<2)){ //If
				 * shape interior is not seen we do not paint it
				 * sym.setFilled(false); } //logger.debug("10: " +
				 * Utils.time()); if(ig.getGeometryType() != FShape.POINT &&
				 * bb2.getWidth() < 1 && bb2.getHeight()<1){ //If shape size is
				 * only a pixel size we paint only a pixel FDot pt = new
				 * FDot(bb2.getMinX(),bb2.getMinY()); //logger.debug("11: " +
				 * Utils.time()); Graph2DUtilities.drawShape(g, pt, sym);
				 * //logger.debug("12: " + Utils.time()); } else {
				 * //logger.debug("13: " + Utils.time()); if
				 * (ig.getGeometryType() == FShape.POINT ||
				 * ig.intersects(vp.getAdjustedExtent())) { ig.transform(_at);
				 * //logger.debug("14: " + Utils.time()); Shape shp =
				 * ig.getInternalShape(); //logger.debug("15: " + Utils.time());
				 * Graph2DUtilities.drawShape(g, shp, sym); //15
				 * //logger.debug("16: " + Utils.time()); } }
				 * 
				 * sym.setFilled(fill); //logger.debug("17: " + Utils.time()); }
				 * else{ //logger.debug("No intersecto"); } }
				 */

				drawSelected(g, vp, sym);
				// logger.debug("18: " + Utils.time());
			} catch (Exception e) {
				logger.error("Error while drawing vect layer: "
						+ e.getMessage());
			}
		}
	}

	private void prepareDrawing(BufferedImage image, Graphics2D g,
			ViewPort viewPort) {

	}

	public FBitSet queryByPoint(Point2D p, double tolerance)
			throws FMapDriverException {

		return null;
	}

	public ArrayList getInfo(Point p, double tolerance, Cancellable cancel,
			String srs) throws FMapDriverException {
		Point2D pReal = this.getMapContext().getViewPort().toMapPoint(
				new Point2D.Double(p.x, p.y));

		VectorialDriver vd = source.getDriver();
		int[] intersect_indices = null;
		ArrayList resp = new ArrayList();
		resp.add("[No data]");
		resp.add(null);

		if (vd instanceof BoundedShapes) {

			Rectangle2D click_rect = new Rectangle2D.Double(pReal.getX() - 0.5
					* tolerance, pReal.getY() - 0.5 * tolerance, tolerance,
					tolerance);

			try {
				intersect_indices = ((BoundedShapes) vd)
						.getIntersectingIndices(click_rect,
								MAX_REQUESTED_IN_INFO);
			} catch (IOException e) {
				logger.error("While computing intersecting indices: "
						+ e.getMessage());
				throw new FMapDriverException(e.getMessage());
			}

			if (intersect_indices == null)
				return resp;

			SelectableDataSource sds = getRecordset();
			try {
				String[] f_names = sds.getFieldNames();

				for (int i = 0; i < intersect_indices.length; i++) {
					if (i == 0)
						resp.clear();

					resp.add("");
					resp.add(null); // empty row before item

					Value[] atts = sds.getRow(intersect_indices[i]);
					ArrayList geom_data = getGeometryMetadata(
							intersect_indices[i], srs);
					Utils.addAttributesToList(atts, f_names, geom_data, resp);
				}

			} catch (DriverException dex) {
				resp.add("Error:");
				resp.add(dex.getMessage().substring(0, 20) + "...");
			}
		}

		return resp;

	}

	protected ArrayList getGeometryMetadata(int i, String srs) {

		ArrayList resp = new ArrayList();
		double[] tot_last = null;

		VectorialDriver vd = source.getDriver();
		IGeometry ig = null;
		try {
			ig = vd.getShape(i);
		} catch (Exception ex) {
			logger.debug("Error: " + ex.getMessage());
		}

		int _type = ig.getGeometryType();

		switch (_type) {

		case FShape.MULTIPOINT:
			resp.add("Points");
			resp.add(null);
			FMultiPoint2D mp = (FMultiPoint2D) ig.getInternalShape();
			for (int n = 0; n < mp.getNumPoints(); n++) {
				FPoint2D p = mp.getPoint(n);
				resp.add("[" + Utils.getFormattedDouble(p.getX(), 3) + ", ");
				resp.add(Utils.getFormattedDouble(p.getY(), 3) + "]");
			}
			break;

		case FShape.POINT:
			resp.add("Point");
			resp.add(null);
			FPoint2D p = (FPoint2D) ig.getInternalShape();
			resp.add("[" + Utils.getFormattedDouble(p.getX(), 3) + ", ");
			resp.add(Utils.getFormattedDouble(p.getY(), 3) + "]");
			break;

		case FShape.LINE:
			double aux = 0;
			resp.add("Line");
			resp.add(null);
			FPolyline2D l = (FPolyline2D) ig.getInternalShape();
			resp.add("Length");
			if (l.isMulti()) {
				for (int j = 0; j < l.getNumSubShapes(); j++) {
					FPolyline2D subElement = l.getSubElement(j);
					tot_last = GeoUtils.getIteratorLengthAndLast(subElement
							.getPathIterator(Utils.IDENTITY_AT), srs);
					aux = tot_last[0] + aux;
				}
				tot_last[0] = aux;
			} else
				tot_last = GeoUtils.getIteratorLengthAndLast(l
						.getPathIterator(Utils.IDENTITY_AT), srs);
			if (tot_last == null) {
				resp.add("[Error]");
			} else {
				resp.add(Utils.getFormattedDouble(tot_last[0], 3) + " m");
			}

			break;

		case FShape.POLYGON:
			resp.add("Polygon");
			resp.add(null);
			FPolygon2D pol = (FPolygon2D) ig.getInternalShape();

			PathIterator pitr = pol.getPathIterator(Utils.IDENTITY_AT);
			resp.add("Perimeter");

			tot_last = GeoUtils.getIteratorLengthAndLast(pitr, srs);
			if (tot_last == null) {
				resp.add("[Error]");
			} else {
				resp.add(Utils.getFormattedDouble(tot_last[0], 3) + " m");
			}

			pitr = pol.getPathIterator(Utils.IDENTITY_AT);
			resp.add("Area");

			double a = GeoUtils.getAreaInSquareMeters(pitr, srs);
			String units = " sq m";
			int prec = 2;
			if (a >= GeoUtils.SQ_METERS_LIMIT) {
				a = a / 1000000;
				units = " sq km";
			}
			resp.add(Utils.getFormattedDouble(a, prec) + units);
			break;
		}
		return resp;
	}

	public void setLegend(VectorialLegend r) {

		legend = r;
	}

	public Legend getLegend() {
		return legend;
	}

	public int getShapeType() throws FMapDriverException {
		if (typeShape == -1) {
			try {
				getSource().start();
				typeShape = getSource().getShapeType();
				getSource().stop();
			} catch (DriverException e) {
				throw new FMapDriverException(e.getMessage());
			}
		}
		return typeShape;
	}

	/*
	 * public XMLEntity getXMLEntity() throws XMLException { return null; }
	 */
	/*
	 * public void setXMLEntity(XMLEntity xml) throws XMLException { }
	 */

	public String toString() {

		String ret = super.toString();

		return "layer" + ret.substring(ret.indexOf('@') + 1);
	}

	public SelectableDataSource getRecordset() throws FMapDriverException {

		if (!isAvailable()) {
			logger.debug("isAvailable() = FALSE, se devuelve null");
			return null;
		}

		if (sds == null) {

			SelectableDataSource ds = source.getRecordset();

			if (ds == null) {
				return null;
			}

			sds = ds;
			sds.setSelectionSupport(selectionSupport);

		}
		return sds;
	}

	public void setRecordset(SelectableDataSource newSds) {
		sds = newSds;

		if (sds != null) {
			sds.setSelectionSupport(selectionSupport);
		}
	}

	public boolean isPropertiesMenuVisible() {
		return true;
	}

	public void reload() throws FMapDriverException {
		super.reload();
		try {
			source.getDriver().reload();
		} catch (Exception e) {
			throw new FMapDriverException(e.getMessage());
		}
		if (this.getLegend() == null)
			setLegend(LegendFactory.createSingleSymbolLegend());
	}

	public boolean isWritable() {
		return false;
	}

	public void wakeUp(boolean driver_is_already_open) {
	}

	public long getId() {
		return id;
	}

	public StringBuffer toXML(String name, String projectPath) {
		StringBuffer buff = new StringBuffer();
		buff.append("<" + name + ">\n");

		buff.append("</" + name + ">\n");
		return buff;
	}

	public void clean() {
		logger
				.warn("This cleaning method is EMPTY. Subclasses should override it!");
	}

	public SelectionSupport getSelectionSupport() {
		return selectionSupport;
	}

	public void setEditing(boolean b) throws Exception {
		super.setEditing(b);
		try {
			if (b) {
				logger.info("Capa en edicin");
				VectorialEditableAdapter vea = null;

				vea = new VectorialEditableAdapter();
				logger.info("Construimos el VEA");
				// }
				vea.setOriginalVectorialAdapter(getSource());
				logger.info("Despues de Source");
				vea.start();
				logger.info("Nos guardamos el Data Source original");
				vea.setProjection(getProjection());
				logger.info("Ponemos la proyeccion de la capa");

				vea.startEdition(EditionEvent.GRAPHIC);
				logger.info("Empieza edicin");
				setSource(vea);
				logger.info("Cambio el Data Source");
				getRecordset().setSelectionSupport(
						vea.getOriginalAdapter().getRecordset()
								.getSelectionSupport());
				logger.info("Pongo la capa en edicin");

			} else {
				VectorialEditableAdapter vea = (VectorialEditableAdapter) getSource();
				setSource(vea.getOriginalAdapter());
			}
			// Si tenemos una leyenda, hay que pegarle el cambiazo a su
			// recordset
			// setRecordset(getSource().getRecordset());
			logger.info("Termino el setRecordset");
			// if (getLegend() instanceof VectorialLegend) {
			// VectorialLegend ley = (VectorialLegend) getLegend();
			// ley.setDataSource(getSource().getRecordset());
			// // Esto lo pongo para evitar que al dibujar sobre un
			// // dxf, dwg, o dgn no veamos nada. Es debido al checkbox
			// // de la leyenda de textos "dibujar solo textos".
			//
			// ley.useDefaultSymbol(true);
			//
			// }
			logger.info("Termino el try sin problemas");
		} catch (Exception e) {
			throw new Exception(getName(), e);
		}
		logger.info("Pongo el spatial cache");
		setSpatialCacheEnabled(b);
		callEditionChanged(LayerEvent
				.createEditionChangedEvent(this, "edition"));
		logger.info("Creo el evento cambio en edicin");
	}

	public void clearSpatialCache() {
		spatialCache.clearAll();
	}

	public boolean isSpatialCacheEnabled() {
		return spatialCacheEnabled;
	}

	public void setSpatialCacheEnabled(boolean spatialCacheEnabled) {
		this.spatialCacheEnabled = spatialCacheEnabled;
	}

	public SpatialCache getSpatialCache() {
		return spatialCache;
	}

	/**
	 * Siempre es un numero mayor de 1000
	 * 
	 * @param maxFeatures
	 */
	public void setMaxFeaturesInEditionCache(int maxFeatures) {
		if (maxFeatures > spatialCache.maxFeatures)
			spatialCache.setMaxFeatures(maxFeatures);

	}

	public String getCustomForm() {
		return super.getCustomForm();
	}

	public void setCustomFormActive(boolean state) {
		super.hasCustomForm = state;
	}

	public void setIntervalLegend(IntervalLegend intervalLegend) {
		this.intervalLegend = intervalLegend;
	}

	public IntervalLegend getIntervalLegend() {
		return this.intervalLegend;
	}

	public void setUseInterval(boolean useInterval) {
		this.useInterval = useInterval;
	}

	public boolean IsUsedIntervalLegend() {
		return this.useInterval;
	}

}
