/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/

package es.prodevelop.gvsig.mobile.fmap.layer;

import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.io.IOException;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.core.DefaultFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialDriver;

/**
 * Clase padre de los adaptadores de los drivers. De momento mantiene solo el
 * ndice creado sobre la capa
 */
public abstract class VectorialAdapter implements ReadableVectorial {
	protected VectorialDriver driver;

	/**
	 * Establece el driver sobre el que acta el adaptador
	 * 
	 * @param driver
	 */
	public void setDriver(VectorialDriver driver) {
		this.driver = driver;
	}
	
	public Rectangle2D getShapeBounds(int i) throws IOException {
		if (driver == null) {
			return null;
		}
		return driver.getShapeBounds(i);
	}

	/**
	 * Obtiene una referencia al objeto que implementa la interfaz vectorial con
	 * el fin de que las Strategy puedan optimizar en funcin del driver.
	 * 
	 * @return VectorialDriver
	 */
	public VectorialDriver getDriver() {
		return driver;
	}

	/**
	 * Devuelve el DataSource a pasrtir del nombre.
	 * 
	 * @return DataSource.
	 * 
	 * @throws DriverLoadException
	 */
	public abstract SelectableDataSource getRecordset()
			throws DriverLoadException;

	/**
	 * Por defecto devuelve null, y se le pone el icono por defecto. Si el
	 * driver reescribe este mtodo, se usar este icono en el TOC.
	 * 
	 * @return
	 */
	public Image getImageIcon() {
		return null;
	}


	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShapeCount()
	 */
	public int getShapeCount() throws FMapDriverException {
		try {
			return getDriver().getShapeCount();
		} catch (IOException e) {
			throw new FMapDriverException(e.getMessage());
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getFullExtent()
	 */
	public Rectangle2D getFullExtent() throws FMapDriverException {
		try {
			Rectangle2D aux = getDriver().getFullExtent();
			// Para evitar que una nueva capa aadida a una vista vaca no
			// tenga un lienzo para pintar.
			if (aux == null)
				aux = new Rectangle2D.Double(1,1,10,10);
			return aux;
		} catch (IOException e) {
			throw new FMapDriverException(e.getMessage());
		}
	}

	/**
	 * En la implementacin por defecto podemos hacer que cada feature tenga ID =
	 * numero de registro. En el DBAdapter podramos "overrride" este mtodo y
	 * poner como ID de la Feature el campo nico escogido en la base de datos.
	 * 
	 * @param numReg
	 * @return
	 * @throws FMapDriverException
	 */
	public IFeature getFeature(int numReg) throws FMapDriverException {
		IGeometry geom;
		IFeature feat = null;
		try {
			geom = getShape(numReg);
			DataSource rs = getRecordset();
			Value[] regAtt = new Value[rs.getFieldCount()];
			for (int fieldId = 0; fieldId < rs.getFieldCount(); fieldId++) {
				regAtt[fieldId] = rs.getFieldValue(numReg, fieldId);
			}

			feat = new DefaultFeature(geom, regAtt, numReg + "");
		} catch (FMapDriverException e) {
			throw new FMapDriverException(e.getMessage());
		} catch (DriverLoadException e) {
			throw new FMapDriverException(e.getMessage());
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			throw new FMapDriverException(e.getMessage());
		}
		return feat;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getFeatureIterator(java.awt.geom.Rectangle2D,
	 *      java.lang.String) Lo sobreescribirn los adapters para base de datos
	 *      espacial. Por defecto, suponemos un buen acceso aleatorio y usamos
	 *      getFeature(i)
	 */
	/* public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG)
			throws FMapDriverException {
		try {
			return new RandomAccessFeatureIterator(driver, getRecordset(), r,
					strEPSG);
		} catch (DriverLoadException e) {
			throw new FMapDriverException(e);
		}
	}

	public IFeatureIterator getFeatureIterator(String strEPSG)
			throws FMapDriverException {
		try {
			return new RandomAccessFeatureIterator(driver, getRecordset(), strEPSG);
		} catch (DriverLoadException e) {
			throw new FMapDriverException(e);
		}
	} */

}
