package es.prodevelop.gvsig.mobile.fmap.legend;

import java.awt.Color;

import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;

public class IntervalLegend extends SingleSymbolLegend {
	
	private Color[] intervalColors;

	private int intervals;
	private double minValue = 0;
	private double maxValue = 0;
	
	private Color minColor;
	private Color maxColor;
	
	private String fieldName;
	
	private int strokeWidth = 1;
	private Color strokeColor;

	public IntervalLegend(FSymbol style) {
		super(style);
		// TODO Auto-generated constructor stub
	}
	
	public IntervalLegend(FSymbol style, Color minColor, Color maxColor,  int intervals) {
		super(style);
		
		this.minColor = minColor;
		this.maxColor = maxColor;
		this.intervals = intervals;
		
		this.calculateIntervalColors();
	}

	public void calculateIntervalColors() {
		
		if (intervals < 0) {
			// TODO: Lanzar excepcin			
		}

		double initBlue, initGreen, initRed, 
			endBlue, endGreen, endRed, 
			intervalBlue, intervalGreen, intervalRed;
			
		double stepBlue, stepGreen, stepRed;

		initBlue = minColor.getBlue();
		initGreen = minColor.getGreen();
		initRed = minColor.getRed();

		endBlue = maxColor.getBlue();
		endGreen = maxColor.getGreen();
		endRed = maxColor.getRed();

		intervalBlue = endBlue - initBlue;
		intervalGreen = endGreen - initGreen;
		intervalRed = endRed - initRed;
		
		stepBlue = intervalBlue/intervals;
		stepGreen = intervalGreen/intervals;
		stepRed = intervalRed/intervals;		
		
		intervalColors = new Color[intervals + 1];
//		intervalColors = new HashMap(intervals+1);
		for( int i = 0; i<=intervals; i++) {
			if(initRed > 255) initRed = 255;
			if(initGreen > 255) initGreen = 255;
			if(initBlue > 255) initBlue = 255;
			
			if(initRed < 0) initRed = 0;
			if(initGreen < 0) initGreen = 0;
			if(initBlue < 0) initBlue = 0;
			
			Color c = new Color((int)Math.round(initRed), (int)Math.round(initGreen), (int)Math.round(initBlue));
//			intervalColors.put(String.valueOf(i), c);
			intervalColors[i] = c;
			initBlue += stepBlue;
			initGreen += stepGreen;
			initRed += stepRed;
		}
	}
	
	public Color getSymbolForValue(double value) {
		
		if (value < minValue || value > maxValue) {
			//TODO: Lanzar excepcin
		}
//		FSymbol s = FSymbol.getDefaultSymbol();
		
		double step = (maxValue - minValue) / new Double(intervals).doubleValue();		
		
		double currentInterval = value - minValue;
		
		int interval = (int) (Math.round(currentInterval / step));
		
		if(interval < 0) {
			interval = 0;			
		} else if (interval > intervalColors.length - 1) {
			interval = intervalColors.length - 1;
		}
		
//		Color currentColor = (Color)intervalColors.get(String.valueOf(interval));
		
		Color currentColor = intervalColors[interval];
//		System.out.println("interval: " + interval);
//		System.out.println("Color: " + currentColor.toString());
//		s.setFillColor(currentColor);
//		s.setBorderColor(currentColor);
//		s.setStrokeWidth(getDefaultSymbol().getStrokeWidth());
		
		return currentColor; 
				
	}
	
	public Color[] getIntervalColors() {
		return intervalColors;
	}

	public void setIntervalColors(Color[] intervalColors) {
		this.intervalColors = intervalColors;
	}

	public int getIntervals() {
		return intervals;
	}

	public void setIntervals(int intervals) {
		this.intervals = intervals;
	}

	public double getMinValue() {
		return minValue;
	}

	public void setMinValue(double minValue) {
		this.minValue = minValue;
	}

	public double getMaxValue() {
		return maxValue;
	}

	public void setMaxValue(double maxValue) {
		this.maxValue = maxValue;
	}

	public Color getMinColor() {
		return minColor;
	}

	public void setMinColor(Color minColor) {
		this.minColor = minColor;
	}

	public Color getMaxColor() {
		return maxColor;
	}

	public void setMaxColor(Color maxColor) {
		this.maxColor = maxColor;
	}
	
	public String getFieldName() {
		return fieldName;
	}

	public void setFieldName(String fieldName) {
		this.fieldName = fieldName;
	}
	
	public boolean isSame(IntervalLegend e) {
		try {
			if (e == null)
				return false;
			
			if (e.fieldName.compareTo(this.fieldName) != 0) return false;
			if (e.intervals != intervals) return false;
			if (e.maxColor != maxColor) return false;
			if (e.minColor != minColor) return false;
			if (e.maxValue != maxValue) return false;
			if (e.minValue != minValue) return false;
			if (e.strokeColor != strokeColor) return false;
			if (e.strokeWidth != strokeWidth) return false;
		} catch (Exception ex) {
			return false;
		}
		
		return true;
	}
	
	public IntervalLegend cloneIntervalLegend() {
		IntervalLegend res = new IntervalLegend(FSymbol.MEASURE_SYMBOL);
		res.fieldName = fieldName;
		res.intervalColors = intervalColors;
		res.intervals = intervals;
		res.maxColor = maxColor;
		res.minColor = minColor;
		res.maxValue = maxValue;
		res.minValue = minValue;
		res.strokeColor = strokeColor;
		res.strokeWidth = strokeWidth;
		return res;
	}
	
	public int getStrokeWidth() {
		return strokeWidth;
	}

	public void setStrokeWidth(int strokeWidth) {
		this.strokeWidth = strokeWidth;
	}

	public Color getStrokeColor() {
		return strokeColor;
	}

	public void setStrokeColor(Color strokeColor) {
		this.strokeColor = strokeColor;
	}
}