/*
 * Created on 19-feb-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/************************************************
 *												*
 *   Modfied By:								*
 *   Prodevelop Integracin de Tecnologas SL	*
 *   Conde Salvatierra de lava , 34-10			*
 *   46004 Valencia								*
 *   Spain										*
 *												*
 *   +34 963 510 612							*
 *   +34 963 510 968							*
 *   gis@prodevelop.es							*
 *   http://www.prodevelop.es					*
 *												*
 *   gvSIG Mobile Team 2006 					*
 *  											*	 
 ************************************************/
package es.prodevelop.gvsig.mobile.fmap.symbol;

import java.awt.Color;

import org.apache.log4j.Logger;

/**
 * Simplified symbol object.
 * 
 * @author jldominguez
 *
 */
public class FSymbol {

	private static Logger logger = Logger.getLogger(FSymbol.class);
	/**
	 * Symbol used for selected features
	 */
	public static final FSymbol SELECTION_SYMBOL = getSymbolForSelection();
	/**
	 * Symbol uised for the polyline shown during measurements
	 */
	public static final FSymbol MEASURE_SYMBOL = getMeasureSymbol();
	
	private boolean filled = true;
	private Color borderColor = new Color(0, 0, 0);
	private Color fillColor = new Color(0, 0, 0);
	private int strokeWidth = 1;
	
	/**
	 * 
	 * @return the selection symbol
	 */
	public static FSymbol getSymbolForSelection() {
		return new FSymbol(Color.YELLOW.darker(), Color.YELLOW, true, 1);
	}

	/**
	 * 
	 * @return the measurement symbol
	 */
	public static FSymbol getMeasureSymbol() {
		return new FSymbol(Color.ORANGE, Color.ORANGE, false, 1);
	}

	/**
	 * 
	 * @return the default symbol object
	 */
	public static FSymbol getDefaultSymbol() {
		return new FSymbol(Color.BLUE, Color.GREEN, false, 1);
	}

	/**
	 * Constructor
	 * 
	 * @param bc border color
	 * @param fc fill color
	 * @param _filled wherher the polygon has to be filled
	 * @param strokeW stroke width
	 */
	public FSymbol(Color bc, Color fc, boolean _filled, int strokeW) {
		filled = _filled;
		borderColor = bc;
		fillColor = fc;
		strokeWidth = strokeW;
	}
	
	/**
	 * Constructor with default values.
	 *
	 */
	public FSymbol() {
	}

	/**
	 * 
	 * @return whether this symbol is filled
	 */
	public boolean isFilled() {
		return filled;
	}

	/**
	 * Sets the filled property
	 * @param f the new filled property
	 */
	public void setFilled(boolean f) {
		filled = f;
	}



	/**
	 * 
	 * @return the border color
	 */
	public Color getBorderColor() {
		return borderColor;
	}

	/**
	 * Gets the border color
	 * @param color the new border color
	 */
	public void setBorderColor(Color color) {
		borderColor = color;
	}
	
	/**
	 * 
	 * @return the fill color
	 */
	public Color getFillColor() {
		return fillColor;
	}

	/**
	 * Sets the fill color
	 * @param color the new fill color
	 */
	public void setFillColor(Color color) {
		fillColor = color;
	}
	
	/**
	 * 
	 * @return the stroke width
	 */
	public int getStrokeWidth() {
		return strokeWidth;
	}
	
	/**
	 * Sets the stroke width
	 * @param w the new stroke width
	 */
	public void setStrokeWidth(int w) {
		strokeWidth = w;
	}
	
	/**
	 * Array of good taste colors (from Brewer)
	 */
	public static Color[] validColor;
	
	private static int validColorInd = 0;
	
	static {
		validColor = new Color[9];
		try {
			validColor[0] = new Color(166, 206, 227);
			validColor[1] = new Color(178, 223, 138);
			validColor[2] = new Color(251, 154, 153);
			validColor[3] = new Color(51, 160, 44);
			validColor[4] = new Color(227, 26, 28);
			validColor[5] = new Color(31, 120, 180);
			validColor[6] = new Color(253, 191, 111);
			validColor[7] = new Color(255, 127, 0);
			validColor[8] = new Color(202, 178, 214);
		} catch (Throwable th) {
			logger.error("COLOR ERROR: " + th.getMessage());
		}
	}
	
	/**
	 * 
	 * @return a randomly chosen color (among good colors)
	 */
	public static Color getRandomColor() {
		return validColor[(validColorInd++) % validColor.length];
	}
	
	/**
	 * creates random symbol 
	 * @param filled whether it has to be filled or not
	 * @return a random symbol (stroke width = 1)
	 */
	public static FSymbol createRandomSymbol(boolean filled) {
		Color c = getRandomColor();
		return new FSymbol(c.darker(), c, filled, 1);
	}
	


}
