/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.util.bytebuffer;

import java.nio.ByteOrder;

/**
 * This class replaces the byte buffer used in J2SE to store a file in memory.
 * 
 * @see java.nio.ByteOrder
 * 
 * @author jldominguez
 *
 */
public class FalseByteBuffer {
	
	private byte[] bytes;
	private ByteOrder byteOrder = ByteOrder.LITTLE_ENDIAN;
	private int pos = 0;
	
	/**
	 * Constructor
	 * @param bb array of bytes to be stored
	 */
	public FalseByteBuffer(byte[] bb) {
		bytes = bb;
	}
	
	/**
	 * Moves file cursor to position p 
	 * @param p the new position
	 */
	public void position(int p) {
		pos = p;
	}
	
	/**
	 * 
	 * @return current cursor position
	 */
	public int getPosition() {
		return pos;
	}
	
	/**
	 * @return byte pointed by cursor then increases cursor
	 */
	public byte getByte() {
		byte resp = bytes[pos];
		pos++;
		return resp;
	}
	
	/**
	 * Gets one byte
	 * @param ind index of the byte of interest 
	 * @return the byte of interest
	 */
	public byte getByte(int ind) {
		return bytes[ind];
	}

	/**
	 * @return integer pointed by cursor then increases cursor
	 */
	public int getInt() {
		int resp = getInt(bytes, pos);
		pos = pos + 4;
		return resp;
	}
	
	/**
	 * @return short pointed by cursor then increases cursor
	 */
	public short getShort() {
		short resp = getShort(bytes, pos);
		pos = pos + 2;
		return resp;
	}
	
	/**
	 * Gets the integer at a certain position
	 * @param index the index of interest
	 * @return the integer at a certain position
	 */
	public int getInt(int index) {
		int resp = getInt(bytes, index);
		return resp;
	}
	
	/**
	 * Gets some bytes.
	 * @param out the array of bytes to be filled
	 */
	public void getBytes(byte[] out) {
		for (int i=0; i<out.length; i++) {
			out[i] = getByte(pos + i);
		}
		pos = pos + out.length;
	}
	
	/**
	 * @return double pointed by cursor then increases cursor
	 */
	public double getDouble() {
		double resp = getDouble(bytes, pos);
		pos = pos + 8;
		return resp;
	}
	
	private int getInt(final byte[] data, final int offset) {
        return (int) getNumber(data, offset, 4);
    }
	

	private short getShort(final byte[] data, final int offset) {
        return (short) getNumber(data, offset, 2);
    }

	
	private long getNumber(final byte[] data, final int offset, final int size) {
        long result = 0;
        
        if (byteOrder == ByteOrder.LITTLE_ENDIAN) {
        	
            for (int j = offset + size - 1; j >= offset; j--) {
                result <<= 8;
                result |= 0xff & data[j];
            }
            
        } else {

            for (int j = offset; j <= offset + size - 1; j++) {
                result <<= 8;
                result |= 0xff & data[j];
            }

        }

        return result;
    }
	
	private double getDouble(final byte[] data, final int offset) {
        return Double.longBitsToDouble(getNumber(data, offset, 8));
    }

	/**
	 * Sets byte order (big endian, little endian)
	 * @param o byte order (big endian, little endian)
	 */
	public void setByteOrder(ByteOrder o) {
		byteOrder = o;
	}



}
