/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package es.prodevelop.gvsig.mobile.fmap.util.graph2d;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import com.vividsolutions.jts.algorithm.CGAlgorithms;
import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateList;

import es.prodevelop.geodetic.utils.conversion.GeoUtils;
import es.prodevelop.gvsig.mobile.fmap.core.FDot;
import es.prodevelop.gvsig.mobile.fmap.core.FMultiPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.driver.raster.ecw.EcwReader;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;

/**
 * Utility class for graphic purposes.
 * 
 * @see java.awt.geom.GeneralPath
 * @see java.awt.geom.AffineTransform
 * @see java.awt.geom.PathIterator
 * @see es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol
 * 
 * @author jldominguez
 *
 */
public class Graph2DUtilities {

	private static Logger logger = Logger.getLogger(Graph2DUtilities.class.getName());
	private static final boolean PAINT_HOLES_PROPERLY = false;
	

	/**
	 * Draws the general path on the graphics object with the given affine transformation
	 * @param g the graphics object
	 * @param gp the general path
	 * @param at the affine transformation
	 */
	public static void drawGeneralPath(Graphics2D g, GeneralPath gp, AffineTransform at) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		double[] current = new double[6];
		
		PathIterator piter = gp.getPathIterator(at);
		
		while (!piter.isDone()) {
			piter.currentSegment(current);
			x.add(new Integer((int) current[0]));
			y.add(new Integer((int) current[1]));
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);

		g.drawPolyline(gx, gy, gx.length);
	}
	
	
	/**
	 * Converts array list of Integer objects into array of int
	 * @param l
	 * @return array of int
	 */
	public static int[] integerArrayListToIntArray(ArrayList l) {

		int size = l.size();
		int[] resp = new int[size];
		for (int i=0; i<size; i++) {
			resp[i] = ((Integer) l.get(i)).intValue();
		}
		return resp;
	}
	
	/**
	 * Tries to simplify an array of x and y coordinates.
	 * @param xs
	 * @param ys
	 */
	public static void simplifyArrayList(ArrayList xs, ArrayList ys) {
		/*
		Integer x = (Integer)xs.get(xs.size()-1);
		Integer y = (Integer)ys.get(xs.size()-1);
		for (int i=xs.size()-2; i> 0; i--){
			Integer x1 = (Integer)xs.get(i);
			Integer y1 = (Integer)ys.get(i);
			//int diff = Math.abs(x1.intValue()-x.intValue())+Math.abs(y1.intValue()-y.intValue());
			//if (diff>1){
			if(x1.equals(x) && y1.equals(y)){
				xs.remove(i);
				ys.remove(i);
			} else {
				x=x1;
				y=y1;
			}
		}*/
	}
	
	/*
	public static void drawGeneralPath(Graphics2D g, GeneralPath gp) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		float[] current = new float[6];
		
		PathIterator piter = gp.getPathIterator(null);
		
		while (!piter.isDone()) {
			piter.currentSegment(current);
			x.add(new Integer((int) current[0]));
			y.add(new Integer((int) current[1]));
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);

		g.drawPolyline(gx, gy, gx.length);
	}
	*/

	/**
	 * Computes the affine transform needed to go from one rectangle to another
	 * 
	 * @param srcRegion source rectangle
	 * @param dstRegion destination rectangle
	 * @return the affine transform
	 */
	public static AffineTransform createAffineTransform(
			Rectangle2D srcRegion, Rectangle2D dstRegion) {
		
		double[][] matrix = new double[3][3];

		double scale     = dstRegion.getWidth() / srcRegion.getWidth();
		double translate = dstRegion.getMinX() - srcRegion.getMinX() * scale;
		matrix[0][0] = scale;
		matrix[0][2] = translate;

		scale     = - dstRegion.getHeight() / srcRegion.getHeight();
		translate = dstRegion.getMinY() - srcRegion.getMaxY() * scale;
		matrix[1][1] = scale;
		matrix[1][2] = translate;

		matrix[2][2] = 1;
		
		return new AffineTransform(
				matrix[0][0],
				matrix[1][0],
				matrix[0][1],
				matrix[1][1],
				matrix[0][2],
				matrix[1][2]
				);

	}

	/**
	 * Draws a shape on the graphics object with the given affine transform
	 * @param g2d
	 * @param shp
	 * @param symbol
	 */
	public static void drawShape(Graphics2D g2d, Shape shp, FSymbol symbol) {

		
		if (shp instanceof FDot) {
			drawDot(g2d, (FDot) shp);
			return;
		}
		
		if (shp instanceof FPolygon2D) {
			drawPolygon(g2d, (FPolygon2D) shp, symbol);
			return;
		}

		if (shp instanceof FPolyline2D) {
			drawPolyLine(g2d, (FPolyline2D) shp);
			return;
		}

		if (shp instanceof FPoint2D) {
			drawPoint(g2d, (FPoint2D) shp);
			return;
		}
		
		if (shp instanceof FMultiPoint2D) {
			drawMultiPoint(g2d, (FMultiPoint2D) shp);
			return;
		}
		
		logger.error("Did not draw shape of class: " + shp.getClass().getName());
	}
	
	/**
	 * Draws a polygon on the graphics object with the given symbol
	 * @param g2d
	 * @param polygon2D
	 * @param symbol
	 */
	public static void drawPolygon(Graphics2D g2d, FPolygon2D polygon2D, FSymbol symbol) {
					
		PathIterator piter = polygon2D.getPathIterator(null);
		
		if (symbol.isFilled() ) {
			
			if (PAINT_HOLES_PROPERLY) {
				paintFilledWithHoles(g2d, piter, symbol);
			} else {
				fillMultiPolygonIterator(g2d, piter, symbol);			
			}
			
		} else {
			drawMultiPolygonIterator(g2d, piter);			
		}
	}
	
	/**
	 * Draws a polyline on the graphics object
	 * 
	 * @param g2d
	 * @param polyline2D
	 */
	public static void drawPolyLine(Graphics2D g2d, FPolyline2D polyline2D) {
		PathIterator piter = polyline2D.getPathIterator(null);
		drawMultiLineIterator(g2d, piter);
	}
	
	/**
	 * Draws a multipoint  on the graphics object 
	 * @param g2d
	 * @param mp
	 */
	public static void drawMultiPoint(Graphics2D g2d, FMultiPoint2D mp) {
		
		int count = mp.getNumPoints();
		for (int i=0; i<count; i++) {
			drawPoint(g2d, mp.getPoint(i));
		}
	}

	private static void drawShapeBorder(Graphics2D g2d, Shape shp) {

		
		if (shp instanceof FDot) {
			drawDot(g2d, (FDot) shp);
			return;
		}
		
		
		if (shp instanceof FPolygon2D) {
			drawPolygonBorder(g2d, (FPolygon2D) shp);
			return;
		}

		if (shp instanceof FPolyline2D) {
			//drawPolyLine(g2d, (FPolyline2D) shp);
			return;
		}

		if (shp instanceof FPoint2D) {
			drawPoint(g2d, (FPoint2D) shp);
			return;
		}
		
		logger.error("Did not draw shape of class: " + shp.getClass().getName());
	}
	
	
	private static void drawShapeFill(Graphics2D g2d, Shape shp) {

		drawPolygonFill(g2d, (FPolygon2D) shp);
		
	}

	/**
	 * Draws a point  on the graphics object 
	 * @param g2d
	 * @param point2D
	 */
	public static void drawPoint(Graphics2D g2d, FPoint2D point2D) {
		int x = Math.round((float) point2D.getX()) - 1;
		int y = Math.round((float) point2D.getY()) - 1;
		g2d.drawRect(x, y, 3, 3);
	}

	
	/**
	 * Draws a dot on the graphics object 
	 * @param g2d
	 * @param dot
	 */
	public static void drawDot(Graphics2D g2d, FDot dot) {
		int x = Math.round((float) dot.getX());
		int y = Math.round((float) dot.getY());
		//TODO: Improve me please
		g2d.drawLine(x, y, x, y);
	}
	
	/**
	 * Draws a polygon on the graphics object with the given symbol
	 * @param g2d
	 * @param polygon2D
	 * @param symbol
	 */
	public static void drawPolygon(Graphics2D g2d, FPolygon2D polygon2D, FSymbol symbol, double minWX, double minWY, double pixSize, double precission, boolean ultrafast) {
					
		
		PathIterator piter = polygon2D.getPathIterator(null);
		
		if (symbol.isFilled() ) {
			
//			if (PAINT_HOLES_PROPERLY) {
//				paintFilledWithHoles(g2d, piter, symbol);
//			} else {
				fillMultiPolygonIterator2(g2d, polygon2D, symbol, minWX,minWY,pixSize,precission,ultrafast);			
//			}
			
		} else {
			g2d.setColor(symbol.getBorderColor());
			drawMultiPolygonIterator2(g2d, polygon2D, minWX,minWY,pixSize,precission,ultrafast);			
		}
	}

	public static void drawPolygonOld(Graphics2D g2d, FPolygon2D polygon2D, FSymbol symbol) {
		PathIterator piter = polygon2D.getPathIterator(null);
		if (symbol.isFilled() ) {
			
//			if (PAINT_HOLES_PROPERLY) {
//				paintFilledWithHoles(g2d, piter, symbol);
//			} else {
				fillMultiPolygonIterator(g2d, piter, symbol);			
//			}
			
		} else {
			g2d.setColor(symbol.getBorderColor());
			drawMultiPolygonIterator(g2d, piter);			
		}
	}
	
	private static void drawPolygonBorder(Graphics2D g2d, FPolygon2D polygon2D) {
		
		PathIterator piter = polygon2D.getPathIterator(null);
		
		drawMultiPolygonIterator(g2d, piter);			
	}
	
	private static void drawPolygonFill(Graphics2D g2d, FPolygon2D polygon2D) {
		
		PathIterator piter = polygon2D.getPathIterator(null);
		
		fillMultiPolygonIteratorFill(g2d, piter);			

			
	}	

	/**
	 * Draws a polyline on the graphics object
	 * 
	 * @param g2d
	 * @param polyline2D
	 */
	public static void drawPolyLine(Graphics2D g2d, FPolyline2D polygon2D, double minWX, double minWY, double pixSize, double precission, boolean ultrafast) {
		//PathIterator piter = polyline2D.getPathIterator(null);

		drawMultiLineIterator2(g2d, polygon2D,minWX,minWY,pixSize,precission,ultrafast);
	}
	
	public static void drawPolyLineOld(Graphics2D g2d, FPolyline2D polygon2D) {
		PathIterator piter = polygon2D.getPathIterator(null);
		drawMultiLineIterator(g2d, piter);
	}

	private static final int MAXVERTS = 10000;
	private static int[] xs = new int[MAXVERTS];
	private static int[] ys = new int[MAXVERTS];
	
	private static void drawMultiPolygonIterator2(
		Graphics2D g,
		PathIterator piter	){
		int i = 0;
		double[] current = new double[6];
		Point last_pt = null;
		while (!piter.isDone()) {
			if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
				if (i > 0) {
					g.drawPolygon(xs, ys, i);
				}
				i = 0;
				last_pt = null; 
			}
			if ((last_pt == null)
					|| (last_pt.x != ((int) current[0]))
					|| (last_pt.y != ((int) current[1]))) {
				
				last_pt = new Point((int) current[0], (int) current[1]);
				xs[i]=last_pt.x;
				ys[i]=last_pt.y;
				i++;
			}
			
			piter.next();
		}
		g.drawPolygon(xs, ys, i);
	}
	
	private static void drawMultiPolygonIterator(
			Graphics2D g,
			PathIterator piter
			) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		
		Point last_pt = null;

		double[] current = new double[6];

		while (!piter.isDone()) {

			if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
				if (x.size() > 0) {
					int[] gx = integerArrayListToIntArray(x);
					int[] gy = integerArrayListToIntArray(y);
					g.drawPolygon(gx, gy, gx.length);
				}
				x = new ArrayList();
				y = new ArrayList();
				last_pt = null; 
			}
			
			if ((last_pt == null)
					|| (last_pt.x != ((int) current[0]))
					|| (last_pt.y != ((int) current[1]))) {
				
				last_pt = new Point((int) current[0], (int) current[1]);
				x.add(new Integer(last_pt.x));
				y.add(new Integer(last_pt.y));
			}
			
			piter.next();
		}
		ArrayList x1 = new ArrayList(),y1=new ArrayList();
		simplifyArrayList(x,y);
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);
		g.drawPolygon(gx, gy, gx.length);
	}
	
	private static void fillMultiPolygonIterator2(
			Graphics2D g,
			FPolygon2D polygon2D,
			FSymbol sym,
			double minWX, double minWY, double pixSize, double precission
			,boolean ultrafast){
		//float diffSize = 0;
		
		PathIterator piter = polygon2D.getPathIterator(null);
		
			int i = 0;
			double[] current = new double[6];
			Point2D last_pt = null;
			while (!piter.isDone()) {
				if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
					if (i > 0) {
						if (i<=2){
							g.setColor(sym.getBorderColor());
							g.drawRect(xs[0], ys[0], i, i);
						} else {
							g.setColor(sym.getFillColor());
							g.fillPolygon(xs, ys, i);
							g.setColor(sym.getBorderColor());
							g.drawPolyline(xs, ys, i);
						}
					}
					i = 0;
					last_pt = null; 
				}
				if (last_pt == null 
						|| Math.abs(last_pt.getX()-current[0])>precission
						|| Math.abs(last_pt.getY()-current[1])>precission
					){
				
					last_pt = new Point2D.Double(
							current[0], 
							current[1]);
					try{
						xs[i]=(int) ((last_pt.getX()-minWX)/pixSize-0.5);
						ys[i]=(int) ((minWY-last_pt.getY())/pixSize-0.5);
					} catch (ArrayIndexOutOfBoundsException e){
						fillMultiPolygonIterator2(g,polygon2D,sym,minWX,minWY,pixSize,precission*2,ultrafast);
						return;
					}
					//logger.debug("("+last_pt.x+", "+last_pt.y+")");
					i++;
				}
				
				piter.next();
				if (ultrafast){
					if (!piter.isDone()){
						piter.next();
						if (!piter.isDone())
							piter.next();
						if (piter.isDone()){
							xs[i]=xs[0];
							ys[i]=ys[0];
							i++;
						}
					}
				}
			}
			if (ultrafast && i < 10){
				fillMultiPolygonIterator2(g,polygon2D,sym,minWX,minWY,pixSize,precission,false);
			} else if (!ultrafast && i<=2){ 
				g.setColor(sym.getBorderColor());
				g.drawRect(xs[0], ys[0], i, i);
			} else {
 				g.setColor(sym.getFillColor());
				g.fillPolygon(xs, ys, i);
				g.setColor(sym.getBorderColor());
				g.drawPolyline(xs, ys, i);
			}
		}

	private static void drawMultiPolygonIterator2(
			Graphics2D g,
			FPolygon2D polygon2D,
			double minWX, double minWY, double pixSize, double precission
			,boolean ultrafast){
		//float diffSize = 0;
		
		PathIterator piter = polygon2D.getPathIterator(null);
		
			int i = 0;
			double[] current = new double[6];
			Point2D last_pt = null;
			while (!piter.isDone()) {
				if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
					if (i > 0) {
						if (i<=2){
							g.drawRect(xs[0], ys[0], i, i);
						} else {
							g.drawPolyline(xs, ys, i);
						}
					}
					i = 0;
					last_pt = null; 
				}
				if (last_pt == null 
						|| Math.abs(last_pt.getX()-current[0])>precission
						|| Math.abs(last_pt.getY()-current[1])>precission
					){
				
					last_pt = new Point2D.Double(
							current[0], 
							current[1]);
					try{
						xs[i]=(int) ((last_pt.getX()-minWX)/pixSize-0.5);
						ys[i]=(int) ((minWY-last_pt.getY())/pixSize-0.5);
					} catch (ArrayIndexOutOfBoundsException e){
						drawMultiPolygonIterator2(g,polygon2D,minWX,minWY,pixSize,precission*2,ultrafast);
						return;
					}
					//logger.debug("("+last_pt.x+", "+last_pt.y+")");
					i++;
				}
				piter.next();
				if (ultrafast){
					if (!piter.isDone()){
						piter.next();
						if (!piter.isDone())
							piter.next();
						if (piter.isDone()){
							xs[i]=xs[0];
							ys[i]=ys[0];
							i++;
						}
					}
				}
			}
			if (ultrafast && i < 10){
				drawMultiPolygonIterator2(g,polygon2D,minWX,minWY,pixSize,precission,false);
			} else if (!ultrafast && i<=2){ 
				g.drawRect(xs[0], ys[0], i, i);
			} else {
				g.drawPolyline(xs, ys, i);
			}
		}

	private static void fillMultiPolygonIterator(
			Graphics2D g,
			PathIterator piter,
			FSymbol sym
			) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		
		Point last_pt = null;

		double[] current = new double[6];

		while (!piter.isDone()) {

			if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
				if (x.size() > 0) {
					int[] gx = integerArrayListToIntArray(x);
					int[] gy = integerArrayListToIntArray(y);
					g.setColor(sym.getFillColor());
					g.fillPolygon(gx, gy, gx.length);
					g.setColor(sym.getBorderColor());
					g.drawPolyline(gx, gy, gx.length);
				}
				x = new ArrayList();
				y = new ArrayList();
				last_pt = null; 
			}
			
			if ((last_pt == null)
					|| (last_pt.x != ((int) current[0]))
					|| (last_pt.y != ((int) current[1]))) {
				
				last_pt = new Point((int) current[0], (int) current[1]);
				x.add(new Integer(last_pt.x));
				y.add(new Integer(last_pt.y));
			}
			
			piter.next();
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);
		g.setColor(sym.getFillColor());
		g.fillPolygon(gx, gy, gx.length);
		g.setColor(sym.getBorderColor());
		g.drawPolyline(gx, gy, gx.length);
	}
	
	private static void fillMultiPolygonIteratorFill(
			Graphics2D g,
			PathIterator piter
			) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		
		Point last_pt = null;

		double[] current = new double[6];

		while (!piter.isDone()) {

			if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
				if (x.size() > 0) {
					int[] gx = integerArrayListToIntArray(x);
					int[] gy = integerArrayListToIntArray(y);
					//g.setColor(sym.getFillColor());
					g.fillPolygon(gx, gy, gx.length);
					/*g.setColor(sym.getBorderColor());
					g.drawPolyline(gx, gy, gx.length);*/
				}
				x = new ArrayList();
				y = new ArrayList();
				last_pt = null; 
			}
			
			if ((last_pt == null)
					|| (last_pt.x != ((int) current[0]))
					|| (last_pt.y != ((int) current[1]))) {
				
				last_pt = new Point((int) current[0], (int) current[1]);
				x.add(new Integer(last_pt.x));
				y.add(new Integer(last_pt.y));
			}
			
			piter.next();
		}
		ArrayList x1 = new ArrayList(),y1=new ArrayList();
		simplifyArrayList(x,y);
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);
		//g.setColor(sym.getFillColor());
		g.fillPolygon(gx, gy, gx.length);
		/*g.setColor(sym.getBorderColor());
		g.drawPolyline(gx, gy, gx.length);*/
	}
	
	
	private static void drawMultiLineIterator2(
			Graphics2D g,
			FPolyline2D polygon2D,
			double minWX, double minWY, double pixSize, double precission
			,boolean ultrafast){
		//float diffSize = 0;
		
		PathIterator piter = polygon2D.getPathIterator(null);
		
			int i = 0;
			double[] current = new double[6];
			Point2D last_pt = null;
			while (!piter.isDone()) {
				if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
					if (i > 0) {
						g.drawPolyline(xs, ys, i);
					}
					i = 0;
					last_pt = null; 
				}
				
//				System.out.println("==================");
//				System.out.println("X = " + current[0]);
//				System.out.println("Y = " + current[1]);
				if (last_pt == null 
						|| Math.abs(last_pt.getX()-current[0])>precission
						|| Math.abs(last_pt.getY()-current[1])>precission
					){
				
					last_pt = new Point2D.Double(
							current[0], 
							current[1]);
					try{
						xs[i]=(int) ((last_pt.getX()-minWX)/pixSize-0.5);
						ys[i]=(int) ((minWY-last_pt.getY())/pixSize-0.5);
					} catch (ArrayIndexOutOfBoundsException e){
						drawMultiLineIterator2(g,polygon2D,minWX,minWY,pixSize,precission*2,ultrafast);
						return;
					}
					//logger.debug("("+last_pt.x+", "+last_pt.y+")");
					i++;
				}
				piter.next();
				if (ultrafast){
					if (!piter.isDone()){
						piter.next();
						if (!piter.isDone())
							piter.next();
//						if (piter.isDone()){
//							xs[i]=xs[0];
//							ys[i]=ys[0];
//							i++;
//						}
					}
				}
			}
			if (ultrafast && i < 5){
				drawMultiLineIterator2(g,polygon2D,minWX,minWY,pixSize,precission,false);
			} else {
				g.drawPolyline(xs, ys, i);
			}
		}

	private static void drawMultiLineIterator(
			Graphics2D g,
			PathIterator piter
			) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		
		Point last_pt = null;

		double[] current = new double[6];

		while (!piter.isDone()) {

			if (piter.currentSegment(current) == PathIterator.SEG_MOVETO) {
				if (x.size() > 0) {
					int[] gx = integerArrayListToIntArray(x);
					int[] gy = integerArrayListToIntArray(y);
					g.drawPolyline(gx, gy, gx.length);
				}
				x = new ArrayList();
				y = new ArrayList();
				last_pt = null; 
			}
			
			if ((last_pt == null)
					|| (last_pt.x != ((int) current[0]))
					|| (last_pt.y != ((int) current[1]))) {
				
				last_pt = new Point((int) current[0], (int) current[1]);
				x.add(new Integer(last_pt.x));
				y.add(new Integer(last_pt.y));
			}
			
			piter.next();
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);
		g.drawPolyline(gx, gy, gx.length);
	}
	
	
	
	private static PathIterator getPathIterator(Shape shp) {
		
		if (shp instanceof FPolyline2D) {
			return ((FPolyline2D) shp).getPathIterator(null);
		}
		
		if (shp instanceof FPolygon2D) {
			return ((FPolygon2D) shp).getPathIterator(null);
		}
		
		if (shp instanceof FPoint2D) {
			return ((FPoint2D) shp).getPathIterator(null);
		}
		
		logger.error("Did not get a path iterator: null"); 
		return null; // new PointIterator(new Point(0, 0), AffineTransform.getTranslateInstance(0, 0));
		
	}
	
	private static void printCoordList(CoordinateList list) {
		for (int i=0; i<list.size(); i++) {
			logger.debug("coordenadas [" + i + "]: " + ((Coordinate) list.get(i)).x + ", " + ((Coordinate) list.get(i)).y + ", ");
		}
	}
	
	/**
	 * Gets the area of the path iterator
	 * @param piter the path iterator
	 * @return the area of the path iterator
	 */
	public static double getPathIteratorArea(PathIterator piter) {
		
		if (piter == null) {
			return 0;
		}

		double resp = 0;

		double[] theData = new double[6];
		Coordinate c1 = null;
		Coordinate firstc = null;
		CoordinateList coordList = null;

		while (!piter.isDone()) {

			int type = piter.currentSegment(theData);
			
			if (type == PathIterator.SEG_MOVETO) {
				if (coordList != null) {
					coordList.add(firstc, true);
					// printCoordList(coordList);
					resp = resp + CGAlgorithms.signedArea(coordList.toCoordinateArray());
				}
				firstc = new Coordinate(theData[0], theData[1]);
				coordList = new CoordinateList();
			}
			c1 = new Coordinate(theData[0], theData[1]);
			coordList.add(c1, true);
			piter.next();
		}
		
		if (coordList != null) {
			coordList.add(firstc, true);
			// printCoordList(coordList);
			// resp = resp + Math.abs(CGAlgorithms.signedArea(coordList.toCoordinateArray()));
			resp = resp + CGAlgorithms.signedArea(coordList.toCoordinateArray());
		}
		return Math.abs(resp);
	}
	
	/**
	 * Gets the area of a given spherical path iterator
	 * 
	 * @param piter 
	 * @return the area
	 */
	public static double getSphericalPathIteratorArea(PathIterator piter) {
		
		if (piter == null) {
			return 0;
		}

		double resp = 0;

		double[] theData = new double[6];
		Coordinate c1 = null;
		Coordinate firstc = null;
		CoordinateList coordList = null;

		while (!piter.isDone()) {

			int type = piter.currentSegment(theData);
			
			if (type == PathIterator.SEG_MOVETO) {
				if (coordList != null) {
					coordList.add(firstc, true);
					// printCoordList(coordList);
					resp = resp + GeoUtils.sphericalAreaFromCoordinatesArray(coordList.toCoordinateArray());
				}
				firstc = new Coordinate(theData[0], theData[1]);
				coordList = new CoordinateList();
			}
			c1 = new Coordinate(theData[0], theData[1]);
			coordList.add(c1, true);
			piter.next();
		}
		
		if (coordList != null) {
			coordList.add(firstc, true);
			// printCoordList(coordList);
			// resp = resp + Math.abs(CGAlgorithms.signedArea(coordList.toCoordinateArray()));
			resp = resp + GeoUtils.sphericalAreaFromCoordinatesArray(coordList.toCoordinateArray());
		}
		return Math.abs(resp);
	}
	
	
	private static void paintFilledWithHoles(Graphics2D g, PathIterator piter, FSymbol sym) {
		
		ArrayList _split = split(piter);
		int[] indices = getShellIndicesAndHoleCount(_split);
		Object[] shellXYAndHolesList = groupParts(_split, indices);
		int nofshells = shellXYAndHolesList.length / 3;

		int[] shellx, shelly;
		ArrayList holesXY;

		for (int i=0; i<nofshells; i++) {
			
			shellx = (int[]) shellXYAndHolesList[3 * i];
			shelly = (int[]) shellXYAndHolesList[3 * i + 1];
			holesXY = (ArrayList) shellXYAndHolesList[3 * i + 2];

			if (holesXY == null) {
				
				g.setColor(sym.getFillColor());
				g.fillPolygon(shellx, shelly, shellx.length);
				
				g.setColor(sym.getBorderColor());
				g.drawPolyline(shellx, shelly, shellx.length);

			} else {
				
					// --- not properly: shell: fill + border, holes: border
					g.setColor(sym.getFillColor());
					g.fillPolygon(shellx, shelly, shellx.length);
					
					g.setColor(sym.getBorderColor());
					g.drawPolyline(shellx, shelly, shellx.length);
					
					int[] hx, hy;
					int nh = holesXY.size() / 2;
					for (int h=0; h<nh; h++) {
						hx = (int[]) holesXY.get(2 * h);
						hy = (int[]) holesXY.get(2 * h + 1);
						g.drawPolyline(hx, hy, hx.length);
					}
					
					// ------- TODO ----------
					/*
					Object[] img_coords = getPaintedPolygonWithHoles(shellx, shelly, holesXY, sym);
					Image img = (Image) img_coords[0];
					int x = ((Integer) img_coords[1]).intValue();
					int y = ((Integer) img_coords[2]).intValue();
					g.drawImage(img, x, y, null);
					*/
			}

		}
	}

	private static Object[] getPaintedPolygonWithHoles(
			int[] shellx,
			int[] shelly,
			ArrayList holesXYlist,
//			int[] holex,
//			int[] holey
			FSymbol sym
			) {
		
		int minx = shellx[0];
		int maxx = shellx[0];
		int miny = shelly[0];
		int maxy = shelly[0];
		
		int nofp = shellx.length;
		int auxx, auxy;
		
		for (int i=0; i<nofp; i++) {
			auxx = shellx[i];
			auxy = shelly[i];
			if (auxx > maxx) maxx = auxx;
			if (auxx < minx) minx = auxx;
			if (auxy > maxy) maxy = auxy;
			if (auxy < miny) miny = auxy;
		}
		
		int w = maxx - minx + 1;
		int h = maxy - miny + 1;
		
		BufferedImage resp_img = Utils.createBufferedImage(w, h);
		Graphics2D g = (Graphics2D) resp_img.getGraphics();
		
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, w, h);
		
		g.translate(-minx, -miny);

		g.setColor(Color.WHITE);
		g.fillPolygon(shellx, shelly, nofp);
		
		g.setColor(Color.BLACK);
		int nofholes = holesXYlist.size() / 2;
		
		int[] holex, holey;
		for (int i=0; i<nofholes; i++) {
			holex = (int[]) holesXYlist.get(2 * i);
			holey = (int[]) holesXYlist.get(2 * i + 1);
			g.fillPolygon(holex, holey, holex.length);
		}

		int[] pixels = new int[w*h];
		pixels = resp_img.getRGB(0, 0, w, h, null, 0, w);

		byte[] alpha = getRedBand(pixels);

		g.setColor(sym.getFillColor());
		g.fillPolygon(shellx, shelly, nofp);

		pixels = resp_img.getRGB(0, 0, w, h, null, 0, w);
		

		pixels = setAlphaBand(pixels, alpha);
		byte[] png = EcwReader.paintArray(pixels, w, h, 0, true);
		
		Object[] resp = new Object[3];
		resp[0] = Toolkit.getDefaultToolkit().createImage(png);
		resp[1] = new Integer(minx);
		resp[2] = new Integer(miny);
		return resp; 
	}
	
	private static int[] setAlphaBand(int[] _p, byte[] _a) {
		int size = _p.length;
		int[] resp = new int[size];
		for (int i=0; i<size; i++) {
			resp[i] = (_p[i] & 0x00ffffff) + (_a[i] << 24);
		}
		return resp;
	}

	private static byte[] getRedBand(int[] _p) {
		
		int size = _p.length;
		byte[] resp = new byte[size];
		
		for (int i=0; i<size; i++) {
			resp[i] = (byte) ((_p[i] >> 16) & 0xff);
		}
		return resp;
	}

	private static String getFourBands(int p) {
		
		return
		"" + ((p & 0xff000000) >> 24) +
		", " + ((p & 0x00ff0000) >> 16) +
		", " + ((p & 0x0000ff00) >> 8) +
		", " + (p & 0x000000ff);
		
	}
	
	private static boolean isShell(int[] verticesx, int[] verticesy) {
		
		int size = verticesx.length;
		if (size < 3) return true;
		
		Coordinate[] cc = new Coordinate[size];
		
		for (int i=0; i<size; i++) {
			cc[i] = new Coordinate(verticesx[i], verticesy[i]);
		}
		
		boolean r = CGAlgorithms.isCCW(cc);
		return r;
	}
	
	private static ArrayList split(PathIterator pit) {

		ArrayList resp = new ArrayList();
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		
		Point last_pt = null;

		double[] current = new double[6];

		while (!pit.isDone()) {

			if (pit.currentSegment(current) == PathIterator.SEG_MOVETO) {
				if (x.size() > 0) {
					int[] gx = integerArrayListToIntArray(x);
					int[] gy = integerArrayListToIntArray(y);
					resp.add(gx); resp.add(gy);
				}
				x = new ArrayList();
				y = new ArrayList();
				last_pt = null; 
			}
			
			if ((last_pt == null)
					|| (last_pt.x != ((int) current[0]))
					|| (last_pt.y != ((int) current[1]))) {
				
				last_pt = new Point((int) current[0], (int) current[1]);
				x.add(new Integer(last_pt.x));
				y.add(new Integer(last_pt.y));
			}
			
			pit.next();
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);
		resp.add(gx); resp.add(gy);

		return resp;
	}
	
	private static int[] getShellIndicesAndHoleCount(ArrayList parts) {
		
		ArrayList resp = new ArrayList();
		int size = parts.size() / 2;
		int[] cx, cy;
		
		if (size == 1) {
			
			// one item, we understand its a shell
			resp.add(new Integer(0));
			
		} else {
			// ---------- more than one item
			for (int i=0; i<size; i++) {
				cx = (int[]) parts.get(2 * i);
				cy = (int[]) parts.get(2 * i + 1);
				if (isShell(cx, cy)) resp.add(new Integer(i));
			}
			// ---------- more than one item
		}
		
		
		resp.add(new Integer(size));
		int resp_size = resp.size() - 1;
		
		int[] indices = new int[2 * resp_size];
		for (int i=0; i<resp_size; i++) {
			int this_index = ((Integer) resp.get(i)).intValue();
			int next_index = ((Integer) resp.get(i + 1)).intValue();
			indices[2 * i] = this_index; 
			indices[2 * i + 1] = next_index - this_index - 1;
		}
		
		return indices;
	}
	
	private static Object[] groupParts(ArrayList parts, int[] indices) {
		
		int nofp = indices.length / 2;
		Object[] resp = new Object[3 * nofp];
		
		int shell_ind = 0;
		int nofholes = 0;
		
		for (int i=0; i<nofp; i++) {
			
			shell_ind = indices[2 * i];
			nofholes = indices[2 * i + 1];
			
			resp[3 * i] = parts.get(2 * i);
			resp[3 * i + 1] = parts.get(2 * i + 1);
			
			if (nofholes == 0) {
				resp[3 * i + 2] = null;
			} else {
				ArrayList holesxy = new ArrayList();
				for (int h=0; h<(2 * nofholes); h++)
					holesxy.add(parts.get(2 * shell_ind + 2 + h));
				resp[3 * i + 2] = holesxy;
			}
		}
		return resp;
	}
	
	private static void printIntArrayToLogger(int[] arr, String name) {
		
		logger.debug("--- ARRAY: <" + name + "> --- START ---");
		for (int i=0; i<arr.length; i++) {
			logger.debug("-- ITEM [ " + i + " ] = " + arr[i]);
		}
		logger.debug("--- ARRAY: <" + name + "> --- END   ---");
		
	}
	
	/**
	 * Draws the path iterator on the graphics object with the given affine transformation
	 * @param g the graphics object
	 * @param piter the path iterator
	 * @param at the affine transformation (can be null)
	 */
	public static void drawPathIterator(Graphics2D g, PathIterator piter, AffineTransform at) {
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		double[] current = new double[6];

		while (!piter.isDone()) {
			piter.currentSegment(current);
			x.add(new Integer((int) current[0]));
			y.add(new Integer((int) current[1]));
			piter.next();
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);

		g.drawPolyline(gx, gy, gx.length);		
	}

	public static void drawPathIteratorLastVertices(
			Graphics2D g,
			PathIterator piter,
			AffineTransform at,
			int currentLength,
			int max_vertices) {
		
		ArrayList x = new ArrayList();
		ArrayList y = new ArrayList();
		double[] current = new double[6];

		int count = 0;
		while (!piter.isDone()) {
			
			if ((currentLength - count) < max_vertices) {
				piter.currentSegment(current);
				x.add(new Integer((int) current[0]));
				y.add(new Integer((int) current[1]));
			}
			
			count++;
			piter.next();
		}
		
		int[] gx = integerArrayListToIntArray(x);
		int[] gy = integerArrayListToIntArray(y);

		g.drawPolyline(gx, gy, gx.length);			
		
		
		// TODO Auto-generated method stub
		
	}
}
