/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.exceptions;

import java.util.HashMap;
import java.util.Map;

import org.gvsig.exceptions.BaseException;

/**
 * This class represents an exception launched when a curve 
 * is tried to get converted but it fails.
 * 
 * <br>2008 PRODEVELOP SL</br> 
 * @author Carlos Snchez Perin <br>csanchez@prodevelop.es</br>
 */
public class CurveConversionWarning extends BaseException {
	private static final long serialVersionUID = 5231975614662709375L;
	private String featureID = null;
	
	public CurveConversionWarning(String featureID) {
		this.featureID = featureID;
		initialize();		
	}
	
	/**
	 * Initialize the exception properties.
	 */
	private void initialize() {
		messageKey = "gpe_gvsig_curve_conversion_warning";
		formatString = "gvSIG doesn't supports curves. The " +
			"geometry with id %(featureID) has been converted " +
			"to a lineString";		
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.gvsig.exceptions.BaseException#values()
	 */
	protected Map values() {
		HashMap params = new HashMap();
		if (featureID != null){
			params.put("featureID", featureID);
		}
		return params;	
	}
	
}
