/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.fmap;

import java.io.IOException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.hardcode.driverManager.Driver;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.IDataSourceListener;
import com.hardcode.gdbms.engine.data.SourceInfo;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.data.persistence.Memento;
import com.hardcode.gdbms.engine.data.persistence.MementoException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueCollection;
import com.hardcode.gdbms.engine.values.ValueFactory;

import es.prodevelop.gvsig.mobile.gpe.model.GPEElement;
import es.prodevelop.gvsig.mobile.gpe.model.GPEFeature;

/**
 * Implementation of DataSource to get data from libGPE parsed files 
 * layers. Creates the data structure to get info about	 
 * features.											
 * 														
 * <br>2008 PRODEVELOP SL</br> 
 * @author Carlos Snchez Perin <br>csanchez@prodevelop.es</br>
 */
public class DataSourceGpe implements DataSource {
	
	//----------------
	//GLOBAL VARIABLES
	//----------------
	private static Logger logger = LoggerFactory.getLogger(DataSourceGpe.class);
	
	//The attributes position
	private LinkedHashMap fieldPosition = null; 
	private HashMap fields = null; 
	private HashMap fieldWidth = null; 
	
	//The data
	private ArrayList features = null;
	
	public DataSourceGpe(ArrayList feats,LinkedHashMap pos, HashMap f){
		fields = f;
		fieldPosition = pos; 
		features = feats;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#addDataSourceListener(com.hardcode.gdbms.engine.data.IDataSourceListener)
	 */
	public void addDataSourceListener(IDataSourceListener arg0) {
		logger.warn("Method addDataSourceListener(), nothing done.");
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getAsString()
	 */
	public String getAsString() throws DriverException {
		logger.warn("Method getAsString() was called, returned getName()");
		return getName();
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getDataSourceFactory()
	 */
	public DataSourceFactory getDataSourceFactory() {
		logger.warn("Method getDataSourceFactory() was called, returned NULL");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getDataWare(int)
	 */
	public DataWare getDataWare(int arg0) throws DriverException {
		logger.warn("Method getDataWare(" + arg0 + ") returned NULL.");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getDriver()
	 */
	public Driver getDriver() {
		logger.warn("Method getDriver() returned NULL.");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getFieldIndexByName(java.lang.String)
	 */
	public int getFieldIndexByName(String name) throws DriverException {
		int size = getFieldCount();
		for (int i=0; i<size; i++) {
			if (name.compareToIgnoreCase(getFieldName(i)) == 0){ 
				return i;
			}
		}
		logger.error("Field name not found: " + name);
		return -1;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getFieldNames()
	 */
	public String[] getFieldNames() throws DriverException {
		int size = getFieldCount();
		String[] names = new String[size];
		for (int i=0; i<size; i++){
			names[i] = getFieldName(i).trim();
		}
		return names;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getMemento()
	 */
	public Memento getMemento() throws MementoException {
		logger.warn("Method getMemento() was called, returned NULL");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getName()
	 */
	public String getName() {
		return "GPE Data Source";
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getPKCardinality()
	 */
	public int getPKCardinality() throws DriverException {
		return getFieldCount();
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getPKName(int)
	 */
	public String getPKName(int arg0) throws DriverException {
		logger.warn("Method getPKName() was called, returned NULL");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getPKNames()
	 */
	public String[] getPKNames() throws DriverException {
		logger.warn("Method getPKNames() was called, returned NULL");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getPKType(int)
	 */
	public int getPKType(int arg0) throws DriverException {
		logger.warn("Method getPKType(" + arg0 + ") was called, returned getFieldType(...)");
		return getFieldType(arg0);
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getPKValue(long)
	 */
	public ValueCollection getPKValue(long arg0) throws DriverException {
		logger.warn("Method getPKValue(" + arg0 + ") was called, returned NULL");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getPrimaryKeys()
	 */
	public int[] getPrimaryKeys() throws DriverException {
		logger.warn("Method getPrimaryKeys() was called, returned all index.");
		int size = getFieldCount();
		int[] index = new int[size];
		for (int i=0; i<size; i++) {
			index[i] = i;
		}
		return index;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getRow(long)
	 */
	public Value[] getRow(long rowind) throws DriverException {
		//this gets the number of all the properties founded into the file.
		int size = getFieldCount();
		Value[] resp = new Value[size];
		for (int i=0; i<size; i++) {
			resp[i] = getValueOfItsType(i, (int) rowind);
		}
		return resp;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getSourceInfo()
	 */
	public SourceInfo getSourceInfo() {
		logger.warn("Method getSourceInfo() was called, returned NULL");
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#getWhereFilter()
	 */
	public long[] getWhereFilter() throws IOException {
		logger.warn("Method getWhereFilter() was called, returned new long[0]");
		return new long[0];
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#isVirtualField(int)
	 */
	public boolean isVirtualField(int arg0) throws DriverException {
		logger.warn("Method isVirtualField(" + arg0 + ") returned FALSE.");
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#reload()
	 */
	public void reload() throws DriverException, IOException {
		stop(); 
		start();		
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#remove()
	 */
	public void remove() throws DriverException {
		logger.warn("Method remove() was called, nothing done.");
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#removeDataSourceListener(com.hardcode.gdbms.engine.data.IDataSourceListener)
	 */
	public void removeDataSourceListener(IDataSourceListener arg0) {
		logger.warn("Method removeDataSourceListener(), nothing done.");
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
	 */
	public void setDataSourceFactory(DataSourceFactory arg0) {
		logger.warn("Method setDataSourceFactory(...) was called, nothing done.");
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#setSourceInfo(com.hardcode.gdbms.engine.data.SourceInfo)
	 */
	public void setSourceInfo(SourceInfo arg0) {
		logger.warn("Method setSourceInfo(...) was called, nothing done.");		
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#start()
	 */
	public void start() throws DriverException {
		logger.warn("Method start() was called, nothing done.");
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.DataSource#stop()
	 */
	public void stop() throws DriverException {
		logger.warn("Method stop() was called, nothing done.");
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldCount()
	 */
	public int getFieldCount() throws DriverException {
		return fieldPosition.size();
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldName(int)
	 */
	public String getFieldName(int fieldId) throws DriverException {
		return (String) fields.get(new Integer(fieldId));
	}

	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldType(int)
	 */
	public int getFieldType(int i) throws DriverException {
		  if (getRowCount() > 1){
	            Value value = getFieldValue(0,i);
	            return value.getSQLType();
		  }
		  return Types.VARCHAR;
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getFieldWidth(int)
	 */
	public int getFieldWidth(int i) throws DriverException {
		if (fieldWidth == null){
			fieldWidth = new HashMap();
			for (int j=0 ; j<getFieldCount() ; j++){
				if (getRowCount() > 1){
					Value value = getFieldValue(0,j);
					fieldWidth.put(new Integer(j), new Integer(value.getWidth()));
	            }else{
	            	fieldWidth.put(new Integer(j), new Integer(0));
	            }	
			}
		}
		return ((Integer) fieldWidth.get(new Integer(i))).intValue();
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.driver.ReadAccess#getRowCount()
	 */
	public long getRowCount() throws DriverException {
		return features.size();
	}
	
	/**
	 * Get Field Value by its row number and its field identifier. 
	 * It's similar as get the value by the row and column.
	 * 
	 * @param rowIndex
	 * @param fieldId
	 * @return com.hardcode.gdbms.engine.values.Value
	 */
	public Value getFieldValue(long rowIndex, int fieldId) throws DriverException {
		String attName = getFieldName(fieldId);
		GPEElement element = (GPEElement) ((GPEFeature) features.get((int)rowIndex)).getElements().get(attName);
		if (element != null){
			return element.getValue();
		}
		return ValueFactory.createNullValue();
	}
	
	/**
	 * Get the a value by its row and attribute type. 
	 * 
	 * @param fieldindex
	 * @param row
	 * @return com.hardcode.gdbms.engine.values.Value
	 */
	private Value getValueOfItsType(int fieldindex, int row) {	
		Value value=null;
		String attName = null;
		try {
			attName = getFieldName(fieldindex);
		} catch (DriverException e) {
			logger.warn("ERROR: Atrribute number "+fieldindex+" on feature "+ row +" can't read");
			e.printStackTrace();
		}
		int size = ((GPEFeature) features.get((int)row)).getElements().size();
		if(fieldindex < size){
				GPEElement element = (GPEElement) ((GPEFeature) features.get((int)row)).getElements().get(attName);
				if (element!= null)
					value = element.getValue();
		}
		return value;	
	}	
}
