/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.handlers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.ArrayList;

import org.gvsig.gpe.parser.GPEContentHandler;
import org.gvsig.gpe.parser.GPEErrorHandler;
import org.gvsig.gpe.parser.ICoordinateIterator;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.core.FGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.GeneralPathX;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.ShapeFactory;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrDefault;
import es.prodevelop.gvsig.mobile.fmap.proj.CRSFactory;
import es.prodevelop.gvsig.mobile.fmap.proj.IProjection;
import es.prodevelop.gvsig.mobile.gpe.exceptions.CurveConversionWarning;
import es.prodevelop.gvsig.mobile.gpe.layer.FlyrGpeVect;
import es.prodevelop.gvsig.mobile.gpe.layer.ILayerGpe;
import es.prodevelop.gvsig.mobile.gpe.layer.LayerGpeFactory;
import es.prodevelop.gvsig.mobile.gpe.model.GPEBBox;
import es.prodevelop.gvsig.mobile.gpe.model.GPECurve;
import es.prodevelop.gvsig.mobile.gpe.model.GPEElement;
import es.prodevelop.gvsig.mobile.gpe.model.GPEFeature;
import es.prodevelop.gvsig.mobile.gpe.model.GPEGeometry;
import es.prodevelop.gvsig.mobile.gpe.model.GPEMultiGeometry;
import es.prodevelop.gvsig.mobile.gpe.util.GPETypesConversion;

/**
 * Class to implement the libGPE ContentHandler for 	
 * gvsig mobile.										
 * Only fill the content events needed.					
 * 														
 * PRODEVELOP 2008										
 * @author Carlos Snchez (csanchez@prodevelop.es)		
 */

public class FmapContentHandler extends GPEContentHandler {
	private Rectangle2D extent = null;
	private MapContext mapControl = null;
	private FlyrGpeVect parentLayer=null;

		
	//------------
	//CONSTRUCTOR
	//------------
	//-----------------------------------------
	// MapContext and ErrorHandler, are needed.
	//------------------------------------------
	public FmapContentHandler(MapContext mc, GPEErrorHandler errorHandler){
		super();
		this.setMapControl(mc);
		setErrorHandler(errorHandler);
	}
	
	//-----------------------------------------
	//-----------------------------------------
	// START METHOD's
	//-----------------------------------------
	//-----------------------------------------
	public Object startLayer(String id, String namespaceURI, String name, String description, String srs, Object parentLayer, Object bBox) {
		FLayer layer = null;
		layer = LayerGpeFactory.createVectorialLayer(getMapControl(), name, srs, getSchemaDocument(), name, (GPEErrorHandler )getErrorHandler(), (ILayerGpe) parentLayer);
		if (parentLayer != null){
			((ILayerGpe)parentLayer).addLayer(layer);
		}
		this.parentLayer=(FlyrGpeVect) layer;
		return 	layer;	
	}
	
	//-----------------------------------------
	//-----------------------------------------
	public Object startPoint(String id,ICoordinateIterator coord, String srs) {
		double[] buffer = new double[coord.getDimension()];
		try {
			if (coord.hasNext()){
				coord.next(buffer);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		double x =buffer[0];
		double y= buffer[1];
		return new GPEGeometry(id, ShapeFactory.createPoint2D(x, y), srs);
	}
	
	//-----------------------------------------
	//-----------------------------------------
	public Object startLineString(String id, ICoordinateIterator coord, String srs) {
		GeneralPathX gp = new GeneralPathX();
		int i=0;
		try {
			while(coord.hasNext()){
				double[] buffer = new double[coord.getDimension()];
				coord.next(buffer);
				double x =buffer[0];
				double y= buffer[1];
				if (i == 0)
					gp.moveTo(x, y);
				else
					gp.lineTo(x, y);
				i++;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return new GPEGeometry(id, ShapeFactory.createPolyline2D(gp),srs);
	}
	
	//-----------------------------------------
	//-----------------------------------------
	public Object startPolygon(String id, ICoordinateIterator coord, String srs) {
		GeneralPathX gp = new GeneralPathX();
		int i=0;
		try {
			while(coord.hasNext()){
				double[] buffer = new double[coord.getDimension()];
				coord.next(buffer);
				double x =buffer[0];
				double y= buffer[1];
				if (i == 0)
					gp.moveTo(x, y);
				else
					gp.lineTo(x, y);
				i++;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		//COMPROBAR SI EL POLIGONO ESTA CERRADO...SI NO LO HACE CREATEPOLYGON2D
		return new GPEGeometry(id, ShapeFactory.createPolygon2D(gp), srs);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startInnerPolygon(String id,  ICoordinateIterator coord, String srs) {
		GeneralPathX gp = new GeneralPathX();
		int i=0;
		try {
			while(coord.hasNext()){
				double[] buffer = new double[coord.getDimension()];
				coord.next(buffer);
				double x =buffer[0];
				double y= buffer[1];
				if (i == 0)
					gp.moveTo(x, y);
				else
					gp.lineTo(x, y);
				i++;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return new GPEGeometry(id, ShapeFactory.createPolygon2D(gp), srs);
	}
	
	//-----------------------------------------
	//-----------------------------------------
	public Object startBbox(String id,  ICoordinateIterator coords, String srs) {
		return new GPEBBox(id,coords,srs);		
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startElement(String namespace, String name, Object value, Object parentElement) {
		GPEElement element = new GPEElement(name, GPETypesConversion.fromJavaTogvSIG(value), (GPEElement)parentElement);
		addParentElementToElement(parentElement, element);
		return element;
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startFeature(String id, String namespace, String xsElementName, Object layer) {
		return new GPEFeature(id, namespace, xsElementName);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startLinearRing(String id, ICoordinateIterator coord, String srs) {
		GeneralPathX gp = new GeneralPathX();
		int i=0;
		try {
			while(coord.hasNext()){
				double[] buffer = new double[coord.getDimension()];
				coord.next(buffer);
				double x =buffer[0];
				double y= buffer[1];
				if (i == 0)
					gp.moveTo(x, y);
				else
					gp.lineTo(x, y);
				i++;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return new GPEGeometry(id, ShapeFactory.createPolygon2D(gp), id);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startMultiPoint(String id, String srs) {
		return new GPEMultiGeometry(id, srs);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startMultiLineString(String id, String srs) {
		return new GPEMultiGeometry(id, srs);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startCurve(String id, ICoordinateIterator coord,	String srs) {
		getErrorHandler().addWarning(new CurveConversionWarning(id));
		GeneralPathX gp = new GeneralPathX();
		int i=0;
		try {
			while(coord.hasNext()){
				double[] buffer = new double[coord.getDimension()];
				coord.next(buffer);
				double x =buffer[0];
				double y= buffer[1];
				if (i == 0)
					gp.moveTo(x, y);
				else
					gp.lineTo(x, y);
				i++;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return new GPEGeometry(id, ShapeFactory.createPolyline2D(gp), srs);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startCurve(String id, String srs) {
		getErrorHandler().addWarning(new CurveConversionWarning(id));
		return new GPECurve(id, srs);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startMultiCurve(String id, String srs) {
		return new GPEMultiGeometry(id, srs);		
	}
	//-----------------------------------------
	//-----------------------------------------
	// ADD's METHODS
	//-----------------------------------------
	//-----------------------------------------
	public void addGeometryToFeature(Object geometry, Object feature) {
		((GPEFeature)feature).setGeometry((GPEGeometry)geometry);
	}	
	//-----------------------------------------
	//-----------------------------------------
	public void addBboxToLayer(Object bbox, Object layer) {
		GPEBBox gpeBBox = (GPEBBox)bbox;
		//If the Extent it's null we'll have to calculate it latter with the geometries coordinates.
		if (gpeBBox!=null){
			if (gpeBBox.getSrs() != null){
				IProjection projection = null;
				try{
					CRSFactory.getCRS(gpeBBox.getSrs());
				}catch(Exception e){
					//If the CRS factory has an error.
				}
				if ((projection != null) && (!(projection.equals(((FLyrDefault) layer).getProjection())))){
					//TODO reproyectar la bbox y asignarsela a la capa	REPROYECTAR NO SE PUEDE...			
				}
			}
			((ILayerGpe)layer).setExtent(gpeBBox.getBbox2D());
		}
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addElementToFeature(Object element, Object feature) {
		((GPEFeature)feature).addElement((GPEElement)element);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addFeatureToLayer(Object feature, Object layer) {
		GPEGeometry geom =((GPEFeature)feature).getGeometry();
		if(geom!= null)
		{
			if(geom.getShapeBounds()!=null)
				setExtent(geom.getShapeBounds());
		}
		((FlyrGpeVect)layer).addFeature((GPEFeature) feature);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addFeatureToFeature(Object feature, Object parent) {
		//Duplicates the feature because one follow the tree to relay the nodes and the other fill the array.
		((FlyrGpeVect)parentLayer).addFeature((GPEFeature) feature);
		//Fill an array.
		((GPEFeature)parent).addFeature((GPEFeature) feature);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addInnerPolygonToPolygon(Object innerPolygon, Object Polygon) {
		((GPEGeometry)Polygon).addGeometry((GPEGeometry)innerPolygon);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addNameToFeature(String name, Object feature) {
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addNameToLayer(String name, Object layer) {		
		((FLayer)layer).setName(name);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addParentElementToElement(Object parent, Object element) {
		if (parent!=null)
			((GPEElement) parent).addChild((GPEElement) element);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addParentLayerToLayer(Object parent, Object layer) {
		try {
			((FlyrGpeVect)parent).addLayer((FlyrGpeVect)layer);
		} catch (Exception e) {
			getErrorHandler().addError(e);
		}
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addSrsToLayer(String srs, Object Layer) { 
	}
	
	//-----------------------------------------
	//-----------------------------------------
	public void addPointToMultiPoint(Object point, Object multiPoint) {
		((GPEMultiGeometry)multiPoint).addGeometry((GPEGeometry)point);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addLineStringToMultiLineString(Object lineString, Object multiLineString) {
		((GPEMultiGeometry)multiLineString).addGeometry((GPEGeometry)lineString);
	}
	//-----------------------------------------
	//-----------------------------------------
	public Object startMultiPolygon(String id, String srs) {
		super.startMultiPolygon(id, srs);
		return new GPEMultiGeometry(id,srs);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addPolygonToMultiPolygon(Object polygon, Object multiPolygon) {
		((GPEMultiGeometry)multiPolygon).addGeometry((GPEGeometry)polygon);
	}	
	//-----------------------------------------
	//-----------------------------------------
	public void addCurveToMultiCurve(Object curve, Object multiCurve) {
		((GPEMultiGeometry)multiCurve).addGeometry((GPEGeometry)curve);
	}
	//-----------------------------------------
	//-----------------------------------------
	public void addSegmentToCurve(Object segment, Object curve) {
		((GPECurve)curve).addSegment((GPEGeometry)segment);
	}
	//-----------------------------------------
	//-----------------------------------------
	// END METHODS
	//-----------------------------------------
	//-----------------------------------------
	
	public void endPolygon(Object pol){
		int i;
		GPEGeometry polygon = (GPEGeometry) pol;
		if (polygon.isMultiGeometry()){
			ArrayList polygons = new ArrayList();
			GeneralPathX gp = new GeneralPathX();
			//First of all adds the main polygon
			polygons.add((FPolygon2D)((FGeometry)polygon.getGeometry()).getInternalShape());
			FPolyline2D subpolygon = (FPolygon2D)((FGeometry)polygon.getGeometry()).getInternalShape();
			//compose the new GeneralPath
			gp.append(subpolygon.getPathIterator(null), false);
			for (i=0;i<polygon.getGeometriesSize();i++){
				polygons.add((FPolygon2D)((FGeometry)polygon.getGeometryAt(i).getGeometry()).getInternalShape());
				subpolygon = (FPolygon2D)((FGeometry)polygon.getGeometryAt(i).getGeometry()).getInternalShape();
				//compose the new GeneralPath
				gp.append(subpolygon.getPathIterator(null), false);
			}
			IGeometry finalPolygon = ShapeFactory.createPolygon2D(gp);
			polygon.setGeometry(finalPolygon);
			polygon.setReprojectedGeometry(finalPolygon);
		}
	}
	
	public void endMultiPolygon(Object pol){
		int i;
		GPEMultiGeometry polygon = (GPEMultiGeometry) pol;
		ArrayList polygons = new ArrayList();
		GeneralPathX gp = new GeneralPathX();
		for (i=0;i<polygon.getGeometriesSize();i++){
			polygons.add((FPolygon2D)((FGeometry)polygon.getGeometryAt(i).getGeometry()).getInternalShape());
			FPolyline2D subpolygon = (FPolygon2D)((FGeometry)polygon.getGeometryAt(i).getGeometry()).getInternalShape();
			//compose the new GeneralPath
			gp.append(subpolygon.getPathIterator(null), false);
		}
		IGeometry finalPolygon = ShapeFactory.createPolygon2D(gp);
		polygon.setGeometry(finalPolygon);
		polygon.setReprojectedGeometry(finalPolygon);
	}
	
	public void endMultiLineString(Object line){
		int i;
		GPEMultiGeometry geom = (GPEMultiGeometry) line;
		ArrayList lines = new ArrayList();
		GeneralPathX gp = new GeneralPathX();
		for (i=0;i<geom.getGeometriesSize();i++){
			lines.add((FPolyline2D)((FGeometry)geom.getGeometryAt(i).getGeometry()).getInternalShape());
			FPolyline2D subline = (FPolyline2D)((FGeometry)geom.getGeometryAt(i).getGeometry()).getInternalShape();
			//compose the new GeneralPath
			gp.append(subline.getPathIterator(null), false);
		}
		IGeometry finalLine = ShapeFactory.createPolyline2D(gp);
		geom.setGeometry(finalLine);
		geom.setReprojectedGeometry(finalLine);
	}
		
	public void  endElement(Object element,boolean isGeometryTag){
		if (isGeometryTag)
			((GPEElement)element).setGeometryTag(true);
	}
	
	public void endLayer(Object layer) {
		ILayerGpe gpeLayer = (ILayerGpe)layer;
		Rectangle2D extent = null;
		if (gpeLayer.getLayersCount() > 0){
			for (int i=0 ; i < gpeLayer.getLayersCount(); i++){
				ILayerGpe gpeChild = (ILayerGpe)gpeLayer.getLayer(i);
				if (i==0){
					extent = gpeChild.getFullExtent();
				}else{
					extent.add(gpeChild.getFullExtent());
				}
				gpeLayer.setExtent(extent);
			}
		}
		parentLayer = (FlyrGpeVect) gpeLayer;
	}
	//-----------------------------------------
	//-----------------------------------------
	public void setMapControl(MapContext mapControl) {
		this.mapControl = mapControl;
	}
	//-----------------------------------------
	//-----------------------------------------
	public void setExtent(Rectangle2D bbox2D) {
		if (this.extent == null)
			this.extent=bbox2D;
		else
			this.extent.add(bbox2D);
	}
	//-----------------------------------------
	//-----------------------------------------
	// GETTERS
	//-----------------------------------------
	//-----------------------------------------
	public Rectangle2D getExtent() {
		return extent;
	}
	//-----------------------------------------
	//-----------------------------------------
	public MapContext getMapControl() {
		return mapControl;
	}
	//-----------------------------------------
	//-----------------------------------------
	public ArrayList getLayers() {
		// TODO Auto-generated method stub
		return mapControl.getLayers();
	}
	//-----------------------------------------
	//-----------------------------------------
	public FlyrGpeVect getParentLayer() {
		return parentLayer;
	}
}
