/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.layer;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.BitSet;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.gvsig.gpe.parser.GPEErrorHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import es.prodevelop.gvsig.mobile.fmap.MapContext;
import es.prodevelop.gvsig.mobile.fmap.core.FDot;
import es.prodevelop.gvsig.mobile.fmap.core.FGeometry;
import es.prodevelop.gvsig.mobile.fmap.core.FMultiPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.VectorialFileDriver;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLayer;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.LayerFactory;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.legend.LegendFactory;
import es.prodevelop.gvsig.mobile.fmap.legend.SingleSymbolLegend;
import es.prodevelop.gvsig.mobile.fmap.symbol.FSymbol;
import es.prodevelop.gvsig.mobile.fmap.util.Utils;
import es.prodevelop.gvsig.mobile.fmap.util.graph2d.Graph2DUtilities;
import es.prodevelop.gvsig.mobile.fmap.viewport.ViewPort;
import es.prodevelop.gvsig.mobile.gpe.fmap.DriverGpeFactory;
import es.prodevelop.gvsig.mobile.gpe.fmap.VectorialGpeAdapter;
import es.prodevelop.gvsig.mobile.gpe.fmap.VectorialGpeDriver;
import es.prodevelop.gvsig.mobile.gpe.handlers.FmapContentHandler;
import es.prodevelop.gvsig.mobile.gpe.model.GPEFeature;

/************************************************************
 * FlyrGPEVectorial.
 * -----------------
 * Vectorial Layer for a GPE layer. It has a collection
 * of child layers that can be added to it.
 *
 * PRODEVELOP S.L. 2008
 * @author Carlos Snchez Perin
 ************************************************************/
public class FlyrGpeVect extends FLyrVect implements ILayerGpe{

	//----------------
	//GLOBAL VARIABLES
	//----------------

	private static Logger logger = LoggerFactory.getLogger(FlyrGpeVect.class);
	private boolean loaded = false;
	private VectorialGpeDriver vDriver = null;
	private boolean driver_already_open = false;

	private static final String DEFAULT_NAME = "GPE Vectorial Layer";
	private GPEErrorHandler errorHandler = null;
	private String featureName = null;
	private int numRec;
	private int numPto;
	private int numPol;
	private Rectangle2D bb;
	private FSymbol sym;
	private boolean fill;
	private AffineTransform affineTransform;
	private Graphics2D graphics;
	private ViewPort viewPort;

	//---------------------
	// MULTILAYER SUPPORT:
	//---------------------
	//private GPEComposedLayer composedLayer = null;
	//private VectorialGpeDriver driver  = null;


	//-----------------------
	//MULTILAYER CONSTRUCTOR
	//-----------------------
	public FlyrGpeVect(MapContext mapContext, GPEErrorHandler errorHanldler, ILayerGpe parentLayer) {
		super();
		super.setMapContext(mapContext);
		this.errorHandler = errorHanldler;
		//---------------------
		// MULTILAYER SUPPORT:
		//---------------------
		//		if (parentLayer == null){
		//			composedLayer = new GPEComposedLayer(mapContext, null);
		//			}
		//		}else{
		//			composedLayer = new GPEComposedLayer(mapContext, parentLayer.getChildren());
		//		}
	}
	//------------------------------------------------------------
	/*************************************************************
	 *	drawSelected().
	 *	---------------
	 *	Reimplemented to draw multiShapes.
	 *************************************************************/
	public void drawSelected(Graphics2D g, ViewPort _vp) throws FMapDriverException {

		IGeometry ig = null;
		Rectangle2D bb = null;

		BitSet sel = getSelectionSupport().getSelection();
		AffineTransform aft = _vp.getAffineTransform();
		try {
			g.setStroke(new BasicStroke(sym.getStrokeWidth()));
			g.setColor(FSymbol.SELECTION_SYMBOL.getBorderColor());
			for (int i = sel.nextSetBit(0); i >= 0; i = sel.nextSetBit(i+1)) {
				bb = getSource().getShapeBounds(i);
				if (Utils.rectanglesItersect(bb, _vp.getAdjustedExtent())) {
					ig = getSource().getShape(i);
					if (ig == null) {
						logger.debug("ig es NULL: " + i);
						continue;
					}
					if (ig.getGeometryType() == FShape.POINT || ig.intersects(_vp.getAdjustedExtent())) {
						ig.transform(aft);
						Shape shp =  ig.getInternalShape();
						if (ig.getGeometryType()==FShape.POLYGON){
							if (((VectorialGpeDriver)getSource().getDriver()).isMultiShape(i)){
								Graph2DUtilities.drawShape(g, shp, FSymbol.SELECTION_SYMBOL);
								int subshapes = ((VectorialGpeDriver)getSource().getDriver()).getNumSubShapes();
								for (int j=0;j<subshapes;j++){
									IGeometry subig = ((VectorialGpeDriver)getSource().getDriver()).getSubShape(j);
									subig.transform(aft);
									shp =  subig.getInternalShape();
									Graph2DUtilities.drawShape(g, shp, FSymbol.SELECTION_SYMBOL);
								}
							}
							else{
								Graph2DUtilities.drawShape(g, shp, FSymbol.SELECTION_SYMBOL);
							}
						}
						else{
							Graph2DUtilities.drawShape(g, shp, FSymbol.SELECTION_SYMBOL);
						}
					}
				}
			}
		} catch (Exception ex) {
			throw new FMapDriverException(ex.getMessage());
		}
	}

	//------------------------------------------------------------
	/*************************************************************
	 *	draw().
	 *	-------
	 *	Reimplemented to draw multiShapes.
	 *************************************************************/
	public void draw(Graphics2D g, ViewPort vp) throws FMapDriverException {

		logger.debug("Empieza draw de FLyrVect: " + getName());
		double scale = vp.getScale();
		if (isWithinScale(scale)) {
			int count = getSource().getShapeCount();
			logger.debug("count "  + count);
			try {
				for (int i=0; i<count; i++) {
					this.sym = super.getLegend().getDefaultSymbol();
					g.setColor(sym.getBorderColor());
					g.setStroke(new BasicStroke(sym.getStrokeWidth()));
					int lyrType = getSource().getShape(i).getGeometryType();
					if (getSelectionSupport().isSelected(i)) {
						continue;
					}
					switch (lyrType){
						case  FShape.POINT:
							drawPoints(g,vp,i);
							break;
						case  FShape.MULTIPOINT:
							drawMultiPoints(g,vp,i);
							break;
						case  FShape.LINE:
							drawLines(g,vp,i);
							break;
						case  FShape.POLYGON:
							drawPolygons(g,vp,i);
							break;
						case FShape.MULTIPOLYGON:
							drawPolygons(g,vp,i);
						}
					this.drawSelected(g, vp);
				}
			} catch (Exception e) {
				logger.error("Error while drawing vect layer: " + e.getMessage());
			}
		}
	}

	//------------------------------------------------------------
	/*************************************************************
	 *	drawPoints().
	 *	------------------
	 *	Reimplemented to draw multiShapes.
	 *************************************************************/
	protected void drawPoints(Graphics2D g, ViewPort vp, int numGeom) throws FMapDriverException{
		AffineTransform _at = vp.getAffineTransform();
		try{
			Rectangle2D bb = getSource().getShapeBounds(numGeom);

			if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
				IGeometry ig = getSource().getShape(numGeom);
				ig.transform(_at);
				Shape shp =  ig.getInternalShape();
				Graph2DUtilities.drawPoint(g, (FPoint2D) shp);
				if (ig==null)
					logger.error("No hay bbox");
			}
		} catch (Exception e){
			logger.error("drawPoints: ", e);
		}
	}

	//------------------------------------------------------------
	/*************************************************************
	 *	drawMultipoints().
	 *	------------------
	 *	Reimplemented to draw multiShapes.
	 *************************************************************/
	protected void drawMultiPoints(Graphics2D g, ViewPort vp, int numGeom) throws FMapDriverException{
		AffineTransform _at = vp.getAffineTransform();
		try{
			Rectangle2D bb = getSource().getShapeBounds(numGeom);
			if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
				IGeometry ig = getSource().getShape(numGeom);
				ig.transform(_at);
				Shape shp =  ig.getInternalShape();
				Graph2DUtilities.drawMultiPoint(g, (FMultiPoint2D) shp);
			}
		} catch (Exception e){
				logger.error("drawPoints: ", e);
		}
	}

	//------------------------------------------------------------
	/*************************************************************
	 *	drawLines().
	 *	------------
	 *	Reimplemented to draw multiShapes.
	 *************************************************************/
	protected void drawLines(Graphics2D g, ViewPort vp, int numGeom) throws FMapDriverException{
		AffineTransform _at = vp.getAffineTransform();
		try{
			Rectangle2D bb = getSource().getShapeBounds(numGeom);

			if (Utils.rectanglesItersect(bb, vp.getAdjustedExtent())) {
				IGeometry ig = getSource().getShape(numGeom);
				double[] xs = {bb.getMinX(),bb.getMaxX()};
				double[] ys = {bb.getMinY(),bb.getMaxY()};
				FMultiPoint2D gbb = new FMultiPoint2D(xs,ys);
				gbb.transform(_at);
				Rectangle2D bb2 = gbb.getBounds2D();
				if(bb2.getWidth() < 2 && bb2.getHeight()<2){
					//If shape size is only a pixel size we paint only a pixel
					FDot pt = new FDot(bb2.getMinX(),bb2.getMinY());
					Graph2DUtilities.drawDot(g, pt);
				} else if (ig.intersects(vp.getAdjustedExtent())) {
					ig.transform(_at);
					FGeometry aux = (FGeometry)ig;
					
					if (aux.isMulti()){
						for (int i =0;i<aux.getNumShapes();i++){
							Shape shp =  aux.getSubShape(i);
							Graph2DUtilities.drawPolyLineOld(g, (FPolyline2D)shp);
						}
					}
					else{
						Shape shp =  ig.getInternalShape();
						Graph2DUtilities.drawPolyLineOld(g, (FPolyline2D)shp);
					}
				}
			}
		} catch (Exception e){
			logger.error(e.toString());
		}
	}

	//------------------------------------------------------------
	/*************************************************************
	 *	drawPolygons().
	 *	---------------
	 *	Reimplemented to draw multiPoligons and innerPolygons.
	 *************************************************************/
	protected void drawPolygons(Graphics2D g, ViewPort vp,int numGeom) throws FMapDriverException{
		this.graphics = g;
		this.viewPort = vp;
		this.affineTransform = vp.getAffineTransform();
		ReadableVectorial source = (ReadableVectorial) getSource();
		int count = 0;
		count = source.getShapeCount();
		logger.debug("count "  + count);
		this.sym = super.getLegend().getDefaultSymbol();
		this.fill = sym.isFilled();
		int numIn=0;
		this.numPto=0;
		this.numRec=0;
		this.numPol=0;

		if (numGeom%10==0){
			logger.debug("printed  "  + numGeom);
		}
		try{
			this.bb = source.getShapeBounds(numGeom);

			if (Utils.rectanglesItersect(this.bb, vp.getAdjustedExtent())) {
				numIn++;
				
				if (!(source instanceof VectorialEditableAdapter)){
					if(((VectorialGpeDriver)source.getDriver()).isMultiShape(numGeom)){
						IGeometry ig = source.getShape(numGeom);
						this.visualizePoligon(ig);
						int subshapes = ((VectorialGpeDriver)source.getDriver()).getNumSubShapes();
						for (int j=0;j<subshapes;j++){
							IGeometry subig = ((VectorialGpeDriver)source.getDriver()).getSubShape(j);
							this.visualizePoligon(subig);
						}
					}
					else
					{
						IGeometry ig = source.getShape(numGeom);
						if (ig == null) {
							logger.debug("ig es NULL: " + numGeom);
						}
						this.visualizePoligon(ig);
					}
				}else{
					IGeometry ig = source.getShape(numGeom);
					if (ig == null) {
						logger.debug("ig es NULL: " + numGeom);
					}
					this.visualizePoligon(ig);
				}
			}else{
				//logger.debug("No intersecto");
			}
		} catch (Exception e) {
			logger.error(e.toString());
		}
		logger.debug("GEometries visible " + numIn);
		logger.debug("GEometries as points " + numPto);
		logger.debug("GEometries as rects " + numRec);
		logger.debug("GEometries drawn" + numPol);
	}

	//------------------------------------------------------------
	/*************************************************************
	 * visualizePolygon(ig).
	 * ---------------------
	 * Draw a polygon depending on the zoom of the view.
	 * We'll paint the polygons as a dot, rectangle or polygon
	 *
	 * @param ig	:	IGeometry
	 *************************************************************/
	public void visualizePoligon(IGeometry ig){
		double[] xs = {bb.getMinX(),bb.getMaxX()};
		double[] ys = {bb.getMinY(),bb.getMaxY()};
		FMultiPoint2D gbb = new FMultiPoint2D(xs,ys);
		gbb.transform(this.affineTransform);
		Rectangle2D bb2 = gbb.getBounds2D();

		if(bb2.getWidth() < 1 && bb2.getHeight()<1){
			//If shape size is only a pixel size we paint only a pixel
			FDot pt = new FDot(bb2.getMinX(),bb2.getMinY());
			Graph2DUtilities.drawDot(this.graphics, pt);
			this.numPto++;
		}else if(bb2.getWidth()*bb2.getHeight()<6){
			this.numRec++;
			if (fill)
			this.graphics.fillRect(
					(int)bb2.getMinX(),
					(int)bb2.getMinY(),
					(int)Math.round(bb2.getWidth()),
					(int)Math.round(bb2.getHeight()));
			else
				this.graphics.drawRect(
						(int)bb2.getMinX(),
						(int)bb2.getMinY(),
						(int)Math.round(bb2.getWidth()),
						(int)Math.round(bb2.getHeight()));
		} else if  (ig.intersects(this.viewPort.getAdjustedExtent())){
			this.numPol++;
			if (fill && (bb2.getWidth() < 3 || bb2.getHeight()<3)){
				//If shape interior is not seen we do not paint it
				sym.setFilled(false);
			}
			ig.transform(this.affineTransform);
			Shape shp =  ig.getInternalShape();
			Graph2DUtilities.drawPolygonOld(this.graphics, (FPolygon2D)shp, sym);
		}
		sym.setFilled(fill);
	}

	//------------------------------------------------------------
	/*************************************************************
	 * setExtent.
	 * ----------
	 * Set a new Rectangle2D as extent.
	 * @param extent		:	Context of the Map
	 *************************************************************/
	public void setExtent(Rectangle2D extent) {
		 getDriver().setExtent(extent);
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getFullExtent().
	 * ----------------
	 * Get the main extent of the layer.
	 * @return extent		:	Rectangle2D
	 *************************************************************/
	public Rectangle2D getFullExtent() {
		try {
			return super.getFullExtent();
		} catch (FMapDriverException e) {
			e.printStackTrace();
		}
		return null;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getErrorHandler().
	 * ------------------
	 * get the errorHandler.
	 * @return errorHandler		:	GPEErrorHandler
	 *************************************************************/
	public GPEErrorHandler getErrorHandler() {
		return errorHandler;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * setExtent.
	 * ----------
	 * Set a new Rectangle2D as extent.
	 * @param extent		:	Context of the Map
	 *************************************************************/
	public String getDefaultLayerName() {

		return DEFAULT_NAME;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * isOk().
	 * ----------
	 * Returns true if there aren't errors else return false.
	 * @return ok		:	boolean
	 *************************************************************/
	public boolean isOk(){
		return (getNumErrors() == 0);
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getNumErrors().
	 * ---------------
	 * return the number of errors .
	 * @param extent		:	Context of the Map
	 *************************************************************/
	public int getNumErrors(){
		if (errorHandler.getErrorsSize()>0)
			return errorHandler.getErrorsSize();
//		else if (errorHandler.getWarningsSize()>0)
//			return errorHandler.getWarningsSize();
		else return 0;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * setErrorHandler(errorHnadler).
	 * ------------------------------
	 * Sets the errorhandler to this layer.
	 *
	 * @param ErrorHandler	:	GPEErrorHandler
	 *************************************************************/
	public void setErrorHanldler(GPEErrorHandler errorHanldler) {
		this.errorHandler = errorHanldler;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * addFeature(feature).
	 * --------------------
	 * Add a new feature to this layer.
	 *
	 * @param feature	:	GPEFeature
	 *************************************************************/
	public void addFeature(GPEFeature feature){
		if (featureName == null){
			featureName = feature.getName();
		}
		//add a new feature
		IGeometry geometry = getDriver().getGeometry(feature, getProjection());
		//if the geometry exists
		if ((geometry != null)){
			//Set the geometry reprojected
			feature.getGeometry().setReprojectedGeometry(geometry);
			//Set the attributes
			Hashtable elements = feature.getElements();
			//Sorted by key to show the elements ordered
			Vector v = new Vector(elements.keySet());
		    Collections.sort(v);
		    Iterator it = v.iterator();
			//Iterator it = elements.keySet().iterator();
			while (it.hasNext()){
				String fieldName = (String)it.next();
				Integer pos = (Integer) getDriver().getFieldPosition().get(fieldName);
				if (pos == null){
					pos = new Integer(getDriver().getFieldPosition().size());
					getDriver().getFieldPosition().put(fieldName, pos);
					getDriver().getFields().put(pos, fieldName);
				}
			}
			getDriver().getFeatures().put(new Integer(getDriver().getNumFeatures()), feature);
		}
	}
	//------------------------------------------------------------
	/*************************************************************
	 * getFeatureName().
	 * -----------------
	 * Get the name of one feature wich belongs to the this layer.
	 *************************************************************/
	public String getFeatureName() {
		return featureName;
	}


	//------------------------------------------------------------
	/*************************************************************
	 * Create Layer.
	 * -------------
	 * Creates a new gpe layer (with a xml, gml, kml... file).
	 * @param f 		: 	The file to open with GPE
	 * @param lyr_name	:	Name of the layer
	 * @param mc		:	Context of the Map
	 * @return	FlayerGpe
	 * @throws FMapDriverException 
	 *************************************************************/
	public static FlyrGpeVect createGpeLayer(File f, String lyr_name, MapContext mc) throws FMapDriverException{
		VectorialFileDriver drv;
		logger.debug(Utils.time() + "MainWindow instancia el driver MEMORY GPE.");
		drv = DriverGpeFactory.createVectorialDriver(mc, null, null);
		try {
			((VectorialGpeDriver) drv).open(f);
			logger.debug(Utils.time() + "MainWindow llama initialize() de driver MEMORY GPE.");
			((VectorialGpeDriver) drv).initialize();
		} catch (IOException e) {
			logger.error("Error while opening file: " + e.getMessage());
			return null;
		}
		logger.debug(Utils.time() + "MainWindow instancia FLayerGpe.");
		//Seeking last created layer
		FlyrGpeVect gpelyr = ((FmapContentHandler)((VectorialGpeDriver)drv).getContenHandler()).getParentLayer();
		
		if(gpelyr==null) 
			throw new FMapDriverException("Error Reading Layer "+lyr_name+" with GPE Parser");
		
		if ((gpelyr.getErrorHandler().getErrorsSize()>0)){
			for(int i =0;i<gpelyr.getErrorHandler().getErrorsSize();i++){
				gpelyr.addError(new FMapDriverException(gpelyr.getErrorHandler().getErrorAt(i).getMessage()
						         +" : \n"
								 + gpelyr.getErrorHandler().getErrorAt(i).getCause()));
			}
//		}else if(gpelyr.getErrorHandler().getWarningsSize()>0){
//			for(int i =0;i<gpelyr.getErrorHandler().getWarningsSize();i++){
//				gpelyr.addError(new FMapDriverException(gpelyr.getErrorHandler().getWarningAt(i).getMessage()
//		         +" : \n"
//				 + gpelyr.getErrorHandler().getWarningAt(i).getCause()));
//			}
		}
		gpelyr.setMapContext(((VectorialGpeDriver)drv).getMapContext());
		gpelyr.setVisible(true);
		gpelyr.setAvailable(true);
		gpelyr.setName(lyr_name);
		gpelyr.setDriver(((VectorialGpeDriver)drv));
		gpelyr.setProjection(((VectorialGpeDriver)drv).getMapContext().getProjection());
		gpelyr.wakeUp(true);
		return gpelyr;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * setDriver(driver).
	 * ----------------------
	 * sets the driver.
	 *
	 * @param vDriver	:	VectorialDriver
	 *************************************************************/
	public void setDriver(VectorialGpeDriver vDriver) {
		this.vDriver = vDriver;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getDriver().
	 * --------------------------
	 * Returns the driver.
	 *
	 * @return VectorialDriver.
	 *************************************************************/
	public VectorialGpeDriver getDriver() {
		return this.vDriver ;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * setDriverByName(name).
	 * ----------------------
	 * sets the driver seeking it by name.
	 ************************************************************/
	public void setDriverByName(String driverName) {
		this.setDriver(
		  (VectorialGpeDriver)LayerFactory.getDM().getDriver(driverName)
		);
	}

	//------------------------------------------------------------
	/*************************************************************
	 * initializeLegendDefault().
	 * --------------------------
	 * (Esto deberia ir en FLyrVect)
	 * Init the legend.
	 *************************************************************/
	private void initializeLegendDefault() {
		setLegend(LegendFactory.createSingleSymbolLegend());
	}

	//------------------------------------------------------------
	/*************************************************************
	 * setRandomColor().
	 * -----------------
	 * Creates a new set of color for the legend.
	 *************************************************************/
	public void setRandomColor() {
		((SingleSymbolLegend) getLegend()).setDefaultSymbol(
				FSymbol.createRandomSymbol(false));
	}

	//------------------------------------------------------------
	/*************************************************************
	 * wakeUp(isOpen).
	 * ---------------
	 * Check if the driver is opened, else load the driver.
	 *
	 * @param driver_open	:	boolean.
	 *************************************************************/
	public void wakeUp(boolean driver_open) {
		driver_already_open = driver_open;
		if (!loaded) {
			try {
				load();
			} catch (FMapDriverException e) {
				logger.debug(Utils.time() + "ERROR: El driver de libGPE no se ha podido arrancar correctamente.");
			}
		}
	}

	//-------------------------------------------------------------
	/**************************************************************
	 * load().
	 * -------
	 * load the driver for a layer into the map context and default
	 * projection.
	 *
	 * @throws FMapDriverException.
	 **************************************************************/
	public void load() throws FMapDriverException {
		if (this.getName() == null || this.getName().length() == 0) {
			this.setAvailable(false);
			throw new FMapDriverException("Layer name haven't been specified");
		}
		if (this.vDriver == null) {
			this.setAvailable(false);
			throw new FMapDriverException("Driver haven't been specified");
		}
		if (this.getProjection() == null) {
			this.setAvailable(false);
			throw new FMapDriverException("Projection haven't been specified");
		}
		//-----------------------
		//Adaptador de de libGPE
		//-----------------------
		VectorialGpeAdapter adapter = null;
		try {
			VectorialGpeDriver filedrv = (VectorialGpeDriver) vDriver;
			adapter = new VectorialGpeAdapter();

			if (driver_already_open)
				adapter.setReferenceCount(1);

			adapter.setDriver(filedrv);
			setSource(adapter);

		} catch (Exception e) {
			this.setAvailable(false);
			throw new FMapDriverException(e.getMessage());
		}
		try {
			initializeLegendDefault();
		} catch (Exception e) {
			setAvailable(false);
			throw new FMapDriverException(e.getMessage());
		}
	}

	//------------------------------------------------------------
	/*************************************************************
	 * toXML(name,projectPath).
	 * ------------------------
	 * Save project info in Xml format.
	 *
	 * @param	name		:	String
	 * @param	projectPath	:	String.
	 *
	 * @throws FMapDriverException.
	 *************************************************************/
	public StringBuffer toXML(String name, String projectPath) {
		StringBuffer buff = new StringBuffer();
		buff.append("<" + name +">\n");
		buff.append("<type>");
		buff.append(this.getClass().getName());
		buff.append("</type>\n");
		if (vDriver instanceof VectorialFileDriver){
			VectorialFileDriver drv = (VectorialFileDriver)vDriver;
			buff.append("<path>");
			String absPath = drv.getFilePath();
			String relPath = Utils.relPath(projectPath, absPath);
			if (relPath == null)
				relPath = absPath;
			buff.append(relPath);
			buff.append("</path>\n");
		}
		buff.append("</" + name +">\n");
		return buff;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getFilePath().
	 * --------------
	 * Returns the path of the file driver.
	 *
	 * @return String	:	The Path of the File.
	 *************************************************************/
	public String getFilePath() {

		if (vDriver instanceof VectorialFileDriver){
			VectorialFileDriver drv = (VectorialFileDriver)vDriver;
			return drv.getFilePath();
		}
		return "[None]";

	}

	//------------------------------------------------------------
	/*************************************************************
	 * clean().
	 * --------
	 * Cleans the layer from driver, legend, etc...
	 *************************************************************/
	public void clean() {
		logger.debug("Cleaning layer: " + getName());

		vDriver = null;
		setLegend(null);
		getSource().setDriver(null);
		try {
			getRecordset().setSelectionSupport(null);
		} catch (Exception e) {
			logger.error("While cleaning layer: " + e.getMessage());
		}
		setRecordset(null);
		System.gc();
	}

	//------------------------------------------------------------
	/*************************************************************
	 * addLayer(layer).
	 * ----------------
	 * Add a new layer.
	 *************************************************************/
	public void addLayer(FLayer layer){
		logger.debug("FlyrGpeVect.addLayer() Not implementd yet");
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getLayer(numlayer).
	 * -------------------
	 * Get the layer at the position "numlayer".
	 *************************************************************/
	public FLayer getLayer(int i) {
		logger.debug("FlyrGpeVect.getLayer() Not implementd yet");
		// TODO Auto-generated method stub
		return null;
	}

	//------------------------------------------------------------
	/*************************************************************
	 * getLayersCount(layer).
	 * ----------------------
	 * Return the number of existing layers.
	 *************************************************************/
	public int getLayersCount() {
		logger.debug("FlyrGpeVect.getLayersCount() Not implementd yet");
		// TODO Auto-generated method stub
		return 0;
	}
}
	
