/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.model;

import java.util.ArrayList;
import com.hardcode.gdbms.engine.values.ComplexValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

/********************************************************
 * GPEELEMENT.											
 * -----------											
 * Class to implement the libGPE Element for		 	
 * gvsig mobile.										
 * Only fill the content events needed.					
 * 														
 * PRODEVELOP 2008										
 * @author Carlos Snchez (csanchez@prodevelop.es)		
 ********************************************************/
public class GPEElement {
	private String parentName = null;
	private String name = null;
	private Value value = null;
	private ArrayList subElements = null;
	private boolean addparentname = false;
	private int ocurrencias = 1;
	private boolean isGeometry = false;
	private String nameOrig = null;
	private boolean repetidos=false;
	
	//------------
	//CONSTRUCTOR
	//------------
	public GPEElement(String name, Value value) {
		super();
		this.name = name;
		this.value = value;
	}
	
	//--------------------------------
	//CONSTRUCTOR WITH PARENT ELEMENT.
	//--------------------------------
	public GPEElement(String name, Value value, GPEElement parentElement) {
		this(name,value);
		//If it is a complex value
		if (parentElement != null){
			if (!(parentElement.getValue() instanceof ComplexValue)){
				parentElement.value = ValueFactory.createComplexValue(parentElement.getName());
				parentName = parentElement.getName();
			}
			((ComplexValue)parentElement.value).put(name, value);					
		}
	}	
	
	//------------------------------------------------
	//Return true if the attribute has sub attributes.
	//------------------------------------------------
	public boolean isComplex(){
		if (subElements==null)
			return false;
		return true;
	}
	
	//--------------------------------------------
	//Return the treeFormatedname of the element.
	//--------------------------------------------
	public String getName() {
		if ((parentName!=null)&&(addparentname==false)){
			this.nameOrig = name;
			name = parentName+" -> "+name;
			addparentname = true;
		}
		if (name==null)
			return "Unamed Data";
		else{
				if (this.repetidos==true)
					return name +" "+ this.ocurrencias;
				else
					return name;
		}
	}
	
	//-------------------------------
	//Return the original name of the element.
	//-------------------------------
	public String getNameOrig(){
		if (nameOrig!=null)
			return this.nameOrig;
		else 
			if (name!= null)
				return name;
			else
				return "Unamed";
	}
	
	//----------------------
	//Sets the parent Name.
	//----------------------
	private void setParentName(String name) {
		this.parentName=name;
	}

	//--------------------------------
	//Return the value of the element.
	//--------------------------------
	public Value getValue() {
		return value;
	}
	
	//-----------------------------------
	// Return the number of subelements.
	//-----------------------------------
	public int getNumElements(){
		return subElements.size();
	}
	
	//---------------------------------
	//Return the SubElement 
	//---------------------------------
	public GPEElement getSubElement(int i){
		return (GPEElement) subElements.get(i);
	}

	//--------------------------------------
	//Adds a new subElement to this element
	//--------------------------------------
	public void addChild(GPEElement element) {
		element.setParentName(getName());
		if (!isComplex()){
			this.subElements = new ArrayList();
			this.subElements.add(element);
		}else{
			this.subElements.add(element);	
		}
	}
	
	//--------------------------------------
	// There're more elements whit the same 
	// key at the HashTable. adds one to 
	// the counter to make a new key.  
	//--------------------------------------
	public void addOcurrencia() {
		this.ocurrencias++;
		this.repetidos=true;
	}

	//--------------------------------------
	//Set this element as a Geometry tag
	//--------------------------------------
	public void setGeometryTag(boolean b) {
		this.isGeometry=b;		
	}
	
	//--------------------------------------
	//Return true if is a geometry tag
	//--------------------------------------
	public boolean isGeometryTag() {
		return this.isGeometry;
	}
}
