/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.model;

import java.util.ArrayList;
import java.util.Hashtable;

import org.safehaus.uuid.UUID;
import org.safehaus.uuid.UUIDGenerator;

/********************************************************
 * GPEFEATURE.											*
 * -----------											*
 * Class to implement the libGPE Feature for		 	*
 * gvsig mobile.										*
 * Only fill the content events needed.					*
 * 														* 
 * PRODEVELOP 2008										*
 * @author Carlos Snchez (csanchez@prodevelop.es)		*
 ********************************************************/
public class GPEFeature{
	private GPEGeometry geometry = null;
	private Hashtable elements = null;
	private ArrayList features = null;
	private Object id = null;
	private String name = null;
	private String typeName = null;
	private boolean isComplex = false;
	private int numSubFeatures = 0;
	private boolean hasGeometry = false;
	
	public GPEFeature(Object id, String namespace, String typeName) {
		this();
		this.id = id;
		this.name = namespace;
		this.typeName = typeName;		
	}	

	public GPEFeature(){
		elements = new Hashtable();
	}

	/**
	 * @return the id
	 */
	public Object getId() {
		if (id==null){
			UUID identifier;
			UUIDGenerator gen = UUIDGenerator.getInstance();
			identifier = gen.generateRandomBasedUUID();
			id = identifier.toString();
		}
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(Object id) {
		this.id = id;
	}

	/**
	 * @return the geometry
	 */
	public GPEGeometry getGeometry() {
		if((isComplex())&&(geometry == null)){
			//At least Needs 2 sub geometries to compose one.
			if (getNumSubFeatures()>=2){
				geometry = new GPEGeometry(this.getId().toString(), null, null);
				GPEGeometry subgeom1 = getSubFeature(0).getGeometry();
				GPEGeometry subgeom2 = getSubFeature(1).getGeometry();
				//
				if (((subgeom1!=null)&&(subgeom2!=null))&&((!getSubFeature(0).isComplex())&&(!getSubFeature(1).isComplex()))){
					geometry.ComposeGeometry(subgeom1,subgeom2);
					//If has more than 2 sub geometries also compose with them
					for (int i = 2;i<getNumSubFeatures();i++){
						subgeom1 = getSubFeature(i).getGeometry();
						if (((geometry!=null)&&(subgeom1!=null))&&(!getSubFeature(i).isComplex())){
							geometry.ComposeGeometry(geometry,subgeom1);
						}
					}
					hasGeometry=true;
				}
			}
		}
		return geometry;	
	}

	/**
	 * @param geometry the geometry to set
	 */
	public void setGeometry(GPEGeometry geometry) {
		this.geometry = geometry;
		hasGeometry=true;
	}

	/**
	 * @return the elements
	 */
	public Hashtable getElements() {
		return elements;
	}

	/**
	 * @param elements the elements to set
	 */
	public void setElements(Hashtable  elements) {
		this.elements = elements;
	}
	
	/**
	 * It adds a new element
	 * @param element
	 * The element to add
	 */
	public void addElement(GPEElement element){
		if(element.isGeometryTag()!=true){
			if (element.isComplex()){
				this.elements=getSubattributes(this.elements,element);
			}
			else{
				String key = element.getName();
				while (this.elements.containsKey(key)==true){
					element.addOcurrencia();
					key = element.getName();
				}	
				this.elements.put(key, element);
			}
		}
	}
	/**
	 * It adds a new element for every sub attribute
	 * @param fields
	 * @param element
	 * The sub elements to add
	 */
	public Hashtable getSubattributes(Hashtable fields, GPEElement element){
		for (int j=0;j<element.getNumElements();j++){
			GPEElement subelement = (GPEElement) element.getSubElement(j);
			if (subelement.isComplex())
				fields=getSubattributes(fields,subelement);
			else{
				String key = subelement.getName();
				while (this.elements.containsKey(key)==true){
					subelement.addOcurrencia();
					key = subelement.getName();
				}
				fields.put(key,subelement);
			}
		}
		return fields;
	}
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	public GPEFeature getSubFeature(int i){
		return (GPEFeature) features.get(i);
	}
	/**
	 * @return the typeName
	 */
	public String getTypeName() {
		return typeName;
	}

	/**
	 * It adds a new feature
	 * @param feature	The sub feature to add
	 */
	public void addFeature(GPEFeature feature) {
		if (features==null){
			features=new ArrayList();
		}
		this.features.add(feature);
		isComplex = true;
		numSubFeatures ++;
	}
	
	/**
	 * It adds a new element for every sub attribute
	 * @param fields	
	 * @param element 	The sub elements to add
	 */
	public ArrayList getFeatures(ArrayList features, GPEFeature feature){
		for (int j=0;j<feature.getNumSubFeatures();j++){
			GPEFeature subfeature = (GPEFeature) feature.getSubFeature(j);
			if (subfeature.isComplex())
				features=getFeatures(features,subfeature);
			else{
				features.add(subfeature);
			}
		}
		return features;
	}

	public int getNumSubFeatures() {
		return numSubFeatures;
	}

	public boolean isComplex() {
		return isComplex;
	}

	public boolean hasGeometry() {
		return hasGeometry ;		
	}
}
