/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

package es.prodevelop.gvsig.mobile.gpe.model;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/********************************************************
 * GPEMULTIGEOMETRY.									*
 * ------------------									*
 * Class to implement the libGPE Multigeometries for 	*
 * gvsig mobile.										*
 * Only fill the content events needed.					*
 * 														* 
 * PRODEVELOP 2008										*
 * @author Carlos Snchez (csanchez@prodevelop.es)		*
 ********************************************************/
public class GPEMultiGeometry extends GPEGeometry{
	private ArrayList geometryList = null;
	private Rectangle2D generalBounds = null;
	
	public GPEMultiGeometry(String id, String srs) {
		super(id, null, srs);
		geometryList = new ArrayList();		
	}
	
	
	/**
	 * Gets one geometry
	 * @param i
	 * Geometry position
	 * @return
	 * A Geometry
	 */
	public GPEGeometry getGeometryAt(int i){
		return (GPEGeometry) geometryList.get(i);
	}
	
	/**
	 * Adds a new geometry
	 * @param geometry
	 * The geometry to add
	 */
	public void addGeometry(GPEGeometry geometry){
		if (generalBounds == null){
			generalBounds = geometry.getShapeBounds();
		}else{
			generalBounds.add(geometry.getShapeBounds());
		}
		geometryList.add(geometry);
	}
	
	/*
	 * (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.gpe.model.GPEGeometry#getShapeBounds()
	 */
	public Rectangle2D getShapeBounds() {
		Rectangle2D subBound = null;
		if (isMultiGeometry()){
			if (geometry!=null)
				generalBounds = geometry.getBounds2D();
			for (int i=0; i<getGeometriesSize();i++){
				GPEGeometry subgeometry = getGeometryAt(i);
				if (subgeometry.isMultiGeometry())
					subBound = subgeometry.getShapeBounds();
				else
					subBound = subgeometry.getGeometry().getBounds2D();
				if (generalBounds==null)
					generalBounds=subBound;
				else
					generalBounds.add(subBound);
			}
		}
		else{
			if (geometry!=null)
				generalBounds = geometry.getBounds2D();
		}	
		return generalBounds;
	}
	
	/**
	 * @return the number of geometries
	 */
	public int getGeometriesSize(){
		return geometryList.size();
	}
	
	public boolean isMultiGeometry(){
		return false;
	}
}
