/* gvSIG. Geographic Information System of the Valencian Government
 *
 * Copyright (C) 2007-2008 Infrastructures and Transports Department
 * of the Valencian Government (CIT)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 */

package es.prodevelop.gvsig.mobile.gpe.writer;

import java.awt.geom.PathIterator;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import org.gvsig.gpe.GPEDefaults;
import org.gvsig.gpe.GPEFactory;
import org.gvsig.gpe.GPERegister;
import org.gvsig.gpe.parser.GPEErrorHandler;
import org.gvsig.gpe.parser.GPEErrorHandlerTest;
import org.gvsig.gpe.writer.GPEWriterHandler;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;

import es.prodevelop.gvsig.mobile.fmap.core.FPoint2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPoint3D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolygon3D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline2D;
import es.prodevelop.gvsig.mobile.fmap.core.FPolyline3D;
import es.prodevelop.gvsig.mobile.fmap.core.FShape;
import es.prodevelop.gvsig.mobile.fmap.core.IFeature;
import es.prodevelop.gvsig.mobile.fmap.core.IGeometry;
import es.prodevelop.gvsig.mobile.fmap.driver.FMapDriverException;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.export.DBFFromReadVect;
import es.prodevelop.gvsig.mobile.fmap.driver.vect.write.export.SHPSHXFromReadVect;
import es.prodevelop.gvsig.mobile.fmap.edition.VectorialEditableAdapter;
import es.prodevelop.gvsig.mobile.fmap.layer.FLyrVect;
import es.prodevelop.gvsig.mobile.fmap.layer.ReadableVectorial;
import es.prodevelop.gvsig.mobile.fmap.util.string.StringUtilities;
import es.prodevelop.gvsig.mobile.gpe.model.GPEElement;
import es.prodevelop.gvsig.mobile.gpe.model.GPEFeature;
import es.prodevelop.gvsig.mobile.gpe.model.GPEGeometry;

/*******************************************************************************
 * ExportGpeWriter Class.
 * 
 * Class to build a gml file with libGPE from a FLyrVect.
 * 
 * PRODEVELOP S.L.
 * 
 * @author Carlos Snchez Perin (csanchez@prodevelop.es)
 ******************************************************************************/
public class ExportGpeWriter {
	private FLyrVect layer = null;
	private GPEWriterHandler writerHandler = null;
	private GPEErrorHandler errorHandler = null;
	private String filepath;
	private String extension;
	// DEFAULT WRITER....
	private String writer = "GML 3.2 (SFP Simple Feature Profile)";
	private boolean onlyNewChanges = false;

	/**
	 * setUpWriter().
	 * 
	 * Register the writer and gets the handler.
	 * 
	 * @throws Exception
	 */
	public void setUpWriter() throws Exception {
		if (writer.equals("GML 3.2 (SFP Simple Feature Profile)")) {
			// Register the writer
			GPERegister
					.addGpeWriterHandler(
							"GML 3.1.2 SFP",
							"This writer writes GML 3.1.2 with the SFP",
							org.gvsig.gpe.gml.writer.GPEGmlSFP0WriterHandlerImplementor.class);
			// Gets the writer
			this.writerHandler = GPERegister.createWriter("GML 3.1.2 SFP");
			extension = ".gml";
		} else if (writer.equals("GML 2.x (Geographic Markup Language)")) {
			// Register the writer
			GPERegister
					.addGpeWriterHandler(
							"GML v2.1.2",
							"This writer writes GML 2.1.2",
							org.gvsig.gpe.gml.writer.GPEGmlv2WriterHandlerImplementor.class);
			// Gets the writer
			this.writerHandler = GPERegister.createWriter("GML v2.1.2");
			extension = ".gml";
		} else if (writer.equals("KML 2.1 (KeyMap Markup Language)")) {
			// Register the writer
			GPERegister
					.addGpeWriterHandler(
							"KML v21 writer",
							"This writer writer KML 2.1",
							org.gvsig.gpe.kml.writer.GPEKml21WriterHandlerImplementor.class);
			// Gets the writer
			this.writerHandler = GPERegister.createWriter("KML v21 writer");
			extension = ".kml";
		} else if (writer.equals("SHP")) {
			extension = ".shp";
		}
		/** NEW WAY TO REGISTER A GPE WRITER* */
		// this.writerHandler=GPERegister.createWriter(GPEFactory.getWriterHandlerByFormat("gml").get(0).toString());
	}

	/**
	 * This test writes some objects into the file and then try to read the
	 * created file. It compare that the written objects are the same that the
	 * read them.
	 * 
	 * @param gpelyr
	 * @param path
	 * @throws Exception
	 */
	public void write(FLyrVect gpelyr, String path, String w) throws Exception {
		filepath = path;
		if (w != null)
			writer = w;
		setUpWriter();
		setLayer(gpelyr);
		writeInputStream();
	}

	/**
	 * Parses the file like an inputstream
	 * 
	 * @throws FileNotFoundException
	 */
	private void writeInputStream() throws Exception {

		if (writer.compareToIgnoreCase("shp") == 0) {

			ReadableVectorial rv = (ReadableVectorial) layer.getSource();

			File shpout = null;
			if (filepath.toLowerCase().endsWith(extension)) {
				shpout = new File(filepath);
			} else {
				String aux = StringUtilities.replace(filepath, ".", "_");
				shpout = new File(aux + extension);
			}

			int fshptype = rv.getShapeType();

			if (fshptype == FShape.MULTI) {
				writeThreeSHP(rv, shpout);
			} else {
				writeSHP(rv, shpout, fshptype, false);
			}

		} else {

			errorHandler = null;
			if (filepath.toLowerCase().endsWith(extension)) {
				getWriterHandler().setFileName(filepath);
			} else {
				String aux = StringUtilities.replace(filepath, ".", "_");
				getWriterHandler().setFileName(aux + extension);
			}
			getWriterHandler().setErrorHandler(getErrorHandler());
			GPEDefaults.setProperty(GPEDefaults.XML_ENCODING, "UTF-8");
			writeObjects();
		}
	}

	private void writeThreeSHP(ReadableVectorial rv, File shpout) {

		int lastdir = shpout.getAbsolutePath().lastIndexOf(File.separator);
		String fname = shpout.getName();
		String folder = shpout.getAbsolutePath().substring(0, lastdir);

		File poly_shp = new File(folder + File.separator + "polygon_" + fname);
		File line_shp = new File(folder + File.separator + "line_" + fname);
		File point_shp = new File(folder + File.separator + "point_" + fname);

		writeSHP(rv, poly_shp, FShape.POLYGON, true);
		writeSHP(rv, line_shp, FShape.LINE, true);
		writeSHP(rv, point_shp, FShape.POINT, true);
	}

	private void writeSHP(ReadableVectorial rv, File shp, int shptype,
			boolean checktype) {
		File dbff = getDbfFile(shp);
		SHPSHXFromReadVect shpshxw = new SHPSHXFromReadVect(rv, shp, shptype);
		int writtenfeat = shpshxw.create();

		if (writtenfeat == 0) {
			return;
		}

		DBFFromReadVect dbfw = new DBFFromReadVect(rv, dbff, shptype,
				checktype, writtenfeat);
		dbfw.create();
	}

	private File getDbfFile(File shp) {
		String dbfPath = StringUtilities.replace(shp.getAbsolutePath(), ".shp",
				".dbf");
		dbfPath = StringUtilities.replace(dbfPath, ".SHP", ".DBF");
		return new File(dbfPath);
	}

	/**
	 * This method write some objects into the writer handler
	 * 
	 * @throws IOException
	 */
	public void writeObjects() {
		double[] x = new double[2];
		double[] y = new double[2];
		double[] z = new double[2];
		getWriterHandler().initialize();
		getWriterHandler().startLayer(layer.getName(), layer.getName(), null,
				layer.getProjection().getAbrev(), null);
		Rectangle2D box = null;
		box = layer.getMapContext().getAllLayersExtent();
		if (box == null) {
			// GET THE FULL EXTENT FROM THE VIEW AS THE DEFAULT BOUNDING BOX
			box = layer.getMapContext().getViewPort().getAdjustedExtent();
		} else {
			// GET THE PARTICULAR ZOOM EXTENT FROM THE VIEW AS THE DEFAULT
			// BOUNDING BOX
			box = layer.getMapContext().getViewPort().getExtent();
		}
		x[0] = box.getBounds2D().getMinX();
		y[0] = box.getBounds2D().getMinY();
		z[0] = 0.0;
		x[1] = box.getBounds2D().getMaxX();
		y[1] = box.getBounds2D().getMaxY();
		z[1] = 0.0;

		WriterPointsIterator coordIterator = new WriterPointsIterator(x, y, z);
		getWriterHandler().startBbox(null, coordIterator,
				layer.getProjection().getAbrev());
		getWriterHandler().endBbox();

		try {
			int numfeatures = 0;

			if (onlyNewChanges) {
				exportOnlyChanges();
			} else {
				if (layer.getSource() instanceof VectorialEditableAdapter) {
					numfeatures = ((VectorialEditableAdapter) layer.getSource())
							.getRowCount();
					int i = 0;

					while (i < numfeatures) {
						GPEFeature feat = new GPEFeature();
						GPEGeometry geom = null;
						System.out.println(i);
						IFeature feature = (IFeature) ((VectorialEditableAdapter) layer
								.getSource()).getRow(i).getLinkedRow();
						geom = new GPEGeometry("the_geom", feature
								.getGeometry(), layer.getProjection()
								.getAbrev());
						feat.setGeometry(geom);

						Hashtable elements = new Hashtable();
						int numCampos = layer.getSource().getRecordset()
								.getFieldCount();
						int j = 0;

						while (j < numCampos) {
							String nombre = layer.getSource().getRecordset()
									.getFieldName(j);
							Value data = feature.getAttribute(j);
							GPEElement elem = new GPEElement(nombre, data);
							elements.put(nombre, elem);
							j++;
						}
						feat.setElements(elements);
						writeFeature(feat);
						i++;
					}

				} else {
					numfeatures = layer.getSource().getDriver().getShapeCount();
					int i = 0;

					while (i < numfeatures) {
						GPEFeature feat = new GPEFeature();
						GPEGeometry geom = null;

						geom = new GPEGeometry("the_geom", layer.getSource()
								.getDriver().getShape(i), layer.getProjection()
								.getAbrev());
						feat.setGeometry(geom);
						Hashtable elements = new Hashtable();
						int numCampos = layer.getSource().getRecordset()
								.getFieldCount();
						int j = 0;
						while (j < numCampos) {
							String nombre = layer.getSource().getRecordset()
									.getFieldName(j);
							Value data = layer.getSource().getRecordset()
									.getFieldValue(i, j);
							GPEElement elem = new GPEElement(nombre, data);
							elements.put(nombre, elem);
							j++;
						}
						feat.setElements(elements);
						writeFeature(feat);
						i++;
					}

				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FMapDriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		getWriterHandler().endLayer();
		getWriterHandler().close();
	}

	private void writeFeature(GPEFeature feat) {
		String name;
		if (feat.getName() == null)
			name = "Feature";
		else
			name = feat.getName();
		getWriterHandler().startFeature(feat.getId().toString(), name, null);
		GPEGeometry geometry = feat.getGeometry();
		writeGeometry(geometry.getGeometry(), layer.getProjection().getAbrev());
		Hashtable elements = feat.getElements();
		Set set = elements.keySet();
		Iterator iter = set.iterator();
		int i = 0;
		while (iter.hasNext()) {
			// If there are sub features here is where will be the recursive
			// call.
			GPEElement element = (GPEElement) elements.get(iter.next());
			writeElement(element);
			i++;
		}
		getWriterHandler().endFeature();
	}

	private void writeElement(GPEElement element) {
		getWriterHandler().startElement(element.getNameOrig(),
				element.getValue(), null);
		if (element.isComplex()) {
			for (int j = 0; j < element.getNumElements(); j++) {
				element.getSubElement(j);
				HashMap elements = null;
				Set set = elements.keySet();
				Iterator iter = set.iterator();
				int i = 1;
				while (iter.hasNext()) {
					// If there are sub features here is where will be the
					// recursive call.
					element = (GPEElement) elements.get(iter.next());
					writeElement(element);
					i++;
				}
			}
		}
		getWriterHandler().endElement();
	}

	/**
	 * It writes a geometry
	 * 
	 * @param geom
	 *            The geometry to write
	 * @param crs
	 *            The coordinates reference system
	 */
	public void writeGeometry(IGeometry geom, String crs) {
		FShape shp = (FShape) geom.getInternalShape();
		if (shp instanceof FPoint2D) {
			writePoint((FPoint2D) shp, crs);
		} else if ((shp instanceof FPolygon2D) || (shp instanceof FPolygon3D)) {
			writePolygon((FPolygon2D) shp, crs);
		} else if ((shp instanceof FPolyline2D) || (shp instanceof FPolyline3D)) {
			writeLine((FPolyline2D) shp, crs);
		}
	}

	/**
	 * Writes a point in 2D
	 * 
	 * @param point
	 *            The point to write
	 * @param crs
	 *            The coordinates reference system
	 */
	private void writePoint(FPoint2D point, String crs) {
		double[] x = new double[1];
		double[] y = new double[1];
		double[] z = new double[1];
		x[0] = point.getX();
		y[0] = point.getY();
		if (point instanceof FPoint3D)
			z[0] = ((FPoint3D) point).getZs()[0];
		else
			z[0] = 0.0;
		WriterPointsIterator coordIterator = new WriterPointsIterator(x, y, z);
		getWriterHandler().startPoint(null, coordIterator, crs);
		getWriterHandler().endPoint();
	}

	/**
	 * Writes a line in 2D
	 * 
	 * @param line
	 *            The line to write
	 * @param crs
	 *            The coordinates reference system
	 */
	private void writeLine(FPolyline2D line, String crs) {
		boolean isMultiGeometry = false;
		PathIterator iterator = line.getPathIterator(null);
		double[] coords = new double[3];
		int num_lines = 0; // Number of lines in a multiline
		int num_points = 0; // Number of points in a line
		ArrayList x_coords = new ArrayList();
		ArrayList y_coords = new ArrayList();
		ArrayList z_coords = new ArrayList();

		while (!iterator.isDone()) {
			if (iterator.currentSegment(coords) == PathIterator.SEG_MOVETO) {
				num_lines++;
				if (num_lines > 1) {
					if (num_lines == 2) {
						isMultiGeometry = true;
						getWriterHandler().startMultiLineString(null, crs);
					}
					double[] x = new double[num_points];
					double[] y = new double[num_points];
					double[] z = new double[num_points];
					for (int i = 0; i < num_points; i++) {
						x[i] = ((Double) x_coords.get(i)).doubleValue();
						y[i] = ((Double) y_coords.get(i)).doubleValue();
						z[i] = ((Double) z_coords.get(i)).doubleValue();
					}
					// Write a new subPolygon
					WriterPointsIterator coordIterator = new WriterPointsIterator(
							x, y, z);
					getWriterHandler()
							.startLineString(null, coordIterator, crs);
					getWriterHandler().endLineString();
					// Re-Initialize the ArrayLists for the next polygon.
					num_points = 0;
					x_coords = new ArrayList();
					y_coords = new ArrayList();
					z_coords = new ArrayList();
				}
			}
			x_coords.add(new Double(coords[0]));
			y_coords.add(new Double(coords[1]));
			z_coords.add(new Double(coords[2]));
			iterator.next();
			num_points++;
		}
		double[] x = new double[num_points];
		double[] y = new double[num_points];
		double[] z = new double[num_points];
		for (int i = 0; i < num_points; i++) {
			x[i] = ((Double) x_coords.get(i)).doubleValue();
			y[i] = ((Double) y_coords.get(i)).doubleValue();
			z[i] = ((Double) z_coords.get(i)).doubleValue();
		}
		WriterPointsIterator coordIterator = new WriterPointsIterator(x, y, z);
		getWriterHandler().startLineString(null, coordIterator, crs);
		getWriterHandler().endLineString();
		if (isMultiGeometry)
			getWriterHandler().endMultiLineString();
	}

	/**
	 * Writes a polygon in 2D
	 * 
	 * @param polygon
	 *            The polygon to write
	 * @param crs
	 *            The coordinates reference system
	 */
	private void writePolygon(FPolygon2D polygon, String crs) {
		boolean isMultiGeometry = false;
		PathIterator iterator = polygon.getPathIterator(null);
		double[] coords = new double[3];
		int num_polygons = 0; // Number of polygons into the polygon
		int num_points = 0; // Number of polygon points
		ArrayList x_coords = new ArrayList();
		ArrayList y_coords = new ArrayList();
		ArrayList z_coords = new ArrayList();

		while (!iterator.isDone()) {
			if (iterator.currentSegment(coords) == PathIterator.SEG_MOVETO) {
				num_polygons++;
				if (num_polygons > 1) {
					if (num_polygons == 2) {
						isMultiGeometry = true;
						getWriterHandler().startMultiPolygon(null, crs);
					}
					double[] x = new double[num_points];
					double[] y = new double[num_points];
					double[] z = new double[num_points];

					for (int i = 0; i < num_points; i++) {
						x[i] = ((Double) x_coords.get(i)).doubleValue();
						y[i] = ((Double) y_coords.get(i)).doubleValue();
						z[i] = ((Double) z_coords.get(i)).doubleValue();
					}
					// Write a new subPolygon
					WriterPointsIterator coordIterator = new WriterPointsIterator(
							x, y, z);
					getWriterHandler().startPolygon(null, coordIterator, crs);
					getWriterHandler().endPolygon();
					// Re-Initialize the ArrayLists for the next polygon.
					num_points = 0;
					x_coords = new ArrayList();
					y_coords = new ArrayList();
					z_coords = new ArrayList();
				}
			}
			x_coords.add(new Double(coords[0]));
			y_coords.add(new Double(coords[1]));
			z_coords.add(new Double(coords[2]));
			iterator.next();
			num_points++;
		}
		double[] x = new double[num_points];
		double[] y = new double[num_points];
		double[] z = new double[num_points];
		for (int i = 0; i < num_points; i++) {
			x[i] = ((Double) x_coords.get(i)).doubleValue();
			y[i] = ((Double) y_coords.get(i)).doubleValue();
			z[i] = ((Double) z_coords.get(i)).doubleValue();
		}
		WriterPointsIterator coordIterator = new WriterPointsIterator(x, y, z);
		getWriterHandler().startPolygon(null, coordIterator, crs);
		getWriterHandler().endPolygon();
		if (isMultiGeometry)
			getWriterHandler().endMultiPolygon();
	}

	public void setLayer(FLyrVect lyr) {
		this.layer = lyr;
	}

	/**
	 * Each test must to return its parser name to register it before to start
	 * the parsing process
	 */
	public String getGPEWriterHandlerName() {
		return "";
	}

	/**
	 * Each test must to return its parser description to register it before to
	 * start the parsing process
	 */
	public String getGPEWriterHandlerDescription() {
		return "default writer handler description";
	}

	/**
	 * Gets the file format. The deafult writer format will be used by default
	 * 
	 * @return
	 */
	public String getFormat() {
		return null;
	}

	/**
	 * Gets the file format. The deafult writer version will be used by default
	 * 
	 * @return
	 */
	public String getVersion() {
		return null;
	}

	/**
	 * Closes a polygon
	 * 
	 * @param x
	 *            Polygon coordinates
	 * @return A closed polygon
	 */
	protected double[] closePolygon(double[] x) {
		double[] xClosed = new double[x.length + 1];
		System.arraycopy(x, 0, xClosed, 0, x.length);
		xClosed[xClosed.length - 1] = x[0];
		return xClosed;
	}

	/**
	 * @return the handler
	 */
	public GPEWriterHandler getWriterHandler() {
		if (writerHandler == null) {
			try {
				writerHandler = GPEFactory
						.createWriter(getGPEWriterHandlerName());
				writerHandler.setFormat(getFormat());
				writerHandler.setVersion(getVersion());
				writerHandler.setErrorHandler(getErrorHandler());
			} catch (Exception e) {
				// never throwed
				e.printStackTrace();
			}
		}
		return writerHandler;
	}

	/**
	 * @return the errorHandler
	 */
	public GPEErrorHandler getErrorHandler() {
		if (errorHandler == null) {
			errorHandler = new GPEErrorHandlerTest();
		}
		return errorHandler;
	}

	/**
	 * Sets if the writer will store all the layer or the new changes
	 * 
	 * @param partial
	 */
	public void setWritePartialLayer(boolean partial) {
		this.onlyNewChanges = partial;
	}

	/**
	 * 
	 */
	private void exportOnlyChanges() {
		VectorialEditableAdapter vea = (VectorialEditableAdapter) layer
				.getSource();
		HashMap hm = vea.getAddRowInHM();
		Object[] keys = hm.keySet().toArray(new Integer[0]);

		for (int i = 0; i < keys.length; i++) {

			try {

				GPEFeature feat = new GPEFeature();
				GPEGeometry geom = null;
				System.out.println(i);
				IFeature feature = (IFeature) ((VectorialEditableAdapter) layer
						.getSource()).getRow(((Integer) keys[i]).intValue())
						.getLinkedRow();
				geom = new GPEGeometry("the_geom", feature.getGeometry(), layer
						.getProjection().getAbrev());
				feat.setGeometry(geom);

				Hashtable elements = new Hashtable();
				int numCampos = layer.getSource().getRecordset()
						.getFieldCount();
				int j = 0;

				while (j < numCampos) {
					String nombre = layer.getSource().getRecordset()
							.getFieldName(j);
					Value data = feature.getAttribute(j);
					GPEElement elem = new GPEElement(nombre, data);
					elements.put(nombre, elem);
					j++;
				}
				feat.setElements(elements);
				writeFeature(feat);
			} catch (IOException e) {
				e.printStackTrace();
			} catch (DriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (FMapDriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}
}
