package org.gvsig.gpe.gpx.parser;

import org.gvsig.gpe.gpx.exceptions.GpxBodyParseException;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This class must be implemented by all the classes that
 * implements a GPX version parser 
 */
public abstract class AbstractGpxParser {
	
	private IXmlStreamReader parser = null;
	private GPEDefaultGpxParser handler = null;
	
	public AbstractGpxParser(IXmlStreamReader parser, GPEDefaultGpxParser handler) {
		super();
		this.parser = parser;
		this.handler = handler;
	}

	/**
	 * @return 
	 * the supported version
	 */
	public abstract String getVersion();

	/**
	 * Initialize the parsing process
	 */
	public abstract void initParse() throws GpxBodyParseException;
	
	/**
	 * @return the handler
	 */
	public GPEDefaultGpxParser getHandler() {
		return handler;
	}

	/**
	 * @return the parser
	 */
	public IXmlStreamReader getParser() {
		return parser;
	}

}
