package org.gvsig.gpe.gpx.parser.v11.coordinates;

import java.io.IOException;
import java.util.StringTokenizer;

import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.parser.GPEDefaultGpxParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.utils.StringUtils;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

/**
 * This class parses coordinates from a wptType tag. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;wpt&gt;lan="60.0" lon="60.0"&lt;/wpt&gt;
 * </code>
 * </pre>
 * </p> 
 * @see http://www.topografix.com/GPX/1/1/#type_wptType
 */
public class CoordinatesTypeIterator extends GpxCoordinatesIterator{
	private StringTokenizer coordinatesString = null;	
	
	/**
	 * It parses the coordinates from a waypoint tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @throws XmlStreamException
	 * @throws IOException
	 */
	public void initialize(IXmlStreamReader parser,
			GPEDefaultGpxParser handler, QName lastTag)
			throws XmlStreamException, IOException {
		super.initialize(parser, handler, lastTag);
						
		String coordinatesTag = handler.getProfile().getPointBinding().parseCoordinates(parser, handler);
		if (dimension == -1){
			dimension = caculateDimension(coordinatesTag);
		}						
		coordinatesString = new StringTokenizer(coordinatesTag);
		return;		
	
	}
	
	/**
	 * Calculates the dimension from a array of coordinates
	 * @param coordinatesTag
	 * @return
	 */
	private int caculateDimension(String coordinatesTag) {
		String firstPair;
		StringTokenizer st = new StringTokenizer(coordinatesTag);
		if (st.hasMoreTokens()){
			firstPair = st.nextToken();
		}else{			
			firstPair = coordinatesTag;
		}		
		return StringUtils.splitString(firstPair, GpxTags.COORDINATES_SEPARATOR).length;		
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.parser.ICoordinateIterator#hasNext()
	 */
	public boolean hasNext() throws IOException {
		return coordinatesString.hasMoreTokens();	
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.parser.ICoordinateIterator#next(double[])
	 */
	public void next(double[] buffer) throws IOException {
		String next = coordinatesString.nextToken();
		String[] coordinates = org.gvsig.gpe.utils.StringUtils.splitString(next.trim(),GpxTags.COORDINATES_SEPARATOR);
		for (int i=0 ; i<coordinates.length ; i++){
			buffer[i] = handler.getProfile().getDoubleBinding().parse(coordinates[i]);
		}	
	}


}
