package org.gvsig.gpe.gpx.parser.v11.coordinates;

import java.io.IOException;
import java.util.Hashtable;

import javax.xml.namespace.QName;

import org.gvsig.gpe.gpx.parser.GPEDefaultGpxParser;
import org.gvsig.gpe.gpx.utils.GPXUtilsParser;
import org.gvsig.gpe.gpx.utils.GpxTags;
import org.gvsig.gpe.xml.stream.IXmlStreamReader;
import org.gvsig.gpe.xml.stream.XmlStreamException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This class parses a bounds tag inside a metadata tag. Example:
 * <p>
 * <pre>
 * <code> 
 * &lt;bounds minlat="0.0" minlon = "0.0" maxlat = "0.0" maxlon = "0.0"&gt;
 * &lt;/bounds&gt;
 * </code>
 * </pre>
 * </p> 
 * @see http://www.topografix.com/GPX/1/1/#type_boundsType
 */
public class LatLonBoxIterator extends GpxCoordinatesIterator{
	double[] min = null;
	double[] max = null;
	int iterations = 0;
	
	/**
	 * It parses the bounds tag
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A Bounding box
	 * @throws IOException 
	 * @throws XmlStreamException 
	 */
	public  Object parse(IXmlStreamReader parser,GPEDefaultGpxParser handler) throws XmlStreamException, IOException {
		boolean endFeature = false;
		int currentTag;
		Object bbox = null;
		min = new double[3];
		max = new double[3];
		double rotation;
		iterations = 0;
		dimension = 2;
		
		//String id = handler.getProfile().getGeometryBinding().getID(parser, handler);
		String id = null;
		
		for (int i=0 ; i<parser.getAttributeCount(); i++){			
			if (parser.getAttributeName(i).getLocalPart().compareTo(GpxTags.MINLAT) == 0){
				min[1] = Double.parseDouble(parser.getAttributeValue(i));
			}else if (parser.getAttributeName(i).getLocalPart().compareTo(GpxTags.MAXLAT) == 0){
				max[1] = Double.parseDouble(parser.getAttributeValue(i));
			}else if (parser.getAttributeName(i).getLocalPart().compareTo(GpxTags.MINLONG) == 0){
				min[0] = Double.parseDouble(parser.getAttributeValue(i));		
			}else if (parser.getAttributeName(i).getLocalPart().compareTo(GpxTags.MAXLONG) == 0){
				max[0] = Double.parseDouble(parser.getAttributeValue(i));
			}
		}		
		
		bbox = handler.getContentHandler().startBbox(id, this, GpxTags.DEFAULT_SRS);
		handler.getContentHandler().endBbox(bbox);
		
		return bbox;	
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.parser.ICoordinateIterator#hasNext()
	 */
	public boolean hasNext() throws IOException {
		if (iterations < 2){
			return true;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see org.gvsig.gpe.parser.ICoordinateIterator#next(double[])
	 */
	public void next(double[] buffer) throws IOException {
		if (iterations == 0){
			for (int i=0 ; i<buffer.length ; i++){
				buffer[i] = min[i];
			}
			iterations = 1;
			return;
		}	
		if (iterations == 1){
			for (int i=0 ; i<buffer.length ; i++){
				buffer[i] = max[i];
			}
			iterations = 2;
		}
	}
}
