/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *
 */

package gnu.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * @author juangui
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public abstract class SerialPort extends CommPort {

	protected InputStream is;
	protected OutputStream os;

	/** 5 data bit format.
	 */
	public static final int DATABITS_5 = 5;

    /** 6 data bit format.
     */
    public static final int DATABITS_6 = 6;

    /** 7 data bit format.
     */
    public static final int DATABITS_7 = 7;

    /** 8 data bit format.
     */
    public static final int DATABITS_8 = 8;

    /** Number of STOP bits - 1.
     */
    public static final int STOPBITS_1 = 1;

    /** Number of STOP bits - 2.
     */
    public static final int STOPBITS_2 = 2;

    /** Number of STOP bits - 1-1/2. Some UARTs permit 1-1/2 STOP bits only
     * with 5 data bit format, but permit 1 or 2 STOP bits with any format.
     */
    public static final int STOPBITS_1_5 = 3;

    /** No parity bit.
     */
    public static final int PARITY_NONE = 0;

    /** ODD parity scheme. The parity bit is added so there are an odd number
     * of TRUE bits.
     */
    public static final int PARITY_ODD = 1;

     /** EVEN parity scheme. The parity bit is added so there are an even number
     * of TRUE bits.
     */
    public static final int PARITY_EVEN = 2;

    /** MARK parity scheme.
     */
    public static final int PARITY_MARK = 3;

    /** SPACE parity scheme.
     */
    public static final int PARITY_SPACE = 4;

    /** Flow control off.
     */
    public static final int FLOWCONTROL_NONE = 0;

    /** RTS/CTS flow control on input.
     */
    public static final int FLOWCONTROL_RTSCTS_IN = 1;

    /** RTS/CTS flow control on output.
     */
    public static final int FLOWCONTROL_RTSCTS_OUT = 2;

    /** XON/XOFF flow control on input.
     */
    public static final int FLOWCONTROL_XONXOFF_IN = 4;

    /** XON/XOFF flow control on output.
     */
    public static final int FLOWCONTROL_XONXOFF_OUT = 8;

    /**
     * Constructor. SerialPort is an abstract class so it can't be instantiated
     * Instead of that you should use the open() method from CommPortIdentifier class
     */
    public SerialPort() {
    }

    /**
     * Gets the currently configured baud rate.
     * @return integer value indicating the baud rate
     */
    public abstract int getBaudRate();

    /**
     * Gets the currently configured number of data bits.
     * @return integer that can be equal to DATABITS_5, DATABITS_6, DATABITS_7,
     * or DATABITS_8
     */
    public abstract int getDataBits();

    /**
     * Gets the currently defined stop bits.
     * @return integer that can be equal to STOPBITS_1, STOPBITS_2, or STOPBITS_1_5
     */
    public abstract int getStopBits();

    /**
     * Get the currently configured parity setting.
     * @return integer that can be equal to PARITY_NONE, PARITY_ODD, PARITY_EVEN,
     * PARITY_MARK or PARITY_SPACE.
     */
    public abstract int getParity();

    /**
     * Sends a break of millis milliseconds duration. Note that it may not be
     * possible to time the duration of the break under certain Operating Systems.
     * Hence this parameter is advisory.
     * @param millis - duration of break to send
     */
    public abstract void sendBreak(int millis);

    /**
     * Sets the flow control mode.
     * @param flowcontrol - control Can be a bitmask combination of
     * - FLOWCONTROL_NONE: no flow control
     * - FLOWCONTROL_RTSCTS_IN: RTS/CTS (hardware) flow control for input
     * - FLOWCONTROL_RTSCTS_OUT: RTS/CTS (hardware) flow control for output
     * - FLOWCONTROL_XONXOFF_IN: XON/XOFF (software) flow control for input
     * - FLOWCONTROL_XONXOFF_OUT: XON/XOFF (software) flow control for output
     *
     * @throws UnsupportedCommOperationException - if any of the flow control mode
     * was not supported by the underline OS, or if input and output flow control
     * are set to different values, i.e. one hardware and one software.
     * The flow control mode will revert to the value before the call was made.
     */
    public abstract void setFlowControlMode(int flowcontrol)
                                     throws UnsupportedCommOperationException;

    /**
     * Gets the currently configured flow control mode.
     * @return an integer bitmask of the modes FLOWCONTROL_NONE,
     * FLOWCONTROL_RTSCTS_IN, FLOWCONTROL_RTSCTS_OUT, FLOWCONTROL_XONXOFF_IN,
     * and FLOWCONTROL_XONXOFF_OUT.
     */
    public abstract int getFlowControlMode();

    /**
     * Deprecated. This was advisory only.
     * Set the Receive Fifo trigger level If the uart has a FIFO and if it can
     * have programmable trigger levels, then this method will cause the uart
     * to raise an interrupt after trigger bytes have been received.
     * @param trigger - level
     */
    public void setRcvFifoTrigger(int trigger){
    }

    /**
     * Sets serial port parameters.
     * @param baudrate - If the baudrate passed in by the application is unsupported
     * by the driver, the driver will throw an UnsupportedCommOperationException
     * @param dataBits - One of the following constants:
     * - DATABITS_5: 5 bits
     * - DATABITS_6: 6 bits
     * - DATABITS_7: 7 bits
     * - DATABITS_8: 8 bits
     * @param stopBits - One of the following constants:
     * - STOPBITS_1: 1 stop bit
     * - STOPBITS_2: 2 stop bits
     * - STOPBITS_1_5: 1.5 stop bits
     * @param parity - One of the following constants:
     * - PARITY_NONE: no parity
     * - PARITY_ODD: odd parity
     * - PARITY_EVEN: even parity
     * - PARITY_MARK: mark parity
     * - PARITY_SPACE: space parity
     * @throws UnsupportedCommOperationException - if any of the above parameters
     * are specified incorrectly. All four of the parameters will revert to the
     * values before the call was made.
     *
     * DEFAULT: 9600 baud, 8 data bits, 1 stop bit, no parity
     */
    public abstract void setSerialPortParams(int baudrate,
                                             int dataBits,
                                             int stopBits,
                                             int parity)
                                      throws UnsupportedCommOperationException;

    /**
     * Sets or clears the DTR (Data Terminal Ready) bit in the UART, if supported
     * by the underlying implementation.
     * @param dtr
     * - true: set DTR
     * - false: clear DTR
     */
    public abstract void setDTR(boolean dtr);

    /**
     * Gets the state of the DTR (Data Terminal Ready) bit in the UART, if supported by the underlying implementation.
     * @return
     */
    public abstract boolean isDTR();

    /**
     * Sets or clears the RTS (Request To Send) bit in the UART, if supported by
     * the underlying implementation.
     * @param rts
     * - true: set RTS
     * - false: clear RTS
     */
    public abstract void setRTS(boolean rts);

    /**
     * Gets the state of the RTS (Request To Send) bit in the UART, if supported
     * by the underlying implementation.
     * @return
     */
    public abstract boolean isRTS();

    /**
     * Gets the state of the CTS (Clear To Send) bit in the UART, if supported
     * by the underlying implementation.
     * @return
     */
    public abstract boolean isCTS();

    /**
     * Gets the state of the DSR (Data Set Ready) bit in the UART, if supported
     * by the underlying implementation.
     * @return
     */
    public abstract boolean isDSR();

    /**
     * Gets the state of the RI (Ring Indicator) bit in the UART, if supported
     * by the underlying implementation.
     * @return
     */
    public abstract boolean isRI();

    /**
     * Gets the state of the CD (Carrier Detect) bit in the UART, if supported
     * by the underlying implementation.
     * @return
     */
    public abstract boolean isCD();

    /**
     * Registers a SerialPortEventListener object to listen for SerialEvents.
     * Interest in specific events may be expressed using the notifyOnXXX calls.
     * The serialEvent method of SerialPortEventListener will be called with a
     * SerialEvent object describing the event.
     *
     * The current implementation only allows one listener per SerialPort.
     * Once a listener is registered, subsequent call attempts to addEventListener
     * will throw a TooManyListenersException without effecting the listener
     * already registered.
     *
     * All the events received by this listener are generated by one dedicated
     * thread that belongs to the SerialPort object. After the port is closed,
     * no more event will be generated. Another call to open() of the port's
     * CommPortIdentifier object will return a new CommPort object, and the lsnr
     * has to be added again to the new CommPort object to receive event from
     * this port.
     *
     * @param lsnr - The SerialPortEventListener object whose serialEvent method
     * will be called with a SerialEvent describing the event.
     * @throws java.util.TooManyListenersException - If an initial attempt
     * to attach a listener succeeds, subsequent attempts will throw
     * TooManyListenersException without effecting the first listener.
     */
    public abstract void addEventListener(SerialPortEventListener lsnr)
                                   throws java.util.TooManyListenersException;

    /**
     * Deregisters event listener registered using addEventListener.
     * This is done automatically at port close.
     */
    public abstract void removeEventListener();

    /**
     * Expresses interest in receiving notification when input data is available.
     * This may be used to drive asynchronous input. When data is available in the
     * input buffer, this event is propagated to the listener registered using
     * addEventListener.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnDataAvailable(boolean enable);

    /**
     * Expresses interest in receiving notification when the output buffer is empty.
     * This may be used to drive asynchronous output. When the output buffer becomes
     * empty, this event is propagated to the listener registered using
     * addEventListener. The event will be generated after a write is completed,
     * when the system buffer becomes empty again.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     *
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnOutputEmpty(boolean enable);

    /**
     * Expresses interest in receiving notification when the CTS (Clear To Send)
     * bit changes.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnCTS(boolean enable);

    /**
     * Expresses interest in receiving notification when the DSR (Data Set Ready)
     * bit changes.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnDSR(boolean enable);

    /**
     * Expresses interest in receiving notification when the RI (Ring Indicator)
     * bit changes.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnRingIndicator(boolean enable);

    /**
     * Expresses interest in receiving notification when the CD (Carrier Detect)
     * bit changes.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnCarrierDetect(boolean enable);

    /**
     * Expresses interest in receiving notification when there is an overrun error.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnOverrunError(boolean enable);

    /**
     * Expresses interest in receiving notification when there is a parity error.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     *
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnParityError(boolean enable);

    /**
     * Expresses interest in receiving notification when there is a framing error.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     *
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnFramingError(boolean enable);

    /**
     * Expresses interest in receiving notification when there is a break
     * interrupt on the line.
     *
     * This notification is hardware dependent and may not be supported by all
     * implementations.
     * @param enable
     * - true: enable notification
     * - false: disable notification
     */
    public abstract void notifyOnBreakInterrupt(boolean enable);

	/**
	 * Returns an input stream. This is the only way to receive data from the
	 * communications port. If the port is unidirectional and doesn't support
	 * receiving data, then getInputStream returns null.
	 * @return InputStream object that can be used to read from the port
	 * @throws java.io.IOException - if an I/O error occurred
	 */
    public InputStream getInputStream() throws IOException {
		// TODO Auto-generated method stub
		return is;
	}

	/**
	 * Returns an output stream. This is the only way to send data to the c
	 * ommunications port. If the port is unidirectional and doesn't support
	 * sending data, then getOutputStream returns null.
	 * @return OutputStream object that can be used to write to the port
	 * @throws java.io.IOException - if an I/O error occurred
	 */
	public OutputStream getOutputStream() throws IOException {
		// TODO Auto-generated method stub
		return os;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#enableReceiveThreshold(int)
	 */
	public void enableReceiveThreshold(int thresh)
			throws UnsupportedCommOperationException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#disableReceiveThreshold()
	 */
	public void disableReceiveThreshold() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#isReceiveThresholdEnabled()
	 */
	public boolean isReceiveThresholdEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#getReceiveThreshold()
	 */
	public int getReceiveThreshold() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#enableReceiveTimeout(int)
	 */
	public void enableReceiveTimeout(int rcvTimeout)
			throws UnsupportedCommOperationException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#disableReceiveTimeout()
	 */
	public void disableReceiveTimeout() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#isReceiveTimeoutEnabled()
	 */
	public boolean isReceiveTimeoutEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#getReceiveTimeout()
	 */
	public int getReceiveTimeout() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#enableReceiveFraming(int)
	 */
	public void enableReceiveFraming(int framingByte)
			throws UnsupportedCommOperationException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#disableReceiveFraming()
	 */
	public void disableReceiveFraming() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#isReceiveFramingEnabled()
	 */
	public boolean isReceiveFramingEnabled() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#getReceiveFramingByte()
	 */
	public int getReceiveFramingByte() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#setInputBufferSize(int)
	 */
	public void setInputBufferSize(int size) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#getInputBufferSize()
	 */
	public int getInputBufferSize() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#setOutputBufferSize(int)
	 */
	public void setOutputBufferSize(int size) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.dinopolis.util.io.CommPort#getOutputBufferSize()
	 */
	public int getOutputBufferSize() {
		// TODO Auto-generated method stub
		return 0;
	}

}
